###################################
#
# CFME Automate Method: ServiceTemplateProvisionRequest_Pending
#
# Notes: This method is used to email the service requester that
# the Service request was not auto-approved
#
# Event: request_pending
#
# Model Notes:
# 1. to_email_address - used to specify an email address in the case where the
#    requester does not have a valid email address. To specify more than one email
#    address separate email address with commas. (I.e. admin@company.com,user@company.com)
# 2. from_email_address - used to specify an email address in the event the
#    requester replies to the email
# 3. signature - used to stamp the email with a custom signature
#
###################################
require 'socket'

begin
  # Method for logging
  def log(level, message)
    @method = 'ServiceTemplateProvisionRequest_Pending'
    $evm.log(level, "#{@method} - #{message}")
  end

  log(:info, "EVM Automate Method Started")

  # dump_root
  def dump_root()
    log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
    $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
    log(:info, "Root:<$evm.root> End $evm.root.attributes")
    log(:info, "")
  end

  def get_name(name_string)
    name_array = name_string.split(',')
    name = nil
    if name_array.length > 1
      name = name_array[1]
    else
      name = name_array[0]
    end

    name
  end

  def get_patch_cycle_text(patch_cycle)
    if ( patch_cycle == "1" )
      return "First week of every month"
    elsif (patch_cycle == "2" )
      return "Second week of every month"
    else
      return "Unknown"
    end
  end

  def get_patch_window_text(patch_window)
    if ( patch_window == "9" )
      return "Tuesday 9PM"
    elsif (patch_window == "11" )
      return "Tuesday 11PM"
    else
      return "Unknown"
    end
  end

  def get_ldap_group
    begin
      miq_request = $evm.root['miq_request']
      requester = miq_request.requester
      requester_ldap_group = requester.send("ldap_group")
      return requester_ldap_group
    rescue SignalException, StandardError => e
      log(:info,"Unable to get ldap group of requester #{e}")
      return "Unknown"
    end
  end

  def lab_env?
    begin
      log(:info, "Is this a lab environment")
      hostname = Socket.gethostbyname(Socket.gethostname).first
      hostname.downcase!
      log(:info, "is this the king of the lab: #{hostname}")
      return hostname.include?("corplab")
    rescue SignalException, StandardError => e
      log(:info, "Unable to tell if this is the lab, defaulting to prod #{e}")
      return false
    end
  end

  # emailrequester
  def emailrequester(miq_request, appliance)
    log(:info, "Requester email logic starting")

    # get request options
    options = miq_request.options[:dialog]

    # Get requester object
    requester = miq_request.requester

    # Get requester email else set to nil
    requester_email = requester.email || nil

    # Get Owner Email else set to nil
    owner_email = miq_request.options[:owner_email] rescue nil
    options_owner_email = options['dialog_option_0_owner_email'] rescue nil
    log(:info, "Requester email:<#{requester_email}> Owner Email:<#{owner_email}> Request Option Owner email: <#{options_owner_email}")

    # if to is nil then use requester_email or owner_email
    to = nil
    to ||= requester_email || owner_email || options_owner_email || miq_request.userid

    # Also send message to CF Engineering  (engineering_email_address)
    engineering_email_address = nil
    engineering_email_address ||= $evm.object['engineering_email_address']
    engineering_email_address ||= "TechCloudforms_Support@intuit.com"
    to += ",#{engineering_email_address}"

    # Also send message to CF Operations  (operations_email_address)
    operations_email_address = $evm.object['operations_email_address']
    to += ",#{operations_email_address}"

    # Get from_email_address from model unless specified below
    from = nil
    if lab_env?
      from ||= $evm.object['from_email_address_lab']
    else
      from ||= $evm.object['from_email_address']
    end

    # Get signature from model unless specified below
    signature = nil
    signature ||= $evm.object['signature']

    # Set email subject
    subject = "Request ID #{miq_request.id} - Your Service Request is Pending Approval"

    # Build email body
    name = get_name(miq_request.requester_name)
    patch_cycle = get_patch_cycle_text(options['dialog_tag_0_intuit_patch_cycle'])
    patch_window = get_patch_window_text(options['dialog_tag_0_intuit_patch_window'])

    # Capture the BU used for the provisioning request
    bu = get_ldap_group

    body = "Hello, #{name}"
    body += "<br><br>Your #{miq_request.message}"
    body += "<br><br>Please review your Request and wait for approval from an Administrator."
    body += "<br><br>Until approved, you can visit the following link to delete your request"
    body += "<br>https://cloudforms.intuit.net/miq_request/show/#{miq_request.id}"

    body += "<br><br>**********************************************************************************************************************
    <br><br>For reference, here is the request details:<br><br>
    <h3>Requester Information</h3><br>
    <table>
    <tr><td>Requested by:</td><td>#{miq_request.requester_name}</td></tr>
    <tr><td>Requester email:</td><td>#{miq_request.userid}</td></tr>
    <tr><td>Requester BU:</td><td>#{bu}</td></tr>
    </table>

    <h3>Service Information</h3><br>
    <table><tr><td>VM Service Group:</td><td>#{options['dialog_service_name']}</td></tr>
    <tr><td>Description:</td><td>#{options['dialog_service_description']}</td></tr>
    <tr><td>Project Abbreviation:</td><td>#{options['dialog_option_0_project_acronym']}</td></tr>
    <tr><td>Patch Cycle:</td><td>#{patch_cycle}</td></tr>
    <tr><td>Patch Window:</td><td>#{patch_window}</td></tr>
    <tr><td>Notification email:</td><td>#{options['dialog_option_0_owner_email']}</td></tr>
    <tr><td>Application Name:</td><td>#{options['dialog_option_0_application_name']}</td></tr>
    <tr><td>Execution Zone:</td><td>#{options['dialog_option_0_execution_zone']}</td></tr>
    <tr><td>Environment:</td><td>#{options['dialog_option_0_environment']}</td></tr>
    <tr><td>Sox Compliant:</td><td>#{return_check_value(options['dialog_option_0_sox_compliant'])}</td></tr></table><br><br>"

    body += tier_information(options)

    body += "
    <h3>Additional Request Information</h3><br>
    <table>
    <tr><td>Request ID:</td><td></td><td>#{miq_request.id}</td></tr>
    </table>"

    body += "<br><br>Thank you,"
    body += "<br>#{signature}"

    # Send email
    log(:info, "Sending email to <#{to}> from <#{from}> subject: <#{subject}>")
    $evm.execute(:send_email, to, from, subject, body)
  end

  def return_check_value(value)
    check = nil
    $evm.log("info","the value is: #{value}")
    if value == "f" || value == "0"
      check = "Unchecked"
    end

    if value == "t" || value == "1"
      check = "Checked"
    end

    check
  end

  def tier_information(options)
    body = ""
    for i in 1..3
      if options['dialog_tag_'+i.to_s+'_intuit_template']
        body += "<h3>Tier #{i} Information</h3><br>"

        body += "<table>"

        # VM names #dialog_option_1_host_names
        if !options['dialog_option_'+i.to_s+'_host_names'].blank?
          body += "<tr><td>Host Names:</td><td>#{options['dialog_option_'+i.to_s+'_host_names']}</td></tr>"
        else
          body += "<tr><td>Host Names:</td><td>Auto generated</td></tr>"
        end

        # VM IP addresses   #dialog_option_1_ip_addresses
        if !options['dialog_option_'+i.to_s+'_ip_addresses'].blank?
          body += "<tr><td>IP Addresses:</td><td>#{options['dialog_option_'+i.to_s+'_ip_addresses']}</td></tr>"
        else
          body += "<tr><td>IP Addresses:</td><td>Auto generated</td></tr>"
        end

        # QDC-A Management Network    #dialog_option_1_hosting_segment
        if options['dialog_option_1_hosting_segment'] == "qdca"
          # MNET Host Names   #dialog_option_1_mgmt_host_names
          if !options['dialog_option_'+i.to_s+'_mgmt_host_names'].blank?
            body += "<tr><td>Management: Host Names:</td><td>#{options['dialog_option_'+i.to_s+'_mgmt_host_names']}</td></tr>"
          else
            body += "<tr><td>Management Host Names:</td><td>Auto generated</td></tr>"
          end
          # MNET IP Addresses  #dialog_option_1_mgmt_ip_addresses
          if !options['dialog_option_'+i.to_s+'_mgmt_ip_addresses'].blank?
            body += "<tr><td>Management IP Addresses:</td><td>#{options['dialog_option_'+i.to_s+'_mgmt_ip_addresses']}</td></tr>"
          else
            body += "<tr><td>Management IP Addresses:</td><td>Auto generated</td></tr>"
          end
        end

        # ESX Target Hosts  #dialog_option_1_target_hosts
        if !options['dialog_option_'+i.to_s+'_target_hosts'].blank?
          body += "<tr><td>Target ESX Hosts:</td><td>#{options['dialog_option_'+i.to_s+'_target_hosts']}</td></tr>"
        else
          body += "<tr><td>Target ESX Hosts:</td><td>Auto generated</td></tr>"
        end

        body += "<tr><td>Template:</td><td>#{options['dialog_tag_'+i.to_s+'_intuit_template']}</td></tr>"
        body += "<tr><td>Instance Size:</td><td>#{options['dialog_tag_'+i.to_s+'_intuit_instance_size']}</td></tr>"
        body += "<tr><td>Hosting Segment:</td><td>#{options['dialog_option_'+i.to_s+'_hosting_segment']}</td></tr>"
        security_zone = options['dialog_option_'+i.to_s+'_security_zone'].split('_')
        body += "<tr><td>Security Zone:</td><td>#{security_zone[1]}</td></tr>"
        body += "<tr><td>Security Level:</td><td>#{security_zone[2]}</td></tr>"
        body += "<tr><td>Fault Domain:</td><td>#{security_zone[3]}</td></tr>"
        software_component = options['dialog_option_'+i.to_s+'_intuit_software_component']
          if software_component.blank?
            software_component = "OS-Only"
          end
        body += "<tr><td>Software:</td><td>#{software_component}</td></tr>"
        body += "<tr><td>Tier Name:</td><td>#{options['dialog_option_'+i.to_s+'_intuit_tier_name']}</td></tr>"
        body += "<tr><td>Create QB Record:</td><td>#{return_check_value(options['dialog_option_'+i.to_s+'_quick_base_create_shell'])}</td></tr>"
        body += "<tr><td>Number of Nodes:</td><td>#{options['dialog_option_'+i.to_s+'_number_of_vms']}</td></tr>"
        body += "<tr><td>Create Storage NIC:</td><td>#{return_check_value(options['dialog_option_'+i.to_s+'_create_storage_nic'])}</td></tr>"

        if !options['dialog_option_'+i.to_s+'_storage_host_names'].blank?
          body += "<tr><td>Storage Host Names:</td><td>#{options['dialog_option_'+i.to_s+'_storage_host_names']}</td></tr>"
        else
          body += "<tr><td>Storage Host Names:</td><td>Auto generated</td></tr>"
        end
        if !options['dialog_option_'+i.to_s+'_storage_ip_addresses'].blank?
          body += "<tr><td>Storage IP Addresses:</td><td>#{options['dialog_option_'+i.to_s+'_storage_ip_addresses']}</td></tr>"
        else
          body += "<tr><td>Storage IP Addresses:</td><td>Auto generated</td></tr>"
        end

        if options['dialog_option_'+i.to_s+'_storage_ip_addresses'].blank?
          log(:info, "Target EPG Field is BLANK" )
        else
          log(:info, "Target EPG Field : #{options['dialog_option_'+i.to_s+'_target_epg']}" )
          body += "<tr><td>Target EPG:</td><td>#{options['dialog_option_'+i.to_s+'_target_epg']}</td></tr>"
        end

        body += "</table><br><br>"

      end
    end

    body
  end

  # emailapprover
  def emailapprover(miq_request, appliance)
    log(:info, "Requester email logic starting")

    # Get request options
    options = miq_request.options[:dialog]

    # Get requester object
    requester = miq_request.requester

    # Get requester email else set to nil
    requester_email = requester.email || nil

    # Get Owner Email else set to nil
    owner_email = miq_request.options[:owner_email] || nil
    log(:info, "Requester email:<#{requester_email}> Owner Email:<#{owner_email}>")

    # Override to email address below or get to_email_address from from model
    to = nil
    to  ||= "#{$evm.object['to_email_address']},#{$evm.object['operations_email_address']},#{$evm.object['engineering_email_address']}"

    # Override from_email_address below or get from_email_address from model
    from = nil
    if lab_env?
      from ||= $evm.object['from_email_address_lab']
    else
      from ||= $evm.object['from_email_address']
    end

    # Get signature from model unless specified below
    signature = nil
    signature ||= $evm.object['signature']

    # Set email subject
    subject = "#{@method} - Request ID #{miq_request.id} - Virtual Machine is Pending Approval"

    # Build email body
    name = get_name(miq_request.requester_name)
    patch_cycle = get_patch_cycle_text(options['dialog_tag_0_intuit_patch_cycle'])
    patch_window = get_patch_window_text(options['dialog_tag_0_intuit_patch_window'])

    # Capture the BU used for the provisioning request
    bu = get_ldap_group

    body = "Hello Request Approver,
    <br><br>A new service provisioning request named #{options['dialog_service_name']} was submitted at #{miq_request.created_on} and is awaiting your approval to start provisioning.
    <br><br>Please validate that the resources needed to fulfill the request are available... note the number of nodes, requested instance sizes, and desired templates.
    <br>Remember each Windows VM requires 100GB and each RHEL VM requires 70GB
    <br><br>Once resource availability is confirmed, log into Cloudforms and approve the request.
    <br><br>If there will be a delay in approving the request, it is recommended that you notify the requester listed below of the delay.
    <br><br>To view this Request go to: https://cloudforms.intuit.net/miq_request/show/#{miq_request.id}"

    body += "<br>**********************************************************************************************************************
    <br><br>For reference, here is the request details:<br><br>
    <h3>Requester Information</h3><br>
    <table>
    <tr><td>Requested by:</td><td>#{name}</td></tr>
    <tr><td>Requester email:</td><td>#{miq_request.userid}</td></tr>
    <tr><td>Requester BU:</td><td>#{bu}</td></tr>
    </table>

    <h3>Service Information</h3><br>
    <table><tr><td>VM Service Group:</td><td>#{options['dialog_service_name']}</td></tr>
    <tr><td>Description:</td><td>#{options['dialog_service_description']}</td></tr>
    <tr><td>Project Abbreviation:</td><td>#{options['dialog_option_0_project_acronym']}</td></tr>
    <tr><td>Patch Cycle:</td><td>#{patch_cycle}</td></tr>
    <tr><td>Patch Window:</td><td>#{patch_window}</td></tr>
    <tr><td>Notification email:</td><td>#{options['dialog_option_0_owner_email']}</td></tr>
    <tr><td>Application Name:</td><td>#{options['dialog_option_0_application_name']}</td></tr>
    <tr><td>Execution Zone:</td><td>#{options['dialog_option_0_execution_zone']}</td></tr>
    <tr><td>Environment:</td><td>#{options['dialog_option_0_environment']}</td></tr>
    <tr><td>Sox Compliant:</td><td>#{return_check_value(options['dialog_option_0_sox_compliant'])}</td></tr></table><br><br>"

    body += tier_information(options)

    body += "
    <h3>Additional Request Information</h3><br>
    <table>
    <tr><td>Request ID:</td><td></td><td>#{miq_request.id}</td></tr>
    </table>"

    body += "<br><br>Thank you,"
    body += "<br>#{signature}"

    # Send email
    log(:info, "Sending email to <#{to}> from <#{from}> subject: <#{subject}>")
    $evm.execute(:send_email, to, from, subject, body)
  end

  log(:info, "CFME Automate Method Started")

  # dump all root attributes to the log
  #dump_root

  # get the request object from root
  miq_request = $evm.root['miq_request']
  log(:info, "miq_request id:<#{miq_request.id}> approval_state:<#{miq_request.approval_state} options[:dialog]:<#{miq_request.options[:dialog].inspect}>")

  # lookup the service_template object
  service_template = $evm.vmdb(miq_request.source_type, miq_request.source_id)
  log(:info, "service_template id:<#{service_template.id}> service_type:<#{service_template.service_type}> description:<#{service_template.description}> services:<#{service_template.service_resources.count}>")

  # Override the default appliance IP Address below
  appliance = nil
  #appliance ||= 'evmserver.company.com'
  appliance ||= $evm.root['miq_server'].ipaddress

  # Email Requester
  emailrequester(miq_request, appliance)

  # Email Requester
  emailapprover(miq_request, appliance)

  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

  # Ruby rescue
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_STOP
end
