###################################
#
# EVM Automate Method: vdi_add_vm_to_desktop_pool
#
# Notes: This method adds a provisioned VM to a VDI Desktop Group and
#        optionally creates the Desktop Group.
#
###################################

def vmware_view_script(create_new_group, desktopgroup_name, ems, desktopgroup_type, vms_id_list, vm_name_list)
<<PS_SCRIPT
#Add the VMware View Broker snap-in to the current Powershell session
$required_plugins = @("VMware.view.broker")
foreach ($plugin in $required_plugins) {if ((Get-PSSnapin -Name $plugin -ErrorAction SilentlyContinue) -eq $null) {Add-PSSnapin $plugin -ErrorAction Stop}}

# Define variables for the script
$create_new_group = "#{create_new_group}"
$strDesktopGroupName = "#{desktopgroup_name}"
$strDesktopGroupDescription = "New Desktop Group #{desktopgroup_name} Created by EVM"
$strAssignmentBehavior = "#{desktopgroup_type}"
$strVmsIdList = "#{vms_id_list}"
$strVmsNameList = "#{vm_name_list}"

# Virtual Center Credentials
$strHypervisorAddress = "#{ems.ipaddress}"

$result = @{"desktop_pool_name"=$strDesktopGroupName; "error"=$false}

function find_view_vc($target_ip) {
  $vc = Get-ViewVC -serverName $target_ip
  if ($vc -ne $null) {
      Get-ViewVC | ForEach-Object {
      $ip = [System.Net.Dns]::GetHostAddresses($_.serverName)
      if ($ip[0].IPAddressToString -eq $target_ip) {
        $vc = $_
      }
    }
  }

  # Add the VC if not found
  if ($vc -eq $null) {
    miq_logger "info" "Adding ViewVC : <$target_ip>"
    Add-ViewVC -serverName $target_ip -username "#{ems.authentication_userid}" -password #{ems.authentication_password_encrypted} -ErrorAction Stop
    $vc = Get-ViewVC -serverName $target_ip
  }

  return $vc
}

miq_logger "info" "Create new Desktop Pool : <$create_new_group>"
miq_logger "info" "Desktop Pool Name       : <$strDesktopGroupName>"
miq_logger "info" "Desktop Pool Description: <$strDesktopGroupDescription>"
miq_logger "info" "VM Name List            : <$strVmsNameList>"
miq_logger "info" "VM ID List              : <$strVmsIdList>"

# Find the ViewVC for this request
$vc = find_view_vc $strHypervisorAddress

if ($vc -ne $null) {
  if ($strAssignmentBehavior -eq "Pooled") {$persistence = "NonPersistent"} else {$persistence = "Persistent"}
  $new_pool = Add-ManualPool -vc_id $vc.vc_id -pool_id $strDesktopGroupName -vm_id_list $strVmsIdList -description $strDesktopGroupDescription  -persistence $persistence -ErrorAction Stop
}

$result
PS_SCRIPT
end

def citrix_4_script(create_new_group, desktopgroup_name, vm_name, hostname, ems, desktopgroup_type, desktopgroup_users, request_task_idx)
<<PS_SCRIPT
#Add the XenDesktop snap-in to the current Powershell session
$required_plugins = @("XDCommands")
foreach ($plugin in $required_plugins) {if ((Get-PSSnapin -Name $plugin -ErrorAction SilentlyContinue) -eq $null) {Add-PSSnapin $plugin -ErrorAction Stop}}

# Define variables for the script
$create_new_group = "#{create_new_group}"
$strDesktopGroupName = "#{desktopgroup_name}"
$strDesktopGroupDescription = "New Desktop Group #{desktopgroup_name} Created by EVM"
$strDesktopGroupUserList = "#{desktopgroup_users}"
$strVMName = "#{vm_name}"         #This is the virtual machine name to add to the desktop group
$strADName = "#{hostname}"         #This is the corresponding Active Directory machine name

# Virtual Center Credentials
$strHypervisorAddress =  "http://"          # Or https
$strHypervisorAddress += "#{ems.hostname}"

# To ensure a specific domain name is used set it here.  Example: ($strDomainName = ".companyname.com")
$strDomainName = ""
if ($strHypervisorAddress -inotlike "*$($strDomainName)*") {$strHypervisorAddress += $strDomainName}

$strHypervisorAddress += "/sdk"
$strHypervisorUser = "#{ems.authentication_userid}"
$strAssignmentBehavior = "#{desktopgroup_type}"
$request_task_idx = #{request_task_idx}

$result = @{"desktop_pool_name"=$strDesktopGroupName; "error"=$false}

function add_users_to_group($strDesktopGroupUserList, $xdgroup) {
  # Add desktop users to group
  $users = $strDesktopGroupUserList.split(",")

  $user_sids = @()
  foreach ($user in $xdgroup.Users) {$user_sids += $user.Sid.Value}

  foreach ($user in $users) {
    $user = $user.Trim()
    if ($user.length -ne 0) {
      $xd_user = New-XdUser $user

      # Check if the user is already in the group
      if ($user_sids -notcontains $xd_user.Sid.Value) {
        miq_logger "info" "Adding User to group: <$user>"
        $xdgroup.Users.Add($xd_user)
      }
    }
  }
}

function add_pre_assigned_user_to_vm($strDesktopGroupUserList, $xdgroup, $vm, $request_task_idx) {
  $users = $strDesktopGroupUserList.split(",")
  $user  = $users[$request_task_idx]
  if ($user -ne $null) {
    $user  = $user.Trim()
    if ($user.length -ne 0) {
      miq_logger "info" "Processing User IDX: $($request_task_idx) <$($user)>"
      $xd_user = New-XdUser -Name $user

      # Check if the user is already in the group and skip assignment if they are.
      $user_sids = @()
      foreach ($desktop in $xdgroup.Desktops) {if ($desktop.AssignedUserSid -ne $null) {$user_sids += $desktop.AssignedUserSid.Value}}

      if ($user_sids -notcontains $xd_user.Sid.Value) {
        # Add User to VM
        if ($xd_user -ne $null) { $vm.AssignedUsername = $xd_user }
      }
      else
      {
        miq_logger "info" "User: <$($user)> already exists in the group and will not be assigned to the new VM."
      }
    }
  }
}

# Connect to the local farm and determine if the desktop group already exists
$farm = New-XdAdminConnection 127.0.0.1
$xdgroup = Get-XdDesktopGroup -Name $strDesktopGroupName -AdminConnection $farm
if ($xdgroup -eq $null) {
  if ($create_new_group -eq "true") {
    miq_logger "info" "Desktop Group <$strDesktopGroupName> does not exist.  Creating."

    # Connect to the hypervisor hosting the VM's using the supplied username/password
    $creds = new-object -typename System.Management.Automation.PSCredential -argumentlist $strHypervisorUser, (ConvertTo-SecureString #{ems.authentication_password_encrypted} -AsPlainText -Force)
    $hs = New-XdHostingServer -address $strHypervisorAddress -provider ( Get-XdHostingProvider -name "VMware virtualization" ) -credential $creds

    # Define the hosting infrastructure settings for the desktop group
    $hgs = New-XdGroupHostingSettings -hostingserver $hs

    miq_logger "info" "Creating Desktop Group:<$strDesktopGroupName>  Assignment:<$strAssignmentBehavior>"
    $xdgroup = New-XdDesktopGroup -Publish $strDesktopGroupName -Description $strDesktopGroupDescription -AdminConnection $farm -hostingsettings $hgs -AssignmentBehavior $strAssignmentBehavior
    Set-XdDesktopGroup $xdgroup | Out-Null

    miq_logger "info" "Desktop Group <$strDesktopGroupName> was created."
  }
  else
  {
    $result["error"] = $true
    $msg = "Desktop Group <$strDesktopGroupName> does not exist"
    miq_logger "error" $msg
    $result["msg"] = $msg
  }
}
else {
  miq_logger "info" "Desktop Group <$strDesktopGroupName> already exists."
}

if ($xdgroup -ne $null) {
  # If Pooled or AssignOnFirstUse we can add the users directly to the group.
  # For PreAssigned we need to add the user to the VM that gets added to the group.
  if ($strAssignmentBehavior -ieq "Pooled" -or  $strAssignmentBehavior -ieq "AssignOnFirstUse") {
    add_users_to_group $strDesktopGroupUserList $xdgroup
  }

  miq_logger "info" "Adding VM <$strVMName> to Desktop Group <$strDesktopGroupName>"
  #Get the VM (virtual desktop) that we want to add to the desktop group
  $vm = Get-XdHostedMachine -HostingServer $xdgroup.HostingSettings.HostingServer -AdminConnection $farm -Name $strVMName
  if ($vm -eq $null) { miq_logger "warn" "Get-XdHostedMachine returned a null VM handle" }

  if ($strAssignmentBehavior -ieq "PreAssigned") {
    add_pre_assigned_user_to_vm $strDesktopGroupUserList $xdgroup $vm $request_task_idx
  }

  #Create the mapping between virtual machine name and Active Directory machine name
  #Note:  When I echo the $vm variable created above, the "Name" property of the virtual machine seems to be already set for me.
  #       This might be because the mapping between VM Name and AD Name has been cached from adding this VM to other desktop groups on prior occasions.
  #       Keeping the statement below won't cause any problems in any case and it should not have to be modified.
  #       Just make sure the Active Directory machine name for this virtual machine is correct as defined above.
  miq_logger "info" "Setting vm.Name to Active Directory machine name = $($strADName)"
  $vm.Name = $strADName;

  #Add the VM (virtual desktop) to the desktop group
  $xdgroup.Desktops.Add($vm)

  #Apply the change to the DDC
  Set-XdDesktopGroup $xdgroup | Out-Null
  miq_logger "info" "VM <$strVMName> added to Desktop Group <$strDesktopGroupName>"
}

# Disconnect from the local farm
Disconnect-XdAdminConnection -AdminConnection $farm

$result
PS_SCRIPT
end

def citrix_5_script(create_new_group, desktopgroup_name, vm_name, hostname, vm_uid_ems, ems, desktopgroup_type, desktopgroup_users, request_task_idx)
<<PS_SCRIPT
# Define variables for the script
$create_new_group = "#{create_new_group}"
$strDesktopGroupName = "#{desktopgroup_name}"
$strDesktopGroupDescription = "New Desktop Group #{desktopgroup_name} Created by EVM"
$strDesktopGroupUserList = "#{desktopgroup_users}"
$strVMName = "#{vm_name}"          # This is the virtual machine name to add to the desktop group
$strADName = "#{hostname}"         # This is the corresponding Active Directory machine name
$strVMHostId = "#{vm_uid_ems}"

# Virtual Center Credentials
$strHypervisorAddress = "http://#{ems.ipaddress}/sdk"
$strHypervisorUser = "#{ems.authentication_userid}"
$strAssignmentBehavior = "#{desktopgroup_type}"
$request_task_idx = #{request_task_idx}

$result = @{"desktop_pool_name"=$strDesktopGroupName; "error"=$false}

function load_citrix_plugin($raise_error = $true, $log_result = $true) {
  $plugin_version = $null

  $requested_plugins = @("XDCommands", "Citrix.Broker.Admin.V1", "Citrix.Host.Admin.V1")
  foreach ($plugin in $requested_plugins) {if ((Get-PSSnapin -Name $plugin -ErrorAction SilentlyContinue) -eq $null) {Add-PSSnapin $plugin -ErrorAction SilentlyContinue}}

  if ((Get-PSSnapin     -Name "Citrix.Broker.Admin.V1" -ErrorAction SilentlyContinue) -ne $null) {$plugin_version = 5}
  elseif ((Get-PSSnapin -Name "XDCommands"             -ErrorAction SilentlyContinue) -ne $null) {$plugin_version = 4}

  if ($plugin_version -eq $null -and $raise_error -eq $true) {throw "No Citrix plug-in found"}
  if ($log_result) {
    if ($plugin_version -eq $null) {miq_logger "warn" "Citrix XenDesktop plugin not found"}
    else                           {miq_logger "info" "Citrix XenDesktop version $($plugin_version) plugin found"}
  }

  return $plugin_version
}

function create_desktop_group($strAssignmentBehavior, $strDesktopGroupName, $strDesktopGroupDescription, $strDesktopGroupUserList) {
  $dg = Get-BrokerDesktopGroup -Name $strDesktopGroupName -ErrorAction SilentlyContinue
  if ($dg -eq $null) {
    if ($strAssignmentBehavior -eq 'Pooled') { $desktop_group_type = 'Shared' }
    else                                     { $desktop_group_type = 'Private' }

    miq_logger "info" "Creating new Desktop Group <$($strDesktopGroupName)> with type <$($desktop_group_type)>"
    $dg = New-BrokerDesktopGroup -DesktopKind $desktop_group_type -Name $strDesktopGroupName -Description $strDesktopGroupDescription

    $apr = Get-BrokerAccessPolicyRule -Name "$($strDesktopGroupName)_Direct" -ErrorAction SilentlyContinue
    if ($apr -eq $null) {$pr = New-BrokerAccessPolicyRule -Name "$($strDesktopGroupName)_Direct" -IncludedDesktopGroups @($strDesktopGroupName) -AllowedConnections 'NotViaAG' -AllowedProtocols @('RDP','HDX') -AllowedUsers 'AnyAuthenticated' -AllowRestart $True -Enabled $True -IncludedDesktopGroupFilterEnabled $True -IncludedSmartAccessFilterEnabled $True -IncludedUserFilterEnabled $True}
    else { $apr | Set-BrokerAccessPolicyRule -AddIncludedDesktopGroups @($strDesktopGroupName)}

    $apr = Get-BrokerAccessPolicyRule -Name "$($strDesktopGroupName)_AG" -ErrorAction SilentlyContinue
    if ($apr -eq $null) {$pr = New-BrokerAccessPolicyRule -Name "$($strDesktopGroupName)_AG" -IncludedDesktopGroups @($strDesktopGroupName) -AllowedConnections 'ViaAG' -AllowedProtocols @('RDP','HDX') -AllowedUsers 'AnyAuthenticated' -AllowRestart $True -Enabled $True -IncludedDesktopGroupFilterEnabled $True -IncludedSmartAccessFilterEnabled $True -IncludedUserFilterEnabled $True}
    else {$apr | Set-BrokerAccessPolicyRule -AddIncludedDesktopGroups @($strDesktopGroupName)}
  }
  else {
    miq_logger "info" "Using existing Desktop Group <$($dg.name)> with type <$($dg.DesktopKind)>"
  }

  if ($dg.DesktopKind -icontains "Shared") {
    $policy_rule = Get-BrokerEntitlementPolicyRule -DesktopGroupUid $dg.Uid
    if ($policy_rule -eq $null) { $policy_rule = New-BrokerEntitlementPolicyRule $strDesktopGroupName -DesktopGroupUid $dg.Uid -PublishedName $strDesktopGroupName }
    add_users_to_group 'Entitlement' $strDesktopGroupUserList $policy_rule
  }
  else {
    $policy_rule = Get-BrokerAssignmentPolicyRule -DesktopGroupUid $dg.Uid
    if ($policy_rule -eq $null) { $policy_rule = New-BrokerAssignmentPolicyRule $strDesktopGroupName -DesktopGroupUid $dg.Uid -PublishedName $strDesktopGroupName }
    add_users_to_group 'Assignment' $strDesktopGroupUserList $policy_rule
  }
  return $dg
}

function add_users_to_group($policy_type, $strDesktopGroupUserList, $policy_rule) {
  $strDesktopGroupUserList.Split(",") | foreach {
    try {
      $user = $_.Trim()
      if ($user.Length -ne 0) {
        miq_logger "info" "Adding User <$user> to group"
        if ($policy_rule -is [System.Array]) {$policy_rule = $policy_rule[0]}
        if ($policy_type -eq 'Assignment') { $policy_rule | Set-BrokerAssignmentPolicyRule  -AddIncludedUsers $_ -ErrorAction SilentlyContinue }
        else                               { $policy_rule | Set-BrokerEntitlementPolicyRule -AddIncludedUsers $_ -ErrorAction SilentlyContinue }
      }
    }
    catch { miq_logger "warn" "Failed to add user: $($user).  Error: $($_)" }
  }
}

function resolve_url_to_ip($url) {
  $result = $null
  if ($url -imatch "https?://(.*)/") {
    $hosting_addr = $Matches[1]
    try { $result = [System.Net.Dns]::GetHostAddresses($hosting_addr)[0].IPAddressToString } catch {}
  }
  return $result
}

function find_connection_server($url, $username) {
  $result = $null
  $backslash_char = [char]92
  $requested_ip = resolve_url_to_ip $url
  Get-ChildItem (get-item -Path ('xdhyp:' + $backslash_char + 'Connections')).PSPath | foreach {
    if ($result -eq $null) {
      if (((resolve_url_to_ip $_.HypervisorAddress[0]) -eq $requested_ip) -and ($username.ToLower() -eq $_.UserName.ToLower())) {
        $result = Get-BrokerHypervisorConnection -HypHypervisorConnectionUid $_.HypervisorConnectionUid
      }
    }
  }
  return $result
}

function create_new_connection($url, $username) {
  $conn_name = ""
  $backslash_char = [char]92
  if ($url -imatch "https?://(.*)/") { $conn_name = $Matches[1] }
  $conn_name = $conn_name.Replace(".", "_")
  miq_logger "info" "Creating new connection server <$($conn_name)> for user <$($username)>"
  $new_conn = New-Item -Path ('xdhyp:' + $backslash_char + 'Connections') -Name $conn_name -HypervisorAddress @($url) -ConnectionType 'VCenter' -Username $username -Persist -Password 'smartvm'
  $hc = New-BrokerHypervisorConnection -HypHypervisorConnectionUid $new_conn.HypervisorConnectionUid
  return $hc
}

function get_connection_server($url, $username) {
  $hc = find_connection_server $url  $username
  if ($hc -eq $null) {$hc = create_new_connection $url $username}
  else               { miq_logger "info" "Using existing connection server <$($hc.Name)> with user credentials <$($username)>" }
  return $hc
}

function add_desktop_to_catalog($bhc, $dg, $strVMHostId, $strVMName, $strADName, $strAssignmentBehavior, $strDesktopGroupUserList, $request_task_idx) {
  $bc_name = $dg.name + " Catalog"
  $bc = Get-BrokerCatalog -Name $bc_name -ErrorAction SilentlyContinue
  if ($bc -eq $null) {
    if ($dg.DesktopKind -icontains "Shared") {$allocation_type = 'Random'}
    else                                     {$allocation_type = 'Permanent'}
    miq_logger "info" "Creating Machine Catalog <$($bc_name)> with AllocationType <$($allocation_type)>"
    $bc = New-BrokerCatalog -AllocationType $allocation_type -CatalogKind PowerManaged -Name $bc_name
  }
  else {
    miq_logger "info" "Using existing Machine Catalog <$($bc.name)> with AllocationType <$($bc.AllocationType)>"
  }

  miq_logger "info" "Adding VM <$($strVMName)> with uid <$($strVMHostId)> and machine name <$($strADName)> to catalog"
  $bm = New-BrokerMachine -HostedMachineId $strVMHostId -MachineName $strADName -CatalogUid $bc.Uid -HypervisorConnectionUid $bhc.Uid
  miq_logger "info" "Adding VM to Desktop Group"
  $add_count = Add-BrokerMachinesToDesktopGroup -Catalog $bc -DesktopGroup $dg -Count 1

  # Pre-assign a user to this VM
  if ($strAssignmentBehavior -eq "PreAssigned") {
    $users = $strDesktopGroupUserList.split(",")
    $user  = $users[$request_task_idx]
    if ($user -ne $null) {
      $user  = $user.Trim()
      if ($user.length -ne 0) {
        miq_logger "info" "Adding User IDX: $($request_task_idx) <$($user)> to Desktop <$($bm.MachineName)>"
        Add-BrokerUser -Name $user -Machine $bm.MachineName
      }
    }
  }
}

$plugin_version = load_citrix_plugin
$bhc = get_connection_server $strHypervisorAddress $strHypervisorUser
$dg = create_desktop_group $strAssignmentBehavior $strDesktopGroupName $strDesktopGroupDescription $strDesktopGroupUserList
if ($dg -is [System.Array]) {$dg = $dg[0]}
add_desktop_to_catalog $bhc $dg $strVMHostId $strVMName $strADName $strAssignmentBehavior $strDesktopGroupUserList $request_task_idx

$result
PS_SCRIPT
end


begin
  @method = 'vdi_add_vm_to_desktop_pool'
  $evm.log("info", "===== EVM Automate Method: <#{@method}> Started")

  prov = $evm.root["miq_provision"]
  $evm.log("info", "MIQ Provision Attribute info: #{prov.attributes.inspect}") if @debug

  # Determine if the Desktop tab was enabled and a valid VDI Farm selected.
  vdi_farm = prov.vdi_farm
  if vdi_farm.nil?
    exit MIQ_OK
  end

  proxy = vdi_farm.active_proxy
  if proxy.nil?
    $evm.log("error", "Could not find Active MiqProxy for VDI Farm #{vdi_farm.name}")
    exit MIQ_ABORT
  end

  # Load resources needed by the powershell script
  vdi_vendor = vdi_farm.nil? ? nil : vdi_farm.vendor
  ems = prov.vm_template.ext_management_system
  create_new_group = prov.get_option(:vdi_desktop_pool_create)
  desktopgroup_users = prov.get_option(:vdi_desktop_pool_user_list).to_s.split(',').collect{|user| user.split(' ')}.flatten.uniq.join(", ")
  request_task_idx   = prov.get_option(:pass).to_i - 1
  if create_new_group
    desktopgroup_name = prov.get_option(:vdi_new_desktop_pool_name)
    desktopgroup_type = prov.get_option(:vdi_new_desktop_pool_assignment)
    $evm.log("info", "Checking or creating Desktop Pool:  #{desktopgroup_name}")
  else
    desktopgroup_name = prov.get_option_last(:vdi_desktop_pool)
    $evm.log("info", "Checking for Desktop Pool:  #{desktopgroup_name}")
  end

  vm_name = prov.get_option(:vm_target_name).to_s.strip
  $evm.log("info", "VM Name:  #{vm_name}")

  #
  if vdi_vendor == 'citrix'

    full_hostname = prov.vm.hostnames.first
    hostname = full_hostname.to_s.split('.').first
    $evm.log("info", "VM Hostname:  #{hostname}")
    vm_uid_ems = prov.vm.uid_ems

    ps_script = if vdi_farm.version_major_minor.to_i == 4
      citrix_4_script(
                      create_new_group,
                      desktopgroup_name,
                      vm_name,
                      hostname,
                      ems,
                      desktopgroup_type,
                      desktopgroup_users,
                      request_task_idx)
    else
      citrix_5_script(
                      create_new_group,
                      desktopgroup_name,
                      vm_name,
                      hostname,
                      vm_uid_ems,
                      ems,
                      desktopgroup_type,
                      desktopgroup_users,
                      request_task_idx)
    end

  elsif vdi_vendor == 'vmware'
    vm_id_list = []
    vm_name_list = []
    ps_script = nil
    prov_req = prov.miq_provision_request

    dp_created_key = "vdi_desktop_create_initiated_#{prov_req.id}".to_sym
    unless prov_req.get_option(dp_created_key) == true
      number_of_vms_being_provisioned = prov_req.get_option(:number_of_vms).to_i

      prov_req.miq_provisions.each do |mp|
        vm = mp.vm
        vm_id_list   << "VirtualMachine-#{vm.ems_ref}" unless vm.nil?
        vm_name_list << vm.name unless vm.nil?
      end

      ps_script = nil
      if number_of_vms_being_provisioned == vm_id_list.length
        vms_id_list = vm_id_list.join(";")
        prov_req.set_option(dp_created_key, true)
        $evm.log("info", "Creating new Desktop Pool for VMs: <#{vm_name_list.inspect}>")
        ps_script = vmware_view_script(
                        create_new_group,
                        desktopgroup_name,
                        ems,
                        desktopgroup_type,
                        vms_id_list,
                        vm_name_list.join(", "))
      end
    else
      $evm.log("info", "Desktop Pool creation already initiated for this request.")
    end
  end

  unless ps_script.nil?
    $evm.log("info", "Powershell Script:\n#{ps_script}")
    result = proxy.powershell(ps_script, 'object')
    result = result.first
    $evm.log("info", "PShell Method returned: <#{result.inspect}>")
    if result[:error] == true
      $evm.log("error", result[:msg])
      exit MIQ_ABORT
    end
  else
    $evm.log("info", "Powershell script skipped on this pass.")
  end

  #
  # Exit method
  #
  $evm.log("info", "===== EVM Automate Method: <#{@method}> Ended")
  exit MIQ_OK

  #
  # Set Ruby rescue behavior
  #
rescue => err
  $evm.log("error", "#{@method} err [#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
