###################################
#
# CFME Automate Method: parse_automation_request
#
# Author: Jason Dillaman
#
# Notes: Creates a new Service Provision Request 
#
# Inputs: automation_task
#
# Mods:
#
###################################

require 'uri'

SERVICE_TEMPLATE_PROVISION_REQUEST_URI = '/Intuit/Integration/CloudForms/Actions/CreateServiceTemplateProvisionRequest'
SERVICE_TEMPLATE_NAME = 'Provision a Single Tier Environment'

def dump_root()
  $evm.log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Root:<$evm.root> End $evm.root.attributes")
  $evm.log(:info, "")
end

def escape_uri(value)
  URI.escape("#{value}", Regexp.new("[^#{URI::PATTERN::UNRESERVED}]"))
end

def error(msg)
  $evm.log(:error, msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def instantiate(uri)
  $evm.log(:info, "Instantiating: [#{uri}]")
  result = $evm.instantiate(uri)
  error("Failed to execute: [#{uri}]") unless result
  error(result['reason']) unless result['result'] == 'ok'
  $evm.log(:info, "Result of instantiate #{$evm.root['miq_provision_id']}")
  return result
end

def get_request(task)
  begin
    request = JSON.parse(URI.decode(task.options[:attrs][:request]))
    $evm.log(:info, "Processing Automation Request: [#{request}]")
    return request
  rescue JSON::ParserError => err
    error("Failed to parse request JSON")
  end
end

def process_tier(dialog_values, tier, tier_index)
  if tier['vms'].is_a?(Array)
    vms = tier['vms']
    dialog_values["option_#{tier_index}_number_of_vms"] = vms.length
    
    dialog_values["option_#{tier_index}_host_names"]   = vms['host_name'].collect { |x| x.to_s }.join(',')
    dialog_values["option_#{tier_index}_ip_addresses"] = vms['ip_address'].collect { |x| x.to_s }.join(',')
    dialog_values["option_#{tier_index}_target_hosts"] = vms['target_host'].collect { |x| x.to_s }.join(',')
  else
    dialog_values["option_#{tier_index}_number_of_vms"] = tier['vms']
    dialog_values["option_#{tier_index}_host_names"]    = tier['host_name']
    dialog_values["option_#{tier_index}_ip_addresses"]  = tier['ip_address']
    dialog_values["option_#{tier_index}_target_hosts"]  = tier['target_host']
  end
  
  dialog_values["tag_#{tier_index}_intuit_template"]              = tier['template']
  dialog_values["tag_#{tier_index}_intuit_instance_size"]         = tier['instance_size']
  dialog_values["option_#{tier_index}_hosting_segment"]           = tier['hosting_segment']
  dialog_values["option_#{tier_index}_security_zone"]             = 
    "#{tier['hosting_segment']}_#{tier['security_zone']}_#{tier['security_level']}_#{tier['fault_domain']}"
#  dialog_values["option_#{tier_index}_intuit_software_component"] = tier['']
  dialog_values["option_#{tier_index}_intuit_software_component"] = ''
  dialog_values["option_#{tier_index}_intuit_tier_name"]          = tier['name']
  dialog_values["option_#{tier_index}_quick_base_create_shell"]   = (tier['create_quick_base_shell'] ? 't' : 'f')
  dialog_values["option_#{tier_index}_create_storage_nic"]    = (tier['create_storage_network'] ? 't' : 'f')
  dialog_values["option_#{tier_index}_storage_host_names"]        = tier['storage_host_names']
  dialog_values["option_#{tier_index}_storage_ip_addresses"]      = tier['storage_ip_addresses']
  dialog_values["option_#{tier_index}_target_epg"]                = tier['target_epg']
  $evm.log(:info, "dialog_values: #{dialog_values}")
end

def process_service(service)
  dialog_values = {}
  dialog_values[:option_0_throttle_provision] = 't'
  dialog_values[:service_name]                = service['name']
  dialog_values[:service_description]         = service['description']
  dialog_values[:option_0_project_acronym]    = service['abbreviation']
  dialog_values[:tag_0_intuit_patch_cycle]    = service['patch_cycle']
  dialog_values[:tag_0_intuit_patch_window]   = service['patch_window']
  dialog_values[:option_0_owner_email]        = service['owner_email']
  dialog_values[:option_0_application_name]   = service['application_name']
  dialog_values[:option_0_execution_zone]     = service['execution_zone']
  dialog_values[:option_0_environment]        = service['environment']
  dialog_values[:option_0_sox_compliant]      = (service['sox_compliant'] ? 't' : 'f')
  tiers = service['tiers'] || []
  tier_index = 1
  tiers.collect do |tier|
    process_tier(dialog_values, tier, tier_index)
    tier_index += 1
  end
  dialog_values
end

def process_services(request)
  services = request['services'] || []
  services.collect do |service|
    $evm.root['userid'] = service['userid']
    process_service(service)
  end.flatten
end

def submit_service_template_provision_request(service_request_values)
  $evm.log(:info, "Processing Service: [#{service_request_values[:service_name]}]")
  uri = "#{SERVICE_TEMPLATE_PROVISION_REQUEST_URI}?service_template_name=#{escape_uri(SERVICE_TEMPLATE_NAME)}"
  service_request_values.each do |k,v|
    uri += "&dialog_#{k}=#{escape_uri(v)}"
  end
  instantiate(uri)
end

def submit_service_template_provision_requests(service_requests)
  service_requests.each do |service_request|
    submit_service_template_provision_request(service_request)
  end
end

def get_miqrequest(reqid)
  request = $evm.vmdb('miq_request').find_by_id(reqid)
  error("Failed to retrieve request") unless request
  $evm.log(:info, "Located request: #{reqid} submitted by: [#{request.userid}]")
  return request
end

begin
  dump_root()
  
  task = $evm.root['automation_task']
  
  request = get_request(task)
  service_requests = process_services(request)
  result = submit_service_template_provision_requests(service_requests)
  
  #miq_provision_id = $evm.root['miq_provision_id']
  #$evm.log(:info, "Updating AutomationRequest message with mig_provision_id: #{miq_provision_id}")
  #automation_request = $evm.root['automation_task'].automation_request
  #automation_request.set_message("miq_provision_id: #{miq_provision_id}")
  $evm.log(:info, "Results of bulk deploy service provision #{result}")
  
  reqid = $evm.root['miq_provision_id']
  $evm.log(:info, "Auto-Approving miq_request #{reqid}")
  miq_request = get_miqrequest(reqid)
  miq_request.approve("admin", "Auto-Approved")

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
