###################################
#
# CFME Automate Method: validate_request_schema
#
# Author: Jason Dillaman
#
# Notes: Validates the incoming bulk provision request against the JSON schema
#
# Inputs: automation_task
#         json_schema
#
# Mods:
#
###################################

require 'json'

module CaseInsensitiveHash  
  def [](key)
    key = keys.find { |s| s.casecmp(key)==0 }
    super(key)
  end
end

class SchemaError < RuntimeError
  def initialize(err)
    @err = err
  end
  
  def to_s()
    return @err
  end
end

def dump_root()
  $evm.log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Root:<$evm.root> End $evm.root.attributes")
  $evm.log(:info, "")
end

def error(msg)
  $evm.log(:error, msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def get_json_schema()
  begin
    return JSON.parse($evm.current['json_schema'])
  rescue JSON::ParserError => err
    error("Unable to parse JSON schema")
  end
end

def get_request(task)
  begin
    request = JSON.parse(URI.decode(task.options[:attrs][:request]))
    $evm.log(:info, "Processing Bulk Provision Request: [#{request}]")
    return request
  rescue JSON::ParserError => err
    error("Failed to parse bulk provision request JSON")
  end
end

def json_error(error, json_path, json_node)
  json_node = json_node.to_s
  json_node = "#{json_node.to_s[0..22]}..." if json_node.length > 25
  raise SchemaError, "#{error}[#{json_path}][#{json_node}]"
end

def validate_json_node(json_path, json_node, json_schema_node)
  if json_schema_node['enum']
    validate_json_enum_node(json_path, json_node, json_schema_node)
  elsif json_schema_node['oneOf']
    validate_json_one_of_node(json_path, json_node, json_schema_node)
  else
    case json_schema_node['type']
    when 'boolean'
      validate_json_boolean_node(json_path, json_node, json_schema_node)
    when 'string'
      validate_json_string_node(json_path, json_node, json_schema_node)
    when 'integer'
      validate_json_integer_node(json_path, json_node, json_schema_node)
    when 'object'
      validate_json_object_node(json_path, json_node, json_schema_node)
    when 'array'
      validate_json_array_node(json_path, json_node, json_schema_node)
    else
      json_error("Unsupported node type: [#{json_schema_node['type']}]", json_path, json_node)
    end
  end
end

def validate_json_enum_node(json_path, json_node, json_schema_node)
  enum_values = json_schema_node['enum'] || []
  json_error("Expecting Enum, encountered: ", json_path, json_node) unless enum_values.detect { |e| e.to_s == json_node.to_s }
end

def validate_json_one_of_node(json_path, json_node, json_schema_node)
  one_of_json_schema_nodes = json_schema_node['oneOf'] || []
  one_of_json_schema_nodes.any? do |one_of_json_schema_node|
    begin
      validate_json_node(json_path, json_node, one_of_json_schema_node)
      true
    rescue SchemaError
      false
    end
  end
end

def validate_json_boolean_node(json_path, json_node, json_schema_node)
  json_error("Expecting Boolean, encountered: [#{json_node.class}]", json_path, json_node) unless [TrueClass, FalseClass].any? { |c| json_node.is_a?(c) }
end

def validate_json_string_node(json_path, json_node, json_schema_node)
  json_error("Expecting String, encountered: [#{json_node.class}]", json_path, json_node) unless json_node.is_a?(String)
  
  min_length = json_schema_node['minlength']
  json_error("String too short: [#{json_node.length} < #{min_length}]", json_path, json_node) if min_length && json_node.length < min_length.to_i

  max_length = json_schema_node['maxlength']
  json_error("String too long: [#{json_node.length} > #{max_length}]", json_path, json_node) if max_length && json_node.length > max_length.to_i
end

def validate_json_integer_node(json_path, json_node, json_schema_node)
  json_error("Expecting Integer, encountered: [#{json_node.class}]", json_path, json_node) unless json_node.is_a?(Fixnum)

  minimum = json_schema_node['minimum']
  json_error("Integer too small: [#{json_node} < #{minimum}]", json_path, json_node) if minimum && json_node < minimum.to_i

  maximum = json_schema_node['maximum']
  json_error("Integer too large: [#{json_node} > #{maximum}]", json_path, json_node) if maximum && json_node > maximum.to_i
end

def validate_json_array_node(json_path, json_node, json_schema_node)
  json_error("Expecting Array, encountered: [#{json_node.class}]", json_path, json_node) unless json_node.is_a?(Array)

  min_items = json_schema_node['minitems']
  json_error("Too few array items: [#{json_node.length} < #{min_items}]", json_path, json_node) if min_items && json_node.length < min_items.to_i
  
  max_items = json_schema_node['maxitems']
  json_error("Too many array items: [#{json_node.length} > #{max_items}]", json_path, json_node) if max_items && json_node.length > max_items.to_i
  
  items_json_schema_node = json_schema_node['items']
  json_node.each_with_index do |item_json_node, idx| 
    $evm.log(:info, "Processing JSON Array node: [#{json_path}[#{idx}]]")
    $evm.log(:info, "Item_json_node: #{item_json_node}")
    next if item_json_node[:name].blank?
    validate_json_node("#{json_path}[#{idx}]", item_json_node, items_json_schema_node)
  end
end

def validate_json_object_node(json_path, json_node, json_schema_node)
  json_error("Expecting Hash, encountered: [#{json_node.class}]", json_path, json_node) unless json_node.is_a?(Hash)
  
  properties = json_schema_node['properties'] || []
  required = json_schema_node['required'] || []
  properties.each do |property_name, property_json_schema_node|
    $evm.log(:info, "Processing JSON Object property: [#{json_path}/#{property_name}]")
    property_required = required.include?(property_name)
    property_json_node = json_node[property_name]
    json_error("Missing required property: ", "#{json_path}/#{property_name}", json_node) if property_json_node.nil? && property_required
    validate_json_node("#{json_path}/#{property_name}", property_json_node, property_json_schema_node) if property_json_node
  end
end

def validate_json(json, json_schema)
  json.extend(CaseInsensitiveHash)
  json_schema.extend(CaseInsensitiveHash)
  begin
    validate_json_node("", json, json_schema)
  rescue SchemaError => err
    error(err.to_s)
  end
end

begin
  dump_root()
  
  json_schema = get_json_schema()
  
  task = $evm.root['automation_task']
  request = get_request(task)
  
  validate_json(request, json_schema)
  $evm.log(:info, "Validated request against JSON schema")
rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
