###################################
#
# CFME Automate Method: validate_service_now_parameters
#
# Author: Jason Dillaman
#
# Notes: Creates a new Service Provision Request 
#
# Inputs: automation_task
#         json_schema
#
# Mods:
#
###################################

require 'json'
require 'uri'

GET_CURRENT_INTUIT_DC_URI = 'Intuit/Integration/CloudForms/Actions/GetCurrentIntuitDc'
GET_SERVICE_NOW_APPLICATION_LIST_URI = '/Intuit/Integration/ServiceNow/Actions/GetApplicationList'
GET_SERVICE_NOW_EXECUTION_SUBZONE_LIST_URI = '/Intuit/Integration/ServiceNow/Actions/GetExecutionSubzoneList'
GET_SERVICE_NOW_ENVIRONMENT_LIST_URI = '/Intuit/Integration/ServiceNow/Actions/GetEnvironmentList'

def dump_root()
  $evm.log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Root:<$evm.root> End $evm.root.attributes")
  $evm.log(:info, "")
end

def error(msg)
  $evm.log(:error, msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def escape_uri(value)
  URI.escape("#{value}", Regexp.new("[^#{URI::PATTERN::UNRESERVED}]"))
end

def get_request(task)
  begin
    request = JSON.parse(URI.decode(task.options[:attrs][:request]))
    $evm.log(:info, "Processing Bulk Provision Request: [#{request}]")
    return request
  rescue JSON::ParserError => err
    error("Failed to parse bulk provision request JSON")
  end
end

def instantiate(uri)
  result = $evm.instantiate(uri)
  error("Failed to execute: [#{uri}]") unless result
  error(result['reason']) unless result['result'] == 'ok'
  return result
end

def get_intuit_dc()
  intuit_dc = instantiate(GET_CURRENT_INTUIT_DC_URI)['intuit_dc']
  error("Unable to determine current Intuit data center") if intuit_dc.blank?
  intuit_dc
end

def get_service_now_application_list(intuit_dc)
  result = instantiate("#{GET_SERVICE_NOW_APPLICATION_LIST_URI}?" \
    "intuit_dc=#{escape_uri(intuit_dc)}");
  $evm.log(:info, "ServiceNow Application List [#{result['application_list']}]")
  result['application_list']
end

def get_service_now_execution_subzone_list(intuit_dc)
  result = instantiate("#{GET_SERVICE_NOW_EXECUTION_SUBZONE_LIST_URI}?" \
    "intuit_dc=#{escape_uri(intuit_dc)}");
  $evm.log(:info, "ServiceNow ExecutionSubzone List [#{result['execution_subzone_list']}]")
  result['execution_subzone_list']
end

def get_service_now_environment_list(intuit_dc)
  result = instantiate("#{GET_SERVICE_NOW_ENVIRONMENT_LIST_URI}?" \
    "intuit_dc=#{escape_uri(intuit_dc)}");
  $evm.log(:info, "ServiceNow Environment List [#{result['environment_list']}]")
  result['environment_list']
end

def validate_service_now_parameters(request, application_list, execution_subzone_list, environment_list)
  services = request['services'] || []
  services.each do |service|
    application_name = service['application_name']
    error("Invalid ServiceNow application name: [#{application_name}]") unless application_list.detect { |x| x['name'] == application_name }
    
    execution_zone = service['execution_zone']
    error("Invalid ServiceNow execution zone: [#{execution_zone}]") unless execution_subzone_list.detect { |x| x['label'] == execution_zone }
    
    environment = service['environment']
    error("Invalid ServiceNow environement: [#{environment}]") unless environment_list.detect { |x| x['name'] == environment }
  end
end

begin
  dump_root()
    
  task = $evm.root['automation_task']
  request = get_request(task)
  
  intuit_dc = get_intuit_dc()

  application_list = get_service_now_application_list(intuit_dc)
  execution_subzone_list = get_service_now_execution_subzone_list(intuit_dc)
  environment_list = get_service_now_environment_list(intuit_dc)

  validate_service_now_parameters(request, application_list, execution_subzone_list, environment_list)
  $evm.log(:info, "Validated ServiceNow parameters from request")
rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
