###################################
#
# CFME Automate Method: add_storage_nic
#
# Author: Carsten Clasohm
#
# Notes: Acquire one storage IP addresses and 
#        add a NIC to the VM.
#
# Inputs: vm
#         storage_ip: optional
#         storage_hostname: optional
#
# Mods:
#
###################################

require 'ipaddr'
require 'uri'

EMAIL_FROM = "cloudforms@intuit.com"

# logging convenience method
def log(level, msg)
  @method = 'add_storage_nic'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, msg)
  email_error(msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def retry_method(msg, retry_time=1.minute)
  log(:warn, "Retrying in #{retry_time} seconds: [#{msg}]")
  $evm.root['ae_result'] = 'retry'
  $evm.root['ae_retry_interval'] = retry_time
  exit MIQ_OK
end

def instantiate(uri, params = {})
  result = $evm.instantiate(uri)

  return nil if result.blank? && params[:ignore_error]

  error("Failed to execute #{uri}") unless result
  
  case result['result']
  when 'error'
    error(result['reason'])
  when 'retry'
    retry_method(result['reason'])
  end
  
  result
end

def get_region_endpoint(vm)
  uri = "Intuit/Integration/CloudForms/Actions/GetRegionEndpoints?vmdb_object_type=vm&Vm::vm=#{vm.id}"
  result = instantiate(uri)
  intuit_dc = result['intuit_dc']
  ip_addresses = result['region_endpoint_ip_addresses']

  log(:info, "Mapped VM to CloudForms region: [#{vm.name}][#{intuit_dc}][#{ip_addresses}]")

  {
    :intuit_dc => intuit_dc, 
    :ip_addresses => ip_addresses
  }
end

def create_automation_request(region_endpoint, vm, storage_ip, storage_hostname, user_email)
  uri = "Intuit/Integration/CloudForms/Actions/CreateAutomationRequest?" + \
    URI.encode_www_form([['intuit_dc', region_endpoint[:intuit_dc]],
                         ['ae_namespace', $evm.current_namespace],
                         ['ae_class', $evm.current_class],
                         ['ae_instance', $evm.current_instance],
                         ['parameters', "Vm::vm=#{vm.id}|storage_ip=#{storage_ip}|storage_hostname=#{storage_hostname}|user_email=#{user_email}"]])

  region_endpoint[:ip_addresses].each do |ip_address|
    break if instantiate("#{uri}&ip_address=#{ip_address}", :ignore_error => true)
  end
end

def get_virtual_machine()
  return $evm.current['vm'] unless $evm.current['vm'].blank?
  case $evm.root['vmdb_object_type']
  when 'miq_provision'
    vm = $evm.root['miq_provision'].vm
  when 'vm'
    vm = $evm.root['vm']
  end
  raise "VM not found" unless vm
  vm
end

def get_vix_platform(vm)
  ae_uri = "Intuit/Integration/VIX/Actions/GetVixPlatform?" + \
    URI.encode_www_form([['Vm::vm', vm.id]])

  result = instantiate(ae_uri)
  vix_platform = result['vix_platform']
  $evm.log(:info, "VIX platform: [#{vix_platform}]")
  return vix_platform
end

def add_storage_nic(vm, intuit_dc, storage_ip, storage_hostname)
  hosting_segment = vm.tags(:intuit_hs).first
  security_zone = vm.tags(:intuit_sz).first
  security_level = vm.tags(:intuit_security_level).first

  # The hostname for the storage NIC is the main hostname with an "s"
  # appended.
  # Example: main hostname pprdcclos625.corplab.intuit.net,
  #          storage hostname pprdcclos625s.corplab.intuit.net
  storage_hostname = vm.name.sub(%r{^([^.]+)(.+)$}, '\1s\2') if storage_hostname.blank?

  network_db = "intuit_network_storage/#{intuit_dc}_#{hosting_segment}_#{security_zone}_#{security_level}"

  ae_uri = "Intuit/Integration/MenAndMice/Actions/AcquireIPAddress?" + \
    URI.encode_www_form([['intuit_dc', intuit_dc],
                         ['ip_address', storage_ip],
                         ['hostname', storage_hostname],
                         ['network_db', network_db]])
  result = instantiate(ae_uri)

  error("failure in AcquireIPAddress") if result.nil?

  ip_address = result['ip_address']
  netmask = result['submask']
  gateway = result['gateway']
  vmware_network = result['vmware_network']
  fqdn = result['fqdn']

  ae_uri = "Intuit/Integration/Fog/Actions/AddNIC?" + \
    URI.encode_www_form([['Vm::vm', vm.id],
                         ['vmware_network', vmware_network]])
  result = instantiate(ae_uri)

  nic_device = result['nic_device']

  log(:info, "Added NIC #{nic_device} to VM #{vm.name}, IP #{ip_address}, netmask #{netmask}")

  {
    :ip_address => ip_address,
    :netmask => netmask,
    :gateway => gateway,
    :hostname => fqdn,
    :nic_device => nic_device
  }
end

def set_custom_attribute(vm, key, value)
  ae_uri = "Intuit/Integration/HPSA/Actions/SetCustomAttribute?Vm::vm=#{vm.id}&" + \
    URI.encode_www_form([['attr_key', key],
                         ['attr_value', value]])
  instantiate(ae_uri)
end

def apply_software_policy(vm, intuit_dc, policy_name)
  ae_uri = "Intuit/Integration/HPSA/Actions/GetSoftwarePolicies?" + \
    URI.encode_www_form([['hpsa_software_policy_filter', policy_name],
                         ['intuit_dc', intuit_dc]])
  result = instantiate(ae_uri)

  log(:info, "Discovered HPSA Software Policies: [#{result['hpsa_software_policies']}]")

  error("Cannot find HPSA software policy #{policy_name}") if result['hpsa_software_policies'].blank?

  hpsa_software_policy_id = result['hpsa_software_policies'][0][:id]

  ae_uri = "Intuit/Integration/HPSA/Actions/ApplySoftwarePolicy?Vm::vm=#{vm.id}&" + \
    URI.encode_www_form([['hpsa_software_policy_id', hpsa_software_policy_id]])
  instantiate(ae_uri)
end

def configure_storage_nic(vm, intuit_dc, nic_info)
  set_custom_attribute(vm, 'CF_STORAGE_IP_ADDRESS', nic_info[:ip_address])
  set_custom_attribute(vm, 'CF_STORAGE_NETMASK', nic_info[:netmask])
  set_custom_attribute(vm, 'CF_STORAGE_NIC', nic_info[:nic_device])
  set_custom_attribute(vm, 'CF_STORAGE_HOSTNAME', nic_info[:hostname])

  vix_platform = get_vix_platform(vm)
  policy_name = "phoenix_storage_nic_#{vix_platform}"

  apply_software_policy(vm, intuit_dc, policy_name)
end

def email_error(msg)
  vm = get_virtual_machine

  task = $evm.root['automation_task']
  if task
    user_email = task.options[:attrs][:user_email]
  else
    user_email = $evm.root['user'].email
  end

  from = EMAIL_FROM

  if user_email.blank?
    to = EMAIL_FROM
  else
    to = user_email
  end

  subject = "Failed to Add Storage Network Interface for #{vm.name}"

  body = \
    "<p>Error while adding storage network interface to virtual machine " \
    "#{vm.name}:</p>" \
    "<pre>#{msg}</pre>"

  body += "<pre>automation_task_#{$evm.root['automation_task_id']}<p>" if $evm.root['automation_task_id']

  log(:info, "Sending email to <#{to}> from <#{from}> subject: <#{subject}>")
  $evm.execute('send_email', to, from, subject, body)
end

def email_success(vm, nic_info, user_email)
  vix_platform = get_vix_platform(vm)

  ip_address = nic_info[:ip_address]
  netmask = nic_info[:netmask]
  gateway = nic_info[:gateway]
  nic_device = nic_info[:nic_device]
  hostname = nic_info[:hostname]
  short_hostname = hostname.sub(%r{\..*$}, '')

  from = EMAIL_FROM

  if user_email.blank?
    to = EMAIL_FROM
  else
    to = user_email
  end

  subject = "Storage Network Interface Added for #{vm.name}"

  body = "
<p>Storage network interface #{nic_info[:nic_device]} has been added 
to virtual machine #{vm.name} with the following parameters:</p>

<p>IP address: #{ip_address}<br>
Subnet mask: #{netmask}<br>
Hostname: #{hostname}</p>

<p>HPSA is currently configuring the IP address within the VM, this should
take no more than 2 minutes.</p>"

  body += "
<p>If the storage you are trying to access is not in the same subnet as
#{ip_address}, you have to add a static route.</p>

<p>On Linux systems, this is done by creating 
/etc/sysconfig/network-scripts/route-#{nic_device} 
with content similar to this, where \"192.168.0.0/24\" is the subnet the storage 
is located in:</p>

<pre>192.168.0.0/24 via #{gateway} dev #{nic_device}</pre>

<p>Activate the static route with the following commands:</p>

<pre>
/sbin/ifdown eth1
/sbin/ifup eth1
</pre>"

  log(:info, "Sending email to <#{to}> from <#{from}> subject: <#{subject}>")
  $evm.execute('send_email', to, from, subject, body)
end

begin
  log(:info, "Main: entering main method")

  task = $evm.root['automation_task']
  if task
    # Initiated from an automation request down to the local region.

    vm = get_virtual_machine

    storage_ip = task.options[:attrs][:storage_ip]
    storage_hostname = task.options[:attrs][:storage_hostname]
    user_email = task.options[:attrs][:user_email]

    ems = vm.ext_management_system
    intuit_dc = ems.tags(:intuit_dc).first

    nic_info = add_storage_nic(vm, intuit_dc, storage_ip, storage_hostname)
    configure_storage_nic(vm, intuit_dc, nic_info)

    email_success(vm, nic_info, user_email)
  else
    # Initiated from a custom button.

    vm = $evm.root['vm']
    user_email = $evm.root['user'].email

    # While our code can handle VMs with more than one NIC, we want to stop
    # the user from clicking the Add Storage NIC button multiple times.
    error("Virtual machine already has more than one network interface.") unless vm.hardware.nics.length == 1

    storage_ip = $evm.root['dialog_ip_address']
    storage_hostname = $evm.root['dialog_hostname']
    region_endpoint = get_region_endpoint(vm)
    create_automation_request(region_endpoint, vm, storage_ip, storage_hostname, user_email)
  end

  $evm.root['ae_result'] = 'ok'
  $evm.root['ae_reason'] = ''

  log(:info, "Main: exiting main method")
  exit MIQ_OK

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
