###################################
#
# CFME Automate Method: VMOperations
#
# This method is called via a Button/Simulation or via web services to perform operations against a VM
#
# Button/Simulation Inputs: dialog_operation
# API Inputs: dialog_operation, [dialog_vm_guid||dialog_vm_name]
#
###################################
# Method for logging
def log(level, msg, update_message=true)
  @method = 'VMOperations'
  $evm.log(level,"#{@method}: #{msg}")
  $evm.root['automation_task'].message = msg if $evm.root['automation_task'] && update_message
end

begin
  log(:info, "CFME Automate Method Started")

  # log all of the root object attributes in storage
  def dump_root
    log(:info,"Root:<$evm.root> Begin Attributes",false)
    $evm.root.attributes.sort.each { |k, v| log(:info,"Root:<$evm.root> Attributes - #{k}: #{v}",false) }
    log(:info,"Root:<$evm.root> End Attributes",false)
  end

  # basic retry logic
  def retry_method(retry_time=1.minute)
    log(:info,"Sleeping for #{retry_time} seconds")
    $evm.root['ae_result'] = 'retry'
    $evm.root['ae_retry_interval'] = retry_time
    exit MIQ_OK
  end

  # power on a vm
  def vm_start(vm)
    unless vm.power_state == 'on'
      vm.start
      log(:info,"Powering on VM:#{vm.name}")
      retry_method(10.seconds)
    else
      log(:info,"Power state for VM:#{vm.name} is on")
    end
  end

  # power off a vm
  def vm_stop(vm)
    unless vm.power_state == 'off'
      vm.stop
      log(:info,"Powering off VM:#{vm.name}")
      retry_method(10.seconds)
    else
      log(:info,"Power state for VM:#{vm.name} is off")
    end
  end

  # initiate a retirement event for a vm
  def vm_retire(vm)
    vm.retire_now
    log(:info,"Retiring VM:#{vm.name}")
  end

  # reconfigure a VMs vram, vcpu
  def vm_reconfigure(vm, new_cpu, new_mem)
    current_cpu = vm.num_cpu.to_i
    current_mem = vm.mem_cpu.to_i
    unless ( current_cpu == new_cpu or new_cpu.zero? )
      log(:info,"Reconfiguring vCPU requires VM:#{vm.name} be powered off")
      vm_stop(vm)
      reconfigure_cpu(vm, new_cpu, current_cpu)
    else
      log(:info,"vCPU count is #{new_cpu} on VM: #{vm}") unless new_cpu.zero?
    end
    unless ( current_mem == new_mem or new_mem.zero? )
      log(:info,"Reconfiguring vRAM requires VM: #{vm.name} be powered off")
      vm_stop(vm)
      reconfigure_mem(vm, new_mem, current_mem)
    else
      log(:info,"vRAM count is <#{current_mem} on VM: #{vm}") unless new_mem.zero?
    end
    vm_start(vm)
  end

  # reconfigure a VMs vCPU
  def reconfigure_cpu(vm, new_cpu, current_cpu)
    log(:info,"Setting number of vCPUs from #{current_cpu} to #{new_cpu} on VM: #{vm}")
    vm.object_send('instance_eval', "with_vim_object { | vimVm | vimVm.setNumCPUs(#{new_cpu}) }")
    retry_method(15.seconds)
  end

  # reconfigure a VMs vRAM
  def reconfigure_mem(vm, new_mem, current_mem)
    log(:info,"Setting amount of vRAM from #{current_mem} to #{new_mem} on VM: #{vm}")
    vm.object_send('instance_eval', "with_vim_object { | vimVm | vimVm.setMemory(#{new_mem}) }")
    retry_method(15.seconds)
  end

  # build_body_hash
  def build_body_hash(dialog_options_hash, auto_approve)
    (body_hash ||={})['version'] = '1.1'
    body_hash['uri_parts']  = "namespace=#{$evm.current_object.namespace}|class=#{$evm.current_object.class_name}|instance=#{$evm.current_object.instance_name}|message=create"
    body_hash['parameters'] = dialog_options_hash.collect { |k,v| "#{k.to_s}=#{v}" }.join('|')
    body_hash['requester']  = "user_name=#{$evm.root['user'].userid}|auto_approve=#{auto_approve}"
    return body_hash
  end

  # Create a SOAP call to CFME and submit an Automation Request
  def create_automation_request(soap_action, body_hash)
    # Require savon and httpi Ruby Gem
    require 'rubygems'
    gem 'savon', '=1.1.0'
    require 'savon'
    require 'httpi'

    HTTPI.log_level = :debug # changing the log level
    HTTPI.log = false
    HTTPI.adapter = :net_http # [:httpclient, :curb, :net_http]

    # Setup Savon Configuration
    Savon.configure do |config|
      config.log = false
      config.log_level = :debug # changing the log level
    end

    # Set username name below else use input from model
    username = nil
    username ||= $evm.object['username']

    # Set password below else use input from model
    password = nil
    password ||= $evm.object.decrypt('password')

    # Set CFME servername below else use input from the model or this appliances ipaddress
    servername = nil
    servername ||= $evm.object['servername']
    servername ||= $evm.root['miq_server'].ipaddress

    # Set up Savon client
    client = Savon::Client.new do |wsdl, http|
      wsdl.document = "https://#{servername}/vmdbws/wsdl"
      http.auth.basic username, password
      http.auth.ssl.verify_mode = :none
      http.read_timeout = 180
      http.open_timeout = 180
    end

    log(:info, "Calling CFME:<#{servername}> SOAP action:<#{soap_action}> SOAP Message:<#{body_hash.inspect}>")
    cfme_response = client.request soap_action do
      soap.body = body_hash
    end
    return cfme_response.to_hash
  end

  log(:info, "CFME Automate Method Started")

  # List all of the root objects attributes
  dump_root

  case $evm.root['vmdb_object_type']

  # If method is run from the API
  when 'automation_task'
    log(:info, "Detected API execution for automation task:#{$evm.root['automation_task'].id} request:#{$evm.root['automation_task'].automation_request.id}", false)
    log(:info, "Inspecting automation_task:<#{$evm.root['automation_task'].inspect}>", false)

    # set vm to nil first to see if we can find it
    vm = nil

    # Get parameters from root object
    vm_guid = $evm.root['dialog_vm_guid']
    vm_name = $evm.root['dialog_vm_name']
    operation  = $evm.root['dialog_operation']

    # If the vm_guid is not nil, try to use it.
    unless vm_guid.nil?
      # Lookup VM by GUID
      vm = $evm.vmdb('vm').find_by_guid(vm_guid)
      log(:info, "Found VM: #{vm.name} via guid:#{vm_guid}") unless vm.nil?
    end

    # If the vm is still nil then use vm_name to lookup VM
    if vm.nil?
      vm = $evm.vmdb('vm').find_by_name(vm_name)
      log(:info, "Found VM: #{vm.name} via name:#{vm_name}") unless vm.nil?
    end
    # Bail out if VM is not found
    raise "VM: #{vm.inspect} not found" if vm.nil?

  when 'vm'
    vm         = $evm.root['vm']
    operation  = $evm.root['dialog_operation']
    api        = $evm.root['dialog_api']
    log(:info, "Detected Button/Simulation execution for VM:<#{vm.name}> operation:<#{operation}> api:<#{api}>", false)
    # if api is true then kickoff an automation request
    if api =~ /(true|t|yes|y|1)$/i

      # Gather all dialog parameters that begin with dialog_ into a hash
      dialog_options_hash = Hash[$evm.root.attributes.sort.collect { |k, v| [k, v] if k.starts_with?('dialog_')}]

      # since we have the VM object put the guid and name in the parameters
      dialog_options_hash['dialog_vm_guid'] = vm.guid
      dialog_options_hash['dialog_vm_name'] = vm.name

      # Set auto_approve request to true
      auto_approve = true

      # Build a hash with the path to the instance you want to run
      body_hash = build_body_hash(dialog_options_hash, auto_approve)

      # Create automation request
      log(:info,"Creating automation request with parameters:<#{body_hash.inspect}>")
      request = create_automation_request(:create_automation_request, body_hash)
      log(:info,"Inspecting automation request:<#{request.inspect}>")
      exit MIQ_OK
    end
  else
    raise "Invalid $evm.root['vmdb_object_type']:<#{$evm.root['vmdb_object_type']}>. Skipping method."
  end # case $evm.root['vmdb_object_type']

  # Look at the operation and kick off the workflow
  case operation
  when 'start'
    vm_start(vm)
  when 'stop'
    vm_stop(vm)
  when 'retire'
    vm_retire(vm)
  when 'reconfigure'
    raise "Neither new_cpu: <#{new_cpu}> nor new_mem <#{new_mem}> was specified." unless ( new_cpu or new_mem )
    vm_reconfigure(vm, new_cpu, new_mem)
  else
    raise "Undefined operation #{operation}"
  end # case operation

  # Exit method
  log(:info, "CFME Automate Method Ended")
  $evm.root['automation_task'].finished("#{vm} #{operation} completed successfully") if $evm.root['automation_task']
  exit MIQ_OK

  # Set Ruby rescue behavior
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  $evm.root['automation_task'].finished("#{vm} #{operation} #{err}") if $evm.root['automation_task']
  exit MIQ_ABORT
end
