###################################
#
# CFME Automate Method: ServiceFlex_AddToService
#
# by Josh Carter & Kevin Morey
#
# Notes: This method MUST be ran after the PostProvision state in the provisioning StateMachine
#
# This method preforms the following:
# 1. Look up the parent service by ws_values[:serviceflex_service_id] to get the service object
# 2. Add newly provisioned vm to the parent service
# 3. Decrement parent service tag serviceflex_pending by 1
# 4. Increment parent service serviceflex_current tag by 1
# 5. Add custom attribute on the provisioned VM as to why it was provisioned. I.e. event driven or manually flexed
# 6. Tag vm with serviceflex_monitor
#
# Required inputs:
# $evm.root['miq_provision'], ws_values[:serviceflex_service_id], ws_values[:serviceflex_monitor],
#
###################################
begin
  # Method for logging
  def log(level, message)
    @method = 'ServiceFlex_AddToService'
    $evm.log(level, "#{@method}: #{message}")
  end

  # process_tags - create categories and tags
  def process_tags(category, single_value, tag)
    # Convert to lower case and replace all non-word characters with underscores
    category_name = category.to_s.downcase.gsub(/\W/, '_')
    tag_name = tag.to_s.downcase.gsub(/\W/, '_')
    log(:info, "Converted category name:<#{category_name}> Converted tag name: <#{tag_name}>")
    # if the category exists else create it
    unless $evm.execute('category_exists?', category_name)
      log(:info, "Category <#{category_name}> doesn't exist, creating category")
      $evm.execute('category_create', :name => category_name, :single_value => single_value, :description => "#{category}")
    end
    # if the tag exists else create it
    unless $evm.execute('tag_exists?', category_name, tag_name)
      log(:info, "Adding new tag <#{tag_name}> in Category <#{category_name}>")
      $evm.execute('tag_create', category_name, :name => tag_name, :description => "#{tag}")
    end
  end

  log(:info, "CFME Automate Method Started")

  # Dump all root attributes
  log(:info, "Listing Root Object Attributes:")
  $evm.root.attributes.sort.each { |k, v| log(:info, "\t#{k}: #{v}") }
  log(:info, "===========================================")

  # Get miq_provision from root
  prov = $evm.root['miq_provision']
  raise "miq_provision object not found" if prov.nil?
  log(:info, "Provision:<#{prov.id}> Request:<#{prov.miq_provision_request.id}> Type:<#{prov.type}>")

  vm = prov.vm
  raise "$evm.root['miq_provision'].vm not found" if prov.vm.nil?

  prov_tags = prov.get_tags
  log(:info, "Inspecting miq_provision tags:<#{prov_tags.inspect}>")

  if prov.options.has_key?(:ws_values)
    ws_values = prov.options[:ws_values]
    raise "Missing ws_values[:serviceflex_monitor]" if ws_values[:serviceflex_monitor].nil?
    raise "Missing ws_values[:serviceflex_service_id]" if ws_values[:serviceflex_service_id].nil?
  else
    raise "Missing ws_values - Skipping method"
  end
  log(:info, "Inspecting $evm.root['miq_provision'].options[:ws_values]:<#{ws_values.inspect}>")

  # Look up the flex service parent by prov.options[:ws_values][:serviceflex_service_id]
  parent_service = $evm.vmdb('service').find_by_id(ws_values[:serviceflex_service_id])
  raise "Parent service id:<#{ws_values[:serviceflex_service_id]}> not found" if parent_service.nil?
  log(:info, "Service:<#{parent_service.name}> vms:<#{parent_service.vms.count}> tags:<#{parent_service.tags.inspect}>")

  # Add vm to the parent service
  log(:info, "Adding VM:<#{vm.name}> to flex parent service:<#{parent_service.name}>")
  vm.add_to_service(parent_service)
  log(:info, "Service:<#{parent_service.name}> vms:<#{parent_service.vms.count}> tags:<#{parent_service.tags.inspect}>")

  # Get the serviceflex_monitor tag
  serviceflex_monitor = parent_service.tags(:serviceflex_monitor).first
  # Get the serviceflex_maximum tag and convert it to an integer
  #serviceflex_maximum = parent_service.tags(:serviceflex_maximum).first.to_i
  # Get the serviceflex_current tag and convert it to an integer
  serviceflex_current = parent_service.tags(:serviceflex_current).first.to_i
  # Get the serviceflex_pending tag and convert it to an integer
  serviceflex_pending = parent_service.tags(:serviceflex_pending).first.to_i

  # Never drop below 0
  unless serviceflex_pending.zero?
    # Decrement serviceflex_pending by 1
    new_serviceflex_pending = serviceflex_pending - 1
    # Tag parent service with new_serviceflex_pending
    unless parent_service.tagged_with?('serviceflex_pending',new_serviceflex_pending)
      # Create serviceflex_pending tags if they do not already exist
      process_tags('serviceflex_pending', true, new_serviceflex_pending)
      log(:info, "Decrementing parent_service:<#{parent_service.name}> category:<serviceflex_pending> from:<#{serviceflex_pending}> to:<#{new_serviceflex_pending}>")
      parent_service.tag_assign("serviceflex_pending/#{new_serviceflex_pending}")
    end
  end

  # Increment serviceflex_current by 1
  new_serviceflex_current = serviceflex_current + 1
  # Tag parent service with new_serviceflex_current
  unless parent_service.tagged_with?('serviceflex_current',new_serviceflex_current)
    # Create serviceflex_current tags if they do not already exist
    process_tags('serviceflex_current', true, new_serviceflex_current)
    log(:info, "Incrementing parent_service:<#{parent_service.name}> category:<serviceflex_current> from:<#{serviceflex_current}> to:<#{new_serviceflex_current}>")
    parent_service.tag_assign("serviceflex_current/#{new_serviceflex_current}")
  end

  # Add custom attribute on the provisioned VM as to why it was provisioned. I.e. event driven or manually flexed
  unless ws_values[:serviceflex_reason].nil?
    log(:info, "Adding custom attribute <:serviceflex_reason => #{ws_values[:serviceflex_reason].to_s}> to VM:<#{vm.name}>")
    vm.custom_set(:serviceflex_reason, ws_values[:serviceflex_reason].to_s)
  end

  # Tag vm with serviceflex_monitor
  unless vm.tagged_with?('serviceflex_monitor',serviceflex_monitor)
    process_tags('serviceflex_monitor', true, serviceflex_monitor)
    log(:info, "Assigning new tag:<serviceflex_monitor/#{serviceflex_monitor}> to VM:<#{vm.name}>")
    vm.tag_assign("serviceflex_monitor/#{serviceflex_monitor}")
  end

  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

  # Error Rescue
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_STOP
end
