###################################
#
# CFME Automate Method: ServiceFlex_Processing
#
# by Marco Berube & Kevin Morey
#
# Notes: This method preforms the following:
# 1. Check to ensure that vm.tags(:serviceflex_monitor) is true
# 2. Check to ensure that vm.tags(:serviceflex_current) < vm.tags(:serviceflex_maximum)
# 3. Provision new vm with parent vm parameters
#  3a. Include ws_values => serviceflex_service_id to be used by ServiceFlex_AddToService
#  3b. Include ws_values => serviceflex_reason to be used by ServiceFlex_AddToService
#  3c. Inherit :environment tags from previous provision
# 4. Increment service :serviceflex_pending tag by 1
#
# Required inputs:
# $evm.root['vm'], $evm.root['vm'].miq_provision
#
###################################
begin
  # Method for logging
  def log(level, message)
    @method = 'ServiceFlex_Processing'
    $evm.log(level, "#{@method}: #{message}")
  end

  # build_workload - Create Provision Request
  def build_workload(prov, ws_values_hash)

    # Loop through 0-999 and look to see if the name already exists in CFME
    for i in (1..999)
      new_vm_name = "#{prov.get_option(:vm_target_name)}-#{i}"
      if $evm.vmdb('vm').find_by_name(new_vm_name).nil?
        log(:info, "Using VM:<#{new_vm_name}> name")
        break
      end
    end

    # arg1 = version
    args = ['1.1']

    # arg2 = templateFields
    args << "name=#{prov.vm_template.name}|guid=#{prov.vm_template.guid}|request_type=template"

    # arg3 = vmFields
    vm_args = "vm_name=#{new_vm_name}"
    vm_args += "|vm_target_hostname=#{new_vm_name}"
    vm_args += "|cores_per_socket=#{prov.get_option(:cores_per_socket)}"
    vm_args += "|vm_memory=#{prov.get_option(:vm_memory)}"
    vm_args += "|number_of_sockets=#{prov.get_option(:number_of_sockets)}"
    vm_args += "|vlan=#{prov.get_option(:vlan)}"
    vm_args += "|provision_type=#{prov.get_option(:provision_type)}"
    vm_args += "|retirement=#{prov.get_option(:retirement)}"
    vm_args += "|retirement_warn=#{prov.get_option(:retirement_warn)}"
    args << vm_args

    # arg4 = requester
    requester_args = "owner_email=#{prov.get_option(:owner_email)}"
    requester_args += "|user_name=#{prov.userid}"
    # The default provisioning dialog requires a first and last name (Need to open an RFE for this) and you cannot set one on the admin account
    if prov.userid == 'admin'
      requester_args += "|owner_last_name=admin|owner_first_name=cfme"
    else
      requester_args += "|owner_last_name=#{prov.get_option(:owner_last_name)}|owner_first_name=#{prov.get_option(:owner_first_name)}"
    end
    args << requester_args

    # arg5 = tags
    environment = prov.get_tags[:environment]
    unless environment.nil?
      args << "environment=#{prov.get_tags[:environment]}"
    else
      args << nil
    end
    # arg6 = WS Values
    args << "#{ws_values_hash.collect { |k,v| "#{k.to_s}=#{v}" }.join('|')}"
    # arg7 = emsCustomAttributes
    args << nil
    # arg8 = miqCustomAttributes
    args << nil

    log(:info,"Building provisioning request with the following arguments: <#{args.inspect}>")
    request = $evm.execute('create_provision_request', *args)
    return request
  end

  # boolean
  def boolean(string)
    return true if string == true || string =~ (/(true|t|yes|y|1)$/i)
    return false
  end

  # process_tags - create categories and tags
  def process_tags(category, single_value, tag)
    # Convert to lower case and replace all non-word characters with underscores
    category_name = category.to_s.downcase.gsub(/\W/, '_')
    tag_name = tag.to_s.downcase.gsub(/\W/, '_')
    log(:info, "Converted category name:<#{category_name}> Converted tag name: <#{tag_name}>")
    # if the category exists else create it
    unless $evm.execute('category_exists?', category_name)
      log(:info, "Category <#{category_name}> doesn't exist, creating category")
      $evm.execute('category_create', :name => category_name, :single_value => single_value, :description => "#{category}")
    end
    # if the tag exists else create it
    unless $evm.execute('tag_exists?', category_name, tag_name)
      log(:info, "Adding new tag <#{tag_name}> in Category <#{category_name}>")
      $evm.execute('tag_create', category_name, :name => tag_name, :description => "#{tag}")
    end
  end

  log(:info, "CFME Automate Method Started")

  # Dump all root attributes
  log(:info, "Listing Root Object Attributes:")
  $evm.root.attributes.sort.each { |k, v| log(:info, "\t#{k}: #{v}") }
  log(:info, "===========================================")

  # Get vm object from root
  vm = $evm.root['vm']
  raise "$evm.root['vm'] not found" if vm.nil?

  # Get miq_provision from vm
  prov = vm.miq_provision
  raise "$evm.root['vm'].miq_provision object not found" if prov.nil?

  # log the tags from the vm
  log(:info, "Inspecting VM:<#{vm.name}> tags:<#{vm.tags.inspect}>")

  # Get the serviceflex_monitor tag from the vm
  serviceflex_monitor = vm.tags(:serviceflex_monitor).first

  # Skip processing if vm is not tagged with serviceflex_monitor=>true
  if boolean(serviceflex_monitor)

    # Get parent_service from vm
    parent_service = vm.service
    raise "vm.service not found" if parent_service.nil?

    # Dump all parent_service tags to the log
    log(:info, "Service:<#{parent_service.name}> vms:<#{parent_service.vms.count}> tags:<#{parent_service.tags.inspect}>")

    # Get the serviceflex_maximum tag else set it to zero
    serviceflex_maximum = parent_service.tags(:serviceflex_maximum).first.to_i
    # Get the serviceflex_current tag else set it to zero
    serviceflex_current = parent_service.tags(:serviceflex_current).first.to_i
    # Get the serviceflex_pending tag else set it to zero
    serviceflex_pending = parent_service.tags(:serviceflex_pending).first.to_i

    # Insert event driven information into ws_values_hash for later use in provision
    if $evm.root['object_name'] == 'Event'
      (ws_values_hash ||= {})['serviceflex_reason'] = $evm.root['miq_alert_description']
      log(:info, "Inpsecting ws_values_hash:<#{ws_values_hash.inspect}>")
    else
      # If object_name is not Event then manually add notes to ws_values_hash
      (ws_values_hash ||= {})['serviceflex_reason'] = 'This workload was flexed manually'
      log(:info, "Inpsecting ws_values_hash:<#{ws_values_hash.inspect}>")
    end

    # Create serviceflex_pending tags if they do not already exist
    process_tags('serviceflex_pending', true, serviceflex_pending)

    # If serviceflex_current + serviceflex_pending is less than serviceflex_maximum
    if serviceflex_current + serviceflex_pending < serviceflex_maximum
      log(:info, "Flex limit:<#{serviceflex_maximum}> has not been reached")

      # Increment serviceflex_pending by 1
      new_serviceflex_pending = serviceflex_pending + 1

      # Create serviceflex_pending tags if they do not already exist
      process_tags('serviceflex_pending', true, new_serviceflex_pending)

      # Insert serviceflex_monitor, serviceflex_service_id into thw ws_values_hash
      ws_values_hash['serviceflex_monitor'] = serviceflex_monitor
      ws_values_hash['serviceflex_service_id'] = parent_service.id

      # Call build_workload
      request = build_workload(prov, ws_values_hash)
      log(:info, "Submitted provision request:<#{request.id}>")
    else
      raise "ServiceFlex limit:<#{serviceflex_maximum}> has been reached"
    end

    # Tag parent service with new_serviceflex_pending
    unless parent_service.tagged_with?('serviceflex_pending',new_serviceflex_pending)
      log(:info, "Assigning new tag:<serviceflex_pending/#{new_serviceflex_pending}> to parent_service:<#{parent_service.name}>")
      parent_service.tag_assign("serviceflex_pending/#{new_serviceflex_pending}")
    end
  else
    log(:info, "parent_service.tags(:serviceflex_monitor) is set to:<#{serviceflex_monitor}>")
  end # if boolean(serviceflex_monitor)

  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

  # Error Rescue
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
