###################################
#
# CFME Automate Method: ServiceFlex_RemoveFromService
#
# by Kevin Morey
#
# Notes: This method removes the workload from the service during retirement
#
# This method preforms the following:
# 1. Decrement parent service tag :serviceflex_current by 1
# 2. Remove vm from parent service
#
# Required inputs:
# $evm.root['vm_id']
#
###################################
begin
  # Standardize logging
  def log(level, message)
    @method = 'ServiceFlex_RemoveFromService'
    $evm.log(level, "#{@method}: #{message}")
  end

  # boolean
  def boolean(string)
    return true if string == true || string =~ (/(true|t|yes|y|1)$/i)
    return false
  end

  # process_tags - create categories and tags
  def process_tags(category, single_value, tag)
    # Convert to lower case and replace all non-word characters with underscores
    category_name = category.to_s.downcase.gsub(/\W/, '_')
    tag_name = tag.to_s.downcase.gsub(/\W/, '_')
    log(:info, "Converted category name:<#{category_name}> Converted tag name: <#{tag_name}>")
    # if the category exists else create it
    unless $evm.execute('category_exists?', category_name)
      log(:info, "Category <#{category_name}> doesn't exist, creating category")
      $evm.execute('category_create', :name => category_name, :single_value => single_value, :description => "#{category}")
    end
    # if the tag exists else create it
    unless $evm.execute('tag_exists?', category_name, tag_name)
      log(:info, "Adding new tag <#{tag_name}> in Category <#{category_name}>")
      $evm.execute('tag_create', category_name, :name => tag_name, :description => "#{tag}")
    end
  end

  log(:info, "CFME Automate Method Started")

  # Dump all root attributes
  log(:info, "Listing Root Object Attributes:")
  $evm.root.attributes.sort.each { |k, v| log(:info, "\t#{k}: #{v}") }
  log(:info, "===========================================")

  # Get vm object from the VM class versus the VmOrTemplate class for vm.remove_from_service to work
  vm = $evm.vmdb("vm", $evm.root['vm_id'])
  raise "$evm.root['vm'] not found" if vm.nil?

  # Get parent_service from vm
  parent_service = vm.service
  raise "$evm.root['vm'].service not found" if parent_service.nil?
  log(:info, "Service:<#{parent_service.name}> vms:<#{parent_service.vms.count}> tags:<#{parent_service.tags.inspect}>")

  # Get the serviceflex_monitor tag
  serviceflex_monitor = parent_service.tags(:serviceflex_monitor).first
  # Get the serviceflex_maximum tag and convert it to an integer
  #serviceflex_maximum = parent_service.tags(:serviceflex_maximum).first.to_i
  # Get the serviceflex_current tag and convert it to an integer
  serviceflex_current = parent_service.tags(:serviceflex_current).first.to_i
  # Get the serviceflex_pending tag and convert it to an integer
  #serviceflex_pending = parent_service.tags(:serviceflex_pending).first.to_i

  # Only process vms with tag serviceflex_monitor is true
  if boolean(serviceflex_monitor)
    # Ensure that serviceflex_current is not 0
    unless serviceflex_current.zero?
      # Decrement serviceflex_current by 1
      new_serviceflex_current = serviceflex_current - 1

      # Tag parent service with new_serviceflex_current
      unless parent_service.tagged_with?('serviceflex_current',new_serviceflex_current)
        # Create serviceflex_current tags if they do not already exist
        process_tags('serviceflex_current', true, new_serviceflex_current)
        log(:info, "Decrementing parent_service:<#{parent_service.name}> category:<serviceflex_current> from:<#{serviceflex_current}> to:<#{new_serviceflex_current}>")
        parent_service.tag_assign("serviceflex_current/#{new_serviceflex_current}")
      end

      # Add vm to the parent vm service
      log(:info, "Removing VM:<#{vm.name}> from parent service:<#{parent_service.name}>")
      vm.remove_from_service
      log(:info, "Service:<#{parent_service.name}> vms:<#{parent_service.vms.count}> tags:<#{parent_service.tags.inspect}>")
    else
      log(:info, "Skipping method: parent_service.tags(:serviceflex_current) is set to:<#{serviceflex_current}>")
    end
  else
    log(:info, "Skipping method: parent_service.tags(:serviceflex_monitor) is set to:<#{serviceflex_monitor}>")
  end

  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

  # Error Rescue
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_STOP
end
