###################################
#
# CFME Automate Method: reconfigure_vm
#
# Author: Wayne Chatham
#
# Notes: Reconfigures a VM, based on inputs provided
#
# Inputs: vm, vcpu, vram
#
# Mods:
#
###################################

require 'rbvmomi'
require 'cgi'
VIM = RbVmomi::VIM
def connect_vcenter(cfvm="")
  # Get VC info from CF VM
  vcserver = cfvm.ext_management_system.ipaddress
  vcuser = cfvm.ext_management_system.authentication_userid
  vcpwd = cfvm.ext_management_system.authentication_password

  #Connect to VC
  vim = VIM.connect host: vcserver, insecure: true, user: vcuser, password: vcpwd
  if vim
    msg = "Connected to vCenter: #{vcserver} as: #{vcuser}"
    log(:info,msg)
  else
    msg = "ERROR: Unable to connect to vCenter: #{vcserver} as: #{vcuser}"
    error(msg)
  end
  return vim
end

def dump_root()
  log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  log(:info, "Root:<$evm.root> End $evm.root.attributes")
  log(:info, "")
end

def error(msg)
  log(:error, "#{msg}")
  task = $evm.root['automation_task']
  task.object_send(:update_and_notify_parent,
                   :state => 'finished',
                   :status => 'Error',
                   :message => "#{msg}")
  exit MIQ_OK
end

def get_vcvm(args)
  vim = args['vim']
  searchfor = args['searchfor']

  # Allow types to be a string or an array.
  types = 'VirtualMachine'
  types = [types].flatten
  startobj = vim.rootFolder
  vms = vim.serviceContent.viewManager.CreateContainerView({
      :container => startobj,
      :type => types,
      :recursive => true
      }).view
  vm = vms.select { |v| v._ref.index(searchfor) == 0 }.first
  if vm
    msg = "Located vm: #{vm.config.name} uuid: #{vm.config.instanceUuid} moref: #{vm._ref}"
    log(:info,msg)
  else
    msg = "ERROR: Unable to locate vm with moref: #{searchfor}"
    error(msg)
  end
  return vm
end

def get_cfvm()
  vm = $evm.root['vm']
  error("Failed to retrieve VM") unless vm
  log(:info, "Located VM: [#{vm.name}]")
  return vm
end

def log(level, msg)
  @method = 'reconfigure_vm'
  $evm.log(level, "#{@method}: #{msg}")
end

def reconfigure_vcvm(args)
  #Connect to the vCenter instance that manages this cf vm
  cfvm = args['cfvm']
  vmref = args['vmref']

  vim = connect_vcenter(cfvm)

  #Get the vm from vCenter
  args={}
  args['vim'] = vim
  args['searchfor'] = vmref
  vc_vm = get_vcvm(args)

  #Validate powered off or on w/ hotadd
  power_state = vc_vm.runtime.powerState #poweredOn / poweredOff
  mem_hotadd = vc_vm.config.memoryHotAddEnabled #boolean
  cpu_hotadd = vc_vm.config.cpuHotAddEnabled #boolean

  if ((!mem_hotadd || !cpu_hotadd) && power_state == "poweredOn")
    error("Memory & CPU Hotadd settings are not correctly configured / VM must be powered off for this operation.")
  end

  vmConfigSpec = VIM.VirtualMachineConfigSpec
  if $evm.root['vcpu'].to_i > 0
    vcpu = $evm.root['vcpu'].to_i
    log(:info, "Configuring number of vCPU's: #{vcpu}")
    vmConfigSpec.numCPUs = vcpu
  end

  if $evm.root['vram'].to_i > 0
    vram = $evm.root['vram']
    log(:info, "Configuring ram: #{vram}")
    vmConfigSpec.memoryMB = vram
  end

  #Reconfigure VM task
  msg = "Reconfiguring VM"
  log(:info,msg)
  task = vc_vm.ReconfigVM_Task(:spec => vmConfigSpec)

  #Wait for ReconfigureVm_Task to complete
  loop do
    if task.info.state == "error"
      msg = "Error: Reconfigure VM task failed"
      ###### Test this case #######
      error(msg)
      taskrun = false
    end
    if task.info.state != "success"
      taskrun = true
      msg ="Waiting for Reconfigure Task to complete: #{task.info.state}"
      log(:info,msg)
      sleep(5)
    else
      taskrun = false
      msg = "Reconfigure VM complete: #{task.info.state}"
      log(:info,msg)
    end
    break if (taskrun == false)
  end

end

def retry_method(msg, retry_time=1.minute)
  log(:warn, "Retrying in #{retry_time} seconds: [#{msg}]")
  $evm.root['ae_result'] = 'retry'
  $evm.root['ae_reason'] = msg.to_s
  $evm.root['ae_retry_interval'] = retry_time
  exit MIQ_OK
end

begin
  log(:info, "Entering reconfigure_vm method")
  # Turn on debugging
  @debug = true
  dump_root()

  # Get CF VM
  cfvm = get_cfvm()
  vmref = cfvm.ems_ref

  # Execute our own VC calls for these options
  if $evm.root['vcpu'] || $evm.root['vram']
    args = {}
    args['cfvm'] = cfvm
    args['vmref'] = vmref
    reconfigure_vcvm(args)
  end

  # Execute built-in methods


  $evm.root['ae_result'] = 'ok'
  $evm.root['ae_reason'] = ''
  exit MIQ_OK

end
