###################################
#
# CFME Automate Method: check_software_policy_status
#
# Author: Earl Waud (based on code from Jason Dillaman)
#
# Notes: Adds a HPSA software policy to a service or VM
#
# Inputs: service | vm: the target service or VM
#
###################################
#
# Mods:
#
###################################

require 'json'
require 'uri'

APPLY_HPSA_SOFTWARE_POLICIES_URI = 'Intuit/Integration/HPSA/Actions/ApplySoftwarePolicy'
CREATE_AUTOMATION_REQUEST_URI = 'Intuit/Integration/CloudForms/Actions/CreateAutomationRequest'
GET_REGION_ENDPOINTS_URI = 'Intuit/Integration/CloudForms/Actions/GetRegionEndpoints'

# logging convenience method
def log(level, msg)
  @method = 'check_software_policy_status'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, "#{msg}")
  $evm.current['result'] = 'error'
  $evm.current['reason'] = msg
  exit MIQ_OK
end

def escape_uri(value)
  URI.escape("#{value}", Regexp.new("[^#{URI::PATTERN::UNRESERVED}]"))
end

def dump_root()
  log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  log(:info, "Root:<$evm.root> End $evm.root.attributes")
  log(:info, "")
end

def instantiate(uri, params = {})
  log(:info, "Instantiating: [#{uri}]")
  result = $evm.instantiate(uri)
  return nil if result.blank? && params[:ignore_error]
  error("Failed to execute: [#{uri}]") unless result
  error(result['reason']) unless result['result'] == 'ok'
  return result
end

def get_virtual_machines_from_vmdb_object()
  vms = []
  case $evm.root['vmdb_object_type']
  when 'service'
    service = $evm.root['service']
    error("Service does not have any associated VMs: [#{service.name}]") if service.direct_vms.empty?

    service.direct_vms.each do |vm|
      vms << vm
    end

  when 'vm'
    vms << $evm.root['vm']
  else
    error("Unsupported Source Object: [#{$evm.root['vmdb_object_type']}]")
  end
  log(:info, "Located VMs: [#{vms.collect { |x| x.name }.join(', ')}]")
  vms
end

def group_vms_by_region_endpoint(vms)
  intuit_dc_vms = {}
  vms.each do |vm|
    uri = "#{GET_REGION_ENDPOINTS_URI}?vmdb_object_type=vm&Vm::vm=#{vm.id}"
    result = instantiate(uri)
    intuit_dc = result['intuit_dc']
    region_endpoint_ip_addresses = result['region_endpoint_ip_addresses']
    region_key = {:intuit_dc => intuit_dc, :region_endpoint_ip_addresses => region_endpoint_ip_addresses}
    region_vms = (intuit_dc_vms[region_key] ||= [])
    region_vms << vm
    log(:info, "Mapped VM to CloudForms region: [#{vm.name}][#{intuit_dc}][#{region_endpoint_ip_addresses}]")
  end
  intuit_dc_vms
end

def apply_software_policy(vm, hpsa_software_policy_id)
  log(:info, "Applying HPSA Software Policy: [#{vm.name}][#{hpsa_software_policy_id}]")
  uri = "#{APPLY_HPSA_SOFTWARE_POLICIES_URI}?" \
    "Vm::vm=#{vm.id}&" \
    "hpsa_software_policy_id=#{escape_uri(hpsa_software_policy_id)}"
  instantiate(uri)
end

def create_automation_requests(region_endpoint_vms, hpsa_software_policy_id)
  region_endpoint_vms.each do |key, vms|
    vms.each do |vm|
      uri = "#{CREATE_AUTOMATION_REQUEST_URI}?" \
        "intuit_dc=#{key[:intuit_dc]}&" \
        "ae_namespace=#{escape_uri($evm.current_namespace)}&" \
        "ae_class=#{escape_uri($evm.current_class)}&" \
        "ae_instance=#{escape_uri($evm.current_instance)}&" \
        "parameters=#{escape_uri("vm_id=#{vm.id}|hpsa_software_policy_id=#{hpsa_software_policy_id}")}"
      key[:region_endpoint_ip_addresses].each do |ip_address|
        break if instantiate("#{uri}&ip_address=#{ip_address}", :ignore_error => true)
      end
    end
  end
end

def get_virtual_machine_from_request(task)
  vm_id = task.options[:attrs][:vm_id]
  vm = $evm.vmdb('vm').find_by_id(vm_id)
  error("Unable to locate virtual machine: [#{vm_id}]") unless vm
  log(:info, "Located VM: [#{vm.name}]")
  vm
end

begin
  dump_root()

  task = $evm.root['automation_task']
  if task
    log(:info, "Initated from an automation request down to the local region")
#    vm = get_virtual_machine_from_request(task)
#    apply_software_policy(vm, task.options[:attrs][:hpsa_software_policy_id])
  else
    log(:info, "Initated from the the custom button")
#    vms = get_virtual_machines_from_vmdb_object()
#    region_endpoint_vms = group_vms_by_region_endpoint(vms)
#    create_automation_requests(region_endpoint_vms, $evm.root['dialog_hpsa_software_policy_id'])
  end
rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
