###################################
#
# CFME Automate Method: get_bd
# Author: Nick Lane
#
# Notes: Get a list of EPGs from Cisco ACI
#
# Inputs: 
#
# Mods:
#
###################################

require 'restclient'
require 'json'

def log(level, msg)
  @method = 'get_bd'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, "ERROR: #{msg}")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def retry_method(msg)
  log(:error, "RETRY: #{msg}")
  $evm.root['ae_result'] = 'retry'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def get_token(username, password, server)
	log(:info, "Getting Login Token")
	uri = "http://#{server}/api/aaaLogin.json?gui-token-request=yes"
	body = {
		"aaaUser"=> {
			"attributes" => {
				"name" => username,
				"pwd"  => password
			}
		}
	}.to_json

	log(:info, "Attempting to acquire Token from #{server} with body: #{body}")
	login_response = RestClient.post(uri, body)
	log(:info, "Got response: #{login_response.headers}")
	response_hash = JSON.parse(login_response)

	imdata = response_hash['imdata'].first

	log(:info, "DATA: #{imdata}")

	url_token = imdata['aaaLogin']['attributes']['urlToken'].chomp
	@apic_cookie = imdata['aaaLogin']['attributes']['token'].chomp

	log(:info, "TOKEN: #{url_token}")
	return url_token
end

def get_bridge_domain(server, token, epg)
	log(:info, "Getting Bridge Domain")
	#uri = "https://#{server}/api/node/mo/info.json"

	uri = "https://#{server}/api/node/mo/#{epg}.json?query-target=children&target-subtree-class=fvRsBd"	
	log(:info, "Attempting to get brodge domains with this request: #{uri}")
	begin
		epg_response = RestClient::Request.execute(
		:method => :get,
		:url => uri,
		:headers => {
			:accept => 'text/html,application/xhtml+xml,application/xml,application/json',
			'APIC-challenge' => token,
			'devcookie' => @apic_cookie
			}
		)
	rescue => err
		log(:info, "ERROR: #{err.inspect}")
	end

	epg_data = JSON.parse(epg_response)

	log(:info, "Got Response: #{epg_response}")
	epg_data = epg_data['imdata'].first
	log(:info, "BD: #{epg_data}")

	b_domain = epg_data['fvRsBd']['attributes']['tDn']

	log(:info, "BDS: #{b_domain}")

	return b_domain
end

def get_networks(server, token, bridge_domain, epg)
	log(:info, "Getting Networks")
	#uri = "https://#{server}/api/node/mo/info.json"

	networks = Array.new
	net_hash = Hash.new

	uri = "https://#{server}/api/node/mo/#{bridge_domain}.json?query-target=children&target-subtree-class=fvSubnet"	
	log(:info, "Attempting to get networks with this request: #{uri}")
	begin
		epg_response = RestClient::Request.execute(
		:method => :get,
		:url => uri,
		:headers => {
			:accept => 'text/html,application/xhtml+xml,application/xml,application/json',
			'APIC-challenge' => token,
			'devcookie' => @apic_cookie
			}
		)
	rescue => err
		log(:info, "ERROR: #{err.inspect}")
	end

	epg_data = JSON.parse(epg_response)

	log(:info, "Got Response: #{epg_response}")
	epg_data = epg_data['imdata']
	log(:info, "Subnets: #{epg_data}")
	epg_data.each {|bd|
		ip_address = bd['fvSubnet']['attributes']['ip']

		gateway = ip_address.split("/").first

		vmware_network = epg.gsub("/","|")

		net_hash = {ip_address => {'vmware_network' => vmware_network,'gateway' => gateway, 'dns' => '10.159.2.27', 'reserved_count_start' => 15, 'reserved_count_end' => 1}}

		log(:info, " network data: #{net_hash}")


		networks.push net_hash
		log(:info, "Networks: #{networks.inspect}")
	}
	return networks
end

def retrieve_configuration(config, class_name, instance_name)
  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  instance = $evm.instantiate(instance_uri.to_s)
  error("Unable to retrieve required configuration: [#{instance_uri}]") unless instance
  instance.attributes.each { |k, _| config[k] = instance.decrypt(k) }
end

begin
	log(:info, "Entering Get EPGs")

	######
	#I'm setting this to IHPLAB for now until we can determine how best to figure out which environment we're in
	######

	intuit_dc = "ihplab"
	config = {}
  	retrieve_configuration(config, 'intuit_dc', intuit_dc)

  	#epg = $evm.current['target_epg']

  	epg = 'uni/tn-qbo/ap-prod/epg-prd.app'

  	username = config['username']
  	password = config['password']
  	server   = config['aci_server']

  	log(:info, "Targeting EPG: #{epg}")

  	token = get_token(username, password, server)

  	bridge_domain = get_bridge_domain(server, token, epg)

  	networks = get_networks(server, token, bridge_domain, epg)

  	log(:info, "Got Token: #{token}")
rescue => err
	 error("[#{err}]\n#{err.backtrace.join("\n")}")
end
