###################################
#
# CFME Automate Method: get_epgs
# Author: Nick Lane
#
# Notes: Get a list of EPGs from Cisco ACI
#
# Inputs:
#
# Mods:
#
###################################

require 'restclient'
require 'uri'
require 'json'

def log(level, msg)
  @method = 'get_epg'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, "ERROR: #{msg}")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def retry_method(msg)
  log(:error, "RETRY: #{msg}")
  $evm.root['ae_result'] = 'retry'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def get_token(username, password, server)
  log(:info, "Getting Login Token")
  uri = "#{server}/api/aaaLogin.json?gui-token-request=yes"
  body = {
      "aaaUser"=> {
          "attributes" => {
              "name" => username,
              "pwd"  => password
          }
      }
  }.to_json

  log(:info, "Attempting to acquire Token from #{server} with body: #{body}")

  login_response = RestClient::Request.execute(
    :method => :post,
    :url => uri,
    :payload => body,
    :verify_ssl => OpenSSL::SSL::VERIFY_NONE
  )
  
  log(:info, "Got response: #{login_response.headers}")
  response_hash = JSON.parse(login_response)

  imdata = response_hash['imdata'].first

  log(:info, "DATA: #{imdata}")

  url_token = imdata['aaaLogin']['attributes']['urlToken'].chomp
  @apic_cookie = imdata['aaaLogin']['attributes']['token'].chomp

  log(:info, "TOKEN: #{url_token}")
  return url_token
end

def get_tenants(server, token, snow_app)
  log(:info, "Getting Tenants")
  #uri = "https://#{server}/api/node/mo/info.json"

  uri = "#{server}/api/node/class/fvTenant.json"

  log(:info, "Attempting to get tenants with this request: #{uri}")
  #tenant_response = RestClient.get(uri, headers = {'APIC-challenge' => token})
  begin
    tenant_response = RestClient::Request.execute(
        :method => :get,
        :url => uri,
        :headers => {
            :accept => 'text/html,application/xhtml+xml,application/xml,application/json',
            'APIC-challenge' => token,
            'devcookie' => @apic_cookie
        },
        :verify_ssl => OpenSSL::SSL::VERIFY_NONE
    )
  rescue => err
    log(:info, "ERROR: #{err.inspect}")
  end

  tenant_data = JSON.parse(tenant_response)

  log(:info, "Got Response: #{tenant_data}")
  tenant_data = tenant_data['imdata']
  log(:info, "Tenant Data: #{tenant_data}")
  tenants = Array.new
  tenant_data.each {|tenant|
    next unless tenant['fvTenant']['attributes']['descr'] == snow_app
    tenants.push tenant['fvTenant']['attributes']['name']
  }
  log(:info, "Tenants: #{tenants}")

  return tenants
end

def get_epgs(server, token, security_level, security_zone, tenant)
  log(:info, "Getting Tenants")
  #uri = "https://#{server}/api/node/mo/info.json"

  #	tenant = 'qbo'
  comp_sec = nil
  subtree_filter = nil
  if security_level.include? "pci"
    comp_sec = security_level.split(",").first
    security_level = security_level.split(",").last
    subtree_filter = "rsp-subtree-filter=or(wcard(tagInst.name,\"#{security_zone}\"),wcard(tagInst.name,\"#{security_level}\"),wcard(tagInst.name,\"#{comp_sec}\"))"
  else
    subtree_filter = "rsp-subtree-filter=or(wcard(tagInst.name,\"#{security_zone}\"),wcard(tagInst.name,\"#{security_level}\"))"
  end
  uri =  URI.encode("#{server}/api/mo/uni/tn-#{tenant}.json?target-subtree-class=fvAEPg&query-target=subtree&rsp-subtree-class=tagInst&rsp-subtree=full&#{subtree_filter}&rsp-subtree-include=required")
  log(:info, "Attempting to get tenants with this request: #{uri}")
  begin
    epg_response = RestClient::Request.execute(
        :method => :get,
        :url => uri,
        :headers => {
            :accept => 'text/html,application/xhtml+xml,application/xml,application/json',
            'APIC-challenge' => token,
            'devcookie' => @apic_cookie
        },
        :verify_ssl => OpenSSL::SSL::VERIFY_NONE
    )
  rescue => err
    log(:info, "ERROR: #{err.inspect}")
  end

  epg_data = JSON.parse(epg_response)

  log(:info, "Got Response: #{epg_response}")
  epg_data = epg_data['imdata']
  log(:info, "EPG: #{epg_data}")
  epgs = Array.new




  epg_data.each {|epg|
    epg_names = Array.new
    log(:info, "Looking at EPG: #{epg}")
    epg['fvAEPg']['children'].each {|child|
      epg_names.push child['tagInst']['attributes']['name']
    }
    log(:info, "EPG NAMES: #{epg_names}")
    if comp_sec.nil?
      if (epg_names.include? security_zone) && (epg_names.include? security_level)
        epgs.push epg['fvAEPg']['attributes']['dn']
      end
    else
      if (epg_names.include? security_zone) && (epg_names.include? security_level) && (epg_names.include? comp_sec)
        epgs.push epg['fvAEPg']['attributes']['dn']
      end
    end
  }
  log(:info, "EPGS: #{epgs}")

  return epgs
end

def build_dialog_values(lf_array)
  # CFME does not properly handle sorted dynamic fields
  list_array = Array.new
  unless lf_array.any? {|entry| entry.include? "BREAK"}
    lf_array.each {|tenant_epg|
      log(:info, "Tenant's and EPG: #{tenant_epg}")
      list_array.push ["Tenant:#{tenant_epg.keys.first} EPG:#{tenant_epg.values.first.split("/").last}", tenant_epg.values.first]
    }
  else
    list_array = lf_array
  end
  log(:info, "List array: #{list_array}")

  values_array = [['','']] + list_array
  values_hash = {}
  values_array.each{ |k,v| values_hash.merge!({v=>k})}

  list_values = {
      'sort_by'       => :none,
      'data_type'     => :string,
      'required'      => false,
      'default_value' => '',
      'values'        => values_hash
  }

  $evm.log('info', "Template drop-down: [#{list_values}]")
  list_values.each { |k,v| $evm.object[k] = v }
end

def retrieve_configuration(config, class_name, instance_name)
  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  instance = $evm.instantiate(instance_uri.to_s)
  error("Unable to retrieve required configuration: [#{instance_uri}]") unless instance
  instance.attributes.each { |k, _| config[k] = instance.decrypt(k) }
end

begin
  log(:info, "Entering Get EPGs")

  ######
  #I'm setting this to IHPLAB for now until we can determine how best to figure out which environment we're in
  ######

  log(:info, "Inputs: #{$evm.inputs.inspect}")
  security_zone_input = $evm.inputs['dialog_option_1_security_zone']
  snow_app = $evm.inputs['dialog_option_0_application_name']

  unless security_zone_input.blank? || snow_app.blank?

    hosting_segment = security_zone_input.split("_")[0].chomp
    security_zone = security_zone_input.split("_")[1]
    security_level = security_zone_input.split("_")[2]

    log(:info, "Hosting Segment: #{hosting_segment}")

    evm = $evm.vmdb('ext_management_system').all

#    log(:info, "got evm: #{evm.select{|ext| ext.name.include? "SBG"}.first.tags.include?("intuit_hs/#{hosting_segment}")}")

    intuit_dc = evm.select{|ext_ms| ext_ms.tags.include?("intuit_hs/#{hosting_segment}")}.first.tags.select{|tag| tag.include? 'intuit_dc'}.first.split("/").last

    log(:info, "Got DC: #{intuit_dc}")

    log(:info, "Got security zone info: Sec_zone #{security_zone} sec_level #{security_level}")

    config = {}
    retrieve_configuration(config, 'intuit_dc', intuit_dc)
    aci_sec_zone = $evm.current[security_zone]
    aci_sec_level = $evm.current[security_level]

    username = config['username']
    password = config['password']
    server   = config['aci_server']

    token = get_token(username, password, server)

    tenants = get_tenants(server, token, snow_app)
  end

  final_array = Array.new
  unless tenants.blank?
    tenants.each {|tenant|
      epgs = get_epgs(server, token, aci_sec_level, aci_sec_zone, tenant)
      epgs.each {|epg|
        log(:info, "Tenant: #{tenant} EPG #{epg}")
        epg_data = {tenant=>epg}
        final_array.push epg_data
      }
    }
  else
    final_array.push ["No Tenants found for Service Now Application:#{snow_app}", "BREAK"]
  end
  if final_array.empty?
    final_array.push ["No EPGs found for Tenants: #{tenants} and environment: #{security_level} #{security_zone}", "BREAK"]
  end
  log(:info, "Final Array: #{final_array}")
  build_dialog_values(final_array)

  log(:info, "Got Token: #{token}")
rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
