##################################
#
# CFME Automate Method: configure_chef
#
# Author: Jason Dillaman, Daniel Trieu
#
# Description: Configure chef to register with master
# Retryable?:  Yes/No - If a method is retryable (ie, it has can possibly call retry_method), all functions must be
#              idempotent (ie, it changes nothing or it makes changes and subsequent changes are not affected by the
#              previous changes)
#
###################################

require 'open-uri'
require 'tempfile'
require 'timeout'
DOWNLOAD_TIMEOUT                 = 2.minutes
VIX_ACTIONS_URI                  = "/Intuit/Integration/VIX/Actions"
COPY_FILE_FROM_HOST_TO_GUEST_URI = "#{VIX_ACTIONS_URI}/CopyFileFromHostToGuest"
GET_VIX_PLATFORM_URI             = "#{VIX_ACTIONS_URI}/GetVixPlatform"
RUN_PROGRAM_IN_GUEST             = "#{VIX_ACTIONS_URI}/RunProgramInGuest"
RHEL5_VMWARE_TOOLS_SCRIPT        = "/etc/init.d/vmware-tools"
RHEL6_VMWARE_TOOLS_SCRIPT        = "/etc/vmware-tools/services.sh"
def log(level, msg)
  @method = 'configure_chef'
  $evm.log(level, "#{@method}: #{msg}")
end
def error(msg)
  log(:error, "Ending Method [ERROR: #{msg}]")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end
def retry_method(msg)
  log(:warn, "Ending Method [RETRY: #{msg}]")
  $evm.root['ae_result'] = 'retry'
  $evm.root['ae_retry_interval'] = '1.minute'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end
def to_bool(value)
  log(:info, "to_bool: #{value}")
  return true if value == true || value =~ (/^(true|t|yes|y|1)$/i)
  return false if value == false || value.blank? || value =~ (/^(false|f|no|n|0)$/i)
  error("Invalid value for Boolean: to_bool(#{value})")
end
def escape_uri(value)
  URI.escape(value, Regexp.new("[^#{URI::PATTERN::UNRESERVED}]"))
end
def dump_root()
  log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  log(:info, "Root:<$evm.root> End $evm.root.attributes")
end
def instantiate(uri)
  log(:info, "instantiate(#{uri})")
  result = $evm.instantiate(uri)
  error("Failed to execute: [#{uri}]") unless result
  case result['result']
    when 'error'
      error(result['reason'])
    when 'retry'
      retry_method(result['reason'])
  end
  log(:info, "instantiate(#{uri}): #{result}")
  return result
end
def get_virtual_machine()
  case $evm.root['vmdb_object_type']
    when 'miq_provision'
      vm = $evm.root['miq_provision'].vm
    when 'vm'
      vm = $evm.root['vm']
  end
  raise "VM not found" unless vm
  vm
end
def get_virtual_machine_parent(vm)
  templates = vm.wrap_results(vm.object_send(:parents))
  if templates.length > 1
    log(:info, "Virtual machine [#{vm.name}] has multiple parent relationships: [#{templates}]")
    template = templates.first
  else
    template = vm.wrap_results(vm.object_send(:parent))
  end
  error("Virtual machine is missing its parent template relationship: [#{vm.name}]") unless template
  return template
end
def get_vix_platform()
  result = instantiate(GET_VIX_PLATFORM_URI)
  vix_platform = result['vix_platform']
  log(:info, "VIX platform: [#{vix_platform}]")
  return vix_platform
end
def get_intuit_chef_client_url(template)
  chef_client_url_tag = template.tags(:intuit_chef_client_url).first
  error("Template missing Chef Client URL tag: [#{template.name}]") unless chef_client_url_tag
  classification = $evm.vmdb('classification').find_by_name("intuit_chef_client_url/#{chef_client_url_tag}")
  error("Unable to retrieve tag description: [#{chef_client_url_tag}]") unless classification
  return classification.description
end
def retrieve_configuration(config, class_name, instance_name)
  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  instance = $evm.instantiate(instance_uri)
  error("Unable to retrieve required configuration: [#{instance_uri}]") unless instance
  log(:info, "Retrieved configuration: [#{class_name}/#{instance_name}[#{instance.attributes}]")
  instance.attributes.each { |k, _| config[k] = instance.decrypt(k) }
end
def run_program_in_guest(vix_platform, vix_guest_program, vix_guest_options)
  log(:info, "Attempting to run within VM: [#{vix_guest_program}][#{vix_guest_options}]")
  ae_uri = "#{RUN_PROGRAM_IN_GUEST}?" \
    "vix_platform=#{vix_platform}&" \
    "vix_guest_program=#{escape_uri(vix_guest_program)}&" \
    "vix_guest_options=#{escape_uri(vix_guest_options)}"
  result = instantiate(ae_uri)
  retry_method("VIX reported failure: [#{result['exit_status']}]") unless result['exit_status'] == 0
end

def get_chef_server_url(ems)
  chef_server_url_tag = ems.tags(:intuit_chef_server_url).first
  error("Template missing Chef Server URL tag: [#{ems.name}]") unless chef_server_url_tag
  classification = $evm.vmdb('classification').find_by_name("intuit_chef_server_url/#{chef_server_url_tag}")
  error("Unable to retrieve tag description: [#{chef_server_url_tag}]") unless classification
  return classification.description
end

datacenter_conversion = {
    'ihplab' => 'ihplab',
    'qdc' => 'qy',
    'lvdc' => 'lv'
}
def get_chef_environment_name(datacenter, hosting_segment, security_zone, security_level)
  # <projectname>_<env><dc><zone> where env is dev,qal,rtb,e2e,prf,prd AND dc is qy,lv AND zone is a,b,c,d,e
  return '_default'
end
begin
  log(:info, "Starting Method")
  vm = get_virtual_machine()
  cluster = vm.ems_cluster
  ems = vm.ext_management_system
  vix_platform = get_vix_platform()
  datacenter = ems.tags(:intuit_dc).first
  hosting_segment = vm.tags(:intuit_hs).first # lvdca, lvdcb, qcyf01, qdca, qdce
  security_zone = vm.tags(:intuit_sz).first # admin, comp, corp, dev, prod, preprod
  security_level = vm.tags(:intuit_security_level).first # nonpublic, nonpubliccomp, public, publiccomp, sensitive, sensitivecomp, omn
  log(:info, "#{hosting_segment} #{security_zone} #{security_level}")
  if ['ihplab'].include?(datacenter) || (['qdc','lvdc'].include?(datacenter) && ['qcyf01','lasf01'].include?(hosting_segment))

    environment_name = get_chef_environment_name(datacenter, hosting_segment, security_zone, security_level)
    platform_config = {}
    retrieve_configuration(platform_config, 'VixPlatform', vix_platform)
    chef_server_url = get_chef_server_url(cluster)

    # Get template path
    clientrb_template_path = "#{platform_config['chef_client_config_path']}client.rb"
    validation_pem_path = "#{platform_config['chef_client_config_path']}validation.pem"

    if vix_platform == 'linux'

      # Replace keys in template using sed
      run_program_in_guest(vix_platform, '/bin/sed', "-i 's/CHEF_SERVER_URL/#{Regexp.escape(chef_server_url).gsub(/\//, '\/').gsub(/:/,'\:')}/g' #{clientrb_template_path}")
      run_program_in_guest(vix_platform, '/bin/sed', "-i 's/NODE_NAME/#{Regexp.escape(vm.name)}/g' #{clientrb_template_path}")
      run_program_in_guest(vix_platform, '/bin/sed', "-i 's/SSL_VERIFY_MODE/#{Regexp.escape(platform_config['ssl_verify_mode'])}/g' #{clientrb_template_path}")
      run_program_in_guest(vix_platform, '/bin/sed', "-i 's/VALIDATION_CLIENT_NAME/#{Regexp.escape(platform_config['validation_client_name'])}/g' #{clientrb_template_path}")
      run_program_in_guest(vix_platform, '/bin/sed', "-i 's/ENVIRONMENT_NAME/#{Regexp.escape(environment_name)}/g' #{clientrb_template_path}")
      run_program_in_guest(vix_platform, '/bin/sed', "-i 's/LOG_LEVEL/#{Regexp.escape(platform_config['log_level'])}/g' #{clientrb_template_path}")
      run_program_in_guest(vix_platform, '/bin/sed', "-i 's/LOG_LOCATION/#{Regexp.escape(platform_config['log_location'])}/g' #{clientrb_template_path}")
      run_program_in_guest(vix_platform, '/bin/sed', "-i 's/VALIDATION_KEY/#{Regexp.escape(validation_pem_path).gsub(/\//, '\/')}/g' #{clientrb_template_path}")

      # Test ifconfig on eth0; echo $?
      run_program_in_guest(vix_platform, '/sbin/ifconfig', "eth0 2>&1>/dev/null; echo $?")

      # Run chef-client
      run_program_in_guest(vix_platform, '/usr/bin/chef-client', '-l debug -L /var/tmp/chef-client.log -o sdt_sbg_postprov-runlist;true')

    else # vix_platform == 'windows'

      # Replace keys in template using file-content-replace.bat
      run_program_in_guest(vix_platform, 'C:\Temp\file-content-replace.bat', "\"#{clientrb_template_path}\" CHEF_SERVER_URL #{chef_server_url}")
      run_program_in_guest(vix_platform, 'C:\Temp\file-content-replace.bat', "\"#{clientrb_template_path}\" NODE_NAME #{vm.name}")
      run_program_in_guest(vix_platform, 'C:\Temp\file-content-replace.bat', "\"#{clientrb_template_path}\" SSL_VERIFY_MODE #{platform_config['ssl_verify_mode']}")
      run_program_in_guest(vix_platform, 'C:\Temp\file-content-replace.bat', "\"#{clientrb_template_path}\" VALIDATION_CLIENT_NAME #{platform_config['validation_client_name']}")
      run_program_in_guest(vix_platform, 'C:\Temp\file-content-replace.bat', "\"#{clientrb_template_path}\" ENVIRONMENT_NAME #{environment_name}")
      run_program_in_guest(vix_platform, 'C:\Temp\file-content-replace.bat', "\"#{clientrb_template_path}\" LOG_LEVEL #{platform_config['log_level']}")
      run_program_in_guest(vix_platform, 'C:\Temp\file-content-replace.bat', "\"#{clientrb_template_path}\" LOG_LOCATION #{platform_config['log_location']}")
      run_program_in_guest(vix_platform, 'C:\Temp\file-content-replace.bat', "\"#{clientrb_template_path}\" VALIDATION_KEY #{validation_pem_path.gsub(/\\/,'/')}")

      # Install chef-service
      #run_program_in_guest(vix_platform, 'C:\opscode\chef\bin\chef-service-manager.bat', '-a install')

      # Install chef-service
      #run_program_in_guest(vix_platform, 'C:\opscode\chef\bin\chef-service-manager.bat', '-a start')

      # Run chef-client
      run_program_in_guest(vix_platform, 'C:\opscode\chef\bin\chef-client.bat', '-l debug -L "C:\chef\chef-client.log" -o sdt_sbg_postprov-runlist || exit 0')

    end

  end
  $evm.current['result'] = 'ok'
  $evm.current['reason'] = ''
  log(:info, "Ending Method [SUCCESS]")
  exit MIQ_OK
rescue StandardError, SignalException => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
