##################################
#
# CFME Automate Method: deregister_chef
#
# Author: Jason Dillaman, Daniel Trieu
#
# Description: Deregisters chef node from master
# Retryable?:  Yes/No - If a method is retryable (ie, it has can possibly call retry_method), all functions must be
#              idempotent (ie, it changes nothing or it makes changes and subsequent changes are not affected by the
#              previous changes)
#
###################################

require 'open-uri'
require 'tempfile'
require 'timeout'

DOWNLOAD_TIMEOUT                 = 2.minutes
VIX_ACTIONS_URI                  = "/Intuit/Integration/VIX/Actions"
COPY_FILE_FROM_HOST_TO_GUEST_URI = "#{VIX_ACTIONS_URI}/CopyFileFromHostToGuest"
GET_VIX_PLATFORM_URI             = "#{VIX_ACTIONS_URI}/GetVixPlatform"
RUN_PROGRAM_IN_GUEST             = "#{VIX_ACTIONS_URI}/RunProgramInGuest"
RHEL5_VMWARE_TOOLS_SCRIPT        = "/etc/init.d/vmware-tools"
RHEL6_VMWARE_TOOLS_SCRIPT        = "/etc/vmware-tools/services.sh"

def log(level, msg)
  @method = 'deregister_chef'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, "Ending Method [ERROR: #{msg}]")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def retry_method(msg)
  log(:warn, "Ending Method [RETRY: #{msg}]")
  $evm.root['ae_result'] = 'retry'
  $evm.root['ae_retry_interval'] = '1.minute'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def to_bool(value)
  log(:info, "to_bool: #{value}")
  return true if value == true || value =~ (/^(true|t|yes|y|1)$/i)
  return false if value == false || value.blank? || value =~ (/^(false|f|no|n|0)$/i)
  error("Invalid value for Boolean: to_bool(#{value})")
end

def escape_uri(value)
  URI.escape(value, Regexp.new("[^#{URI::PATTERN::UNRESERVED}]"))
end

def dump_root()
  log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  log(:info, "Root:<$evm.root> End $evm.root.attributes")
end

def instantiate(uri)
  log(:info, "instantiate(#{uri})")
  result = $evm.instantiate(uri)

  error("Failed to execute: [#{uri}]") unless result

  case result['result']
    when 'error'
      error(result['reason'])
    when 'retry'
      retry_method(result['reason'])
  end

  log(:info, "instantiate(#{uri}): #{result}")
  return result
end

def get_virtual_machine()
  case $evm.root['vmdb_object_type']
    when 'miq_provision'
      vm = $evm.root['miq_provision'].vm
    when 'vm'
      vm = $evm.root['vm']
  end
  raise "VM not found" unless vm
  vm
end


def get_vix_platform()
  result = instantiate(GET_VIX_PLATFORM_URI)
  vix_platform = result['vix_platform']
  log(:info, "VIX platform: [#{vix_platform}]")
  return vix_platform
end

def retrieve_configuration(config, class_name, instance_name)
  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  instance = $evm.instantiate(instance_uri)
  error("Unable to retrieve required configuration: [#{instance_uri}]") unless instance

  log(:info, "Retrieved configuration: [#{class_name}/#{instance_name}[#{instance.attributes}]")
  instance.attributes.each { |k, _| config[k] = instance.decrypt(k) }
end





def run_program_in_guest(vix_platform, vix_guest_program, vix_guest_options)
  log(:info, "Attempting to run within VM: [#{vix_guest_program}][#{vix_guest_options}]")

  ae_uri = "#{RUN_PROGRAM_IN_GUEST}?" \
    "vix_platform=#{vix_platform}&" \
    "vix_guest_program=#{escape_uri(vix_guest_program)}&" \
    "vix_guest_options=#{escape_uri(vix_guest_options)}"
  result = instantiate(ae_uri)
  retry_method("VIX reported failure: [#{result['exit_status']}]") unless result['exit_status'] == 0
end

begin
  log(:info, "Starting Method")

  vm = get_virtual_machine()
  ems = vm.ext_management_system
  vix_platform = get_vix_platform()

  datacenter = ems.tags(:intuit_dc).first
  hosting_segment = vm.tags(:intuit_hs).first

  if vix_platform == 'linux' && ( ['ihplab'].include?(datacenter) || (['qdc'].include?(datacenter) && ['qycf01'].include?(hosting_segment)) )

    platform_config = {}
    retrieve_configuration(platform_config, 'VixPlatform', vix_platform)

    # Run the knife to delete the node
    clientrb_template_path = "#{platform_config['chef_client_config_path']}/client.rb"
    run_program_in_guest(vix_platform, '/usr/bin/knife', "node delete #{vm.name} -y -c #{clientrb_template_path}; true")

  end

  $evm.current['result'] = 'ok'
  $evm.current['reason'] = ''

  log(:info, "Ending Method [SUCCESS]")
  exit MIQ_OK
rescue StandardError, SignalException => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
