##################################
#
# CFME Automate Method: install_chef
#
# Author: Jason Dillaman, Daniel Trieu
#
# Description: Installs chef binary
# Retryable?:  Yes/No - If a method is retryable (ie, it has can possibly call retry_method), all functions must be
#              idempotent (ie, it changes nothing or it makes changes and subsequent changes are not affected by the
#              previous changes)
#
###################################

require 'open-uri'
require 'tempfile'
require 'timeout'
require 'restclient'

DOWNLOAD_TIMEOUT                 = 2.minutes
VIX_ACTIONS_URI                  = "/Intuit/Integration/VIX/Actions"
COPY_FILE_FROM_HOST_TO_GUEST_URI = "#{VIX_ACTIONS_URI}/CopyFileFromHostToGuest"
GET_VIX_PLATFORM_URI             = "#{VIX_ACTIONS_URI}/GetVixPlatform"
RUN_PROGRAM_IN_GUEST             = "#{VIX_ACTIONS_URI}/RunProgramInGuest"
RHEL5_VMWARE_TOOLS_SCRIPT        = "/etc/init.d/vmware-tools"
RHEL6_VMWARE_TOOLS_SCRIPT        = "/etc/vmware-tools/services.sh"

def log(level, msg)
  @method = 'install_chef'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, "Ending Method [ERROR: #{msg}]")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def retry_method(msg)
  log(:warn, "Ending Method [RETRY: #{msg}]")
  $evm.root['ae_result'] = 'retry'
  $evm.root['ae_retry_interval'] = '1.minute'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def to_bool(value)
  log(:info, "to_bool: #{value}")
  return true if value == true || value =~ (/^(true|t|yes|y|1)$/i)
  return false if value == false || value.blank? || value =~ (/^(false|f|no|n|0)$/i)
  error("Invalid value for Boolean: to_bool(#{value})")
end

def escape_uri(value)
  URI.escape(value, Regexp.new("[^#{URI::PATTERN::UNRESERVED}]"))
end

def dump_root()
  log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  log(:info, "Root:<$evm.root> End $evm.root.attributes")
end

def instantiate(uri)
  log(:info, "instantiate(#{uri})")
  result = $evm.instantiate(uri)

  error("Failed to execute: [#{uri}]") unless result

  case result['result']
    when 'error'
      error(result['reason'])
    when 'retry'
      retry_method(result['reason'])
  end

  log(:info, "instantiate(#{uri}): #{result}")
  return result
end

def get_virtual_machine()
  case $evm.root['vmdb_object_type']
    when 'miq_provision'
      vm = $evm.root['miq_provision'].vm
    when 'vm'
      vm = $evm.root['vm']
  end
  raise "VM not found" unless vm
  vm
end

def get_virtual_machine_parent(vm)
  templates = vm.wrap_results(vm.object_send(:parents))
  if templates.length > 1
    log(:info, "Virtual machine [#{vm.name}] has multiple parent relationships: [#{templates}]")
    template = templates.first
  else
    template = vm.wrap_results(vm.object_send(:parent))
  end
  error("Virtual machine is missing its parent template relationship: [#{vm.name}]") unless template
  return template
end

def get_vix_platform()
  result = instantiate(GET_VIX_PLATFORM_URI)
  vix_platform = result['vix_platform']
  log(:info, "VIX platform: [#{vix_platform}]")
  return vix_platform
end

def get_intuit_chef_client_url(template)
  chef_client_url_tag = template.tags(:intuit_chef_client_url).first
  error("Template missing Chef Client URL tag: [#{template.name}]") unless chef_client_url_tag

  classification = $evm.vmdb('classification').find_by_name("intuit_chef_client_url/#{chef_client_url_tag}")
  error("Unable to retrieve tag description: [#{chef_client_url_tag}]") unless classification
  return classification.description
end

def get_batch_url(template)
  chef_batch_url_tag = template.tags(:intuit_chef_batch_url).first
  error("Template missing Chef Batch URL tag: [#{template.name}]") unless chef_batch_url_tag

  classification = $evm.vmdb('classification').find_by_name("intuit_chef_batch_url/#{chef_batch_url_tag}")
  error("Unable to retrieve tag description: [#{chef_batch_url_tag}]") unless classification
  return classification.description
end

def get_powershell_url(template)
  chef_powershell_url_tag = template.tags(:intuit_chef_pwrshell_url).first
  error("Template missing Chef Powershell URL tag: [#{template.name}]") unless chef_powershell_url_tag

  classification = $evm.vmdb('classification').find_by_name("intuit_chef_pwrshell_url/#{chef_powershell_url_tag}")
  error("Unable to retrieve tag description: [#{chef_powershell_url_tag}]") unless classification
  return classification.description
end

def get_replace_url(template)
  chef_replace_url_tag = template.tags(:intuit_chef_replace_url).first
  error("Template missing Chef Replace URL tag: [#{template.name}]") unless chef_replace_url_tag

  classification = $evm.vmdb('classification').find_by_name("intuit_chef_replace_url/#{chef_replace_url_tag}")
  error("Unable to retrieve tag description: [#{chef_replace_url_tag}]") unless classification
  return classification.description
end

def get_pemfile_url(cluster)
  chef_vpem_url_tag = cluster.tags(:intuit_chef_vpem_url).first
  error("Template missing Chef VPem URL tag: [#{cluster.name}]") unless chef_vpem_url_tag

  classification = $evm.vmdb('classification').find_by_name("intuit_chef_vpem_url/#{chef_vpem_url_tag}")
  error("Unable to retrieve tag description: [#{chef_vpem_url_tag}]") unless classification
  return classification.description
end

def get_chef_cfg_tmpl_url(cluster)
  chef_clrb_url_tag = cluster.tags(:intuit_chef_clrb_url).first
  error("Template missing Chef ClientRB URL tag: [#{cluster.name}]") unless chef_clrb_url_tag

  classification = $evm.vmdb('classification').find_by_name("intuit_chef_clrb_url/#{chef_clrb_url_tag}")
  error("Unable to retrieve tag description: [#{chef_clrb_url_tag}]") unless classification
  return classification.description
end

def retrieve_configuration(config, class_name, instance_name)
  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  instance = $evm.instantiate(instance_uri)
  error("Unable to retrieve required configuration: [#{instance_uri}]") unless instance

  log(:info, "Retrieved configuration: [#{class_name}/#{instance_name}[#{instance.attributes}]")
  instance.attributes.each { |k, _| config[k] = instance.decrypt(k) }
end

def extract_filename(uri)
  return File.basename(URI.parse(uri).path)
end

def transfer_chef_client_to_guest(download_timeout, vix_platform, chef_client_base_path, vm)
  template = get_virtual_machine_parent(vm)
  chef_client_url = get_intuit_chef_client_url(template)

  chef_client_base_path += '/' unless chef_client_base_path.match(/(\\|\/)$/)
  vix_guest_program = "#{chef_client_base_path}#{extract_filename(chef_client_url)}"

  file = Tempfile.new('chef_client', :mode => File::BINARY)
  begin
    download_file(download_timeout, chef_client_url, file)
    transfer_file_to_vm(vix_platform, file.path, vix_guest_program)
  ensure
    file.close()
    file.unlink()
  end
  return vix_guest_program
end

def transfer_batch_to_guest(download_timeout, vix_platform, chef_config_base_path, vm)
  template = get_virtual_machine_parent(vm)
  batchfile_url = get_batch_url(template)

  chef_config_base_path += '/' unless chef_config_base_path.match(/(\\|\/)$/)
  vix_guest_program = "#{chef_config_base_path}chef-client-install.bat"

  file = Tempfile.new('batchfile', :mode => File::BINARY)
  begin
    download_text_file(download_timeout, batchfile_url, file)
    transfer_file_to_vm(vix_platform, file.path, vix_guest_program)
  ensure
    file.close()
    file.unlink()
  end
  return vix_guest_program
end

def transfer_powershell_to_guest(download_timeout, vix_platform, chef_config_base_path, vm)
  template = get_virtual_machine_parent(vm)
  powershellfile_url = get_powershell_url(template)

  chef_config_base_path += '/' unless chef_config_base_path.match(/(\\|\/)$/)
  vix_guest_program = "#{chef_config_base_path}chef-client-install.ps1"

  file = Tempfile.new('powershellfile', :mode => File::BINARY)
  begin
    download_text_file(download_timeout, powershellfile_url, file)
    transfer_file_to_vm(vix_platform, file.path, vix_guest_program)
  ensure
    file.close()
    file.unlink()
  end
  return vix_guest_program
end

def transfer_replace_to_guest(download_timeout, vix_platform, chef_config_base_path, vm)
  template = get_virtual_machine_parent(vm)
  replace_url = get_replace_url(template)

  chef_config_base_path += '/' unless chef_config_base_path.match(/(\\|\/)$/)
  vix_guest_program = "#{chef_config_base_path}file-content-replace.bat"

  file = Tempfile.new('replacefile', :mode => File::BINARY)
  begin
    download_text_file(download_timeout, replace_url, file)
    transfer_file_to_vm(vix_platform, file.path, vix_guest_program)
  ensure
    file.close()
    file.unlink()
  end
  return vix_guest_program
end

def transfer_pemfile_to_guest(download_timeout, vix_platform, chef_config_base_path, vm)
  pemfile_url = get_pemfile_url(vm.ems_cluster)

  chef_config_base_path += '/' unless chef_config_base_path.match(/(\\|\/)$/)
  vix_guest_program = "#{chef_config_base_path}validation.pem"

  file = Tempfile.new('pemfile', :mode => File::BINARY)
  begin
    download_text_file(download_timeout, pemfile_url, file)
    transfer_file_to_vm(vix_platform, file.path, vix_guest_program)
  ensure
    file.close()
    file.unlink()
  end
  return vix_guest_program
end

def transfer_chef_cfg_tmpl_to_guest(download_timeout, vix_platform, chef_config_base_path, vm)
  chef_cfg_tmpl_url = get_chef_cfg_tmpl_url(vm.ems_cluster)

  chef_config_base_path += '/' unless chef_config_base_path.match(/(\\|\/)$/)
  vix_guest_program = "#{chef_config_base_path}#{extract_filename(chef_cfg_tmpl_url)}"

  file = Tempfile.new('chef_cfg_tmpl', :mode => File::BINARY)
  begin
    download_text_file(download_timeout, chef_cfg_tmpl_url, file)
    transfer_file_to_vm(vix_platform, file.path, vix_guest_program)
  ensure
    file.close()
    file.unlink()
  end
  return vix_guest_program
end

def transfer_file_to_vm(vix_platform, local_path, remote_path)
  log(:info, "Attempting to transfer Chef client rpm to VM: [#{local_path}][#{remote_path}]")

  transfer_uri = "#{COPY_FILE_FROM_HOST_TO_GUEST_URI}?" \
    "vix_platform=#{vix_platform}&" \
    "vix_local_file=#{escape_uri(local_path)}&" \
    "vix_remote_file=#{escape_uri(remote_path)}"
  result = instantiate(transfer_uri)
  retry_method("VIX reported failure: [#{result['exit_status']}]") unless result['exit_status'] == 0
end

def download_file(timeout, uri, local_file)
  log(:info, "Starting download of: [#{uri}]")
  begin
    Timeout::timeout(timeout) do
      open(uri, "rb", :ssl_verify_mode => OpenSSL::SSL::VERIFY_NONE) do |read_file|
        local_file.write(read_file.read)
        read_file.close()
      end
    end
  rescue Timeout::Error
    retry_method("Timed out downloading #{uri}")
  end
  log(:info, "#{uri} download complete")
end

def download_text_file(timeout, uri, local_file)
  log(:info, "Starting download of: [#{uri}]")
  begin
    #response_file = RestClient.get uri
    response_file = RestClient::Resource.new(uri, :verify_ssl => false).get
    local_file.write(response_file)
    local_file.flush
  rescue Timeout::Error
    retry_method("Timed out downloading #{uri}")
  end
  log(:info, "#{uri} download complete")
end

def run_program_in_guest(vix_platform, vix_guest_program, vix_guest_options)
  log(:info, "Attempting to run within VM: [#{vix_guest_program}][#{vix_guest_options}]")

  ae_uri = "#{RUN_PROGRAM_IN_GUEST}?" \
    "vix_platform=#{vix_platform}&" \
    "vix_guest_program=#{escape_uri(vix_guest_program)}&" \
    "vix_guest_options=#{escape_uri(vix_guest_options)}"
  result = instantiate(ae_uri)
  retry_method("VIX reported failure: [#{result['exit_status']}]") unless result['exit_status'] == 0
end

begin
  log(:info, "Starting Method")

  vm = get_virtual_machine()
  ems = vm.ext_management_system
  vix_platform = get_vix_platform()

  datacenter = ems.tags(:intuit_dc).first
  hosting_segment = vm.tags(:intuit_hs).first

  if ['ihplab'].include?(datacenter) || (['qdc','lvdc'].include?(datacenter) && ['qcyf01','lasf01'].include?(hosting_segment))

    config = {}
    retrieve_configuration(config, 'VixPlatform', vix_platform)

    # Transfer chef client to the guest vm
    vix_guest_program = transfer_chef_client_to_guest(DOWNLOAD_TIMEOUT, vix_platform, config['chef_client_base_path'], vm)

    if vix_platform == 'linux'
      # Run the chef client rpm
      installer_args = "-i #{vix_guest_program}"
      run_program_in_guest(vix_platform, config['native_installer'], installer_args)

      # Run mkdir /etc/chef
      run_program_in_guest(vix_platform, '/bin/mkdir', "-p #{config['chef_client_config_path']}")

    else # vix_platform == 'windows'
      # Copy chef-client-install.bat
      batch_installer = transfer_batch_to_guest(DOWNLOAD_TIMEOUT, vix_platform, config['chef_client_base_path'], vm)

      # Copy chef-client-install.ps1
      transfer_powershell_to_guest(DOWNLOAD_TIMEOUT, vix_platform, config['chef_client_base_path'], vm)

      # Copy file-content-replace.ps1
      transfer_replace_to_guest(DOWNLOAD_TIMEOUT, vix_platform, config['chef_client_base_path'], vm)

      # Run the chef client exe
      binary_path = config['chef_client_base_path'][0..-2]
      installer_args = "\"#{binary_path}\" \"#{File.basename(vix_guest_program.gsub('\\', '/'))}\""
      run_program_in_guest(vix_platform, batch_installer, installer_args)

    end

    # Copy validation pem
    transfer_pemfile_to_guest(DOWNLOAD_TIMEOUT, vix_platform, config['chef_client_config_path'], vm)

    # Copy template
    transfer_chef_cfg_tmpl_to_guest(DOWNLOAD_TIMEOUT, vix_platform, config['chef_client_config_path'], vm)

  end

  $evm.current['result'] = 'ok'
  $evm.current['reason'] = ''

  log(:info, "Ending Method [SUCCESS]")
  exit MIQ_OK
rescue StandardError, SignalException => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
