##################################
#
# CFME Automate Method: push_to_escalation_qb
#
# Author: Bryan Antigua, Wayne Chatham
#
# Notes: Creates an escalation record in the Escalation Quickbase
# => Added logging in get_app_value method
#
# Inputs: intuit_dc
#
###################################

require 'httpclient'
require 'nokogiri'
require 'uri'

AUTHENTICATE_ACTION = 'API_Authenticate'
QUERY_ACTION = 'API_DoQuery'
ADD_RECORD = 'API_AddRecord'
APP_TOKEN = 'c5upj75cfp4iqccgabs9ub65rrkz'

# Methods for logging
def log(level, message)
  @method = 'push_to_escalation_qb'
  $evm.log(level, "#{@method}: #{message}")
end

def error(msg)
  log(:error, "#{msg}")
  log(:error, "Error received pushing VM record to Escalation QB, exiting #{msg}")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def retry_method(msg)
  $evm.log(:warn,"Error received pushing VM record to Escalation QB,retrying. [#{msg}]")
  $evm.current['result'] = 'retry'
   $evm.current['reason'] = msg
  exit MIQ_OK
end

def escape_uri(value)
  URI.escape("#{value}", Regexp.new("[^#{URI::PATTERN::UNRESERVED}]"))
end

def get_intuit_dc()
  if $evm.root['vmdb_object_type'] == 'miq_provision'
    prov = $evm.root['miq_provision']
    ems_id = prov.get_option(:src_ems_id)
    ems = $evm.vmdb('ext_management_system', ems_id)
  else
    vm = get_virtual_machine()
    ems = vm.ext_management_system
  end

  intuit_dc = ems.tags(:intuit_dc).first
  error("Intuit Data Center tag missing on provider: [#{ems.name}]") unless intuit_dc
  return intuit_dc
end
   
def get_virtual_machine()
  return $evm.current['vm'] unless $evm.current['vm'].blank?
  case $evm.root['vmdb_object_type']
  when 'miq_provision'
    vm = $evm.root['miq_provision'].vm
  when 'vm'
    vm = $evm.root['vm']
  end
  raise "VM not found" unless vm
  vm
end
    
def instantiate(uri)
  result = $evm.instantiate(uri.to_s)
  error("Failed to execute: [#{uri}]") unless result
  error(result['reason']) unless result['result'] == 'ok'
  return result
end
    
def retrieve_configuration(config, class_name, instance_name)
  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  instance = $evm.instantiate(instance_uri.to_s)
  error("Unable to retrieve required configuration: [#{instance_uri}]") unless instance
  instance.attributes.each { |k, _| config[k] = instance.decrypt(k) }
end
 
def authenticate(http,username,password,base_url)
  args = "act=#{AUTHENTICATE_ACTION}&"
  args+= "username=#{escape_uri(username)}&"
  args+= "password=#{escape_uri(password)}"
  log(:info, "querying with :#{base_url+args}")
  resp = http.get(base_url+args,:follow_redirect=>true).content
  results = Nokogiri::XML.parse(resp)
  token = results.xpath("/qdbapi/ticket")[0].content
  token
end

def get_app_value(http,token,application_name,query_url)
  msg = "Entering get_app_value method"
  log(:info, msg)
  args = "act=#{QUERY_ACTION}&"
  args+= "ticket=#{token}&"
  args+= "query={8.EX.'#{application_name}'}"
  args = URI.escape(args)
  msg = "Looking up application name: #{application_name}"
  log(:info, msg)
  resp = http.get(query_url+args,:follow_redirect=>true).content
  results = Nokogiri::XML.parse(resp)
  app_value = ""
  if results.xpath("/qdbapi/record/assets")[0] != nil
    app_value = results.xpath("/qdbapi/record/assets")[0].content
  end
  msg = "Application ID: #{app_value}"
  log(:info, msg)
  app_value
end

def push_to_escalation_qb_table(http,token,app_value,host_name,escalation_table_url,dev_string)
  args = "act=#{ADD_RECORD}&"
  args+= "ticket=#{token}&"
  args+= "apptoken=#{APP_TOKEN}&"
  args+= "_fid_25=#{host_name}&"
  args+= "_fid_208=#{app_value}&"
  args+= "_fid_45=No&"
  args+= "_fid_44=#{dev_string}"
  args = URI.escape(args)
  resp = http.get(escalation_table_url+args,:follow_redirect=>true).content
  resp
end

begin
  log(:info, "Main: entering main method")

  prov = $evm.root['miq_provision']
  values = prov.options[:ws_values]
  
  if values[:quick_base_create_shell].eql? "f"
    log(:info, "not creating EQB shell")
  else
    log(:info, "creating EQB shell")

    intuit_dc = get_intuit_dc()
    config = {}
    retrieve_configuration(config, 'intuit_dc', intuit_dc)

    host_name = prov.get_option(:vm_target_hostname)
    domain = prov.get_option(:dns_domain)
    if domain.blank?
      domain = prov.get_option(:dns_suffixes)
    end
    if !(host_name.include? domain)
      host_name += "." + domain
    end

    http = HTTPClient.new(config['proxy_url'])

    #getting token needed for authentication
    $evm.current['password'] = config['eqb_password']
    token = authenticate(http,config['eqb_username'], $evm.current.decrypt('password'),config['eqb_base_url'])

    #getting the app value associated with the app name 
    #this is needed because the escalation db uses a number value not names
    app_value = get_app_value(http,token,prov.options[:ws_values][:application_name],config['eqb_query_url'])

    #push to escalation db 
    resp = push_to_escalation_qb_table(http,token,app_value,host_name,config['escalation_table_url'],config['eqb_dev_description'])
    xml_resp = Nokogiri.parse(resp)

    errcode = xml_resp.xpath("qdbapi/errcode")[0].content
    errtext = xml_resp.xpath("qdbapi/errtext")[0].content
    if errcode == "0"
      rid = Nokogiri::XML.parse(resp).xpath("/qdbapi/rid")[0].content
      vm = get_virtual_machine()
      url = "https://intuitcorp.quickbase.com/db/bdmsudrgp?a=dr&rid=#{rid}"
      vm.custom_set("eqb_record_url", url)

      log(:info, "eqb_record_url = #{url}")

      # Prepare rollback in case we run into an error.
      rollback_parameters = prov.get_option(:rollback_parameters)
      rollback_parameters['eqb_record_id'] = rid
      prov.set_option(:rollback_parameters, rollback_parameters)
    elsif errcode == "51"
      log(:warn, "Escalation QB Record for #{host_name} already exists")
    else
      retry_method("Error #{errcode} deleting record #{record_id}: #{errtext}")
    end
  end
  
  $evm.root['ae_result'] = 'ok'
  $evm.root['ae_reason'] = ''

  log(:info, "Main: exiting main method")
  exit MIQ_OK

rescue StandardError, SignalException => err
  retry_method("[#{err}]\n#{err.backtrace.join("\n")}")
end
