###################################
#
# CFME Automate Method: initiate_resize_second_disk
# Author: Carsten Clasohm
#
# Notes: Acquire one storage IP addresses and 
#        add a NIC to the VM.
#
# Inputs: vm
#         storage_ip: optional
#         storage_hostname: optional
#
# Mods:
#
###################################

require 'ipaddr'
require 'uri'

EMAIL_FROM = "cloudforms@intuit.com"

# logging convenience method
def log(level, msg)
  @method = 'initiate_resize_second_disk'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, msg)
  email_error(msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def retry_method(msg, retry_time=1.minute)
  log(:warn, "Retrying in #{retry_time} seconds: [#{msg}]")
  $evm.root['ae_result'] = 'retry'
  $evm.root['ae_retry_interval'] = retry_time
  exit MIQ_OK
end

def instantiate(uri, params = {})
  result = $evm.instantiate(uri)

  return nil if result.blank? && params[:ignore_error]

  error("Failed to execute #{uri}") unless result
  
  case result['result']
  when 'error'
    error(result['reason'])
  when 'retry'
    retry_method(result['reason'])
  end
  
  result
end

def get_region_endpoint(vm)
  uri = "Intuit/Integration/CloudForms/Actions/GetRegionEndpoints?vmdb_object_type=vm&Vm::vm=#{vm.id}"
  result = instantiate(uri)
  intuit_dc = result['intuit_dc']
  ip_addresses = result['region_endpoint_ip_addresses']

  log(:info, "Mapped VM to CloudForms region: [#{vm.name}][#{intuit_dc}][#{ip_addresses}]")

  {
    :intuit_dc => intuit_dc, 
    :ip_addresses => ip_addresses
  }
end

def create_automation_request(region_endpoint, vm, dialog_size)
  uri = "Intuit/Integration/CloudForms/Actions/CreateAutomationRequest?" + \
    URI.encode_www_form([['intuit_dc', region_endpoint[:intuit_dc]],
                         ['ae_namespace', $evm.current_namespace],
                         ['ae_class', $evm.current_class],
                         ['ae_instance', $evm.current_instance],
                         ['parameters', "Vm::vm=#{vm.id}|dialog_size=#{dialog_size}"]])

  region_endpoint[:ip_addresses].each do |ip_address|
    break if instantiate("#{uri}&ip_address=#{ip_address}", :ignore_error => true)
  end
end

def get_virtual_machine()
  return $evm.current['vm'] unless $evm.current['vm'].blank?
  case $evm.root['vmdb_object_type']
  when 'miq_provision'
    vm = $evm.root['miq_provision'].vm
  when 'vm'
    vm = $evm.root['vm']
  end
  raise "VM not found" unless vm
  vm
end

def get_vix_platform(vm)
  ae_uri = "Intuit/Integration/VIX/Actions/GetVixPlatform?" + \
    URI.encode_www_form([['Vm::vm', vm.id]])

  result = instantiate(ae_uri)
  vix_platform = result['vix_platform']
  $evm.log(:info, "VIX platform: [#{vix_platform}]")
  return vix_platform
end

def resize_second_disk(vm, intuit_dc, dialog_size)

  ae_uri = "Intuit/Integration/ExpandDisk/StateMachine/Default?" + \
    URI.encode_www_form([['Vm::vm', vm.id],
                         ['dialog_size', dialog_size]])
  result = instantiate(ae_uri)

  #log(:info, "Result #{result.attributes.each.inspect}")
  
  log(:info, "Resized Second Disk to #{dialog_size} on #{vm.name}")
end

def set_custom_attribute(vm, key, value)
  ae_uri = "Intuit/Integration/HPSA/Actions/SetCustomAttribute?Vm::vm=#{vm.id}&" + \
    URI.encode_www_form([['attr_key', key],
                         ['attr_value', value]])
  instantiate(ae_uri)
end


def email_error(msg)
  vm = get_virtual_machine

  task = $evm.root['automation_task']
  if task
    user_email = task.options[:attrs][:user_email]
  else
    user_email = $evm.root['user'].email
  end

  from = EMAIL_FROM

  if user_email.blank?
    to = EMAIL_FROM
  else
    to = user_email
  end

  subject = "Failed to Add Storage Network Interface for #{vm.name}"

  body = \
    "<p>Error while adding storage network interface to virtual machine " \
    "#{vm.name}:</p>" \
    "<pre>#{msg}</pre>"

  body += "<pre>automation_task_#{$evm.root['automation_task_id']}<p>" if $evm.root['automation_task_id']

  log(:info, "Sending email to <#{to}> from <#{from}> subject: <#{subject}>")
  $evm.execute('send_email', to, from, subject, body)
end

def email_success(vm, user_email)
  vix_platform = get_vix_platform(vm)

  short_hostname = hostname.sub(%r{\..*$}, '')

  from = EMAIL_FROM

  if user_email.blank?
    to = EMAIL_FROM
  else
    to = user_email
  end

  subject = "Storage Network Interface Added for #{vm.name}"

  body = "
<p>Storage network interface #{nic_info[:nic_device]} has been added 
to virtual machine #{vm.name} with the following parameters:</p>

<p>IP address: #{ip_address}<br>
Subnet mask: #{netmask}<br>
Hostname: #{hostname}</p>

<p>HPSA is currently configuring the IP address within the VM, this should
take no more than 2 minutes.</p>"

  body += "
<p>If the storage you are trying to access is not in the same subnet as
#{ip_address}, you have to add a static route.</p>

<p>On Linux systems, this is done by creating 
/etc/sysconfig/network-scripts/route-#{nic_device} 
with content similar to this, where \"192.168.0.0/24\" is the subnet the storage 
is located in:</p>

<pre>192.168.0.0/24 via #{gateway} dev #{nic_device}</pre>

<p>Activate the static route with the following commands:</p>

<pre>
/sbin/ifdown eth1
/sbin/ifup eth1
</pre>"

  log(:info, "Sending email to <#{to}> from <#{from}> subject: <#{subject}>")
  $evm.execute('send_email', to, from, subject, body)
end

begin
  log(:info, "Main: entering main method")

  task = $evm.root['automation_task']
  if task
    # Initiated from an automation request down to the local region.

    vm = get_virtual_machine

    dialog_size = $evm.root['dialog_size']

    ems = vm.ext_management_system
    intuit_dc = ems.tags(:intuit_dc).first

  #  nic_info = add_storage_nic(vm, intuit_dc, storage_ip, storage_hostname)
    resize_second_disk(vm, intuit_dc, dialog_size)

    #email_success(vm, nic_info, user_email)
  else
    # Initiated from a custom button.

    vm = $evm.root['vm']
    #dialog_disk_number = $evm.root['dialog_disk_number']
    dialog_size = $evm.root['dialog_size']
    #user_email = $evm.root['user'].email

    region_endpoint = get_region_endpoint(vm)
    create_automation_request(region_endpoint, vm, dialog_size)
  end

  $evm.root['ae_result'] = 'ok'
  $evm.root['ae_reason'] = ''

  log(:info, "Main: exiting main method")
  exit MIQ_OK

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
