####################################
#
# CFME Automate Method: resize_in_os
#
# Author: Nick Lane
#
# Notes: Downloads the HPSA agent installer, transfers it to the new
#        VM, and executes the installer
#
# Inputs: vm: the target VM
#
# Mods:
#
###################################

require 'open-uri'
require 'tempfile'
require 'timeout'

DOWNLOAD_TIMEOUT                 = 2.minutes
VIX_ACTIONS_URI                  = "/Intuit/Integration/VIX/Actions"
GET_VIX_PLATFORM_URI             = "#{VIX_ACTIONS_URI}/GetVixPlatform"
RUN_PROGRAM_IN_GUEST             = "#{VIX_ACTIONS_URI}/RunProgramInGuest"


def dump_root()
  log(:info, "Resize - Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| log(:info, "Resize - Root:<$evm.root> Attribute - #{k}: #{v}")}
  log(:info, "Resize - Root:<$evm.root> End $evm.root.attributes")
  log(:info, "")
end

def error(msg)
    log(:error, "Resize - Resize failed: [#{msg}]")
    $evm.current['result'] = 'error'
    $evm.current['reason'] = msg.to_s
  exit MIQ_OK
end

def log(level, msg)
  @method = 'resize_in_os'
  $evm.log(level, "#{@method}: #{msg}")
end

def escape_uri(value)
  URI.escape(value, Regexp.new("[^#{URI::PATTERN::UNRESERVED}]"))
end

def get_virtual_machine()
  case $evm.root['vmdb_object_type']
  when 'miq_provision'
    vm = $evm.root['miq_provision'].vm
  when 'vm'
    vm = $evm.root['vm']
  when 'automation_task'
    vm = $evm.root['vm']
  end
  raise "VM not found" unless vm
  vm
end

def get_vix_platform()
  result = instantiate(GET_VIX_PLATFORM_URI)
  vix_platform = result['vix_platform']
  log(:info, "VIX platform: [#{vix_platform}]")
  return vix_platform
end


def instantiate(uri)
  result = $evm.instantiate(uri)
  error("Failed to execute #{GET_VIX_PLATFORM_URI}") unless result
  error(result['reason']) unless result['result'] == 'ok'
  return result
end

def get_commands(config, class_name, instance_name)
  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  commands = $evm.instantiate(instance_uri)
  error("Unable to retrieve required configuration: [#{instance_uri}]") unless commands
  
  log(:info, "Retrieved configuration: [#{class_name}/#{instance_name}[#{commands.attributes}]")
  return commands
end


def execute_in_os(vix_platform, vix_guest_program, vix_guest_options)
  log(:info, "Attempting to resize disk in OS with the following commands: [#{vix_guest_program}][#{vix_guest_options}]")
  begin
    Timeout::timeout(180) do
      while true do
  run_program_uri = "#{RUN_PROGRAM_IN_GUEST}?" \
    "vix_platform=#{vix_platform}&" \
    "vix_guest_program=#{escape_uri(vix_guest_program)}&" \
    "vix_guest_options=#{escape_uri(vix_guest_options)}"
  result = $evm.instantiate(run_program_uri)
        unless result['result'] == 'retry'
          break
        end
        log(:info, "Retrying #{vix_guest_program}")
      end
    end
  rescue Timeout::Error
    error('Timed out running #{vix_guest_program}')
  end
  error("Failed to execute #{RUN_PROGRAM_IN_GUEST}") unless result

  error(result['reason']) unless result['result'] == 'ok'
end

begin
  dump_root()
  
  vm = get_virtual_machine()  
  vix_platform = get_vix_platform()

  config = {}
  commands = get_commands(config, 'VixPlatform', vix_platform)

  commands.attributes.each { |k,v| 
  #log(:info, "Returned these keys: #{k} v: #{v}")
  values = v.split(" ")
  command = values.shift
  options = values.join(" ")
  log(:info, "command: #{command} options: #{options}")
  execute_in_os(vix_platform, command, options)
  }

 

  $evm.current['result'] = 'ok'
  $evm.current['reason'] = ''

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
