###################################
#
# CFME Automate Method: add_nic
#
# Author: Carsten Clasohm
#
# Notes: Adds a NIC to an existing VM.
#
# Inputs:
#   vm
#   vmware_network
#
# Mods:
#
###################################

require 'fog'

# logging convenience method
def log (level, msg)
  @method = 'add_nic'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, "ERROR: #{msg}")
  $evm.current['result'] = 'error'
  $evm.current['reason'] = msg
  exit MIQ_OK
end

def get_vmware_vm_id(conn, vm_name)
  response = conn.list_virtual_machines

  response.each do | vm |
    return vm['id'] if vm['name'] == vm_name
  end

  nil
end

def add_interface (conn, vm_id, options)
  result = conn.add_vm_interface(vm_id, options)

  log(:info, "catch1: #{result.inspect}")

  error("Failed to add NIC: #{result.inspect}") unless result['task_state'] == "success"
end

def get_virtual_machine()
  return $evm.current['vm'] unless $evm.current['vm'].blank?
  case $evm.root['vmdb_object_type']
  when 'miq_provision'
    vm = $evm.root['miq_provision'].vm
  when 'vm'
    vm = $evm.root['vm']
  end
  raise "VM not found" unless vm
  vm
end

begin
  log(:info, "Main: entering main method")

  vmware_network = $evm.current['vmware_network']

  vm = get_virtual_machine
  ems = vm.ext_management_system
  intuit_dc = ems.tags(:intuit_dc).first

  vim_username = ems.authentication_userid
  vim_password = ems.authentication_password

  error("Missing VIM username or password for intuit_dc #{intuit_dc}") if vim_username.nil? or vim_password.nil?

  vcenter_db = "Customer/Intuit/Integration/Fog/Configuration/vCenter"
  vcenter_props = $evm.instance_find("#{vcenter_db}/#{ems.hostname}")[ems.hostname]

  error("Configuration properties not found in #{vcenter_db}/#{ems.hostname}") if vcenter_props.nil?

  # If the VM currently has one NIC, eth0, the new device will be eth1.
  nic_device = "eth#{vm.hardware.nics.length}"

  credentials = {
    :provider         => "vsphere",
    :vsphere_username => vim_username,
    :vsphere_password => vim_password,
    :vsphere_server   => ems.hostname,
    :vsphere_ssl      => true,
    :vsphere_expected_pubkey_hash => vcenter_props['pubkey_hash']
  }

  conn = Fog::Compute.new(credentials)

  vmware_vm_id = get_vmware_vm_id(conn, vm.name)

  add_interface(conn, vmware_vm_id, { 
                  :network => vmware_network, 
                  :type => 'VirtualVmxnet3'
                })

  $evm.current['result'] = 'ok'
  $evm.current['reason'] = ''
  $evm.current['nic_device'] = nic_device

  log(:info, "Main: exiting main method")
  exit MIQ_OK

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
