###################################
#
# CFME Automate Method: apply_software_policy
#
# Author: Jason Dillaman
#
# Notes: Applies the HPSA baseline software policy to a VM
#
# Inputs: vm: the target VM
#         hpsa_software_policy_id: optional specific HPSA software policy to apply
#
# Changes:
#   2015-12-07 Daniel Trieu updated Savon 2.0
#   2015-12-23 Earl Waud added a skip for Canada DCs
#
###################################

require 'httpi'
require 'savon'
require 'httpclient'

# logging convenience method
def log(level, msg)
  @method = 'apply_software_policy'
  $evm.log(level, "#{@method}: #{msg}")
end

def dump_root()
  log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  log(:info, "Root:<$evm.root> End $evm.root.attributes")
  log(:info, "")
end

def error(msg)
  log(:error, "Apply HPSA baseline software policy to a VM failed, exiting [#{msg}]")
  $evm.current['result'] = 'error'
  $evm.current['reason'] = msg.to_s
  exit MIQ_OK
end

def retry_method(msg, retry_time=1.minute)
  log(:warn, "Retrying in #{retry_time} seconds: [#{msg}]")
  $evm.current['result'] = 'retry'
  $evm.current['retry_interval'] = retry_time
  exit MIQ_OK
end

#def configure_soap()

  # SOAP client setup
#  @client = nil
#  @session = nil

#  HTTPI.log_level = :warn
#  HTTPI.log = false
#  HTTPI.adapter = :net_http

  #Savon.configure do |config|
  #  config.log = true
  #  config.log_level = :warn
  #end

#end

def get_virtual_machine()
  return $evm.current['vm'] unless $evm.current['vm'].blank?
  case $evm.root['vmdb_object_type']
    when 'miq_provision'
      vm = $evm.root['miq_provision'].vm
    when 'vm'
      vm = $evm.root['vm']
  end
  raise "VM not found" unless vm
  vm
end

def get_virtual_machine_parent(vm)
  templates = vm.wrap_results(vm.object_send(:parents))
  if templates.length > 1
    log(:info, "Virtual machine [#{vm.name}] has multiple parent relationships: [#{templates}]")
    template = templates.first
  else
    template = vm.wrap_results(vm.object_send(:parent))
  end
  error("Virtual machine is missing its parent template relationship: [#{vm.name}]") unless template
  return template
end

def get_intuit_dc(vm)
  ems = vm.ext_management_system
  intuit_dc = ems.tags(:intuit_dc).first
  error("Intuit Data Center tag missing on provider: [#{ems.name}]") unless intuit_dc
  return intuit_dc
end

def get_software_policy_names(vm)
  template = get_virtual_machine_parent(vm)

  hpsa_baseline_tag = template.tags(:intuit_hpsa_baseline).first
  error("Template missing HPSA Baseline Policy tag: [#{template.name}]") unless hpsa_baseline_tag

  classification = $evm.vmdb('classification').find_by_name("intuit_hpsa_baseline/#{hpsa_baseline_tag}")
  error("Unable to retrieve tag description: [#{hpsa_baseline_tag}]") unless classification
  return [classification.description]
end

def create_hpsa_service(hostname, service, username, password)
  log(:info, "Creating SOAP service to #{hostname} with #{username}/******")
  urlServerService = "/osapi/com/opsware/#{service}"
  log(:info, "Create ServerService SOAP Object to https://#{hostname}#{urlServerService}?wsdl")
#  serverService = Savon::Client.new do |wsdl, http|
#    wsdl.document = "https://#{hostname}#{urlServerService}?wsdl"
#    wsdl.endpoint = "https://#{hostname}#{urlServerService}"
#    http.auth.basic username, password
#    http.auth.ssl.verify_mode = :none
#  end

  # configure HTTPI
  HTTPI.log_level = :info
  HTTPI.log = false
  HTTPI.adapter = :net_http

  # Configure Savon
  client = Savon.client(
      wsdl: "https://#{hostname}#{urlServerService}?wsdl",
      endpoint: "https://#{hostname}#{urlServerService}",
      read_timeout: 300,
      log: false,
      log_level: :debug,
      basic_auth: [username, password],
      ssl_verify_mode: :none
  )

  client
end

def create_server_service(hostname, username, password)
  create_hpsa_service(hostname, 'server/ServerService', username, password)
end

def create_software_policy_service(hostname, username, password)
  create_hpsa_service(hostname, 'swmgmt/SoftwarePolicyService', username, password)
end

def find_server(server_service, name)
  log(:info, "Searching HPSA using filter ServerVO.name CONTAINS #{name}")
  #response = server_service.request :find_server_refs do |soap|
  #  soap.body = {
  #      'filter' => {
  #          'expression' => "ServerVO.name CONTAINS \"#{name}\"",
  #          'objectType' => nil
  #      }
  #  }
  #end.to_hash

  response = server_service.call(
    :find_server_refs,
    message: {
        'filter' => {
            'expression' => "ServerVO.name CONTAINS \"#{name}\"",
            'objectType' => nil
        }
    }
  ).to_hash

  log(:info, "Got Response to find_server_refs: #{response}")
  ref = response[:multi_ref]
  retry_method("Unable to locate server: [#{name}]") unless ref
  log(:info, "Ref Details #{ref}")
  if "#{ref.class}" == "Array"
    error("Multiple HPSA Servers found.")
  else
    return ref[:id], ref
  end
end

def find_software_policy(software_policy_service, name)
  # Get a ref to the Software Policy Service
  #response = software_policy_service.request :find_software_policy_refs do |soap|
  #  soap.body = {
  #      'filter' => {
  #          'expression' => "SoftwarePolicyVO.name EQUAL_TO \"#{name}\"",
  #          'objectType' => nil
  #      }
  #  }
  #end.to_hash

  response = software_policy_service.call(
      :find_software_policy_refs,
      message: {
          'filter' => {
              'expression' => "SoftwarePolicyVO.name EQUAL_TO \"#{name}\"",
              'objectType' => nil
          }
      }
  ).to_hash

  log(:info, "Got response from find_software_policy_refs #{response}")
  pref = response[:multi_ref]
  retry_method("Unable to locate software policy: [#{name}]") unless pref
  log(:info, "pref (multi_ref): #{pref.class} #{pref}")
  if "#{pref.class}" == "Array"
    log(:warn, "Found multiple software policies named #{name}, using first one.")
    return pref[0][:id]
  else
    return pref[:id]
  end
end

def attach_software_policy(intuit_dc, server_service, server_id, server_refs, policy_id)
  policy = { :item => { :id => policy_id } }
  log(:info, "Attaching policy #{policy.inspect} to #{server_id}")

  policy_type = "ns2:ArrayOf_tns10_SoftwarePolicyRef"

  #response = server_service.request :attach_policies do |soap|
  #  soap.body = {
  #      :self =>  { :id => server_id },
  #      :policies => policy,
  #      :attributes! => {
  #          :self => {
  #              'xsi:type' => "#{server_refs[:"@xsi:type"]}",
  #              'xmlns:ns2' => "#{server_refs[:"@xmlns:ns2"]}"
  #          },
  #          :policies => {
  #              'xsi:type' => policy_type,
  #              'xmlns:ns2' => "#{server_refs[:"@xmlns:ns2"]}"
  #          }
  #      }
  #  }
  #end

  response = server_service.call(
      :attach_policies,
      message: {
          :self =>  { :id => server_id },
          :policies => policy,
          :attributes! => {
              :self => {
                  'xsi:type' => "#{server_refs[:"@xsi:type"]}",
                  'xmlns:ns2' => "#{server_refs[:"@xmlns:ns2"]}"
              },
              :policies => {
                  'xsi:type' => policy_type,
                  'xmlns:ns2' => "#{server_refs[:"@xmlns:ns2"]}"
              }
          }
      }
  )

  log(:info, "Attached SoftwarePolicies Response #{response.to_hash}")
end

def remediate_software_policy(software_policy_service, server_id, server_refs, policy_id)
  policy = { :item => { :id => policy_id } }
  log(:info, "Remediating Software Policy #{policy.inspect} on #{server_id}")
  #response = software_policy_service.request :start_remediate_now do |soap|
  #  soap.body = {
  #      :selves => policy,
  #      :attachables => [ { :item => { :id => server_id } } ],
  #      :attributes! => {
  #          :attachables => {
  #              'xsi:type'          => 'soapenc:Array',
  #              'xmlns:soapenc'     => 'http://schemas.xmlsoap.org/soap/encoding/',
  #              'soapenc:arrayType' => "#{server_refs[:"@xsi:type"]}[1]",
  #              'xmlns:ns2'         => "#{server_refs[:"@xmlns:ns2"]}"
  #          }
  #      }
  #  }
  #end

  response = software_policy_service.call(
      :start_remediate_now,
      message: {
          :selves => policy,
          :attachables => [ { :item => { :id => server_id } } ],
          :attributes! => {
              :attachables => {
                  'xsi:type'          => 'soapenc:Array',
                  'xmlns:soapenc'     => 'http://schemas.xmlsoap.org/soap/encoding/',
                  'soapenc:arrayType' => "#{server_refs[:"@xsi:type"]}[1]",
                  'xmlns:ns2'         => "#{server_refs[:"@xmlns:ns2"]}"
              }
          }
      }
  )

  log(:info, "Remediate Now Response for policy #{response.to_hash}")
end

def get_server_software_policies(server_service, server_id, server_refs)
  log(:info, "Retrieving Attached Software Policies: [#{server_id}]")
  #response = server_service.request :get_software_policies do |soap|
  #  soap.body = {
  #      :selves => { :item => { :id => server_id } },
  #      :attributes! => {
  #          :selves => {
  #              'xsi:type'          => 'soapenc:Array',
  #              'xmlns:soapenc'     => 'http://schemas.xmlsoap.org/soap/encoding/',
  #              'soapenc:arrayType' => "#{server_refs[:"@xsi:type"]}[1]",
  #              'xmlns:ns2'         => "#{server_refs[:"@xmlns:ns2"]}"
  #          }
  #      }
  #  }
  #end

  response = server_service.call(
      :get_software_policies,
      message: {
          :selves => { :item => { :id => server_id } },
          :attributes! => {
              :selves => {
                  'xsi:type'          => 'soapenc:Array',
                  'xmlns:soapenc'     => 'http://schemas.xmlsoap.org/soap/encoding/',
                  'soapenc:arrayType' => "#{server_refs[:"@xsi:type"]}[1]",
                  'xmlns:ns2'         => "#{server_refs[:"@xmlns:ns2"]}"
              }
          }
      }
  ).to_hash

  log(:info, "Get Software Policies Response for policy #{response}")

  multi_ref = response[:multi_ref]
  multi_ref = [multi_ref].compact unless multi_ref.is_a?(Array)
  retry_method("Unable to retrieve server software policies: [#{server_id}]") unless multi_ref
  return multi_ref.collect { |policy| policy[:name] }
end

def update_vm_hpsa_attributes(vm, software_policy_names)
  software_policy_names = software_policy_names.join(', ')
  log(:info, "Updating VM HPSA Software Policies: [#{vm.name}][#{software_policy_names}]")
  vm.custom_set('HPSA Software Policies', software_policy_names)
end

def retrieve_configuration(config, class_name, instance_name)
  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  instance = $evm.instantiate(instance_uri)
  error("Unable to retrieve required configuration: [#{instance_uri}]") unless instance

  log(:info, "Retrieved configuration: [#{class_name}/#{instance_name}[#{instance.attributes}]")
  instance.attributes.each { |k, _| config[k] = instance.decrypt(k) }
end

begin
  log(:info, "Starting Method")
  #dump_root()

  vm = get_virtual_machine()
  ems = vm.ext_management_system
  datacenter = ems.tags(:intuit_dc).first
  log(:info, "Canada check, dc found: #{datacenter}")

  dc_skip_list = $evm.current['datacenters_to_skip']
  if (dc_skip_list.nil?)
    dc_skip_list = []
  end
  log(:info, "Canada check, list of dc's to skip #{dc_skip_list}")

  if ( dc_skip_list.include?(datacenter) )
    log(:info, "Detected Canada Provision, so the Apply HPSA baseline software policy is being skipped")
  else
    log(:info, "Continue with Apply HPSA baseline software policy")
    intuit_dc = get_intuit_dc(vm)

    config = {}
    retrieve_configuration(config, 'intuit_dc', intuit_dc)

    hostname = config['hpsa_hostname']
    username = config['hpsa_username']
    password = config['hpsa_password']

    #configure_soap()
    server_service = create_server_service(hostname, username, password)
    software_policy_service = create_software_policy_service(hostname, username, password)

    server_id, server_refs = find_server(server_service, vm.name)

    if $evm.root['vmdb_object_type'] == 'miq_provision'
      prov = $evm.root['miq_provision']

      # Prepare rollback in case we run into an error.
      rollback_parameters = prov.get_option(:rollback_parameters)
      rollback_parameters['hpsa_vm_id'] = vm.id
      prov.set_option(:rollback_parameters, rollback_parameters)
    end

    software_policy_ids = [$evm.current['hpsa_software_policy_id']].compact
    if software_policy_ids.blank?
      software_policy_names = get_software_policy_names(vm)
      software_policy_ids = software_policy_names.collect do |name|
        find_software_policy(software_policy_service, name)
      end
    end

    software_policy_ids.each do |policy_id|
      attach_software_policy(intuit_dc, server_service, server_id, server_refs, policy_id)
    end

    # Wait up to one minute for the software policies to be visible in
    # Opsware.
    tries = 0
    begin
      software_policy_names = get_server_software_policies(server_service, server_id, server_refs)
      sleep 5 if software_policy_names.empty?
      tries += 1
    end while software_policy_names.empty? and tries < 12

    log(:info, "No attached policies found in HPSA after 60 seconds") if software_policy_names.empty?

    software_policy_ids.each do |policy_id|
      remediate_software_policy(software_policy_service, server_id, server_refs, policy_id)
    end

    update_vm_hpsa_attributes(vm, software_policy_names)
  end

  $evm.current['result'] = 'ok'
  $evm.current['reason'] = ''

rescue StandardError, SignalException => err
  retry_method("[#{err}]\n#{err.backtrace.join("\n")}")
end
