###################################
#
# CFME Automate Method: get_software_policies
#
# Author: Jason Dillaman
#
# Notes: Retrieves all available software policies in HPSA
#
# Inputs: hpsa_software_policy_filter: optional software policy name filter 
#         intuit_dc: the Intuit DC
#
# Mods:
#
###################################

require 'httpi'
require 'savon'

def dump_current()
  $evm.log(:info, "Current:<$evm.current> Begin $evm.current.attributes")
  $evm.current.attributes.sort.each { |k, v| $evm.log(:info, "Current:<$evm.current> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Current:<$evm.current> End $evm.current.attributes")
  $evm.log(:info, "")
end

def configure_soap()
  HTTPI.log_level = :warn
  HTTPI.log = false
  HTTPI.adapter = :net_http

  #Savon.configure do |config|
  #  config.log = true
  #  config.log_level = :warn
  #end
end

def error(msg)
  $evm.log(:error, "HPSA apply software policy failed: [#{msg}]")
  $evm.current['result'] = 'error'
  $evm.current['reason'] = msg.to_s
  exit MIQ_OK
end

def retrieve_configuration(config, class_name, instance_name)
  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  instance = $evm.instantiate(instance_uri)
  error("Unable to retrieve required configuration: [#{instance_uri}]") unless instance
  
  $evm.log(:info, "Retrieved configuration: [#{class_name}/#{instance_name}[#{instance.attributes}]")
  instance.attributes.each { |k, _| config[k] = instance.decrypt(k) }
end

def create_hpsa_service(hostname, service, username, password)
  $evm.log(:info, "Creating SOAP service to #{hostname} with #{username}/******")
  urlServerService = "/osapi/com/opsware/#{service}"
  $evm.log(:info, "Create ServerService SOAP Object to https://#{hostname}#{urlServerService}?wsdl")
  #serverService = Savon::Client.new do |wsdl, http|
  #  wsdl.document = "https://#{hostname}#{urlServerService}?wsdl"
  #  wsdl.endpoint = "https://#{hostname}#{urlServerService}"
  #  http.auth.basic username, password
  #  http.auth.ssl.verify_mode = :none
  #end

  # Configure Savon
  serverService = Savon.client(
      wsdl: "https://#{hostname}#{urlServerService}?wsdl",
      endpoint: "https://#{hostname}#{urlServerService}",
      read_timeout: 300,
      log: false,
      log_level: :debug,
      basic_auth: [username, password],
      ssl_verify_mode: :none
  )
end

def create_software_policy_service(hostname, username, password)
  create_hpsa_service(hostname, 'swmgmt/SoftwarePolicyService', username, password)
end

def find_software_policies(software_policy_service, hpsa_software_policy_filter)
  # Get a ref to the Software Policy Service
  filter = {}
  unless hpsa_software_policy_filter.blank?
    filter = {
      'filter' => {
         'expression' => "SoftwarePolicyVO.name CONTAINS \"#{hpsa_software_policy_filter}\"",
         'objectType' => nil
      }
    }
    $evm.log(:info, "Using HPSA software policy filter: [#{filter}]") 
  end
  
  #response = software_policy_service.request :find_software_policy_refs do |soap|
  #  soap.body = filter
  #end.to_hash
  
  response = software_policy_service.call(
    :find_software_policy_refs,
    message: filter
  ).to_hash
  
  $evm.log(:info, "Got response from find_software_policy_refs #{response}")
  multi_ref = response[:multi_ref]
  multi_ref = [multi_ref].compact unless multi_ref.is_a?(Array)
  multi_ref = multi_ref.collect { |ref| clean_software_policy(ref) }
  error("Unable to locate software policies: [#{hpsa_software_policy_filter}]") if multi_ref.blank?
  $evm.log(:info, "multi_ref: [#{multi_ref}]")
  return multi_ref
end

def clean_software_policy(policy)
  {
    :id   => policy[:id].to_i,
    :name => policy[:name].to_s
  }
end

begin
  dump_current()
  
  intuit_dc = $evm.current['intuit_dc']
  error("Intuit DC must be provided") if intuit_dc.blank?
  
  config = {}
  retrieve_configuration(config, 'intuit_dc', intuit_dc)
  
  configure_soap()
  software_policy_service = create_software_policy_service(config['hpsa_hostname'], config['hpsa_username'], config['hpsa_password'])
  
  software_policies = find_software_policies(software_policy_service, $evm.current['hpsa_software_policy_filter'])

  $evm.current['hpsa_software_policies'] = software_policies
  $evm.current['result'] = 'ok'
  
rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
