###################################
#
# CFME Automate Method: install_hpsa_agent
#
# Author: Jason Dillaman
#
# Notes: Downloads the HPSA agent installer, transfers it to the new
#        VM, and executes the installer
#
# Inputs: vm: the target VM
#
# Changes:
#   2015-12-23 Earl Waud added a skip for Canada DCs
#
###################################

require 'open-uri'
require 'tempfile'
require 'timeout'

DOWNLOAD_TIMEOUT                 = 2.minutes
VIX_ACTIONS_URI                  = "/Intuit/Integration/VIX/Actions"
COPY_FILE_FROM_HOST_TO_GUEST_URI = "#{VIX_ACTIONS_URI}/CopyFileFromHostToGuest"
GET_VIX_PLATFORM_URI             = "#{VIX_ACTIONS_URI}/GetVixPlatform"
RUN_PROGRAM_IN_GUEST             = "#{VIX_ACTIONS_URI}/RunProgramInGuest"
RHEL5_VMWARE_TOOLS_SCRIPT        = "/etc/init.d/vmware-tools"
RHEL6_VMWARE_TOOLS_SCRIPT        = "/etc/vmware-tools/services.sh"

# logging convenience method
def log(level, msg)
  @method = 'install_hpsa_agent'
  $evm.log(level, "#{@method}: #{msg}")
end

def dump_root()
  log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  log(:info, "Root:<$evm.root> End $evm.root.attributes")
  log(:info, "")
end

def error(msg)
  log(:error, "HPSA agent installation failed, exiting [#{msg}]")
  $evm.current['result'] = 'error'
  $evm.current['reason'] = msg.to_s
  exit MIQ_OK
end

def retry_method(msg, retry_time=1.minute)
  log(:warn,"HPSA agent installation failed, retrying in #{retry_time} seconds: [#{msg}]")
  $evm.current['result'] = 'retry'
  $evm.current['ae_retry_interval'] = retry_time
  exit MIQ_OK
end

def escape_uri(value)
  URI.escape(value, Regexp.new("[^#{URI::PATTERN::UNRESERVED}]"))
end

def get_virtual_machine()
  case $evm.root['vmdb_object_type']
    when 'miq_provision'
      vm = $evm.root['miq_provision'].vm
    when 'vm'
      vm = $evm.root['vm']
  end
  raise "VM not found" unless vm
  vm
end

def get_virtual_machine_parent(vm)
  templates = vm.wrap_results(vm.object_send(:parents))
  if templates.length > 1
    log(:info, "Virtual machine [#{vm.name}] has multiple parent relationships: [#{templates}]")
    template = templates.first
  else
    template = vm.wrap_results(vm.object_send(:parent))
  end
  error("Virtual machine is missing its parent template relationship: [#{vm.name}]") unless template
  return template
end

def get_vix_platform()
  result = instantiate(GET_VIX_PLATFORM_URI)
  vix_platform = result['vix_platform']
  log(:info, "VIX platform: [#{vix_platform}]")
  return vix_platform
end

def get_intuit_hpsa_agent_url(template)
  hpsa_agent_url_tag = template.tags(:intuit_hpsa_agent_url).first
  error("Template missing HPSA Agent URL tag: [#{template.name}]") unless hpsa_agent_url_tag

  classification = $evm.vmdb('classification').find_by_name("intuit_hpsa_agent_url/#{hpsa_agent_url_tag}")
  error("Unable to retrieve tag description: [#{hpsa_agent_url_tag}]") unless classification
  return classification.description
end

def get_intuit_hpsa_gateway_address(vm)
  cluster = vm.ems_cluster
  hpsa_gateway_tag = cluster.tags(:intuit_hpsa_gateway).first

  ems = vm.ext_management_system
  hpsa_gateway_tag = ems.tags(:intuit_hpsa_gateway).first unless hpsa_gateway_tag

  error("Infrastructure missing HPSA Agent URL tag: [#{vm.name}][#{cluster.name}][#{ems.name}]") unless hpsa_gateway_tag

  classification = $evm.vmdb('classification').find_by_name("intuit_hpsa_gateway/#{hpsa_gateway_tag}")
  error("Unable to retrieve tag description: [#{hpsa_gateway_tag}]") unless classification
  return classification.description
end

def instantiate(uri)
  result = $evm.instantiate(uri)
  error("Failed to execute #{uri}") unless result

#  case result['result']
#  when 'error'
#    error(result['reason'])
#  when 'retry'
#    retry_method(result['reason'])
#  end

  result
end

def download_hpsa_agent(timeout, uri, local_file)
  log(:info, "Starting download of HPSA agent: [#{uri}]")
  begin
    Timeout::timeout(timeout) do
      open(uri, "rb", :ssl_verify_mode => OpenSSL::SSL::VERIFY_NONE) do |read_file|
        local_file.write(read_file.read)
        read_file.close()
      end
    end
  rescue Timeout::Error
    retry_method("Timed out downloading HPSA agent")
  end
  log(:info, "HPSA agent download complete")
end

def extract_filename(uri)
  return File.basename(URI.parse(uri).path)
end

def transfer_file_to_vm(vix_platform, local_path, remote_path)
  log(:info, "Attempting to transfer HPSA installer to VM: [#{local_path}][#{remote_path}]")

  transfer_uri = "#{COPY_FILE_FROM_HOST_TO_GUEST_URI}?" \
    "vix_platform=#{vix_platform}&" \
    "vix_local_file=#{escape_uri(local_path)}&" \
    "vix_remote_file=#{escape_uri(remote_path)}"
  result = instantiate(transfer_uri)
  retry_method("VIX reported failure: [#{result['exit_status']}]") unless result['exit_status'] == 0
end

def transfer_hpsa_agent_to_guest(download_timeout, vix_platform, hpsa_agent_base_path, vm)
  template = get_virtual_machine_parent(vm)
  hpsa_agent_url = get_intuit_hpsa_agent_url(template)

  hpsa_agent_base_path += '/' unless hpsa_agent_base_path.match(/(\\|\/)$/)
  vix_guest_program = "#{hpsa_agent_base_path}#{extract_filename(hpsa_agent_url)}"

  file = Tempfile.new('hpsa_agent', :mode => File::BINARY)
  begin
    download_hpsa_agent(download_timeout, hpsa_agent_url, file)
    transfer_file_to_vm(vix_platform, file.path, vix_guest_program)
  ensure
    file.close()
    file.unlink()
  end
  return vix_guest_program
end

def retrieve_configuration(config, class_name, instance_name)
  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  instance = $evm.instantiate(instance_uri)
  error("Unable to retrieve required configuration: [#{instance_uri}]") unless instance

  log(:info, "Retrieved configuration: [#{class_name}/#{instance_name}[#{instance.attributes}]")
  instance.attributes.each { |k, _| config[k] = instance.decrypt(k) }
end

def run_program_in_guest(vix_platform, vix_guest_program, vix_guest_options)
  log(:info, "Attempting to run within VM: [#{vix_guest_program}][#{vix_guest_options}]")

  ae_uri = "#{RUN_PROGRAM_IN_GUEST}?" \
    "vix_platform=#{vix_platform}&" \
    "vix_guest_program=#{escape_uri(vix_guest_program)}&" \
    "vix_guest_options=#{escape_uri(vix_guest_options)}"
  result = instantiate(ae_uri)
  retry_method("VIX reported failure: [#{result['exit_status']}]") unless result['exit_status'] == 0
end

def build_hpsa_agent_installer_arguments(config, vm)
  "#{config['hpsa_agent_additional_options']} #{config['hpsa_agent_gateway_option']} #{get_intuit_hpsa_gateway_address(vm)}"
end

def file_exists_in_guest(vix_platform, vix_guest_file)
  ae_uri = "/Intuit/Integration/VIX/Actions/FileExistsInGuest?" + \
           URI.encode_www_form([['vix_platform', vix_platform],
                                ['vix_guest_file', vix_guest_file]])

  log(:info, "Instantiating VIX call: [#{ae_uri}]")
  result = $evm.instantiate(ae_uri)

  return result['exit_status']
end

# Customized version of run_program_in_guest that adds the "noWait" option.
def run_program_in_guest_nowait(vix_platform, vix_guest_program, vix_guest_options)
  vmrun_command    = 'runProgramInGuest'
  vmrun_arguments  = " -noWait \"#{vix_guest_program.gsub('\\', '\\\\')}\""
  vmrun_arguments += " #{vix_guest_options}" unless vix_guest_options.blank?

  log(:info, "Executing vmrun: [#{vmrun_command}][#{vmrun_arguments}]")
  result = instantiate(
      "/Intuit/Integration/VIX/Actions/ExecuteVmrun?" \
    "vix_platform=#{vix_platform}&" \
    "vmrun_command=#{vmrun_command}&" \
    "vmrun_arguments=#{escape_uri(vmrun_arguments)}")
  retry_method("VIX reported failure: [#{result['exit_status']}]") unless result['exit_status'] == 0
end

def run_script_in_guest(vix_platform, vix_guest_interpreter, vix_guest_script)
  vmrun_command    = 'runScriptInGuest'
  vmrun_arguments  = " \"#{vix_guest_interpreter.gsub('\\', '\\\\')}\""
  vmrun_arguments += " \"#{vix_guest_script}\""

  log(:info, "Executing vmrun: [#{vmrun_command}][#{vmrun_arguments}]")
  instance = instantiate(
      "/Intuit/Integration/VIX/Actions/ExecuteVmrun?" \
    "vix_platform=#{vix_platform}&" \
    "vmrun_command=#{vmrun_command}&" \
    "vmrun_arguments=#{escape_uri(vmrun_arguments)}")
  retry_method("VIX reported failure: [#{instance['exit_status']}]") unless instance['exit_status'] == 0
end

def restart_vmware_tools(vix_platform)
  # Check if vmware-tools are installed in /etc/vmware-tools.

  service_script = nil
  if file_exists_in_guest(vix_platform, RHEL5_VMWARE_TOOLS_SCRIPT) == 0
    service_script = RHEL5_VMWARE_TOOLS_SCRIPT
  elsif file_exists_in_guest(vix_platform, RHEL6_VMWARE_TOOLS_SCRIPT) == 0
    service_script = RHEL6_VMWARE_TOOLS_SCRIPT
  else
    log(:info, "Warning: Cannot find vmware-tools script, NOT restarting vmware-tools")
    return
  end

  log(:info, "Attempting to restart vmware-tools within VM by running #{service_script} restart")

  run_program_in_guest_nowait(vix_platform, service_script, "restart")
end

begin
  log(:info, "Starting Method")
  #dump_root()

  vm = get_virtual_machine()
  #hosting_segment = vm.tags(:intuit_hs).first
  ems = vm.ext_management_system
  datacenter = ems.tags(:intuit_dc).first
  log(:info, "Canada check, dc found: #{datacenter}")

  dc_skip_list = $evm.current['datacenters_to_skip']
  if (dc_skip_list.nil?)
    dc_skip_list = []
  end
  log(:info, "Canada check, list of dc's to skip #{dc_skip_list}")

  if ( dc_skip_list.include?(datacenter) )
    log(:info, "Detected Canada Provision, so the Install HPSA Agent is being skipped")
  else
    log(:info, "Continue with Install HPSA Agent")

    vix_platform = get_vix_platform()

    config = {}
    retrieve_configuration(config, 'VixPlatform', vix_platform)

    # Log some diagnostics.
    log(:info, "Logging diagnostics to /tmp/hpsa-diagnostics.log")
    if vix_platform == "linux"

      hpsa_gateway_return = get_intuit_hpsa_gateway_address(vm)

      #hpsa_gateway = ""
      if hpsa_gateway_return.include? ","
        wget_entries = ""
        hpsa_gateway_return.split(",").each {|hpsa_gateways|
            wget_entries << "wget -t 1 -T 60 https://#{hpsa_gateways}/;"
        }
        run_script_in_guest(vix_platform, "/bin/bash", "(date; /sbin/ifconfig -a; #{wget_entries} true) >&/tmp/hpsa-diagnostics.log")
      else
        run_script_in_guest(vix_platform, "/bin/bash", "(date; /sbin/ifconfig -a; wget -t 1 -T 60 https://#{hpsa_gateway_return}/; true) >&/tmp/hpsa-diagnostics.log")
      end

      # Restart vmware-tools which should set the system clock to the correct time.
      restart_vmware_tools(vix_platform)

      # And wait a little bit for the vmware-tools service to start again.
      sleep(3)
    end


    vix_guest_program = transfer_hpsa_agent_to_guest(DOWNLOAD_TIMEOUT, vix_platform, config['hpsa_agent_base_path'], vm)
    vix_guest_options = build_hpsa_agent_installer_arguments(config, vm)
    run_program_in_guest(vix_platform, vix_guest_program, vix_guest_options)
  end

  $evm.current['result'] = 'ok'
  $evm.current['reason'] = ''

rescue StandardError, SignalException => err
  retry_method("[#{err}]\n#{err.backtrace.join("\n")}")
end
