###################################
#
# CFME Automate Method: remove_duplicate_record
#
# Author: Bryan Antigua, Carsten Clasohm
#
# Notes: Checks to see if one or more HPSA records already exist for the given hostname. 
#        If so, it deletes them.
#
# Inputs: vm: the target VM
#
# Changes:
#   2015-12-23 Earl Waud added a skip for Canada DCs
#
###################################

require 'httpi'
require 'savon'
require 'httpclient'

# logging convenience method
def log(level, msg)
  @method = 'remove_duplicate_record'
  $evm.log(level, "#{@method}: #{msg}")
end

def dump_root()
  log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  log(:info, "Root:<$evm.root> End $evm.root.attributes")
  log(:info, "")
end

def error(msg)
  log(:error, "#{msg}")
  log(:error, "Error received removing duplicate records, exiting. #{msg}")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg.to_s
  exit MIQ_OK
end

def retry_method(msg)
  $evm.log(:warn,"Error received removing duplicate records, retrying. [#{msg}]")
  $evm.current['result'] = 'retry'
  $evm.current['reason'] = msg.to_s
  exit MIQ_OK
end

def escape_uri(value)
  URI.escape("#{value}", Regexp.new("[^#{URI::PATTERN::UNRESERVED}]"))
end

def instantiate(uri)
  result = $evm.instantiate(uri)
  error("Failed to execute: [#{uri}]") unless result
  error(result['reason']) unless result['result'] == 'ok'
  return result
end

def configure_soap()
  HTTPI.log_level = :warn
  HTTPI.log = false
  HTTPI.adapter = :net_http

  #Savon.configure do |config|
  #  config.log = true
  #  config.log_level = :warn
  #end
end

def get_virtual_machine()
  return $evm.current['vm'] unless $evm.current['vm'].blank?
  case $evm.root['vmdb_object_type']
  when 'miq_provision'
    vm = $evm.root['miq_provision'].vm
  when 'vm'
    vm = $evm.root['vm']
  end
  raise "VM not found" unless vm
  vm
end

def get_virtual_machine_parent(vm)
  template = vm.wrap_results(vm.object_send(:parent))
  error("Virtual machine is missing its parent template relationship: [#{vm.name}]") unless template
  return template
end

def retrieve_configuration(config, class_name, instance_name)
  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  instance = $evm.instantiate(instance_uri)
  error("Unable to retrieve required configuration: [#{instance_uri}]") unless instance

  log(:info, "Retrieved configuration: [#{class_name}/#{instance_name}[#{instance.attributes}]")
  instance.attributes.each { |k, _| config[k] = instance.decrypt(k) }
end

def create_hpsa_service(hostname, service, username, password)
  log(:info, "Creating SOAP service to #{hostname} with #{username}/******")
  urlServerService = "/osapi/com/opsware/#{service}"
  log(:info, "Create ServerService SOAP Object to https://#{hostname}#{urlServerService}?wsdl")
  #serverService = Savon::Client.new do |wsdl, http|
  #  wsdl.document = "https://#{hostname}#{urlServerService}?wsdl"
  #  wsdl.endpoint = "https://#{hostname}#{urlServerService}"
  #  http.auth.basic username, password
  #  http.auth.ssl.verify_mode = :none
  #end

  # Configure Savon
  serverService = Savon.client(
      wsdl: "https://#{hostname}#{urlServerService}?wsdl",
      endpoint: "https://#{hostname}#{urlServerService}",
      read_timeout: 300,
      log: false,
      log_level: :debug,
      basic_auth: [username, password],
      ssl_verify_mode: :none
  )
end

def create_server_service(hostname, username, password)
  create_hpsa_service(hostname, 'server/ServerService', username, password)
end

def find_servers(server_service, name)
  log(:info, "Searching HPSA using filter ServerVO.name CONTAINS #{name}")
  #response = server_service.request :find_server_refs do |soap|
  #  soap.body = {
  #    'filter' => {
  #      'expression' => "ServerVO.name CONTAINS \"#{name}\"",
  #      'objectType' => nil
  #    }
  #  }
  #end.to_hash

  response = server_service.call(
    :find_server_refs,
    message: {
        'filter' => {
            'expression' => "ServerVO.name CONTAINS \"#{name}\"",
            'objectType' => nil
        }
    }
  ).to_hash

  log(:info, "Got Response to find_server_refs: #{response}")

  ref = response[:multi_ref]

  if ref.nil?
    return []
  elsif "#{ref.class}" == "Array"
    return ref.map { |r| r[:id] }
  else
    return [ref[:id]]
  end
end

def decommission_server(server_service, server_id)
  log(:info, "Starting Decommission")
  #response = server_service.request :decommission do |soap|
  #  soap.body = {
  #    :self => { :id => server_id }
  #  }
  #end

  response = server_service.call(
    :decommission,
    message: {
        :self => { :id => server_id }
    }
  )

  log(:info, "Decommission Response: #{response.to_hash}")
end

def remove_server(server_service, server_id)
  log(:info, "Starting Remove")
  #response = server_service.request :remove do |soap|
  #  soap.body = {
  #    :self => { :id => server_id }
  #  }
  #end

  response = server_service.call(
    :remove,
    message: {
        :self => { :id => server_id }
    }
  )

  log(:info, "Remove Response: #{response.to_hash}")
end

begin
  log(:info, "Starting Method")
  #dump_root()

  vm = get_virtual_machine()
  emsys = vm.ext_management_system
  datacenter = emsys.tags(:intuit_dc).first
  log(:info, "Canada check, dc found: #{datacenter}")

  dc_skip_list = $evm.current['datacenters_to_skip']
  if (dc_skip_list.nil?)
    dc_skip_list = []
  end
  log(:info, "Canada check, list of dc's to skip #{dc_skip_list}")

  if ( dc_skip_list.include?(datacenter) )
    log(:info, "Detected Canada Provision, so the remove duplicate record is being skipped")
  else
    log(:info, "Continue with the remove duplicate record")
    prov = $evm.root['miq_provision']

    ems_id = prov.get_option(:src_ems_id)
    ems = $evm.vmdb('ext_management_system', ems_id)
    intuit_dc = ems.tags(:intuit_dc).first

    config = {}
    retrieve_configuration(config, 'intuit_dc', intuit_dc)

    hostname = config['hpsa_hostname']
    username = config['hpsa_username']
    password = config['hpsa_password']

    configure_soap()
    server_service = create_server_service(hostname, username, password)
    server_ids = find_servers(server_service, vm.name)

    server_ids.each { |id|
      log(:info, "Deleting old HPSA record #{id} for #{vm.name}")
      decommission_server(server_service, id)
      remove_server(server_service, id)
    }

  end

  $evm.root['ae_result'] = 'ok'
  $evm.root['ae_reason'] = ''

rescue StandardError, SignalException => err
  retry_method("[#{err}]\n#{err.backtrace.join("\n")}")
end
