###################################
#
# CFME Automate Method: set_custom_attribute
#
# Author: Carsten Clasohm
#
# Notes: Sets a custom attribute for a VM registered in HPSA.
#
# Inputs:
#   vm: the target VM
#   attr_key: name of the custom attribute
#   attr_value: value of the custom attribute
#
# Mods:
#
###################################

require 'httpi'
require 'savon'
require 'httpclient'

# logging convenience method
def log(level, msg)
  @method = 'set_custom_attribute'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, msg)
  log(:error, "Error received setting custom attributes, exiting #{msg}")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def retry_method(msg)
  $evm.log(:warn,"Error received setting custom attributes,retrying. [#{msg}]")
  $evm.current['result'] = 'retry'
   $evm.current['reason'] = msg
  exit MIQ_OK
end

def configure_soap()
  HTTPI.log_level = :warn
  HTTPI.log = false
  HTTPI.adapter = :net_http

  #Savon.configure do |config|
  #  config.log = true
  #  config.log_level = :warn
  #end
end

def get_virtual_machine()
  return $evm.current['vm'] unless $evm.current['vm'].blank?
  case $evm.root['vmdb_object_type']
  when 'miq_provision'
    vm = $evm.root['miq_provision'].vm
  when 'vm'
    vm = $evm.root['vm']
  end
  raise "VM not found" unless vm
  vm
end

def get_virtual_machine_parent(vm)
  template = vm.wrap_results(vm.object_send(:parent))
  error("Virtual machine is missing its parent template relationship: [#{vm.name}]") unless template
  return template
end

def get_intuit_dc(vm)
  ems = vm.ext_management_system
  intuit_dc = ems.tags(:intuit_dc).first
  error("Intuit Data Center tag missing on provider: [#{ems.name}]") unless intuit_dc
  return intuit_dc
end

def create_hpsa_service(hostname, service, username, password)
  log(:info, "Creating SOAP service to #{hostname} with #{username}/******")
  urlServerService = "/osapi/com/opsware/#{service}"
  log(:info, "Create ServerService SOAP Object to https://#{hostname}#{urlServerService}?wsdl")
  #serverService = Savon::Client.new do |wsdl, http|
  #  wsdl.document = "https://#{hostname}#{urlServerService}?wsdl"
  #  wsdl.endpoint = "https://#{hostname}#{urlServerService}"
  #  http.auth.basic username, password
  #  http.auth.ssl.verify_mode = :none
  #end

  # Configure Savon
  serverService = Savon.client(
      wsdl: "https://#{hostname}#{urlServerService}?wsdl",
      endpoint: "https://#{hostname}#{urlServerService}",
      log: true,
      log_level: :warn,
      basic_auth: [username, password],
      ssl_verify_mode: :none
  )
end

def create_server_service(hostname, username, password)
  create_hpsa_service(hostname, 'server/ServerService', username, password)
end

def find_server(server_service, name)
  log(:info, "Searching HPSA using filter ServerVO.name CONTAINS #{name}")
  #response = server_service.request :find_server_refs do |soap|
  #  soap.body = {
  #    'filter' => {
  #      'expression' => "ServerVO.name CONTAINS \"#{name}\"",
  #      'objectType' => nil
  #    }
  #  }
  #end.to_hash

  response = server_service.call(
    :find_server_refs,
    message: {
          'filter' => {
              'expression' => "ServerVO.name CONTAINS \"#{name}\"",
              'objectType' => nil
          }
      }
  ).to_hash

  log(:info, "Got Response to find_server_refs: #{response}")
  ref = response[:multi_ref]
  retry_method("Unable to locate server: [#{name}]") unless ref
  log(:info, "Ref Details #{ref}")
  return ref[:id], ref
end

def retrieve_configuration(config, class_name, instance_name)
  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  instance = $evm.instantiate(instance_uri)
  error("Unable to retrieve required configuration: [#{instance_uri}]") unless instance
  
  log(:info, "Retrieved configuration: [#{class_name}/#{instance_name}[#{instance.attributes}]")
  instance.attributes.each { |k, _| config[k] = instance.decrypt(k) }
end

def set_custom_attribute(server_service, server_id, server_refs, attr_key, attr_value)
  log(:info, "Setting custom attribute #{attr_key} for #{server_id}")
  #response = server_service.request :set_cust_attr do |soap|
  #  soap.body = {
  #    :self =>  { :id => server_id },
  #    :key => attr_key,
  #    :value => attr_value,
  #    :attributes! => {
  #      :self => {
  #         'xsi:type' => "#{server_refs[:"@xsi:type"]}",
  #         'xmlns:ns2' => "#{server_refs[:"@xmlns:ns2"]}"
  #      },
  #      :key => {
  #         'xsi:type' => "xsd:string"
  #      },
  #      :value => {
  #         'xsi:type' => "xsd:string"
  #      }
  #    }
  #  }
  #end

  response = server_service.call(
    :set_cust_attr,
    message: {
        :self =>  { :id => server_id },
        :key => attr_key,
        :value => attr_value,
        :attributes! => {
            :self => {
                'xsi:type' => "#{server_refs[:"@xsi:type"]}",
                'xmlns:ns2' => "#{server_refs[:"@xmlns:ns2"]}"
            },
            :key => {
                'xsi:type' => "xsd:string"
            },
            :value => {
                'xsi:type' => "xsd:string"
            }
        }
    }
  )
  
  log(:info, "SetCustAttr Response #{response.to_hash}")
end

begin
  attr_key = $evm.current['attr_key']
  attr_value = $evm.current['attr_value']

  vm = get_virtual_machine()
  intuit_dc = get_intuit_dc(vm)
  
  config = {}
  retrieve_configuration(config, 'intuit_dc', intuit_dc)
  
  hostname = config['hpsa_hostname']
  username = config['hpsa_username']
  password = config['hpsa_password']

  configure_soap()
  server_service = create_server_service(hostname, username, password)

  server_id, server_refs = find_server(server_service, vm.name)

  set_custom_attribute(server_service, server_id, server_refs, attr_key, attr_value)

  $evm.current['result'] = 'ok'
  $evm.current['reason'] = ''

rescue StandardError, SignalException => err
  retry_method("[#{err}]\n#{err.backtrace.join("\n")}")
end
