####################################
#
# CFME Automate Method: add_management_route
#
# Author: Nick Lane
#
# Notes: Add route for the management nic
#
# Inputs: vm: the target VM
#
# Mods:
#
###################################

require 'open-uri'
require 'tempfile'
require 'timeout'
require 'resolv'

DOWNLOAD_TIMEOUT                 = 2.minutes
VIX_ACTIONS_URI                  = "/Intuit/Integration/VIX/Actions"
GET_VIX_PLATFORM_URI             = "#{VIX_ACTIONS_URI}/GetVixPlatform"
RUN_PROGRAM_IN_GUEST             = "#{VIX_ACTIONS_URI}/RunProgramInGuest"


def dump_root()
  $evm.log(:info, "Resize - Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "Resize - Root:<$evm.root> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Resize - Root:<$evm.root> End $evm.root.attributes")
  $evm.log(:info, "")
end

def error(msg)
  #if msg.include? "Timed out"
  #  $evm.log(:info, "Resize - Retrying due to vmrun time out")
  #  $evm.current['result'] = 'retry'
  #  $evm.current['reason'] = msg.to_s
    #$evm.root['ae_result'] = 'retry'
    #$evm.root['ae_retry_interval'] = 10.seconds
  # else
    $evm.log(:error, "Resize - Resize failed: [#{msg}]")
    $evm.current['result'] = 'error'
    $evm.current['reason'] = msg.to_s
  # end
  exit MIQ_OK
end

def escape_uri(value)
  URI.escape(value, Regexp.new("[^#{URI::PATTERN::UNRESERVED}]"))
end

def get_virtual_machine()
  case $evm.root['vmdb_object_type']
  when 'miq_provision'
    vm = $evm.root['miq_provision'].vm
  when 'vm'
    vm = $evm.root['vm']
  end
  raise "VM not found" unless vm
  vm
end

def get_vix_platform()
  result = instantiate(GET_VIX_PLATFORM_URI)
  vix_platform = result['vix_platform']
  $evm.log(:info, "VIX platform: [#{vix_platform}]")
  return vix_platform
end


def instantiate(uri)
  result = $evm.instantiate(uri)
  error("Failed to execute #{GET_VIX_PLATFORM_URI}") unless result
  error(result['reason']) unless result['result'] == 'ok'
  return result
end

def get_commands(config, class_name, instance_name)
  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  commands = $evm.instantiate(instance_uri)
  error("Unable to retrieve required configuration: [#{instance_uri}]") unless commands
  
  $evm.log(:info, "Retrieved configuration: [#{class_name}/#{instance_name}[#{commands.attributes}]")
  return commands
end

def execute_in_os(vix_platform, vix_guest_program, vix_guest_options)
  $evm.log(:info, "Attempting to resize disk in OS with the following commands: [#{vix_guest_program}][#{vix_guest_options}]")

  run_program_uri = "#{RUN_PROGRAM_IN_GUEST}?" \
    "vix_platform=#{vix_platform}&" \
    "vix_guest_program=#{escape_uri(vix_guest_program)}&" \
    "vix_guest_options=#{escape_uri(vix_guest_options)}"
  result = $evm.instantiate(run_program_uri)
  error("Failed to execute #{RUN_PROGRAM_IN_GUEST}") unless result
  error(result['reason']) unless result['result'] == 'ok'
  result  
end

begin
  dump_root()
  
  prov = $evm.root['miq_provision']
  vm = get_virtual_machine()
  vix_platform = get_vix_platform()

  cluster = vm.ems_cluster
  hpsa_gateway_tag = cluster.tags(:intuit_hpsa_gateway).first
  
  ems = vm.ext_management_system
  hpsa_gateway_tag = ems.tags(:intuit_hpsa_gateway).first unless hpsa_gateway_tag
  
  
  $evm.log(:info, "GAteway tag: #{hpsa_gateway_tag}")
  error("Infrastructure missing HPSA Agent URL tag: [#{vm.name}][#{cluster.name}][#{ems.name}]") unless hpsa_gateway_tag
  
  classification = $evm.vmdb('classification').find_by_name("intuit_hpsa_gateway/#{hpsa_gateway_tag}")
  error("Unable to retrieve tag description: [#{hpsa_gateway_tag}]") unless classification
  hpsa_agent_url = classification.description
  $evm.log(:info, "hpsa agent:#{hpsa_agent_url}")
  hpsa_agent = hpsa_agent_url.split(':').first
  $evm.log(:info, "agent split#{hpsa_agent}")
  hpsa_ip = Resolv.getaddress(hpsa_agent)
  $evm.log(:info, "hpsa ip: #{hpsa_ip}")
  #rescue Resolv::ResolvError => err
  #management_ip      = $evm.current['management_ip']
  
  #management_netmask = $evm.current['management_netmask']
  #management_gateway = $evm.current['management_gateway']
  #management_network = $evm.current['management_network']
  $evm.log(:info, "Prov: #{prov.inspect}")  
  management_ip      = prov.get_option(:management_ip)
  management_netmask = prov.get_option(:management_netmask)
  management_gateway = prov.get_option(:management_gateway)
  management_network = prov.get_option(:management_network)

  config = {}
  execute_result = nil
  commands = get_commands(config, 'VixPlatform', vix_platform)

  commands.attributes.each { |k,v| 
    $evm.log(:info, "Returned these keys: #{k} v: #{v}")
    values = v.split(" ")
    command = values.shift
    options = values.join(" ")
    $evm.log(:info, "Got options: #{options}")
    if options.include? "LINUXREPLACE"
      options = options.gsub('LINUXREPLACE', "#{hpsa_ip} via #{management_gateway}")
    elsif options.include? "SAVENETWORK"
      options = options.gsub('SAVENETWORK', "#{hpsa_ip}")  
      options = options.gsub('SAVEGATEWAY', "#{management_gateway}")     
    elsif options.include? "WINDOWSREPLACE"
      management_network = management_network.gsub(/\/\d\d/, '')
      options = options.gsub!('WINDOWSREPLACE', "#{hpsa_ip} mask #{management_netmask} #{management_gateway}")    
    end
    $evm.log(:info, "ADD MANAGEMENT command: #{command} options: #{options.inspect}")
    execute_result = execute_in_os(vix_platform, command, options)
  }

  $evm.log(:info, "Execute result inspect: #{execute_result.inspect}")
  $evm.current['result'] = execute_result['result']
  $evm.current['reason'] = execute_result['reason']
rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
