##################################
#
# CFME Automate Method: acquire_ip_address_aci
#
# Author: Nick Lane
#
# Notes: Acquire IP address from Men & Mice using ACI
#
# Inputs:
#   intuit_dc
#   ip_address
#   hostname
#   network_db
#
# Mods:
#
###################################

require 'httpi'
require 'ipaddr'
require 'rubygems'
require 'savon'
require 'httpclient'
require 'restclient'
require 'net/ping'

# logging convenience method
def log (level, msg)
  @method = 'acquire_ip_address_aci'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  #log(:error, msg)

  # There are some errors from Men & Mice that are only intermittent. We
  # wait a little bit and retry.
  log(:error, "Received error from Men and Mice, exiting #{msg}.")
  $evm.current['result'] = 'error'
  $evm.current['reason'] = msg
  exit MIQ_OK
end
def retry_method(msg, retry_time=1.minute)
  # There are some errors from Men & Mice that are only intermittent. We
  # wait a little bit and retry.
  $evm.log(:warn,"Received error from Men and Mice, retrying #{msg}.")
  $evm.current['result'] = 'retry'
  $evm.current['ae_retry_interval'] = retry_time
  exit MIQ_OK
end

def to_bool(value)
  return true if value == true || value =~ (/^(true|t|yes|y|1)$/i)
  return false if value == false || value.blank? || value =~ (/^(false|f|no|n|0)$/i)
  raise ArgumentError.new("invalid value for Boolean: \"#{value}\"")
end

def get_session_id(central, username, password)
  log(:info, "get_session_id: entering method")
  #response = @client.request :login do |soap|
  #  soap.body = {
  #      'server' => central,
  #      'loginName' => username,
  #      'password' => password
  #  }
  #end

  response = @client.call(
      :login,
      message: {
          'server' => central,
          'loginName' => username,
          'password' => password
      }
  )
  log(:info, "get_session_id: #{response.to_hash[:login_response]}")
  log(:info, "get_session_id: exiting method")
  return response.to_hash[:login_response][:session]
end

def get_token(username, password, server)
  log(:info, "Getting Login Token")
  uri = "#{server}/api/aaaLogin.json?gui-token-request=yes"
  body = {
      "aaaUser"=> {
          "attributes" => {
              "name" => username,
              "pwd"  => password
          }
      }
  }.to_json

  log(:info, "Attempting to acquire Token from #{server} with body: #{body}")
  login_response = RestClient::Request.execute(
      :method => :post,
      :url => uri,
      :payload => body,
      :verify_ssl => OpenSSL::SSL::VERIFY_NONE
  )
  log(:info, "Got response: #{login_response.headers}")
  response_hash = JSON.parse(login_response)

  imdata = response_hash['imdata'].first

  log(:info, "DATA: #{imdata}")

  url_token = imdata['aaaLogin']['attributes']['urlToken'].chomp
  @apic_cookie = imdata['aaaLogin']['attributes']['token'].chomp

  log(:info, "TOKEN: #{url_token}")
  return url_token
end

def get_bridge_domain(server, token, epg)
  log(:info, "Getting Bridge Domain")
  #uri = "https://#{server}/api/node/mo/info.json"

  uri = "#{server}/api/node/mo/#{epg}.json?query-target=children&target-subtree-class=fvRsBd"
  log(:info, "Attempting to get brodge domains with this request: #{uri}")
  begin
    epg_response = RestClient::Request.execute(
        :method => :get,
        :url => uri,
        :headers => {
            :accept => 'text/html,application/xhtml+xml,application/xml,application/json',
            'APIC-challenge' => token,
            'devcookie' => @apic_cookie
        },
        :verify_ssl => OpenSSL::SSL::VERIFY_NONE
    )
  rescue => err
    log(:info, "ERROR: #{err.inspect}")
  end

  epg_data = JSON.parse(epg_response)

  log(:info, "Got Response: #{epg_response}")
  epg_data = epg_data['imdata'].first
  log(:info, "BD: #{epg_data}")

  b_domain = epg_data['fvRsBd']['attributes']['tDn']

  log(:info, "BDS: #{b_domain}")

  return b_domain
end

def get_networks(server, token, bridge_domain, epg, intuit_dns)
  log(:info, "Getting Networks")
  #uri = "https://#{server}/api/node/mo/info.json"

  networks = Array.new
  net_hash = Hash.new

  uri = "#{server}/api/node/mo/#{bridge_domain}.json?query-target=children&target-subtree-class=fvSubnet"
  log(:info, "Attempting to get networks with this request: #{uri}")
  begin
    epg_response = RestClient::Request.execute(
        :method => :get,
        :url => uri,
        :headers => {
            :accept => 'text/html,application/xhtml+xml,application/xml,application/json',
            'APIC-challenge' => token,
            'devcookie' => @apic_cookie
        },
        :verify_ssl => OpenSSL::SSL::VERIFY_NONE
    )
  rescue => err
    log(:info, "ERROR: #{err.inspect}")
  end

  epg_data = JSON.parse(epg_response)

  log(:info, "Got Response: #{epg_response}")
  epg_data = epg_data['imdata']
  log(:info, "Subnets: #{epg_data}")
  epg_data.each {|bd|
    ip_address = bd['fvSubnet']['attributes']['ip']

    gateway = ip_address.split("/").first

    range_name = ip_address.gsub(/\.\d+\//, ".0/")

    network_name_array = epg.split("/")

    #tenant = network_name_array[1].split("-").last

    tenant = network_name_array[1].gsub("tn-","")

    application_profile = network_name_array[2].gsub("ap-","")

    port_group = network_name_array[3].gsub("epg-","")

    vmware_network = "#{tenant}|#{application_profile}|#{port_group}"

    net_hash = {bridge_domain => {'vmware_network' => vmware_network,'gateway' => gateway, 'range_name' => range_name, 'dns1' => intuit_dns.first, 'dns2' => intuit_dns.last, 'reserved_count_start' => 15, 'reserved_count_end' => 1}}

    log(:info, " network data: #{net_hash}")


    networks.push net_hash
    log(:info, "Networks: #{networks.inspect}")
  }
  return networks
end
# Get a reference to a Men & Mice address range, which will be used in
# other SOAP API calls.
def get_range_ref(name)
  log(:info, "get_range_ref(#{name}): entering method")
  #response = @client.request :get_ranges do |soap|
  #  soap.body = {
  #      'session' => @session,
  #      'filter' => "name:^#{name}$"
  #  }
  #end

  response = @client.call(
      :get_ranges,
      message: {
          'session' => @session,
          'filter' => "name:^#{name}$"
      }
  )
  log(:info, "get_range_ref(#{name}): response details #{response.to_hash[:get_ranges_response].inspect}")
  hash = response.to_hash[:get_ranges_response][:ranges]
  error("Cannot find Men & Mice address range #{name}") if hash.nil?
  log(:info, "get_range_ref(#{name}): debug_hash[:range]: #{hash[:range].inspect}")
  return hash[:range][:ref]
end

# Get a reference to a DNS zone, given a domain name.
def get_dns_zone_ref(zone_name, mm_zone_refs)
  log(:info, "get_dns_zone_ref(#{zone_name}, #{mm_zone_refs}): entering method")

  # Check locally before asking M&M
  zone_key = zone_name.gsub('.','_')[0..-2]
  zone_ref = mm_zone_refs[zone_key]
  log(:info, "get_dns_zone_ref(#{zone_name}): Found local zone_ref #{zone_key} => #{zone_ref}") unless zone_ref.nil?
  return zone_ref unless zone_ref.nil?
  #response = @client.request :get_dns_zones do |soap|
  #  soap.body = {
  #      'session' => @session,
  #      'filter' => "type:^Master$ name:^#{zone_name}$"
  #  }
  #end

  response = @client.call(
      :get_dns_zones,
      message: {
          'session' => @session,
          'filter' => "type:^Master$ name:^#{zone_name}$"
      }
  )
  log(:info, "get_dns_zone_ref(#{zone_name}): Response Details #{response.to_hash[:get_dns_zones_response].inspect}")

  zones = response.to_hash[:get_dns_zones_response][:dns_zones]
  error("Cannot find DNS zone [#{zone_name}] in Men & Mice.") unless zones

  zone = zones[:dns_zone]
  error("Too many DNS zone matches for domain #{zone_name}: #{zone}") if zone.kind_of?(Array)

  log(:info, "get_dns_zone_ref(#{zone_name}): Got zone #{zone}")
  return zone[:ref]
end

# Convert the integer representation of an IPv4 IPAddr to a four-octet
# string.
def int_ipaddr_to_string(int_ipaddr)
  return (0..3).map { |i|
    (int_ipaddr >> (24 - 8 * i)) & 0xff
  }.join('.')
end

# To handle cases where a machine might not have a DNS record in Men and Mice,
# we check via Ping and Ports to make sure that a given IP is truly free. No response
# means a free machine. A response via ping or port means there is a ghost machine.
def check_for_ghost_machine(ipaddr)

  log(:info, "check_for_ghost_machines(#{ipaddr}): Performing Ping Test")
  # Perform Ping Test.
  ping_obj = Net::Ping::ICMP.new(ipaddr)

  if ping_obj.ping
    return [ true, "Bad IP: IP Address #{ipaddr} is responding to ping test, IP is not free"]
  end

  log(:info, "check_for_ghost_machines(#{ipaddr}): Performing Port Check")
  # Test ports.
  rdp_port1 = 3389
  rdp_port2 = 799
  ssh_port = 22
  net_bios_port = 445


  test_ports = [rdp_port1, rdp_port2, ssh_port, net_bios_port]

  test_ports.each do |port|
    begin
      Timeout::timeout(SOCKET_TIMEOUT) {
        begin
          connection = TCPSocket.new ipaddr, port
          if connection
            return [ true, "Bad IP: IP Address #{ipaddr} is responding on port #{port}, IP is not free"]
          end
        ensure
          connection.close if connection
        end
      }
    rescue => err
      log(:info, "Port #{port} socket connection message: #{err}")
    end
  end

  return [ false, ""]
end

# Given a Men & Mice network range, find the first available IP address
# starting with the given start_address. We also check that the address
# returned by Men & Mice is before the given end_address.
def get_next_free(range_name, cidr_prefix, reserved_count_start, reserved_count_end, zone_ref, fqdn)
  log(:info, "get_next_free(#{range_name}, #{cidr_prefix}, #{reserved_count_start}, #{reserved_count_start}, #{zone_ref}, #{fqdn}): entering method")

  range_ref = get_range_ref(range_name)
  net = IPAddr.new(range_name)

  start_address_int = net.to_i() + reserved_count_start + 1

  # Convert the numerical IP address into a string like 10.136.130.100.
  start_address = int_ipaddr_to_string(start_address_int)

  # We use the CIDR prefix to determine how many addresses are in the
  # range, and compute the last usable address.
  end_address_int = net.to_i() + (2**(32-cidr_prefix)) - reserved_count_end - 1
  end_address = int_ipaddr_to_string(end_address_int)

  # With Men & Mice 6.5 or newer, the temporaryClaimTime parameter should
  # be added.
  # See https://docs.menandmice.com/display/MM650/SOAP+API#GetDNSZones
  #response = @client.request :get_next_free_address do |soap|
  #  soap.body = {
  #      'session' => @session,
  #      'rangeRef' => range_ref,
  #      'ping' => true,
  #      'startAddress' => start_address,
  #      'temporaryClaimTime' => 60
  #  }
  #end

  response = @client.call(
      :get_next_free_address,
      message: {
          'session' => @session,
          'rangeRef' => range_ref,
          'ping' => true,
          'startAddress' => start_address,
          'temporaryClaimTime' => 60
      }
  )

  # This is the IP address in string format.
  next_free = response.to_hash[:get_next_free_address_response][:address]

  if next_free.blank?
    log(:info, "No address available in #{range_name}")
    return nil
  end

  # Check if the returned IP address is larger than the last usable IP
  # address that we are allowed to use. We use the address' integer
  # presentation for this.

  ip_next_free = IPAddr.new(next_free)

  if ip_next_free.to_i > end_address_int
    log(:info, "Only available IP address #{next_free} is beyond usable end address #{end_address} - check if range #{range_name} has exhausted all usable addresses")
    return nil
  end

  #Check if IP still responds via Ping & Port
  ghost_response, ghost_msg = check_for_ghost_machine(next_free)

  if ghost_response
    domain_array = fqdn.split(".", 2)
    uniq_str = (0...8).map { (97 + rand(26)).chr }.join
    dummyfqdn = "unknown#{uniq_str}.#{domain_array[1]}"

    add_dns_record(zone_ref, next_free, dummyfqdn)
    retry_method(ghost_msg)
  end

  log(:info, "get_next_free(#{range_ref}, #{cidr_prefix}, #{start_address}, #{end_address}, #{zone_ref}, #{fqdn}): found #{next_free}")
  return next_free
end

# Create a DNS record in the given zone for the given address and
# FQDN. This will succeed even if another A record already exists for the
# same IP address.
#
# We could turn get_next_free and this method into an atomic operation if
# we used the autoAssignRangeRef parameter, but that would not allow us to
# specify the start address.
def add_dns_record(zone_ref, address, fqdn)
  log(:info, "add_dns_record(#{zone_ref},#{address},#{fqdn}): entering method")

  #response = @client.request :add_dns_record do |soap|
  #  soap.body = {
  #      'session' => @session,
  #      'saveComment' => "Added by CloudForms at #{Time.now}",
  #      'dnsRecord' => {
  #          'name' => fqdn,
  #          'data' => address,
  #          'ttl' => "3600",
  #          'enabled' => true,
  #          'type' => "A",
  #          'dnsZoneRef' => zone_ref,
  #      }
  #  }
  #end

  response = @client.call(
      :add_dns_record,
      message: {
          'session' => @session,
          'saveComment' => "Added by CloudForms at #{Time.now}",
          'dnsRecord' => {
              'name' => fqdn,
              'data' => address,
              'ttl' => "3600",
              'enabled' => true,
              'type' => "A",
              'dnsZoneRef' => zone_ref,
          }
      }
  )
  log(:info, "add_dns_record(#{zone_ref},#{address},#{fqdn}): #{response.to_hash[:add_dns_record_response]}")
  return response.to_hash[:add_dns_record_response][:ref]
end

def remove_dns_record(ref)
  log(:info, "remove_dns_record(#{ref}): entering method")
  #response = @client.request :remove_object do |soap|
  #  soap.body = {
  #      'session' => @session,
  #      'ref' => ref,
  #  }
  #end

  response = @client.call(
      :remove_object,
      message: {
          'session' => @session,
          'ref' => ref,
      }
  )
  log(:info, "remove_dns_record(#{ref}):DEBUG: #{response.to_hash[:remove_object_response]}")
end

# Get the address for a given FQDN from DNS.
def get_dns_address(zone_ref, fqdn)
  log(:info, "get_dns_address(#{zone_ref},#{fqdn}): entering method")
  short_name = fqdn.split('.').first
  filter = "type:^A$ name:^#{short_name}$"
  log(:info, "get_dns_address(#{zone_ref},#{fqdn}): Using filter #{filter}")
  #response = @client.request :get_dns_records do |soap|
  #  soap.body = {
  #      'session' => @session,
  #      'dnsZoneRef' => zone_ref,
  #      'filter' => filter,
  #  }
  #end

  response = @client.call(
      :get_dns_records,
      message: {
          'session' => @session,
          'dnsZoneRef' => zone_ref,
          'filter' => filter,
      }
  )
  filter_2 = "type:^A$ name:^ipparked-#{short_name}$"
  #response_2 = @client.request :get_dns_records do |soap|
  #  soap.body = {
  #      'session' => @session,
  #      'dnsZoneRef' => zone_ref,
  #      'filter' => filter_2,
  #  }
  #end

  response_2 = @client.call(
      :get_dns_records,
      message: {
          'session' => @session,
          'dnsZoneRef' => zone_ref,
          'filter' => filter_2,
      }
  )
  log(:info, "get_dns_address(#{zone_ref},#{fqdn}): #{response.to_hash[:get_dns_records_response].inspect}")
  log(:info, "get_dns_address(#{zone_ref},#{fqdn}): #{response_2.to_hash[:get_dns_records_response].inspect}")
  log(:info, "RESPONSE 2 TOTAL RESULTS: #{response_2.to_hash[:get_dns_records_response][:total_results]}")
  return nil if (response.to_hash[:get_dns_records_response][:total_results] == 0) && (response_2.to_hash[:get_dns_records_response][:total_results] == 0)
  return nil if (response.to_hash[:get_dns_records_response][:dns_records].nil?) && (response_2.to_hash[:get_dns_records_response][:dns_records].nil?)
  if (response_2.to_hash[:get_dns_records_response][:total_results] == 1) || (response_2.to_hash[:get_dns_records_response][:total_results].include? "1")
    log(:info, "Entering IP Parked renaming code")
    hash = response_2.to_hash[:get_dns_records_response][:dns_records][:dns_record]
    parked_name = hash[:name]
    log(:info, "PARKED NAME: #{parked_name}")
    ##Figure this part out
    record = get_ipam_record(hash[:data])
    second_record  = get_ipam_record(hash[:data])
    orig_record = record.to_hash[:get_ipam_record_response][:ipam_record]
    log(:info, "OLD RECORD: #{orig_record}")
    new_record  = second_record.to_hash[:get_ipam_record_response][:ipam_record]
    ####
    real_name = parked_name.split("-").last
    new_record[:dns_hosts][:dns_host][:dns_record][:name] = real_name
    log(:info, "NEW RECORD: #{new_record}")
    update_ipam_record(orig_record, new_record)
    hash[:name] = real_name
  else
    hash = response.to_hash[:get_dns_records_response][:dns_records][:dns_record]
  end
  if "#{hash.class}" != "Hash"
    error("Response is of type #{hash.class}, probably multiple DNS entries with this name")
  end
  log(:info, "HASH: #{hash}")
  return hash[:data]
end

# Get all DNS A records for a given IP address.
def get_dns_names(zone_ref, address)
  log(:info, "get_dns_names(#{zone_ref},#{address}): entering method")
  filter = "data:^#{address}$"
  log(:info, "get_dns_names(#{zone_ref},#{address}): Using filter #{filter}")
  response = @client.request :get_dns_records do |soap|
    soap.body = {
        'session' => @session,
        'dnsZoneRef' => zone_ref,
        'filter' => filter,
    }
  end
  log(:info, "get_dns_names(#{zone_ref},#{address}): #{response.to_hash[:get_dns_records_response].inspect}")
  return nil if response.to_hash[:get_dns_records_response][:total_results] == 0
  return nil if response.to_hash[:get_dns_records_response][:dns_records].nil?
  return response.to_hash[:get_dns_records_response][:dns_records]
end

def get_ipam_record(ipaddr)
  log(:info, "get_ipam_record(#{ipaddr}): entering method")
  #response = @client.request :get_ipam_record do |soap|
  #  soap.body = {
  #      'session' => @session,
  #      'addrRef' => ipaddr,
  #  }
  #end

  response = @client.call(
      :get_ipam_record,
      message: {
          'session' => @session,
          'addrRef' => ipaddr,
      }
  )
  log(:info, "get_dns_record(#{ipaddr}): #{response.inspect}")
  #return nil if response.to_hash[:get_dns_records_response][:total_results] == 0
  #return nil if response.to_hash[:get_dns_records_response][:dns_records].nil?
  #log(:info, "DNS NAME: #{response.to_hash[:get_dns_records_response][:dns_records][:dns_record][:name]}")
  #return "parked" if response.to_hash[:get_dns_records_response][:dns_records][:dns_record][:name].include? "ipparked"
  #hash = response.to_hash[:get_ipam_record_response]
  #log(:info, "IPAM HASH(#{ipaddr}): #{hash.inspect}")
  #if "#{hash.class}" == "Hash"
  #  return hash[:ref]
  #else
  #  error("Response is of type #{hash.class}, probably multiple DNS entries with this IP address")
  #end
  return response
end

def update_ipam_record(orig_record, new_record)
  log(:info, "update_ipam_record(#{orig_record}): entering method")
  log(:info, "Session #{@session} client #{@client}")
  begin
    #response = @client.request :set_ipam_record do |soap|
    #  soap.body = {
    #      'session' => @session,
    #      'ipamRecordBefore' => {
    #          'addrRef' => orig_record[:addr_ref],
    #          'address' => orig_record[:address],
    #          'claimed' => orig_record[:claimed],
    #          'dnsHosts'=> {
    #              'dnsHost' => {
    #                  'dnsRecord' => {
    #                      'ref'  => orig_record[:dns_hosts][:dns_host][:dns_record][:ref],
    #                      'name' => orig_record[:dns_hosts][:dns_host][:dns_record][:name],
    #                      'type' => orig_record[:dns_hosts][:dns_host][:dns_record][:type],
    #                      'ttl'  => orig_record[:dns_hosts][:dns_host][:dns_record][:ttl],
    #                      'data' => orig_record[:dns_hosts][:dns_host][:dns_record][:data],
    #                      'enabled' => orig_record[:dns_hosts][:dns_host][:dns_record][:enabled],
    #                      'dnsZoneRef' => orig_record[:dns_hosts][:dns_host][:dns_record][:dns_zone_ref]
    #                  },
    #                  'ptrStatus' => orig_record[:dns_hosts][:dns_host][:ptr_status],
    #                  'relatedRecords' => orig_record[:dns_hosts][:dns_host][:related_records]
    #              },
    #          },
    #          'dhcpReservations' => orig_record[:dhcp_reservations],
    #          'dhcpLeases' => orig_record[:dhcp_leases],
    #          'discoveryType' => orig_record[:discovery_type],
    #          'lastSeenDate' => orig_record[:last_seen_date],
    #          'lastDiscoveryDate' => orig_record[:last_discovery_date],
    #          'lastKnownClientIdentifier' => orig_record[:last_know_client_identifier],
    #          'customProperties' => orig_record[:custom_properties]
    #      },
    #      'ipamRecordAfter' => {
    #          'addrRef' => new_record[:addr_ref],
    #          'address' => new_record[:address],
    #          'claimed' => new_record[:claimed],
    #          'dnsHosts'=> {
    #              'dnsHost' => {
    #                  'dnsRecord' => {
    #                      'ref'  => new_record[:dns_hosts][:dns_host][:dns_record][:ref],
    #                      'name' => new_record[:dns_hosts][:dns_host][:dns_record][:name],
    #                      'type' => new_record[:dns_hosts][:dns_host][:dns_record][:type],
    #                      'ttl'  => new_record[:dns_hosts][:dns_host][:dns_record][:ttl],
    #                      'data' => new_record[:dns_hosts][:dns_host][:dns_record][:data],
    #                      'enabled' => new_record[:dns_hosts][:dns_host][:dns_record][:enabled],
    #                      'dnsZoneRef' => new_record[:dns_hosts][:dns_host][:dns_record][:dns_zone_ref]
    #                  },
    #                  'ptrStatus' => new_record[:dns_hosts][:dns_host][:ptr_status],
    #                  'relatedRecords' => new_record[:dns_hosts][:dns_host][:related_records]
    #              },
    #          },
    #          'dhcpReservations' => new_record[:dhcp_reservations],
    #          'dhcpLeases' => new_record[:dhcp_leases],
    #          'discoveryType' => new_record[:discovery_type],
    #          'lastSeenDate' => new_record[:last_seen_date],
    #          'lastDiscoveryDate' => new_record[:last_discovery_date],
    #          'lastKnownClientIdentifier' => new_record[:last_know_client_identifier],
    #          'customProperties' => new_record[:custom_properties]
    #      },
    #  }
    #  #'ipamRecordBefore' => orig_record,
    #  #'ipamRecordAfter' => new_record
    #  log(:info, "SOAP BODY: #{soap.body}")
    #end

    response = @client.call(
        :set_ipam_record,
        message: {
            'session' => @session,
            'ipamRecordBefore' => {
                'addrRef' => orig_record[:addr_ref],
                'address' => orig_record[:address],
                'claimed' => orig_record[:claimed],
                'dnsHosts'=> {
                    'dnsHost' => {
                        'dnsRecord' => {
                            'ref'  => orig_record[:dns_hosts][:dns_host][:dns_record][:ref],
                            'name' => orig_record[:dns_hosts][:dns_host][:dns_record][:name],
                            'type' => orig_record[:dns_hosts][:dns_host][:dns_record][:type],
                            'ttl'  => orig_record[:dns_hosts][:dns_host][:dns_record][:ttl],
                            'data' => orig_record[:dns_hosts][:dns_host][:dns_record][:data],
                            'enabled' => orig_record[:dns_hosts][:dns_host][:dns_record][:enabled],
                            'dnsZoneRef' => orig_record[:dns_hosts][:dns_host][:dns_record][:dns_zone_ref]
                        },
                        'ptrStatus' => orig_record[:dns_hosts][:dns_host][:ptr_status],
                        'relatedRecords' => orig_record[:dns_hosts][:dns_host][:related_records]
                    },
                },
                'dhcpReservations' => orig_record[:dhcp_reservations],
                'dhcpLeases' => orig_record[:dhcp_leases],
                'discoveryType' => orig_record[:discovery_type],
                'lastSeenDate' => orig_record[:last_seen_date],
                'lastDiscoveryDate' => orig_record[:last_discovery_date],
                'lastKnownClientIdentifier' => orig_record[:last_know_client_identifier],
                'customProperties' => orig_record[:custom_properties]
            },
            'ipamRecordAfter' => {
                'addrRef' => new_record[:addr_ref],
                'address' => new_record[:address],
                'claimed' => new_record[:claimed],
                'dnsHosts'=> {
                    'dnsHost' => {
                        'dnsRecord' => {
                            'ref'  => new_record[:dns_hosts][:dns_host][:dns_record][:ref],
                            'name' => new_record[:dns_hosts][:dns_host][:dns_record][:name],
                            'type' => new_record[:dns_hosts][:dns_host][:dns_record][:type],
                            'ttl'  => new_record[:dns_hosts][:dns_host][:dns_record][:ttl],
                            'data' => new_record[:dns_hosts][:dns_host][:dns_record][:data],
                            'enabled' => new_record[:dns_hosts][:dns_host][:dns_record][:enabled],
                            'dnsZoneRef' => new_record[:dns_hosts][:dns_host][:dns_record][:dns_zone_ref]
                        },
                        'ptrStatus' => new_record[:dns_hosts][:dns_host][:ptr_status],
                        'relatedRecords' => new_record[:dns_hosts][:dns_host][:related_records]
                    },
                },
                'dhcpReservations' => new_record[:dhcp_reservations],
                'dhcpLeases' => new_record[:dhcp_leases],
                'discoveryType' => new_record[:discovery_type],
                'lastSeenDate' => new_record[:last_seen_date],
                'lastDiscoveryDate' => new_record[:last_discovery_date],
                'lastKnownClientIdentifier' => new_record[:last_know_client_identifier],
                'customProperties' => new_record[:custom_properties]
            },
        }
    )
  rescue => err
    $evm.log(:info, "ERR: #{err.inspect}")
  end
  log(:info, "DEBUG: #{response.inspect}")
  #log(:info, "update_upam_record(#{re}): exiting method")
end


def logout
  log(:info, "logout: entering method")
  log(:info, "logout: Logging out #{@session}") unless @session.nil?
  if @client.nil? || @session.nil?
    log(:info, "logout: exiting method")
    return
  end
  #@client.request :logout do |soap|
  #  soap.body = {
  #      'session' => @session
  #  }
  #end

  @client.call(
      :logout,
      message: {
          'session' => @session
      }
  )

  @session = nil
  log(:info, "logout: exiting method")
end

begin
  log(:info, "Main: entering main method")

  # Parameters that were collected by the AcquireIPAddress instance, which
  # gets them from a Integration/MenAndMice/Configuration/intuit_dc
  # instance.

  wsdl_url = $evm.current['wsdl']
  central = $evm.current['central']
  username = $evm.current['mm_username']
  password = $evm.current.decrypt('mm_password')
  aci_username = $evm.current['aci_username']
  aci_password = $evm.current.decrypt('aci_password')
  aci_server = $evm.current['aci_server']

  prov = $evm.root['miq_provision']

  ems_id = prov.get_option(:src_ems_id)
  ems = $evm.vmdb('ext_management_system', ems_id)
  intuit_dns = ems.tags.select{|tag| tag.include? "intuit_dns"}.map{|tag| tag.gsub("_",".")}.map{|tag| tag.split("/").last}
  #intuit_dns = intuit_dns

  log(:info, "INTUIT DNS: #{intuit_dns}")

  log(:info, "ACI Server: #{aci_server} ACI USername: #{aci_username}")

  dns_zone = $evm.current['default_dns_zone']

  error("Missing Men & Mice credentials for intuit_dc: [#{intuit_dc}]") if username.blank? || password.blank?

  # Parameters that were passed via $evm.instantiate()
  intuit_dc = $evm.current['intuit_dc']
  ip_address = $evm.current['ip_address']
  hostname = $evm.current['hostname']
  target_epg = $evm.current['target_epg']

  # SOAP client setup
  @client = nil
  @session = nil

  # configure HTTPI
  HTTPI.log_level = :info
  HTTPI.log = false
  HTTPI.adapter = :net_http

  # Configure Savon
  #Savon.configure do |config|
  #  config.log = false
  #  config.log_level = :debug
  #end

  #@client = Savon::Client.new do |wsdl,http|
  #  wsdl.document = wsdl_url
  #
  #  # Increase the read timeout to prevent a slow Men & Mice server from
  #  # causing provisioning to fail.
  #  http.read_timeout = 300
  #end

  # Configure Savon
  @client = Savon.client(
      wsdl: wsdl_url,
      read_timeout: 300,
      log: false,
      log_level: :debug
  )

  aci_token = get_token(aci_username,aci_password, aci_server)


  bridge_domain = get_bridge_domain(aci_server, aci_token, target_epg)

  networks = get_networks(aci_server, aci_token, bridge_domain, target_epg, intuit_dns)



  log(:info, "Found networks #{networks.inspect}")

  @session = get_session_id(central, username, password)

  # If the given hostname contains at least one dot, we take it as it is,
  # and extract the DNS zone from it. Otherwise we append the DNS zone that
  # we got for the local data center.
  #
  # The FQDN has a trailing dot to prevent Men & Mice from appending the DNS
  # zone a second time when creating the DNS record.
  if hostname.include? "."
    dns_zone = hostname.sub(%r{^[^.]+\.}, '') + '.'
    log(:info, "Main: extracted dns_zone #{dns_zone} from hostname #{hostname}")
    fqdn = hostname + '.'
  else
    fqdn = "#{hostname}.#{dns_zone}"
  end

  short_hostname = hostname.sub(%r{\..*$}, '')

  mm_zone_refs_instance = "Customer/Intuit/Integration/MenAndMice/Configuration/intuit_mm/#{intuit_dc}"
  mm_zone_refs = $evm.instance_find(mm_zone_refs_instance)[intuit_dc]
  zone_ref = get_dns_zone_ref(dns_zone, mm_zone_refs)

  ip_address_valid = false
  vmware_network = nil
  vmware_network_is_dvs = nil
  next_free = nil
  submask = nil
  gateway = nil
  dns1 = nil
  dns2 = nil
  dns_record_added = false
  check_address = nil
  provided_ip_not_in_range = nil
  fqdn_address = get_dns_address(zone_ref, fqdn)
  if not ip_address.blank?
    # Check if the given IP address already has a DNS entry.
    existing_records = get_dns_names(zone_ref, ip_address)

    log(:info, "Existing DNS Records for IP #{ip_address}: #{existing_records}")

    # Check if given IP address has a ghost machine( machine with no DNS record)
    if existing_records.nil?
      # Check if IP still responds via Ping & Port checks
      ghost_response, ghost_msg = check_for_ghost_machine(ip_address)
      log(:info, "Ghost Res: #{ghost_response}, Ghost Msg: #{ghost_msg}")

      #Create a dummy DNS for the IP then error out
      if ghost_response
        domain_array = fqdn.split(".", 2)
        uniq_str = (0...8).map { (97 + rand(26)).chr }.join
        dummyfqdn = "unknown#{uniq_str}.#{domain_array[1]}"

        add_dns_record(zone_ref, ip_address, dummyfqdn)
        error(ghost_msg)
      end
    end
  end

  networks.each { |network|
    # A simple pattern match is not enough to find the right network. We
    # have both "nonpublic" and "nonpubliccomp" as security levels, so
    # "nonpublic*" would not only match "nonpublic_1", but also
    # "nonpubliccomp_1". We account for that with an additional check
    # here. The instance_pattern must either match exactly, or be followed
    # by a "_".

    #next unless network_name == network_name_pattern || network_name =~ (/^#{network_name_pattern}_.*$/)

    network_name = network.keys[0]
    network_props = network.values[0]

    log(:info, "Examining network #{network_name}")

    ###########
    #This should be returned with '/', but for VCenter they need to become '|'
    ###########
    vmware_network = network_props['vmware_network'] #THIS WILL NEED TO BE CHANGED FOR WHEN I ACTUALLY GET THE BRIDGE DOMAIN INFORMATION

    ###########
    #Since all EPGs will be a Distributed Switch this automatically becomes 1
    ###########
    vmware_network_is_dvs = true

    range_name = network_props['range_name'] #THIS WILL NEED TO BE CHANGED FOR WHEN I ACTUALLY GET THE BRIDGE DOMAIN INFORMATION
    gateway = network_props['gateway'] #THIS WILL MOST LIKELY JUST BE THE RANGE's .1 ADDRESS
    dns1 = network_props['dns1'] #STILL NOT SURE WHERE WE'RE GETTING THIS FROM
    dns2 = network_props['dns2']
    reserved_count_start = network_props['reserved_count_start'].to_i #WHERE THIS DATA IS COMING FROM IS STILL UNSURE
    reserved_count_end = network_props['reserved_count_end'].to_i #WHERE THIS DATA IS COMING FROM IS STILL UNSURE

    if gateway.blank?
      # If the gateway is not defined in the network properties, we use the
      # first address in the given network range.

      net = IPAddr.new(range_name)
      gateway = int_ipaddr_to_string(net.to_i() + 1)
    end

    # Extract the CIDR prefix from the range name ('10.136.130.0/23'),
    # and compute the corresponding subnet mask.
    cidr_prefix = range_name.sub(%r{^.*/([0-9]+)$}, '\1').to_i
    submask = IPAddr.new('255.255.255.255').mask(cidr_prefix).to_s

    # If the user specified an IP address for the VM, we will verify that
    # it's within one of the given networks. Otherwise, if Men & Mice
    # already has an IP address for the given hostname, we will verify that
    # IP address.

    check_address = ip_address

    # Check if Men & Mice already has a DNS entry for this host.
    if fqdn_address
      if not ip_address.blank? and fqdn_address != ip_address
        error("#{fqdn} already exists in Men & Mice with IP != #{ip_address}")
      end


      # We will use the existing address from Men & Mice, either because the
      # user did not request a specific one, or because the two match.
      next_free = fqdn_address
      check_address = fqdn_address
    end

    unless check_address.blank?


      # Check that the user-supplied IP address is within the given network
      # range, taking the reserved start and end counts into account. If it
      # isn't, we skip this network.
      log(:info, "Checking address #{check_address}")
      net = IPAddr.new(range_name)
      start_address_int = net.to_i() + reserved_count_start + 1
      end_address_int = net.to_i() + (2**(32-cidr_prefix)) - reserved_count_end - 1

      begin
        a = IPAddr.new(check_address)
        provided_ip_not_in_range = 0
      rescue
        error("Received an error verifying the IP address reserved by the user in Men & Mice. Verify the IP address reserved in M&M is accurate")
      end

      if a.to_i < start_address_int or a.to_i > end_address_int
        log(:info, "Pre-defined IP address #{a} is outside of network range #{range_name}, skipping #{network_name}")
        provided_ip_not_in_range = 1
        next
      end
    end

    if not ip_address.blank?

      if existing_records.nil?
        # Create a new DNS entry for the given IP address.
        add_dns_record(zone_ref, ip_address, fqdn)
        log(:info, "Main: added DNS A record for #{fqdn} #{ip_address}")
        dns_record_added = true
      elsif "#{existing_records.class}" == "Array"
        error("Found multiple DNS records for #{ip_address}")
      else
        existing_name = existing_records[:dns_record][:name]
        if existing_name != short_hostname
          error("Existing DNS record #{existing_name} for #{ip_address} is different from #{short_hostname}")
        end
      end

      next_free = ip_address
    end

    ip_address_valid = true

    while next_free.nil?
      # We need to get a new IP address from Men & Mice.

      next_free = get_next_free(range_name, cidr_prefix, reserved_count_start, reserved_count_end, zone_ref, fqdn)


      # If get_next_free returns nil, this network does not have any
      # available addresses. Skip to the next network.
      break if next_free.nil?

      log(:info, "Main: got next free IP #{next_free}")

      begin
        record_ref = add_dns_record(zone_ref, next_free, fqdn)
        log(:info, "Main: added DNS A record for #{fqdn} #{next_free}")
        dns_record_added = true
      rescue Savon::SOAP::Fault => e
        if e.message.match("DNS server did not accept the change")
          error("Cannot create DNS A record for #{fqdn} #{next_free} - check for duplicate hostname")
        else
          retry_method("Unknown SOAP error in add_dns_record: #{e.message}")
        end
      end

      # The add_dns_record succeeds even if another host already uses the same
      # IP address. This can happen even with get_next_free() and a check for
      # existing records before the add_dns_record() call because multiple
      # instances of this automation class can run at the same time.
      #
      # The only reliable way to handle this with the Men & Mice 6.3 SOAP API is
      # to check the DNS zone after we create our record. If we find multiple
      # records for the same IP address, we have to delete our record and start
      # over.
      #
      # To avoid an infinite loop where two instances of this class do this
      # over and over again, we keep the address if ours is the first record.
      # This assumes that the records returned by get_dns_names are always
      # in the same order.

      # Returns either a hash for a single DNS record, or an array with
      # multiple records.
      #existing_records = get_dns_names(zone_ref, next_free)[:dns_record]
      #if "#{existing_records.class}" == "Array"
      #  log(:info, "Main: found multiple DNS records for #{next_free}")
      #
      #  if existing_records[0][:name] != short_hostname
      #    log(:info, "Main: releasing #{next_free} which is in use by multiple hosts")
      #    remove_dns_record(record_ref)
      #    next_free = nil
      #  end
      #end
    end

    # No need to check other networks if we have found an IP address.
    break unless next_free.nil?
  }

  if provided_ip_not_in_range == 1
    error("Pre-defined IP address #{check_address} is outside of network ranges #{target_epg}")
  end

  unless ip_address_valid
    if check_address.nil?
      error("No network available for given security zone")
    else
      error("Pre-defined IP address #{check_address} outside of available network ranges")
    end
  end

  # Return values to the caller.
  #
  # We remove the trailing dot from the DNS zone and the FQDN, because we
  # only need them when talking with Men & Mice.

  $evm.current['vmware_network'] = vmware_network
  $evm.current['vmware_network_is_dvs'] = vmware_network_is_dvs
  $evm.current['ip_address'] = next_free.to_s
  $evm.current['submask'] = submask
  $evm.current['gateway'] = gateway
  $evm.current['dns_domain'] = dns_zone.sub(%r{\.$}, '')
  $evm.current['dns1'] = dns1
  $evm.current['dns2'] = dns2
  $evm.current['fqdn'] = fqdn.sub(%r{\.$}, '')
  $evm.current['dns_record_added'] = dns_record_added

  logout

  log(:info, "Main: exiting main method")
  $evm.current['result'] = 'ok'
  exit MIQ_OK

rescue StandardError, SignalException => err
  retry_method("[#{err}]\n#{err.backtrace.join("\n")}")
ensure
  begin
    log(:info, "Ensuring logout occurs")
    logout unless @session.nil?
  rescue StandardError, SignalException => lerr
    # do nothing
  end
end
