###################################
#
# CFME Automate Method: release_ip_address
#
# Author: Dave Costakos, Carsten Clasohm
#
# Notes: Release IP address in Men & Mice
#
# Inputs: vm
#
# Changes:
#   2015-12-15 Daniel Trieu updates to Savon2.0
#
###################################

require 'savon'
require 'rubygems'
require 'httpi'

# logging convenience method
def log (level, msg)
  @method = 'release_ip_address'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, msg)


  log(:error, "Received error from Men and Mice, exiting #{msg}.")
  $evm.current['result'] = 'error'
  $evm.current['reason'] = msg
  exit MIQ_OK
end

def retry_method(msg, retry_time=1.minute)
  # There are some errors from Men & Mice that are only intermittent. We
  # wait a little bit and retry.
  $evm.log(:warn,"Received error from Men and Mice, retrying #{msg}.")
  $evm.current['result'] = 'retry'
  $evm.current['ae_retry_interval'] = retry_time
  exit MIQ_OK
end

def get_session_id(central, username, password)
  log(:info, "get_session_id: entering method")
  #response = @client.request :login do |soap|
  #  soap.body = {
  #      'server' => central,
  #      'loginName' => username,
  #      'password' => password
  #  }
  #end

  response = @client.call(
    :login,
    message: {
        'server' => central,
        'loginName' => username,
        'password' => password
    }
  )

  log(:info, "get_session_id: #{response.to_hash[:login_response]}")
  log(:info, "get_session_id: exiting method")
  return response.to_hash[:login_response][:session]
end

def get_dns_zone_ref(zonename, mm_zone_refs)
  log(:info, "get_dns_zone_ref(#{zonename}, #{mm_zone_refs}): entering method")

  # Check locally before asking M&M
  zone_key = zonename.gsub('.','_')[0..-2]
  zone_ref = mm_zone_refs[zone_key]
  log(:info, "get_dns_zone_ref(#{zonename}): Found local zone_ref #{zone_key} => #{zone_ref}") unless zone_ref.nil?
  return zone_ref unless zone_ref.nil?

  #response = @client.request :get_dns_zones do |soap|
  #  soap.body = {
  #      'session' => @session,
  #      'filter' => "type:^Master$ name:^#{zonename}$"
  #  }
  #end

  response = @client.call(
    :get_dns_zones,
    message: {
        'session' => @session,
        'filter' => "type:^Master$ name:^#{zonename}$"
    }
  )

  log(:info, "get_dns_zone_ref(#{zonename}): Response Details #{response.to_hash[:get_dns_zones_response].inspect}")
  zone = response.to_hash[:get_dns_zones_response][:dns_zones][:dns_zone]
  log(:info, "get_dns_zone_ref(#{zonename}): Got zone #{zone}")
  log(:info, "get_dns_zone_ref(#{zonename}): exiting method")
  return zone[:ref]
end

def get_dns_record(zone_ref, ipaddr, short_name)
  log(:info, "get_dns_record(#{zone_ref},#{short_name},#{ipaddr}): entering method")
  #filter = "#{filter} data:^#{ipaddr}$" unless ipaddr.nil?
  filter = "name:#{short_name}$"
  log(:info, "get_dns_record(#{zone_ref},#{short_name}): Using filter #{filter}")

  #response = @client.request :get_dns_records do |soap|
  #  soap.body = {
  #      'session' => @session,
  #      'dnsZoneRef' => zone_ref,
  #      'filter' => filter,
  #  }
  #end

  response = @client.call(
    :get_dns_records,
    message: {
        'session' => @session,
        'dnsZoneRef' => zone_ref,
        'filter' => filter,
    }
  )

  log(:info, "get_dns_record(#{zone_ref},#{ipaddr}): #{response.to_hash[:get_dns_records_response].inspect}")

  return nil if (response.to_hash[:get_dns_records_response][:total_results] == 0)
  return nil if (response.to_hash[:get_dns_records_response][:dns_records].nil?)

  if (response.to_hash[:get_dns_records_response][:total_results].to_i > 1)
    multiple_records = response.to_hash[:get_dns_records_response][:dns_records][:dns_record]
    multiple_records.each { | record |
      if record[:name] == short_name
        selected_record = record
      elsif record[:name].downcase.include? "ipparked"
        # If any of the records contain ipparked, this IP is destined to be cleared.
        return "parked"
      end
    }
  else
    selected_record = response.to_hash[:get_dns_records_response][:dns_records][:dns_record]
    return "parked" if selected_record[:name].downcase.include? "ipparked"
  end

  # If no records selected, it means that we couldn't find an exact name match nor a name that includes ipparked,
  #   which means there is another name modifier that we are unaware of like "goofy-pprfzzzos100"
  return nil if (selected_record.nil?)

  log(:info, "Selected Record: #{selected_record[:ref]} #{selected_record[:name]} #{selected_record[:data]}")

  return selected_record[:ref]
end

def remove_dns_record(ref)
  log(:info, "remove_dns_record(#{ref}): entering method")
  #response = @client.request :remove_object do |soap|
  #  soap.body = {
  #      'session' => @session,
  #      'ref' => ref,
  #  }
  #end

  response = @client.call(
    :remove_object,
    message: {
        'session' => @session,
        'ref' => ref,
    }
  )

  log(:info, "remove_dns_record(#{ref}):DEBUG: #{response.to_hash[:remove_object_response]}")
  log(:info, "remove_dns_record(#{ref}): exiting method")
end

def logout
  log(:info, "logout: entering method")
  log(:info, "logout: Logging out #{@session}") unless @session.nil?
  if @client.nil? || @session.nil?
    log(:info, "logout: exiting method")
    return
  end
  #response = @client.request :logout do |soap|
  #  soap.body = {
  #      'session' => @session
  #  }
  #end

  response = @client.call(
    :logout,
    message: {
        'session' => @session
    }
  )
  @session = nil
  log(:info, "logout: exiting method")
end

def get_virtual_machine()
  case $evm.root['vmdb_object_type']
    when 'vm'
      vm = $evm.root['vm']
  end
  raise "VM not found" unless vm
  vm
end

# --- BEGIN MAIN -----#
begin
  # Parameters that were collected by the ReleaseIPAddress instance, which
  # gets them from a Integration/MenAndMice/Configuration/intuit_dc
  # instance.
  wsdl_url = $evm.current['wsdl']
  central = $evm.current['central']
  username = $evm.current['mm_username']
  password = $evm.current.decrypt('mm_password')

  # Parameters that were passed via $evm.instantiate()
  intuit_dc = $evm.current['intuit_dc']
  ip_address = $evm.current['ip_address'].split(",")
  dns_zone = $evm.current['dns_zone']
  vm_name = $evm.current['vm_name']

  error("Missing Men & Mice credentials for intuit_dc: [#{intuit_dc}]") if username.blank? || password.blank?

  @client = nil
  @session = nil

  # configure HTTPI
  HTTPI.log_level = :info
  HTTPI.log = false
  HTTPI.adapter = :net_http

  # Configure Savon
  #Savon.configure do |config|
  #  config.log = false
  #  config.log_level = :debug
  #end

  #@client = Savon::Client.new do |wsdl,http|
  #  wsdl.document = wsdl_url
  #end

  # Configure Savon
  @client = Savon.client(
      wsdl: wsdl_url,
      log: false,
      log_level: :debug
  )

  data_name = vm_name.split(".").shift

  #domain_name = vm_name.join(".")

  storage_name = data_name.clone

  mnet_name = data_name.clone

  storage_name = "#{storage_name}s"

  mnet_name = "#{mnet_name}m"



  @session = get_session_id(central, username, password)

  mm_zone_refs_instance = "Customer/Intuit/Integration/MenAndMice/Configuration/intuit_mm/#{intuit_dc}"
  mm_zone_refs = $evm.instance_find(mm_zone_refs_instance)[intuit_dc]
  zone_ref = get_dns_zone_ref(dns_zone, mm_zone_refs)

  [data_name,storage_name,mnet_name].each {|short_name|
    log(:info, "Getting Records for: #{short_name}")
    record = get_dns_record(zone_ref, ip_address, short_name)

    log(:info, "RECORD: #{record.inspect}")
    if record.nil?
      log(:info, "No DNS record found for #{short_name}")
    elsif record == "parked"
      log(:info, "IP address parked, will not release IP address.")
    else
      remove_dns_record(record)
    end
  }

  log(:info, "Main: exiting main method")
  # --- END MAIN -----#
  exit MIQ_OK
rescue StandardError, SignalException => err
  retry_method("[#{err}]\n#{err.backtrace.join("\n")}")
ensure
  begin
    log(:info, "Ensuring logout occurs")
    logout unless @session.nil?
  rescue StandardError, SignalException => lerr
    # do nothing
  end
end
