###################################
#
# CFME Automate Method: dispose_vm_record
#
# Author: Bryan Antigua
#
# Notes: Removes VM record from VM Table in ServiceNow
#
# Inputs: intuit_dc
#
# Mods:
#
###################################

require 'json'
require 'httpclient'

STATUS = 'Disposed'

# Methods for logging
def log(level, msg)
  @method = 'dispose_vm_record'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, "#{msg}")
  log(:error, "Received an error disposing of VM record in Service Now, exiting #{msg}")
  $evm.current['result'] = 'error'
  $evm.current['reason'] = msg.to_s
  exit MIQ_OK
end

def retry_method(msg)
  $evm.log(:warn,"Error received disposing VM record to Escalation QB,retrying. [#{msg}]")
  $evm.current['result'] = 'retry'
   $evm.current['reason'] = msg
  exit MIQ_OK
end

def get_virtual_machine()
  return $evm.current['vm'] unless $evm.current['vm'].blank?
  case $evm.root['vmdb_object_type']
  when 'miq_provision'
    vm = $evm.root['miq_provision'].vm
  when 'vm'
    vm = $evm.root['vm']
  end
  raise "VM not found" unless vm
  vm
end

def get_intuit_dc()
  return $evm.current['intuit_dc'] unless $evm.current['intuit_dc'].blank?
  if $evm.root['vmdb_object_type'] == 'miq_provision'
    prov = $evm.root['miq_provision']
    ems_id = prov.get_option(:src_ems_id)
    ems = $evm.vmdb('ext_management_system', ems_id)
  else
    vm = get_virtual_machine()
    ems = vm.ext_management_system
  end

  intuit_dc = ems.tags(:intuit_dc).first
  error("Intuit Data Center tag missing on provider: [#{ems.name}]") unless intuit_dc
  return intuit_dc
end
    
def retrieve_configuration(config, class_name, instance_name)
  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  instance = $evm.instantiate(instance_uri)
  error("Unable to retrieve required configuration: [#{instance_uri}]") unless instance
  
  instance.attributes.each { |k, _| config[k] = instance.decrypt(k) }
end

def instantiate(uri)
  result = $evm.instantiate(uri)
  error("Failed to execute: [#{uri}]") unless result
  error(result['reason']) unless result['result'] == 'ok'
  return result
end
    
def delete_from_service_now_vm_table(http, host_name, table_url) 
  body = {
        :u_name => host_name,
        :u_hardware_status => STATUS
    }.to_json

  headers = ["Content-Type", "application/json"]

  resp = http.post(table_url, body, headers).content
  log(:info, "ServiceNow response: #{resp.inspect}")
end

begin
  log(:info, "Main: entering main method")

  intuit_dc = get_intuit_dc()
  config = {}
  retrieve_configuration(config, 'intuit_dc', intuit_dc)
  vm = get_virtual_machine()
  host_name = vm.custom_get("sn_host_name")
    
  #authenticating for ServiceNow Call
  http =  HTTPClient.new(config['proxy_url'])
  $evm.current['password'] = config['sn_password']
  http.set_auth(config['sn_domain_url'], config['sn_username'], $evm.current.decrypt('password'))

  delete_from_service_now_vm_table(http, host_name, config['sn_vm_table_insert_url'])
  
  log(:info, "Main: exiting main method")
  exit MIQ_OK

rescue StandardError, SignalException => err
  retry_method("[#{err}]\n#{err.backtrace.join("\n")}")
end
