###################################
#
# CFME Automate Method: push_record_to_vm_table	
#
# Author: Bryan Antigua
#
# Notes: Retrieves information from VM and pushes it to the VM Table in ServiceNow
#
# Inputs: intuit_dc
#
# Mods:
#
###################################

require 'json'
require 'httpclient'

GET_SERVICE_NOW_APPLICATION_LIST_URI = '/Intuit/Integration/ServiceNow/Actions/GetApplicationList'
GET_SERVICE_NOW_SECURITY_TRUST_ZONE_LIST_URI = '/Intuit/Integration/ServiceNow/Actions/GetSecurityTrustZoneList'
PARENT_TYPE = 'cmdb_ci_appl'
CHILD_TYPE = 'cmdb_ci_server'
RELATIONSHIP_TYPE = 'Depends On::Used By'
SOURCE = 'CloudForms Integration'
ID_TYPE ='name'
MODEL_ID = 'VMware'
MANUFACTURER = 'VMware Inc'
RHEL_OS = 'rhel'
WINDOWS_OS = 'w2k'
STATUS = 'Deployed'

# Methods for logging
def log(level, message)
  @method = 'push_record_to_vm_table'
  $evm.log(level, "#{@method} - #{message}")
end

def dump_current()
  $evm.log(:info, "Current:<$evm.current> Begin $evm.current.attributes")
  $evm.current.attributes.sort.each { |k, v| $evm.log(:info, "Current:<$evm.current> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Current:<$evm.current> End $evm.current.attributes")
  $evm.log(:info, "")
end

def dump_root()
  log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  log(:info, "Root:<$evm.root> End $evm.root.attributes")
  log(:info, "")
end

def error(msg)
  $evm.log(:error, "Registering ServiceNow failed: [#{msg}]")
  $evm.current['result'] = 'error'
  $evm.current['reason'] = msg.to_s
  exit MIQ_OK
end

def retry_method(msg)
  $evm.log(:warn,"Registering Service Now received an error: [#{msg}], retrying")
  $evm.current['result'] = 'retry'
  $evm.current['reason'] = msg
  exit MIQ_OK
end
def get_virtual_machine()
  return $evm.current['vm'] unless $evm.current['vm'].blank?
  case $evm.root['vmdb_object_type']
    when 'miq_provision'
      vm = $evm.root['miq_provision'].vm
    when 'vm'
      vm = $evm.root['vm']
  end
  raise "VM not found" unless vm
  vm
end

def get_intuit_dc()
  if $evm.root['vmdb_object_type'] == 'miq_provision'
    prov = $evm.root['miq_provision']
    ems_id = prov.get_option(:src_ems_id)
    ems = $evm.vmdb('ext_management_system', ems_id)
  else
    vm = get_virtual_machine()
    $evm.log(:info,"vm object is: #{vm.inspect}")
    ems = vm.ext_management_system
  end

  intuit_dc = ems.tags(:intuit_dc).first
  error("Intuit Data Center tag missing on provider: [#{ems.name}]") unless intuit_dc
  return intuit_dc
end

def retrieve_configuration(config, class_name, instance_name)
  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  instance = $evm.instantiate(instance_uri)
  error("Unable to retrieve required configuration: [#{instance_uri}]") unless instance

  #$evm.log(:info, "Retrieved configuration: [#{class_name}/#{instance_name}[#{instance.attributes}]")
  instance.attributes.each { |k, _| config[k] = instance.decrypt(k) }
end

def instantiate(uri)
  result = $evm.instantiate(uri)
  error("Failed to execute: [#{uri}]") unless result
  error(result['reason']) unless result['result'] == 'ok'
  return result
end

def escape_uri(value)
  URI.escape("#{value}", Regexp.new("[^#{URI::PATTERN::UNRESERVED}]"))
end

def get_service_now_application(intuit_dc, app_name)
  result = instantiate("#{GET_SERVICE_NOW_APPLICATION_LIST_URI}?" \
    "intuit_dc=#{escape_uri(intuit_dc)}");
  #$evm.log(:info, "ServiceNow Application List [#{result['application_list']}]")
  app_list = result['application_list']
  result = nil
  app_list.each do |app|
    if app_name == app['name']
      result = app
    end
  end

  result
end

def get_service_now_security_zone(intuit_dc, security_zone_string)
  result = instantiate("#{GET_SERVICE_NOW_SECURITY_TRUST_ZONE_LIST_URI}?" \
    "intuit_dc=#{escape_uri(intuit_dc)}");
  #$evm.log(:info, "ServiceNow Security Zone List [#{result['security_trust_zone_list']}]")
  security_trust_zone_list = result['security_trust_zone_list']
  result = nil
  trust_zone_array = security_zone_string.split('_')

  #example: qdce_prod_omn_fd2
  security_trust_zone_list.each do |zone|
    cleaned_zone_label = zone['label'].tr('-','').downcase
    if (cleaned_zone_label.include? trust_zone_array[1]) && (cleaned_zone_label.include? trust_zone_array[2])
      result = zone['label']
      if trust_zone_array[2].include? "TTO"
        result << " (TTO)"
      end
    end
  end

  result
end

def push_to_rel_all_table(http,host_name,table_url,prov)
  values = prov.options[:ws_values]
  body = {
      :u_parenttype => PARENT_TYPE,
      :u_childid => host_name,
      :u_parentid => values[:application_name],
      :u_childtype => CHILD_TYPE,
      :u_relationshiptype => RELATIONSHIP_TYPE,
      :u_parentname => values[:application_name],
      :u_source => SOURCE,
      :u_childname => host_name,
      :u_childidtype => ID_TYPE,
      :u_parentidtype => ID_TYPE
  }.to_json

  #making POST request to ServiceNow
  resp = http.post(table_url,body,["Content-Type", "application/json"]).content
  json_response = JSON.parse(resp)
  #$evm.log(:info,"ServiceNow Rel_All Insert Response: #{json_response}")
end

def get_device_type(template)
  device_type = ''
  device_types = {}
  retrieve_configuration(device_types, 'device_types', 'Default')
  if template.include? device_types['rhel_os']
    device_type = 'Linux'
  else
    device_type = 'Windows'
  end

  device_type
end

def format_hosting_segment(hosting_segment)
  hosting_segment.upcase.insert(-2,'-')
end

def get_os_version(template)
  template.split('_'+template.reverse.split('_')[0].reverse)[0]
end

def get_fault_domain(security_zone_string)
  security_zone_string.split('_')[3].upcase
end

def get_container_size(container_size)
  container_sizes = {
      "xsmall" => "XS",
      "small" => "S",
      "small_highmem" => "XS",
      "medium" => "M",
      "medium_highmem" => "M",
      "large" => "L",
      "large_highmem" => "L",
      "xlarge" => "XL",
      "xlarge_highmem" => "XL",
      "xlarge_extrememem" => "XL"
  }
  container_sizes[container_size]
end

def push_to_service_now_vm_table(http,host_name,table_url,bufgs,prov, intuit_dc)
  #authenticating for ServiceNow Call
  values = prov.options[:ws_values]
  body = {
      :u_hosting_segment => format_hosting_segment(values[:hosting_segment]),
      :u_bufg_l3 => bufgs[:l3],
      :u_device_type => get_device_type(values[:template]),
      :u_model_id => MODEL_ID,
      :u_bufg_l2 => bufgs[:l2],
      :u_fault_domain => get_fault_domain(values[:security_zone]),
      :u_owned_by => prov.userid[/[^@]+/],
      :u_environment => values[:environment],
      :u_vm_service_group => values[:service_name],
      :u_security_trust_zone => get_service_now_security_zone(intuit_dc, values[:security_zone]),
      :u_name => host_name,
      :u_container_size => get_container_size(values[:tag_intuit_instance_size]),
      :u_os_version => get_os_version(values[:template]),
      :u_execution_subzone => values[:execution_zone],
      :u_bufgl4 => bufgs[:l4],
      :u_sox_compliance => values[:sox_compliant],
      :u_manufacturer => MANUFACTURER,
      :u_hardware_status => STATUS
  }.to_json

  #making POST request to ServiceNow
  resp = http.post(table_url,body,["Content-Type", "application/json"]).content
  json_response = JSON.parse(resp)
  #$evm.log(:info,"ServiceNow Insert Response: #{json_response}")
end

begin
  intuit_dc = get_intuit_dc()
  config = {}
  retrieve_configuration(config, 'intuit_dc', intuit_dc)

  prov = $evm.root['miq_provision']
  host_name = prov.options[:host_name]

  #authenticating for ServiceNow Call
  http =  HTTPClient.new(config['proxy_url'])
  $evm.current['password'] = config['sn_password']
  http.set_auth(config['sn_domain_url'], config['sn_username'], $evm.current.decrypt('password'))

  app = get_service_now_application(intuit_dc, prov.options[:ws_values][:application_name])
  bufgs = {
      :l2 => app['u_bufg_l2'],
      :l3 => app['u_bufg_l3'],
      :l4 => app['u_bufg_l4']
  }

  push_to_service_now_vm_table(http, host_name, config['sn_vm_table_insert_url'], bufgs,prov,intuit_dc)
  push_to_rel_all_table(http, host_name, config['sn_rel_all_table_insert_url'],prov)

  # adding hostname to VM
  vm = get_virtual_machine()
  vm.custom_set("sn_host_name", host_name)

  # Prepare rollback in case we run into an error.
  rollback_parameters = prov.get_option(:rollback_parameters)
  rollback_parameters['sn_vm_id'] = vm.id
  prov.set_option(:rollback_parameters, rollback_parameters)

  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

rescue StandardError, SignalException => err
  retry_method("[#{err}]\n#{err.backtrace.join("\n")}")
end
