###################################
#
# CFME Automate Method: delete_from_spectrum
#
# Author: Carsten Clasohm, Daniel Trieu
#
# Notes: Remove a system from the Spectrum monitoring system during VM
#        retirement.
#
# Inputs: vm
#
# Mods:
#
###################################

require 'resolv'
require 'date'
require 'json'
require 'httpclient'

# logging convenience method
def log(level, msg)
  @method = 'delete_from_spectrum'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, "#{msg}")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def retry_method(msg)
  log(:error, "#{msg}, retrying")
  $evm.root['ae_result'] = 'retry'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def get_virtual_machine()
  case $evm.root['vmdb_object_type']
    when 'vm'
      vm = $evm.root['vm']
  end
  raise "VM not found" unless vm
  vm
end

def get_intuit_dc(vm)
  ems = vm.ext_management_system
  intuit_dc = ems.tags(:intuit_dc).first
  error("Intuit Data Center tag missing on provider: [#{ems.name}]") unless intuit_dc
  return intuit_dc
end

def spectrum_get_model_by_ip(http_client, custom_header, spectrum_server, ip_addr)
  log(:info, "Spectrum Get Model By IP: http_client=#{http_client} , custom_header=#{custom_header} , spectrum_server=#{spectrum_server} , ip_addr=#{ip_addr}")

  url = "http://#{spectrum_server}/spectrum/restful/model?ipaddress=#{ip_addr}"
  resp = http_client.post(url, '', custom_header).content
  json_response = JSON.parse(resp)
  log(:info, "Spectrum Get Model By IP: json_response=#{json_response}")

  # Production
  unless json_response['ns1.create-model-response'].nil?
    if json_response['ns1.create-model-response']['@error'].end_with? "Failure"
      return nil
    else
      model_handle = json_response['ns1.create-model-response']['ns1.model']['@mh']
      log(:info, "Spectrum Get Model By IP: model_handle=#{model_handle}")
      return model_handle
    end
  end
  
  # Lab
  unless json_response['ns2.create-model-response'].nil?
    if json_response['ns2.create-model-response']['@error'].end_with? "Failure"
      return nil
    else
      model_handle = json_response['ns2.create-model-response']['ns2.model']['@mh']
      log(:info, "Spectrum Get Model By IP: model_handle=#{model_handle}")
      return model_handle
    end
  end

  error("Spectrum Get Model By IP: Response unexpected")
end

def spectrum_annotate_model(http_client, custom_header, spectrum_server, model_handle, note)
  log(:info, "Spectrum Annotate Model: http_client=#{http_client} , custom_header=#{custom_header} , spectrum_server=#{spectrum_server} , model_handle=#{model_handle} , note=#{note}")

  url = "http://#{spectrum_server}/spectrum/restful/model/#{model_handle}?attr=0x11564&val=#{note}"
  resp = http_client.put(url, '', custom_header).content
  json_response = JSON.parse(resp)
  log(:info, "Spectrum Annotate Model: json_response=#{json_response}")

  return json_response
end

def spectrum_delete_model(http_client, custom_header, spectrum_server, model_handle)
  log(:info, "Spectrum Delete Model: http_client=#{http_client} , custom_header=#{custom_header} , spectrum_server=#{spectrum_server} , model_handle=#{model_handle}")

  url = "http://#{spectrum_server}/spectrum/restful/model/#{model_handle}"
  resp = http_client.delete(url, '', custom_header).content
  json_response = JSON.parse(resp)
  log(:info, "Spectrum Delete Model: json_response=#{json_response}")

  return json_response
end


begin
  log(:info, "CFME Automate Method Started")

  vm = get_virtual_machine()
  ems = vm.ext_management_system
  intuit_dc = get_intuit_dc(vm)

  if intuit_dc == "ihplab"
    hosting_segment = "ihplab"
  else
    hosting_segment = ems.tags(:intuit_hs).first
  end

  conf_path = "/Intuit/Integration/Spectrum/Configuration/hosting_segment/#{hosting_segment}"

  conf_props = $evm.instance_find(conf_path)
  error("Configuration not found in #{conf_path}") if conf_props.nil?

  instance = $evm.instantiate(conf_path)

  spectrum_server = instance['spectrum_server']
  spectrum_username = instance['username']
  spectrum_password = instance.decrypt('password')

  log(:info, "Spectrum Configuration: hosting_segment=#{hosting_segment} , spectrum_server=#{spectrum_server} , spectrum_username=#{spectrum_username} , spectrum_password=#{instance['password']}")

  vm_addresses = vm.ipaddresses
  if vm_addresses.blank?
    # If VMware tools are not running, we don't have a list of IP
    # addresses. Try to use the VM name for a DNS lookup.

    begin
      vm_addresses = [Resolv.getaddress(vm.name)]
    rescue Resolv::ResolvError => err
      # We only log this and don't throw an error so the rest of the
      # retirement process can continue. vm_addresses will be empty in this
      # case.

      log(:warn, "Cannot delete #{vm.name} from Spectrum, no known IP addresses and DNS lookup failed [#{err}]")

    end
  end

  http_client =  HTTPClient.new
  http_client.set_auth("http://#{spectrum_server}/spectrum/restful/", spectrum_username, spectrum_password)
  custom_header = [['Accept', 'application/json'], ['Content-Type', 'application/xml']]

  vm_addresses.each do |vm_ip|

    # Get the model_handle
    model_handle = spectrum_get_model_by_ip(http_client, custom_header, spectrum_server, vm_ip)

    unless model_handle.nil?
      # Annotate the model specifying who and when
      note = "Device_#{vm.name}_Deleted_By_Cloudforms_On_#{Date.today.to_s}"
      spectrum_annotate_model(http_client, custom_header, spectrum_server, model_handle, note)

      # Delete the model
      spectrum_delete_model(http_client, custom_header, spectrum_server, model_handle)
    end

  end

  $evm.root['ae_result'] = 'ok'
  $evm.root['ae_reason'] = ''

  log(:info, "Main: exiting main method")
  exit MIQ_OK

rescue StandardError, SignalException => err
  retry_method("[#{err}]\n#{err.backtrace.join("\n")}")
end
