##################################
#
# CFME Automate Method: add_vm_to_storage
#
# Author: Daniel Trieu
#
# Notes: Regional execution of adding storage mount points to a vm.
#
###################################

require 'open-uri'

VIX_ACTIONS_URI                  = "/Intuit/Integration/VIX/Actions"
RUN_PROGRAM_IN_GUEST             = "#{VIX_ACTIONS_URI}/RunProgramInGuest"

def log(level, msg)
  @method = 'add_vm_to_storage'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, "ERROR: #{msg}")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def retry_method(msg)
  log(:error, "RETRY: #{msg}")
  $evm.root['ae_result'] = 'retry'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def to_bool(value)
  log(:info, "to_bool: #{value}")
  return true if value == true || value =~ (/^(true|t|yes|y|1)$/i)
  return false if value == false || value.blank? || value =~ (/^(false|f|no|n|0)$/i)
  error("Invalid value for Boolean: to_bool(#{value})")
end

def dump_root()
  log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  log(:info, "Root:<$evm.root> End $evm.root.attributes")
end

def instantiate(uri)
  log(:info, "Instantiating: [#{uri}]")
  result = $evm.instantiate(uri)

  error("Failed to execute: [#{uri}]") unless result

  case result['result']
    when 'error'
      error(result['reason'])
    when 'retry'
      retry_method(result['reason'])
  end

  log(:info, "Instantiate Success: [#{uri}]")
  return result
end

def escape_uri(value)
  URI.escape(value, Regexp.new("[^#{URI::PATTERN::UNRESERVED}]"))
end

def execute_in_os(vix_platform, vix_guest_program, vix_guest_options)
  log(:info, "Attempting to add storage route and mount info in OS with the following commands: [#{vix_guest_program}][#{vix_guest_options}]")

  run_program_uri = "#{RUN_PROGRAM_IN_GUEST}?" \
    "vix_platform=#{vix_platform}&" \
    "vix_guest_program=#{escape_uri(vix_guest_program)}&" \
    "vix_guest_options=#{escape_uri(vix_guest_options)}"
  result = instantiate(run_program_uri)
  error("Failed to execute #{RUN_PROGRAM_IN_GUEST}") unless result
  error(result['reason']) unless result['result'] == 'ok'
end

def get_commands(class_name, instance_name)
  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  commands = instantiate(instance_uri)
  error("Unable to retrieve required configuration: [#{instance_uri}]") unless commands

  log(:info, "Retrieved configuration: [#{class_name}/#{instance_name}[#{commands.attributes}]")
  return commands
end

def attach_mounts_to_vm_linux(vm_name, route_options, fstab_options, mount_point)
  log(:info, "attach_mounts_to_vm_linux: #{vm_name} #{route_options}, #{fstab_options}")
  vix_platform = 'linux'

  commands = get_commands('VixPlatform', "#{vix_platform}_attach")
  commands.attributes.each { |k,v|
    log(:info, "Linux returned these keys: #{k} v: #{v}")
    values = v.split(" ")
    command = values.shift
    get_options = values.join(" ")
    if get_options.include? "EXCHANGE"
      options = get_options.gsub(/EXCHANGE/, fstab_options)
    elsif get_options.include? "PATH"
      options = get_options.gsub(/PATH/, route_options)
    elsif get_options.include? "MOUNTPOINT"
      options = get_options.gsub(/MOUNTPOINT/, mount_point)
    else
      options = get_options
    end
    log(:info, "command: #{command} options: #{options}")

    $evm.root['current_storage_vm'] = vm_name
    execute_in_os(vix_platform, command, options)
  }
end

def attach_mounts_to_vm_windows(vm_name, fstab_options)
  log(:info, "attach_mounts_to_vm_windows: #{vm_name}, #{fstab_options}")
  vix_platform = 'windows'

  commands = get_commands('VixPlatform', "#{vix_platform}_attach")
  commands.attributes.each { |k,v|
    log(:info, "Windows returned these keys: #{k} v: #{v}")
    values = v.split(" ")
    command = values.shift
    get_options = values.join(" ")
    if get_options.include? "PATH"
      windows_fstab_options = fstab_options.gsub(/\/+/,'\\')
      log(:info, "Windows converted route_options: #{windows_fstab_options}")
      options = get_options.gsub(/PATH/, windows_fstab_options)
    else
      options = get_options
    end
    log(:info, "command: #{command} options: #{options}")

    $evm.root['current_storage_vm'] = vm_name
    execute_in_os(vix_platform, command, options)
  }
end

begin
  log(:info, "Starting Method")

  task = $evm.root['automation_task']
  if task
    # Initiated from an automation request down to the local region.

    vm_name = task.options[:attrs][:vm_name]
    service_name = task.options[:attrs][:service_name]
    route_options = task.options[:attrs][:route_options]
    fstab_options = task.options[:attrs][:fstab_options]
    mount_point = fstab_options.split(':')[1]

    log(:info, "Received parameters: #{vm_name}, #{service_name}, #{route_options}, #{fstab_options}")

    vm_object = $evm.vmdb('vm').find_by_name(vm_name)

    os = vm_object.operating_system.product_name
    if os.include? 'Linux'
      vix_platform = 'linux'
    else
      vix_platform = 'windows'
    end
    log(:info, "VM #{vm_name} has Operating System #{os}; vix_platform #{vix_platform}")

    if vix_platform == 'linux'
      attach_mounts_to_vm_linux(vm_name, route_options, "#{fstab_options} #{mount_point} nfs defaults 0 0", mount_point)
    elsif vix_platform == 'windows'
      attach_mounts_to_vm_windows(vm_name, fstab_options)
    else
      log(:info, "VIX Platform Unknown: #{vix_platform}")
      error("Vix Platform Unknown")
    end

  end

  log(:info, "Ending Method")
  exit MIQ_OK
rescue StandardError, SignalException => err
  retry_method("[#{err}]\n#{err.backtrace.join("\n")}")
end
