##################################
#
# CFME Automate Method: create_storage_service
#
# Author: Nick Lane, Daniel Trieu
#
# Notes: Creates storage services
#
###################################
require 'restclient'
require 'json'
require 'resolv'

# logging convenience method
def log(level, msg)
  @method = 'create_storage_service'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def retrieve_configuration(config, class_name, instance_name)
  log(:info, "retrieve_configuration: #{config} #{class_name} #{instance_name}")

  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  instance = $evm.instantiate(instance_uri)
  error("Unable to retrieve required configuration: [#{instance_uri}]") unless instance

  log(:info, "Retrieved configuration: [#{class_name}/#{instance_name}[#{instance.attributes}]")
  instance.attributes.each { |k, _| config[k] = instance.decrypt(k) }
end

def get_ip_addresses_for_vms(vms)
  log(:info, "get_ip_addresses_for_vms: #{vms}")

  vm_ip_addresses = []
  vms.split(',').each { |vm_name|
    vm_split_name = vm_name.split('.')
    vm_split_name[0] = "#{vm_split_name[0]}s"
    vm_storage_name = vm_split_name.join('.')

    ipaddr = Resolv.getaddress(vm_storage_name)

    log(:info, "VM #{vm_name} Storage NIC IP Address: #{ipaddr}")
    vm_ip_addresses.push(ipaddr)
  }

  log(:info, "Found IP Addresses: #{vm_ip_addresses}")
  return vm_ip_addresses
end

def get_iqns_for_vms(vms)
  log(:info, "get_iqns_for_vms: #{vms}")

  vm_iqns = []
  vms.split(',').each { |vm_name|
    vm_split_name = vm_name.split('.')
    year_month = Time.now.strftime("%Y-%m")
    ms_since_epoch = Time.now.to_i.to_s
    vm_iqn_name = "iqn.#{year_month}.com.intuit.iscsi:#{vm_split_name[0]}-#{ms_since_epoch}"
    vm_iqns.push(vm_iqn_name)
  }

  log(:info, "Found IQN: #{vm_iqns}")
  return vm_iqns
end

begin
  log(:info, "Starting Method")

  task = $evm.root['service_template_provision_task']

  # TODO Tag service with more attributes?

  # Setup REST parameters
  tier = task.get_option(:storage_tier)
  protocol = task.get_option(:storage_protocol)
  hosting_segment = task.get_option(:storage_hosting_segment)
  fault_domain = task.get_option(:storage_fault_domain)
  zone = task.get_option(:storage_zone)
  security = task.get_option(:storage_security)
  compliance = task.get_option(:storage_compliance)
  environment = task.get_option(:storage_environment)
  hostname = task.get_option(:storage_hostname)
  numqtrees = task.get_option(:storage_numqtrees)
  storage_size = task.get_option(:storage_size)
  quota_size_list = Array.new(numqtrees.to_i, storage_size).join(',')
  export_ip = get_ip_addresses_for_vms(task.get_option(:storage_vms)).join(',')
  iqn_list = get_iqns_for_vms(task.get_option(:storage_vms)).join(',')
  bufg = task.get_option(:storage_bufg)
  application = task.get_option(:storage_app)

  # Find datacenter based on hosting segment, otherwise it is lab.
  log(:info, "Searching Datacenter with hosting segment #{hosting_segment}")
  ems_returned = $evm.vmdb('ext_management_system').all.select do |ems|
    next unless ems.tagged_with?('intuit_hs', hosting_segment)
    ems
  end
  unless ems_returned.empty?
    log(:info, "Provider Found: #{ems_returned}")
    ems_provider = ems_returned.first

    intuit_dc = ems_provider.tags(category='intuit_dc').first.to_s
  else
    intuit_dc = 'ihplab'
  end

  log(:info, "Found Datacenter: #{intuit_dc}")

  # Load up configuration for the found datacenter to get the Storage Server address and credentials
  config = {}
  retrieve_configuration(config, 'intuit_dc', intuit_dc)

  log(:info, "Storage Server Configuration: #{config}")
  storage_server = config['storage_server']
  user_key = config['user_key']

  storage_url = "http://#{storage_server}/api/storage/v1/create"

  if protocol == 'nfs'
    storage_hash = {"tier" => tier,
                    "protocol" => protocol,
                    "HostingSegment" => hosting_segment,
                    "FaultDomain" => fault_domain,
                    "function" => 'Storage',
                    "purpose" => 'Shared',
                    "zone" => zone,
                    "security" => security,
                    "compliance" => compliance,
                    "environment" => environment,
                    "hostname" => hostname,
                    "numQtrees" => numqtrees,
                    "qtreeSizeList" => quota_size_list,
                    "secString" => export_ip, # list of vms ip
                    "bufg"=> bufg,
                    "application"=> application}
  elsif protocol == 'iscsi'
    storage_hash = {"tier" => tier,
                    "protocol" => protocol,
                    "HostingSegment" => hosting_segment,
                    "FaultDomain" => fault_domain,
                    "function" => 'Storage',
                    "purpose" => 'Shared',
                    "zone" => zone,
                    "security" => security,
                    "compliance" => compliance,
                    "environment" => environment,
                    "hostname" => hostname,
                    "ostype" => 'windows',
                    "numluns" => numqtrees,
                    "lunSizeList" => quota_size_list,
                    "iqnlist" => iqn_list, # list iqns
                    "bufg"=> bufg,
                    "application"=> application}
  end

  storage_body = JSON.generate(storage_hash)
  headers_hash = JSON.generate({'userkey' => user_key})

  log(:info, "Using url #{storage_url}")
  log(:info, "Using body #{storage_body}")
  log(:info, "Using headers #{headers_hash}")

  begin
    response = RestClient::Request.execute(
        :method => :post,
        :url => storage_url,
        :payload => storage_body,
        :headers => {
            :userkey => user_key,
            :content_type => :json
        })
  rescue => err
    log(:info, "RestClient Error: #{err.inspect}")
  end
  log(:info, "Returned response: #{response.inspect}")

  job = JSON.parse(response)

  task.set_option(:storage_job_id, job['JobId'])
  task.set_option(:datacenter, intuit_dc)

  log(:info, "Storage Service Created [#{job['JobId']}] for Datacenter #{intuit_dc}")
  log(:info, "Ending Method")
  exit MIQ_OK
rescue StandardError, SignalException => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
