##################################
#
# CFME Automate Method: delete_storage_service
#
# Author: Daniel Trieu
#
# Notes: Attempts to delete the storage service from the API
#
###################################

require 'restclient'
require 'json'
require 'open-uri'
require 'tempfile'
require 'timeout'
require 'resolv'

def log(level, msg)
  @method = 'delete_storage_service'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def retry_method(msg)
  log(:warn, "RETRY: #{msg}")
  $evm.root['ae_result'] = 'retry'
  $evm.root['ae_retry_interval'] = '1.minute'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def dump_root()
  log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  log(:info, "Root:<$evm.root> End $evm.root.attributes")
  log(:info, "")
end

def to_bool(value)
  return true if value == true || value =~ (/^(true|t|yes|y|1)$/i)
  return false if value == false || value.blank? || value =~ (/^(false|f|no|n|0)$/i)
  error("Invalid value for Boolean: to_bool(#{value})")
end

def escape_uri(value)
  URI.escape(value, Regexp.new("[^#{URI::PATTERN::UNRESERVED}]"))
end

def retrieve_configuration(class_name, instance_name)
  log(:info, "retrieve_configuration: #{class_name} #{instance_name}")
  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  instance = $evm.instance_find(instance_uri)
  error("Unable to retrieve required configuration: [#{instance_uri}]") unless instance

  log(:info, "Retrieved configuration: [#{class_name}/#{instance_name}[#{instance}]")
  instance[instance_name]
end

def update_configuration(new_config, class_name, instance_name)
  log(:info, "update_configuration: #{new_config} #{class_name} #{instance_name}")
  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  result = $evm.instance_update(instance_uri, new_config)

  error("Unable to update configuration: [#{instance_uri}]") unless to_bool(result)
end

def get_hosting_segment_from_service(service)
  log(:info, "get_hosting_segment_from_service: #{service}")
  vms_in_service = service.tags(category='intuit_storage_vms')

  log(:info, "Found VM IDs: #{vms_in_service}")
  unless vms_in_service.nil?
    vms_in_service.each { |vm_id|
      vm_object = $evm.vmdb('vm').find_by_id(vm_id)
      unless vm_object.nil?
        vm_tags = vm_object.tags(category='intuit_hs')
        log(:info, "Found Hosting Segment: #{vm_tags}")
        return vm_tags.first
      end
    }
  end
end

begin
  log(:info, "Starting Method")

  service = $evm.root['service']
  description = service.description

  hosting_segment = get_hosting_segment_from_service(service)
  object_id		= service.tags(category='intuit_storage_object_id').first

  ems_returned = $evm.vmdb('ext_management_system').all.select do |ems|
    next unless ems.tagged_with?('intuit_hs', hosting_segment)
    log(:info, "EMS #{ems} has Hosting Segment: #{hosting_segment}")
    ems
  end
  unless ems_returned.empty?
    log(:info, "Provider returned #{ems_returned}")
    ems_provider = ems_returned.first

    intuit_dc = ems_provider.tags(category='intuit_dc').first.to_s
  else
    intuit_dc = 'ihplab'
  end
  log(:info, "Datacenter is #{intuit_dc}")

  config = retrieve_configuration('intuit_dc', intuit_dc)

  storage_server = config['storage_server']
  user_key = config['user_key']
  log(:info, "Got config #{config}")

  delete_url = "http://#{storage_server}/api/storage/v1/delete/objectid/#{object_id}"

  description_array = description.split(':')

  lif_name = description_array[1]
  path 	 = description_array[2]
  body_hash  = {"vserverlifname" => lif_name, "path" => path}

  delete_body = JSON.generate(body_hash)
  log(:info, "URL: #{delete_url}, BODY: #{delete_body}")

  begin
    response = RestClient::Request.execute(
        :method => :delete,
        :url => delete_url,
        :payload => delete_body,
        :headers => {
            :userkey => user_key,
            :content_type => :json
        })
  rescue => err
    log(:info, "RestClient Error: #{err.inspect}")
  end
  log(:info, "Returned response: #{response.inspect}")

  job = JSON.parse(response)

  unless job['Error'].nil?
    if job['Error'].include?('TRY AGAIN')
      retry_method(job['Error'])
    else
      error(job['Error'])
    end
  end

  old_config = retrieve_configuration('intuit_ss', object_id)

  new_config = {
      'storage_service_name' => old_config['storage_service_name'],
      'storage_path' => old_config['storage_path'],
      'datacenter' => old_config['datacenter'],
      'storage_job_id' => job['JobId']
  }

  update_configuration(new_config, 'intuit_ss', object_id)

  log(:info, "Ending Method")
  exit MIQ_OK
rescue StandardError, SignalException => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
