##################################
#
# CFME Automate Method: read_storage_info
#
# Author: Nick Lane, Daniel Trieu
#
# Notes: Creates storage services
#
###################################
require 'restclient'
require 'json'

# logging convenience method
def log(level, msg)
  @method = 'read_storage_info'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def to_bool(value)
  return true if value == true || value =~ (/^(true|t|yes|y|1)$/i)
  return false if value == false || value.blank? || value =~ (/^(false|f|no|n|0)$/i)
  error("Invalid value for Boolean: to_bool(#{value})")
end

def retrieve_configuration(class_name, instance_name)
  log(:info, "retrieve_configuration: #{class_name} #{instance_name}")
  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  instance = $evm.instance_find(instance_uri)
  error("Unable to retrieve required configuration: [#{instance_uri}]") unless instance

  log(:info, "Retrieved configuration: [#{class_name}/#{instance_name}[#{instance}]")
  instance[instance_name]
end

def create_configuration(new_config, class_name, instance_name)
  log(:info, "create_configuration: #{new_config} #{class_name} #{instance_name}")

  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  result = $evm.instance_create(instance_uri, new_config)
  error("Unable to create configuration") unless to_bool(result)

  result
end

def tag_service(service, category, tag_value)
  $evm.execute('category_create', :name => category, :single_value => false, :description=> category) unless $evm.execute('category_exists?', category)
  tag = tag_value.downcase unless tag_value.nil?

  log(:info, "Tag value: #{tag}")
  if $evm.execute('tag_exists?', category, tag)
    service.tag_assign(category+"/"+tag)
  else
    $evm.execute('tag_create', category, :name => tag, :description => tag)
    service.tag_assign(category+"/"+tag)
  end
end

begin
  log(:info, "Starting Method")

  task = $evm.root['service_template_provision_task']
  service = task.destination

  storage_object_id	= task.options[:storage_object_id]
  datacenter = task.options[:datacenter]
  protocol = task.options[:storage_protocol]


  log(:info, "Got Object ID [#{storage_object_id}] and datacenter [#{datacenter}]")
  config = retrieve_configuration('intuit_dc', datacenter)

  log(:info, "Storage Server Configuration: #{config}")
  storage_server = config['storage_server']
  user_key = config['user_key']

  read_url = "http://#{storage_server}/api/storage/v1/read/objectid/#{storage_object_id}"

  log(:info, "Using url #{read_url}")
  log(:info, "Using header userkey: #{user_key}")

  read_ext = RestClient::Request.execute(
      :method => :get,
      :url => read_url,
      :headers => {
          :userkey => user_key,
          :content_type => :json
      })

  log(:info, "Read storage: #{read_ext}")

  storage_objects = JSON.parse(read_ext)

  #We are making the assumption that the first key is the valid key, since the storage service provides keys in the form of a timestamp. Timestamps are not good keys, please have Barry or Achal fix this.
  storage_key = storage_objects.keys.first

  storage_obj = storage_objects[storage_key]

  service.description = "#{protocol}:#{storage_obj['path']}" # lowercase path is the key

  new_config = {
      'storage_service_name' => service.name,
      'storage_path' => storage_obj['path'],
      'datacenter' => datacenter,
      'storage_job_id' => nil
  }

  create_configuration(new_config, 'intuit_ss', storage_object_id)
  tag_service(service, 'intuit_storage_object_id', storage_object_id)

  log(:info, "Ending Method")
  exit MIQ_OK
rescue StandardError, SignalException => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
