##################################
#
# CFME Automate Method: request_vm_add_storage
#
# Author: Daniel Trieu
#
# Notes: Tags service with the associated VMs.
#        Instantiates webservice request to attach vm to storage.
#
###################################

require 'uri'

def log(level, msg)
  @method = 'request_vm_add_storage'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, "#{msg}")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def retry_method(msg)
  log(:error, "RETRY: #{msg}")
  $evm.root['ae_result'] = 'retry'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def to_bool(value)
  return true if value == true || value =~ (/^(true|t|yes|y|1)$/i)
  return false if value == false || value.blank? || value =~ (/^(false|f|no|n|0)$/i)
  error("Invalid value for Boolean: to_bool(#{value})")
end

def dump_root()
  log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  log(:info, "Root:<$evm.root> End $evm.root.attributes")
end

def instantiate(uri)
  log(:info, "Instantiating: [#{uri}]")
  result = $evm.instantiate(uri)

  error("Failed to execute: [#{uri}]") unless result

  case result['result']
    when 'error'
      error(result['reason'])
    when 'retry'
      retry_method(result['reason'])
  end

  log(:info, "Instantiate Success: [#{uri}]")
  return result
end

def tag_service(service, category, tag)
  log(:info, "Tag Service: #{service}, #{category}, #{tag}")

  $evm.execute('category_create', :name => category, :single_value => false, :description=> category) unless $evm.execute('category_exists?', category)
  tag_value = tag.values.first
  tag_description = tag.keys.first

  log(:info, "Tagging with Value #{tag_value} and Description #{tag_description}")

  if $evm.execute('tag_exists?', category, tag_value)
    service.tag_assign(category+"/"+tag_value)
  else
    $evm.execute('tag_create', category, :name => tag_value, :description => tag_description)
    service.tag_assign(category+"/"+tag_value)
  end
end

def get_region_endpoint(vm)
  log(:info, "get_region_endpoint: #{vm}")

  uri = "Intuit/Integration/CloudForms/Actions/GetRegionEndpoints?vmdb_object_type=vm&Vm::vm=#{vm.id}"
  result = instantiate(uri)
  ip_address = result['region_endpoint_ip_addresses'].first
  log(:info, "Mapped VM to CloudForms region: [#{vm.name}][#{ip_address}]")

  return ip_address
end

def create_attach_storage_request(intuit_dc, service, vm_name)
  log(:info, "Create Attach Storage Request: #{intuit_dc}, #{vm_name}")

  log(:info, "Find VM by Name: #{vm_name}")
  vm_object = $evm.vmdb('vm').find_by_name(vm_name)

  error("VM not found: #{vm_name}") if vm_object.nil?

  log(:info, "Found VM: #{vm_object}")

  endpoint_ip_address = get_region_endpoint(vm_object)

  parameters = "vm_name=#{vm_name}|service_name=#{service.name}|route_options=#{@route_options}|fstab_options=#{@fstab_options}"

  uri = "Intuit/Integration/CloudForms/Actions/CreateAutomationRequest?" + URI.encode_www_form([ ['intuit_dc', intuit_dc ],
                                                                                                 ['ae_namespace', 'Intuit/Integration/StorageServices'],
                                                                                                 ['ae_class', 'StateMachine'],
                                                                                                 ['ae_instance', 'AttachStorageRegional'],
                                                                                                 ['ip_address', endpoint_ip_address],
                                                                                                 ['parameters', parameters] ])
  result = instantiate(uri)
  log(:info, "Instantiate Result: #{result['result']}")

  vm_tag = {}
  vm_tag[vm_object.name] = vm_object.id.to_s
  tag_service(service, 'intuit_storage_vms', vm_tag)
end

def get_dc_by_service_hosting_segment(service)
  log(:info, "get_dc_by_service_hosting_segment: #{service}")
  hosting_segment = service.tags(category='intuit_hs')

  ems_returned = $evm.vmdb('ext_management_system').all.select do |ems|
    log(:info, "Looking for #{hosting_segment}")
    next unless ems.tagged_with?('intuit_hs', hosting_segment)
    ems
  end
  unless ems_returned.empty?
    log(:info, "Provider returned #{ems_returned}")
    ems_provider = ems_returned.first

    intuit_dc = ems_provider.tags(category='intuit_dc').first.to_s
  else
    intuit_dc = 'ihplab'
  end

  log(:info, "Datacenter: #{intuit_dc}")
  return intuit_dc
end

begin
  log(:info, "Starting Method")

  object_type = $evm.root['vmdb_object_type']
  case object_type
    when 'service_template_provision_task'
      task = $evm.root[object_type]
      vm_list = task.get_option(:storage_vms)
      service = task.destination
    when 'service'
      vm_list = $evm.root['dialog_storage_vms']
      service = $evm.root['service']
  end

  description = service.description
  log(:info, "Service Description: #{description}")

  description_array = description.split(':')
  path = "#{description_array[1]}:#{description_array[2]}"
  @route_options = description_array[1]
  @fstab_options = path

  log(:info, "Adding VMs #{vm_list} to service #{service.name}")

  intuit_dc = get_dc_by_service_hosting_segment(service)

  if vm_list.include? (',')
    vms = vm_list.split(',')
    vms.each do |vm_name|
      create_attach_storage_request(intuit_dc, service, vm_name)
    end
  else
    create_attach_storage_request(intuit_dc, service, vm_list)
  end

  log(:info, "Ending Method")
  exit MIQ_OK
rescue StandardError, SignalException => err
  retry_method("[#{err}]\n#{err.backtrace.join("\n")}")
end
