##################################
#
# CFME Automate Method: validate_parameters
#
# Author: Daniel Trieu
#
# Notes: Validates the storage service parameters
#
###################################

require 'resolv'

def log(level, msg)
  @method = 'validate_parameters'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, "#{msg}")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def retry_method(msg)
  log(:error, "RETRY: #{msg}")
  $evm.root['ae_result'] = 'retry'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def to_bool(value)
  return true if value == true || value =~ (/^(true|t|yes|y|1)$/i)
  return false if value == false || value.blank? || value =~ (/^(false|f|no|n|0)$/i)
  error("Invalid value for Boolean: to_bool(#{value})")
end

def dump_root()
  log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  log(:info, "Root:<$evm.root> End $evm.root.attributes")
end

def get_vm_in_storage_service(service)
  log(:info, "get_vm_in_storage_service: #{service}")
  vms_in_service = service.tags(category='intuit_storage_vms')

  log(:info, "Found VM IDs: #{vms_in_service}")
  unless vms_in_service.nil?
    vms_in_service.each { |vm_id|
      vm_object = $evm.vmdb('vm').find_by_id(vm_id)
      return vm_object unless vm_object.nil?
    }
  end

  return nil
end

def get_vm_names_in_storage_service(service)
  log(:info, "get_vm_names_in_storage_service: #{service}")
  vms_in_service = service.tags(category='intuit_storage_vms')

  log(:info, "Found VM IDs: #{vms_in_service}")
  unless vms_in_service.nil?
    vm_list = vms_in_service.collect { |vm_id|
      vm_object = $evm.vmdb('vm').find_by_id(vm_id)
      vm_object.name unless vm_object.nil?
    }.compact.uniq.join(',')
  end

  log(:info, "VM Names are: #{vm_list}")
  return vm_list
end

def get_ss_valid_hosting_segment(vm)
  log(:info, "get_ss_valid_hosting_segment")
  vm_tags = vm.tags(:intuit_hs)
  vm_tags.collect! { |vm_tag|
    vm_tag.insert(-2, '-').downcase
  }

  log(:info, "Valid SS Hosting Segments: #{vm_tags}")
  return vm_tags
end

def get_ss_valid_fault_domain(vm)
  log(:info, "get_ss_valid_fault_domain")
  vm_tags = vm.tags(:intuit_fd)
  vm_tags.collect! { |vm_tag|
    case vm_tag
      #when 'fd0'
      #  'any'
      when 'fd1'
        'fd1'
      when 'fd2'
        'fd2'
      when 'fd3'
        'fd3'
      #when 'none'
      #  'any'
      else
        'ERROR'
    end
  }

  log(:info, "Valid SS Fault Domains: #{vm_tags}")
  return vm_tags
end

def get_ss_valid_security(vm)
  log(:info, "get_ss_valid_security")
  vm_tags = vm.tags(:intuit_security_level)
  vm_tags.collect! { |vm_tag|
    case vm_tag
      #when 'corp'
      #  'ERROR'
      when 'nonpublic'
        'non_public'
      when 'nonpubliccomp'
        'non_public'
      when 'omn'
        'others'
      when 'public'
        'public'
      when 'publiccomp'
        'public'
      when 'sensitive'
        'sensitive'
      when 'sensitivecomp'
        'sensitive'
      else
        'ERROR'
    end
  }

  log(:info, "Valid SS Security: #{vm_tags}")
  return vm_tags
end

def get_ss_valid_environment(vm)
  log(:info, "get_ss_valid_environment")
  vm_tags = vm.tags(:intuit_sz)
  vm_tags.collect! { |vm_tag|
    case vm_tag
      when 'admin'
        'ERROR'
      when 'comp'
        'ERROR'
      when 'corp'
        'corpzone'
      when 'dev'
        'devzone'
      when 'preprod'
        'non_production'
      when 'prod'
        'production'
      else
        'ERROR'
    end
  }

  log(:info, "Valid SS Environment: #{vm_tags}")
  return vm_tags
end

def get_ss_valid_compliance(vm)
  log(:info, "get_ss_valid_compliance")
  vm_tags = vm.tags(:intuit_security_level)
  vm_tags.collect! { |vm_tag|
    if vm_tag.end_with? 'comp'
      'compliance'
    else
      'others'
    end
  }

  log(:info, "Valid SS Compliance: #{vm_tags}")
  return vm_tags
end

def get_ss_valid_zone(vm)
  log(:info, "get_ss_valid_zone")
  vm_tags = vm.tags(:intuit_security_level)
  vm_tags.collect! { |vm_tag|
    if vm_tag == 'omn'
      'omn'
    else
      'ptn'
    end
  }

  log(:info, "Valid SS Zone: #{vm_tags}")
  return vm_tags
end

def get_ostype(vm)
  os = vm.operating_system.product_name
  if os.include? 'Linux'
    return 'linux'
  else
    return 'windows'
  end
end

def is_compatible_nfs(vm)
  log(:info, "is_compatible_nfs: #{vm}")
  compatible_ostypes = %w(linux)
  ostype = get_ostype(vm)

  if compatible_ostypes.include?(ostype)
    log(:info, "NFS Compatible: #{ostype}")
    return 'COMPATIBLE'
  else
    log(:info, "NFS Incompatible: #{ostype}")
    return 'ERROR'
  end
end

def is_compatible_iscsi(vm)
  log(:info, "is_compatible_iscsi: #{vm}")
  compatible_ostypes = %w(linux windows)
  ostype = get_ostype(vm)

  if compatible_ostypes.include?(ostype)
    log(:info, "iSCSI Compatible: #{ostype}")
    return 'COMPATIBLE'
  else
    log(:info, "iSCSI Incompatible: #{ostype}")
    return 'ERROR'
  end
end

def add_to_set_hosting_segment(vm, set)
  log(:info, "add_to_set_hosting_segment: #{vm} #{set}")
  vm_tags = get_ss_valid_hosting_segment(vm)
  set.concat(vm_tags)
end

def add_to_set_fault_domain(vm, set)
  log(:info, "add_to_set_fault_domain: #{vm} #{set}")
  vm_tags = get_ss_valid_fault_domain(vm)
  set.concat(vm_tags)
end

def add_to_set_security(vm, set)
  log(:info, "add_to_set_security: #{vm} #{set}")
  vm_tags = get_ss_valid_security(vm)
  set.concat(vm_tags)
end

def add_to_set_environment(vm, set)
  log(:info, "add_to_set_environment: #{vm} #{set}")
  vm_tags = get_ss_valid_environment(vm)
  set.concat(vm_tags)
end

def add_to_set_compliance(vm, set)
  log(:info, "add_to_set_compliance: #{vm} #{set}")
  vm_tags = get_ss_valid_compliance(vm)
  set.concat(vm_tags)
end

def add_to_set_zone(vm, set)
  log(:info, "add_to_set_zone: #{vm} #{set}")
  vm_tags = get_ss_valid_zone(vm)
  set.concat(vm_tags)
end

def add_to_set_nfs(vm, set)
  log(:info, "add_to_set_nfs: #{vm} #{set}")
  result = is_compatible_nfs(vm)
  set.push(result)
end

def add_to_set_iscsi(vm, set)
  log(:info, "add_to_set_iscsi: #{vm} #{set}")
  result = is_compatible_iscsi(vm)
  set.push(result)
end

def invalid_set(set)
  if set.empty?
    return true
  elsif set.uniq.length > 1
    return true
  elsif set.include? "ERROR"
    return true
  else
    return false
  end
end

def any_set_invalid(set_hosting_segment, set_fault_domain, set_security, set_environment, set_compliance, set_zone, set_storage_tech)
  log(:info, "any_set_invalid")

  if invalid_set(set_hosting_segment)
    log(:info, "Invalid Set: #{set_hosting_segment}")
    return true
  elsif invalid_set(set_fault_domain)
    log(:info, "Invalid Set: #{set_fault_domain}")
    return true
  elsif invalid_set(set_security)
    log(:info, "Invalid Set: #{set_security}")
    return true
  elsif invalid_set(set_environment)
    log(:info, "Invalid Set: #{set_environment}")
    return true
  elsif invalid_set(set_compliance)
    log(:info, "Invalid Set: #{set_compliance}")
    return true
  elsif invalid_set(set_zone)
    log(:info, "Invalid Set: #{set_zone}")
    return true
  elsif invalid_set(set_storage_tech)
    log(:info, "OS Incompatible with Storage Tech: #{set_storage_tech}")
    return true
  else
    log(:info, "No Invalid Set")
    return false
  end

end

def has_storage_address(vm_name)
  log(:info, "has_storage_address: #{vm_name}")

  vm_split_name = vm_name.split('.')
  vm_split_name[0] = "#{vm_split_name[0]}s"
  vm_storage_name = vm_split_name.join('.')

  begin
    ipaddr = Resolv.getaddress(vm_storage_name)
  rescue StandardError => err
    ipaddr = nil
  end

  log(:info, "Storage Address for #{vm_name}: #{ipaddr}")
  if ipaddr.nil?
    false
  else
    true
  end
end

def existing_service_has_vm(service, vm_name)
  log(:info, "existing_service_has_vm: #{service} #{vm_name}")
  if service.nil?
    return false
  else
    assigned_vms = service.tags(:intuit_storage_vms)
    log(:info, "Service has these VMs already: #{assigned_vms}")
    return assigned_vms.include?(vm_name)
  end
end

begin
  log(:info, "Starting Method")

  set_hosting_segment = []
  set_fault_domain = []
  set_security = []
  set_environment = []
  set_compliance = []
  set_zone = []
  set_storage_tech = []

  vmdb_object_type = $evm.root['vmdb_object_type']
  if vmdb_object_type == 'service_template_provision_task'
    log(:info, "Identified VMDB Object Type: #{vmdb_object_type}")

    task = $evm.root[vmdb_object_type]
    service = task.destination

    unless $evm.root['dialog_storage_service_name'].nil?
      service_name = $evm.root['dialog_storage_service_name']
    else
      service.name = "NilServiceName"
      error("Bad Service Name Provided. Please delete failed service from catalog.")
    end

    # Set service name to what was provided by dialogs
    service.name = service_name

    # Validation not needed, so just add to task options
    #task.set_option(:storage_bufg, bufg_from_vm)
    task.set_option(:storage_bufg, "Storage & Backup")
    #task.set_option(:storage_app, app_from_vm)
    task.set_option(:storage_app, "Storage - Las Vegas Datacenter")

    task.set_option(:storage_service_name, $evm.root['dialog_storage_service_name'])
    task.set_option(:storage_tier, $evm.root['dialog_storage_tier'])
    task.set_option(:storage_protocol, $evm.root['dialog_storage_protocol'])
    task.set_option(:storage_hostname, $evm.root['dialog_storage_hostname'])
    task.set_option(:storage_size, $evm.root['dialog_storage_size'])
    task.set_option(:storage_numqtrees, $evm.root['dialog_storage_numqtrees'])

    protocol = $evm.root['dialog_storage_protocol']

  elsif vmdb_object_type == 'service'
    log(:info, "Identified VMDB Object Type: #{vmdb_object_type}")

    svc = $evm.root[vmdb_object_type]

    # Get a VM in service, to get the security properties. All VMs should have the same security levels.
    vm_in_service = get_vm_in_storage_service(svc)

    # The VM already has security settings in place in the form of tags.
    # Add them to the sets for comparison.
    unless vm_in_service.nil?
      add_to_set_hosting_segment(vm_in_service, set_hosting_segment)
      add_to_set_fault_domain(vm_in_service, set_fault_domain)
      add_to_set_security(vm_in_service, set_security)
      add_to_set_environment(vm_in_service, set_environment)
      add_to_set_compliance(vm_in_service, set_compliance)
      add_to_set_zone(vm_in_service, set_zone)
    end

    protocol = svc.description.split(':')[0]

  else
    error("Unidentifiable VMDB Object Type: #{vmdb_object_type}")
  end

  delete_storage_service = $evm.root['dialog_delete_storage_service']

  unless delete_storage_service.nil?
    if to_bool(delete_storage_service)
      vm_list = get_vm_names_in_storage_service(svc)
    else
      error("Delete Storage Task was not confirmed: #{delete_storage_service}")
    end
  else
    vm_list = $evm.root['dialog_storage_vms']
  end

  log(:info, "Verify VMs [#{vm_list}] for existence.")
  # Validate all VMs exist, otherwise error out
  existing_vms = vm_list.split(',').collect { | vm_name |
    unless existing_service_has_vm(svc, vm_name)
      vm = $evm.vmdb('vm').find_by_name(vm_name)
      if vm
        log(:info, "VM Found: #{vm.name}")
        if has_storage_address(vm.name)
          log(:info, "VM Storage Address Found: #{vm.name}")
          vm
        else
          error("VM Storage Address Not Found: #{vm.name}")
        end
      else
        error("VM Not Found: #{vm_name}")
      end
    end
  }.compact.uniq

  error("No VMs Provided: [#{vm_list}]") if existing_vms.empty?

  # Collect existing VMs security parameters.
  existing_vms_names = existing_vms.each { | vm |
    add_to_set_hosting_segment(vm, set_hosting_segment)
    add_to_set_fault_domain(vm, set_fault_domain)
    add_to_set_security(vm, set_security)
    add_to_set_environment(vm, set_environment)
    add_to_set_compliance(vm, set_compliance)
    add_to_set_zone(vm, set_zone)

    if protocol == 'nfs'
      add_to_set_nfs(vm, set_storage_tech)
    elsif protocol == 'iscsi'
      add_to_set_iscsi(vm, set_storage_tech)
    end

    if any_set_invalid(set_hosting_segment, set_fault_domain, set_security, set_environment, set_compliance, set_zone, set_storage_tech)
      error("VMs [#{existing_vms}] do not share equivalent security attributes.")
    end

    vm.name
  }.join(',')

  # If this is a task, then populate the storage create values
  # TODO Remove hardcode in production
  if task
    task.set_option(:storage_vms, existing_vms_names)
    #task.set_option(:storage_hosting_segment, set_hosting_segment.first)
    task.set_option(:storage_hosting_segment, "SDL-A")
    task.set_option(:storage_fault_domain, set_fault_domain.first)
    #task.set_option(:storage_security, set_security.first)
    task.set_option(:storage_security, "Non_Public")
    task.set_option(:storage_environment, set_environment.first)
    task.set_option(:storage_compliance, set_compliance.first)
    #task.set_option(:storage_zone, set_zone.first)
    task.set_option(:storage_zone, "PTN")
  end

  if svc
    # Nothing to do
  end

  log(:info, "Ending Method")
  exit MIQ_OK
rescue StandardError, SignalException => err
  retry_method("[#{err}]\n#{err.backtrace.join("\n")}")
end
