##################################
#
# CFME Automate Method: wait_for_storage
#
# Author: Nick Lane, Daniel Trieu
#
# Notes: Creates storage services
#
###################################
require 'restclient'
require 'json'

# logging convenience method
def log(level, msg)
  @method = 'wait_for_storage'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def retry_method(msg)
  log(:warn, "RETRY: #{msg}")
  $evm.root['ae_result'] = 'retry'
  $evm.root['ae_retry_interval'] = '1.minute'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def retrieve_configuration(class_name, instance_name)
  log(:info, "retrieve_configuration: #{class_name} #{instance_name}")
  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  instance = $evm.instance_find(instance_uri)
  error("Unable to retrieve required configuration: [#{instance_uri}]") unless instance

  log(:info, "Retrieved configuration: [#{class_name}/#{instance_name}[#{instance}]")
  instance[instance_name]
end


begin
  log(:info, "Starting Method")

  object_type = $evm.root['vmdb_object_type']

  case object_type
    when 'service_template_provision_task'
      task = $evm.root[object_type]
      storage_job_id = task.options[:storage_job_id]
      datacenter = task.options[:datacenter]
    when 'service'
      svc = $evm.root[object_type]
      storage_object_id = svc.tags(category='intuit_storage_object_id').first

      config = retrieve_configuration('intuit_ss', storage_object_id)

      storage_job_id = config['storage_job_id']
      datacenter = config['datacenter']
  end

  log(:info, "Got Storage Job ID [#{storage_job_id}] and datacenter [#{datacenter}]")

  config = retrieve_configuration('intuit_dc', datacenter)

  log(:info, "Storage Server Configuration: #{config}")
  storage_server = config['storage_server']
  user_key = config['user_key']

  status_url = "http://#{storage_server}/api/storage/v1/status/jobid/#{storage_job_id}"

  log(:info, "Using url #{status_url}")
  log(:info, "Using header userkey: #{user_key}")

  response = RestClient::Request.execute(
      :method => :get,
      :url => status_url,
      :headers => {
          :userkey => user_key,
          :content_type => :json
      })

  log(:info, "Returned response: #{response.inspect}")
  response_obj = JSON.parse(response)

  if response_obj['Response'].include?('Completed')
    storage_object_id = response_obj['ObjectId']

    case object_type
      when 'service_template_provision_task'
        task.set_option(:storage_object_id, storage_object_id)
    end
  elsif response_obj['Response'].include?('failed')
    error("Storage job failed.")
  else
    log(:info, "Waiting for Storage Complete")
    retry_method("Waiting for Storage Complete")
  end

  log(:info, "Ending Method")
  exit MIQ_OK
rescue StandardError, SignalException => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
