###################################
#
# CFME Automate Method: file_exists_in_guest
#
# Author: Jason Dillaman
#
# Notes: Use VIX to verify the existance of a file
#
# Inputs: vm: the target VM
#         vmrun_path: path to the vmrun program within the CFME appliance
#         vmrun_timeout: timeout (in seconds) for vmrun to complete
#         vix_guest_file: Full path to file to check
#
# Mods:
#
###################################

# dump_current
def dump_current()
  $evm.log(:info, "Current:<$evm.current> Begin $evm.current.attributes")
  $evm.current.attributes.sort.each { |k, v| $evm.log(:info, "Current:<$evm.current> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Current:<$evm.current> End $evm.current.attributes")
  $evm.log(:info, "")
end

# dump_parent
def dump_parent()
  if $evm.parent
    $evm.log(:info, "Parent:<$evm.parent> Begin $evm.parent.attributes")
    $evm.parent.attributes.sort.each { |k, v| $evm.log(:info, "Parent:<$evm.parent> Attribute - #{k}: #{v}")}
    $evm.log(:info, "Parent:<$evm.parent> End $evm.parent.attributes")
    $evm.log(:info, "")
  end
end

def error(msg)
  $evm.log(:error, "VIX execution failed: [#{msg}]")
  $evm.current['result'] = 'error'
  $evm.current['reason'] = msg.to_s
  exit MIQ_OK
end

def retry_method(msg)
  $evm.log(:warn, "VIX execution needs to retry: [#{msg}]")
  $evm.current['result'] = 'retry'
  $evm.current['reason'] = msg.to_s
  exit MIQ_OK
end

def escape_uri(value)
  URI.escape(value, Regexp.new("[^#{URI::PATTERN::UNRESERVED}]"))
end

def instantiate(uri)
  $evm.log(:info, "Instantiating: [#{uri}]")
  
  instance = $evm.instantiate(uri)
  error("Failed to locate automation instance") unless instance
  
  case instance['result']
  when 'error'
    error(instance['reason'])
  when 'retry'
    retry_method(instance['reason'])
  end
  
  instance
end

def copy_file_from_host_to_guest(vmrun_timeout, vix_platform, vix_local_file, vix_remote_file)
  vmrun_command    = 'copyFileFromHostToGuest'
  vmrun_arguments  = " \"#{vix_local_file.gsub('\\', '\\\\')}\" \"#{vix_remote_file.gsub('\\', '\\\\')}\""

  $evm.log(:info, "Executing vmrun: [#{vmrun_command}][#{vmrun_arguments}]")
  instance = instantiate(
    "#{$evm.current_namespace}/#{$evm.current_class}/ExecuteVmrun?" \
    "vix_platform=#{vix_platform}&" \
    "vmrun_timeout=#{vmrun_timeout}&" \
    "vmrun_command=#{vmrun_command}&" \
    "vmrun_arguments=#{escape_uri(vmrun_arguments)}")
    error("VIX reported failure: [#{instance['exit_status']}]") unless instance['exit_status'] == 0
    return instance
end

begin
  dump_current()
  dump_parent()
  
  vmrun_timeout  = $evm.current['vmrun_timeout'].to_i || 60
  vix_platform   = $evm.current['vix_platform']
  vix_local_file = $evm.current['vix_local_file'] || ($evm.parent ? $evm.parent['vix_local_file'] : nil)
  vix_remote_file = $evm.current['vix_remote_file'] || ($evm.parent ? $evm.parent['vix_remote_file'] : nil)
  
  error('Missing VIX local file') if vix_local_file.blank?
  error('Missing VIX remote file') if vix_remote_file.blank?
  
  instance = copy_file_from_host_to_guest(vmrun_timeout, vix_platform, vix_local_file, vix_remote_file)
  $evm.log(:info, "Successfully copied file from host to guest: [#{vix_local_file}][#{vix_remote_file}]")
  
  $evm.current['exit_status'] = instance['exit_status']
  $evm.current['result'] = 'ok'
  $evm.current['reason'] = ''

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
