###################################
#
# CFME Automate Method: execute_vmrun
#
# Author: Jason Dillaman
#
# Notes: Use VIX vmrun to manipulate a guest VM
#
# Inputs: vm: the target VM
#         vmrun_path: path to the vmrun program within the CFME appliance
#         vmrun_timeout: timeout (in seconds) for vmrun to complete
#         vmrun_command: the vmrun command to execute (i.e. runProgramInGuest)
#         vmrun_arguments: the arguments to the vmrun command
#         vix_enabled: whether or not VIX support is enabled
#         vix_guest_username: VM guest username
#         vix_guest_password: VM guest password
#
# Changes:
#   2015-12-08 Daniel Trieu
#
###################################

require 'linux_admin'
require 'timeout'

# dump_current
def dump_current()
  $evm.log(:info, "Current:<$evm.current> Begin $evm.current.attributes")
  $evm.current.attributes.sort.each { |k, v| $evm.log(:info, "Current:<$evm.current> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Current:<$evm.current> End $evm.current.attributes")
  $evm.log(:info, "")
end

def error(msg)
  $evm.log(:error, "VIX vmrun execution failed: [#{msg}]")
  $evm.current['result'] = 'error'
  $evm.current['reason'] = msg.to_s
  exit MIQ_OK
end

def retry_method(msg)
  $evm.log(:warn, "VIX vmrun needs to retry: [#{msg}]")
  $evm.current['result'] = 'retry'
  $evm.current['reason'] = msg.to_s
  exit MIQ_OK
end

def get_vm()
  return $evm.root['vm'] unless $evm.root['vm'].blank?
  case $evm.root['vmdb_object_type']
    when 'miq_provision'
      vm = $evm.root['miq_provision'].vm
    when 'vm'
      vm = $evm.root['vm']
    when 'service'
      if $evm.root['current_storage_vm']
        vm_name = $evm.root['current_storage_vm']
        vm = $evm.vmdb('vm').find_by_name(vm_name)
      end
    when 'automation_task'
      if $evm.root['automation_task']
        task = $evm.root['automation_task']
        vm_name = task.options[:attrs][:vm_name]
        vm = $evm.vmdb('vm').find_by_name(vm_name)
      end
  end
  raise "VM not found" unless vm
  vm
end

def get_vm_vix_path(vm)
  storage = vm.storage
  location = vm.location

  # Possible that VMware will fail to return the storage name,
  # so CFME munges the location
  primary_storage = vm.wrap_results(vm.object_send(:storages)).first
  storage = primary_storage.name if storage.blank? && primary_storage

  if location =~ /file:\/\/\/\[\] /
    location = location.split('/')[-2..-1].join('/')
  end
  "[#{storage}] #{location}"
end

def run_linux_admin(cmd, sanatized_cmd = nil)
  $evm.log(:info, "Executing #{sanatized_cmd || cmd}")
  result = LinuxAdmin.run(cmd)
  $evm.log(:info, "Inspecting output: #{result.output.inspect}")
  $evm.log(:info, "Inspecting error: #{result.error.inspect}")
  $evm.log(:info, "Inspecting exit_status: #{result.exit_status.inspect}")
  retry_method(result.output) if [
      "Error: The VMware Tools are not running in the virtual machine",
      "Error: Invalid user name or password for the guest OS",
      "Error: Authentication failure or insufficient permissions in guest operating system\n",
      "Insufficient permissions in the host operating system"].any? { |e| result.output.to_s.match(e) }
  return result
end

def run_vmrun(vm, vmrun_path, vmrun_command, vmrun_arguments, guest_username, guest_password)
  ems = vm.ext_management_system
  cmd  = "#{vmrun_path}"
  cmd += " -h https://#{ems.ipaddress}:443/sdk"
  cmd += " -u \"#{ems.authentication_userid}\""
  cmd += " -p \"#{ems.authentication_password}\""
  cmd += " -gu \"#{guest_username}\""
  cmd += " -gp \'#{guest_password}\'"
  #cmd += " -gp \"#{guest_password}\""
  cmd += " #{vmrun_command}"
  cmd += " \"#{get_vm_vix_path(vm)}\""
  cmd += " #{vmrun_arguments}"
  sanatized_cmd = cmd.gsub(ems.authentication_password, '******').gsub(guest_password, '******')


  run_linux_admin(cmd, sanatized_cmd)
end

begin
  dump_current()

  vm = get_vm()

  vmrun_path         = $evm.current['vmrun_path']
  vmrun_timeout      = $evm.current['vmrun_timeout'].to_i || 60
  vmrun_command      = $evm.current['vmrun_command']
  vmrun_arguments    = $evm.current['vmrun_arguments']
  vix_platform       = $evm.current['vix_platform']
  vix_enabled        = $evm.current['vix_enabled'] || true
  vix_guest_username = $evm.current['vix_guest_username']
  vix_guest_password = $evm.current.decrypt('vix_guest_password')

  unless vix_enabled.to_s == 'true'
    $evm.log(:info, "VIX disabled for platform: [#{vix_platform || 'unknown'}]")
    exit MIQ_OK
  end

  error('Missing VIX guest username') if vix_guest_username.blank?
  error('Missing VIX guest password') if vix_guest_password.blank?
  error('Missing vmrun command') if vmrun_command.blank?
  error('Missing vmrun arguments') if vmrun_arguments.blank?

  result = nil
  begin
    Timeout.timeout(vmrun_timeout) do
      result = run_vmrun(vm, vmrun_path, vmrun_command, vmrun_arguments, vix_guest_username, vix_guest_password)
    end
  rescue LinuxAdmin::CredentialError => cerr
    error("CredentialError, Failed to execute vmrun: [#{cerr}]")
  rescue Timeout::Error
    retry_method("Timed out waiting for vmrun to complete")
  end

  $evm.current['result'] = 'ok'
  $evm.current['exit_status'] = result.exit_status

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
