###################################
#
# CFME Automate Method: run_program_in_guest
#
# Author: Jason Dillaman
#
# Notes: Use VIX to run a program within the guest VM
#
# Inputs: vm: the target VM
#         vmrun_timeout: timeout (in seconds) for vmrun to complete
#         vix_guest_program: Full path to program
#         vix_guest_options: Optional program arguments
#
# Mods:
#
###################################

# dump_current
def dump_current()
  $evm.log(:info, "Current:<$evm.current> Begin $evm.current.attributes")
  $evm.current.attributes.sort.each { |k, v| $evm.log(:info, "Current:<$evm.current> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Current:<$evm.current> End $evm.current.attributes")
  $evm.log(:info, "")
end

# dump_parent
def dump_parent()
  if $evm.parent
    $evm.log(:info, "Parent:<$evm.parent> Begin $evm.parent.attributes")
    $evm.parent.attributes.sort.each { |k, v| $evm.log(:info, "Parent:<$evm.parent> Attribute - #{k}: #{v}")}
    $evm.log(:info, "Parent:<$evm.parent> End $evm.parent.attributes")
    $evm.log(:info, "")
  end
end

def error(msg)
  $evm.log(:error, "VIX execution failed: [#{msg}]")
  $evm.current['result'] = 'error'
  $evm.current['reason'] = msg.to_s
  exit MIQ_OK
end

def retry_method(msg)
  $evm.log(:warn, "VIX execution needs to retry: [#{msg}]")
  $evm.current['result'] = 'retry'
  $evm.current['reason'] = msg.to_s
  exit MIQ_OK
end

def escape_uri(value)
  URI.escape(value, Regexp.new("[^#{URI::PATTERN::UNRESERVED}]"))
end

def instantiate(uri)
  $evm.log(:info, "Instantiating: [#{uri}]")
  
  instance = $evm.instantiate(uri)
  error("Failed to locate automation instance") unless instance
  
  case instance['result']
  when 'error'
    error(instance['reason'])
  when 'retry'
    retry_method(instance['reason'])
  end
  
  instance
end

def file_exists_in_guest(vix_platform, vix_guest_program)
  $evm.log(:info, "Executing FileExistsInGuest: [#{vix_guest_program}]")
  instance = instantiate(
    "#{$evm.current_namespace}/#{$evm.current_class}/FileExistsInGuest?" \
    "vix_platform=#{vix_platform}&" \
    "vix_guest_file=#{escape_uri(vix_guest_program)}")
  return instance['exit_status']
end

def run_program_in_guest(vmrun_timeout, vix_platform, vix_guest_program, vix_guest_options)
  vmrun_command    = 'runProgramInGuest'
  vmrun_arguments  = " \"#{vix_guest_program.gsub('\\', '\\\\')}\""
  vmrun_arguments += " #{vix_guest_options}" unless vix_guest_options.blank?

  $evm.log(:info, "Executing vmrun: [#{vmrun_command}][#{vmrun_arguments}]")
  instance = instantiate(
    "#{$evm.current_namespace}/#{$evm.current_class}/ExecuteVmrun?" \
    "vix_platform=#{vix_platform}&" \
    "vmrun_timeout=#{vmrun_timeout}&" \
    "vmrun_command=#{vmrun_command}&" \
    "vmrun_arguments=#{escape_uri(vmrun_arguments)}")
    error("VIX reported failure: [#{instance['exit_status']}]") unless instance['exit_status'] == 0
    return instance
end

def set_execute_flag(vmrun_timeout, vix_platform, vix_guest_program)
  attributes = $evm.instance_get("#{$evm.current_namespace}/Configuration/VixPlatform/#{vix_platform}")
  error("Unable to retrieve required platform configuration: [#{vix_platform}]") unless attributes
  
  vix_set_execute_flag = attributes['vix_set_execute_flag']
  unless vix_set_execute_flag.blank?
    $evm.log(:info, "Ensuring that program is executable: #{vix_guest_program}")
    set_execute_program_args = vix_set_execute_flag.split(' ')
    set_execute_program = set_execute_program_args.shift
    vix_guest_options  = "#{set_execute_program_args.join(' ')} \"#{vix_guest_program.gsub('\\', '\\\\')}\""
  
    run_program_in_guest(vmrun_timeout, vix_platform, set_execute_program, vix_guest_options)
  end
end

begin
  dump_current()
  dump_parent()
  
  vmrun_timeout      = $evm.current['vmrun_timeout'].to_i || 60
  vix_platform       = $evm.current['vix_platform']
  vix_guest_program  = $evm.current['vix_guest_program'] || ($evm.parent ? $evm.parent['vix_guest_program'] : nil)
  vix_guest_options  = $evm.current['vix_guest_options'] || ($evm.parent ? $evm.parent['vix_guest_options'] : nil)
  
  error('Missing VIX guest program') if vix_guest_program.blank?
  
  file_exists = file_exists_in_guest(vix_platform, vix_guest_program)
  error("Guest program does not exist within VM") unless file_exists
  
  set_execute_flag(vmrun_timeout, vix_platform, vix_guest_program)
  instance = run_program_in_guest(vmrun_timeout, vix_platform, vix_guest_program, vix_guest_options)
  
  $evm.current['exit_status'] = instance['exit_status']
  $evm.current['result'] = 'ok'
  $evm.current['reason'] = ''

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
