##################################
#
# CFME Automate Method: build_provision_request
#
# Author: Kevin Morey, Jason Dillaman
#
# Notes: Simple method to kick off an provisioning request from a service dialog or an automation request via API call
#
# Inputs: dialog_option_[0-9]_template, dialog_option_[0-9]_sizing, dialog_tag_[0-9]_environment, etc...
#
# Mods:
# Wayne Chatham, Changed options_hash[:vm_auto_start] to false
#
###################################

PHAROS_SOFTWARE_COMPONENTS_CONFIG_URI = 'Intuit/Integration/PharosSoftwareComponents/Configuration'
INTUIT_SERVER_TYPE_URI = "#{PHAROS_SOFTWARE_COMPONENTS_CONFIG_URI}/intuit_server_type"
INTUIT_SOFTWARE_COMPONENT_URI = "#{PHAROS_SOFTWARE_COMPONENTS_CONFIG_URI}/intuit_software_component"
INTUIT_TEMPLATE_URI = "#{PHAROS_SOFTWARE_COMPONENTS_CONFIG_URI}/intuit_template"
GET_SERVICE_NOW_APPLICATION_LIST_URI = '/Intuit/Integration/ServiceNow/Actions/GetApplicationList'
GET_SERVICE_NOW_ENVIRONMENT_LIST_URI = '/Intuit/Integration/ServiceNow/Actions/GetEnvironmentList'
GET_SERVICE_NOW_EXECUTION_SUBZONE_LIST_URI = '/Intuit/Integration/ServiceNow/Actions/GetExecutionSubzoneList'
GET_CURRENT_INTUIT_DC_URI = 'Intuit/Integration/CloudForms/Actions/GetCurrentIntuitDc'

# logging convenience method
def log(level, msg)
  @method = 'build_provision_request'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end


def dump_root()
  log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  log(:info, "Root:<$evm.root> End $evm.root.attributes")
  log(:info, "")
end

# Look for service dialog variables in the dialog options hash that start with "dialog_tag_[0-9]",
def get_tags_hash(dialog_options)
  # Setup regular expression for service dialog tags
  tags_regex = /^dialog_tag_(\d+)_(.+)$/
  tags_hash = {}
  # Loop through all of the tags and build an options_hash from them
  dialog_options.each do |k, v|
    if tags_regex =~ k
      build = $1.to_i
      tag_category = $2.to_sym
      tag_value = v.downcase
      log(:info, "build:<#{build.inspect}> tag_category:<#{tag_category.inspect}> tag_value:<#{tag_value.inspect}>")
      unless tag_value.blank?
        (tags_hash[build] ||={})[tag_category] = tag_value
      end
    end
  end
  log(:info, "Inspecting tags_hash:<#{tags_hash.inspect}>")
  return tags_hash
end

# Look for service dialog variables in the dialog options hash that start with "dialog_option_[0-9]",
def get_options_hash(dialog_options)
  options_hash = {}
  # Loop through all of the options and build an options_hash from them
  dialog_options_regex = /^dialog_option_(\d+)_(.+)$/
  dialog_options.each do |k, v|
    if dialog_options_regex =~ k
      build = $1.to_i
      option_key = $2.to_sym
      option_value = v
      unless option_value.blank?
        log(:info, "Adding option_key:<#{option_key.inspect}> option_value:<#{option_value.inspect}> to options_hash")
        (options_hash[build] ||={})[option_key] = option_value
      end
    end
  end
  log(:info, "Inspecting options_hash:<#{options_hash.inspect}>")
  return options_hash
end

def parse_security_zone(options_hash, tags_hash)
  # lvdca_preprod_fd1
  security_zone_regex = /^(\w+)_(\w+)_(\w+)_(\w+)$/i
  if options_hash[:security_zone] =~ security_zone_regex
    environment_tags = {}
    environment_tags[:intuit_hs] = $1
    environment_tags[:intuit_sz] = $2
    environment_tags[:intuit_security_level] = $3
    environment_tags[:intuit_fd] = $4
    environment_tags.each { |k, v| log(:info, "Adding category:<#{k}> tag:<#{v}> to tags_hash") }
    tags_hash.merge!(environment_tags)
  else
    error("Invalid security zone: [#{options_hash[:security_zone]}]")
  end
end

# Loop through all tags from the dialog and create the categories and tags automatically
def process_tags(category, single_value, tag)
  # Convert to lower case and replace all non-word characters with underscores
  category_name = category.to_s.downcase.gsub(/\W/,'_')
  tag_name = tag.downcase.gsub(/\W/,'_')
  log(:info, "Converted category name:<#{category_name}> Converted tag name: <#{tag_name}>")

  # if the category exists else create it
  unless $evm.execute('category_exists?', category_name)
    log(:info, "Category <#{category_name}> doesn't exist, creating category")
    $evm.execute('category_create', :name => category_name, :single_value => single_value, :description => "#{category}")
  end
  # if the tag exists else create it
  unless $evm.execute('tag_exists?', category_name, tag_name)
    log(:info, "Adding new tag <#{tag_name}> in Category <#{category_name}>")
    $evm.execute('tag_create', category_name, :name => tag_name, :description => "#{tag}")
  end
end

# name_service - name the service to avoid duplicate names
def name_service(service, new_service_name, new_service_description)
  unless new_service_name.blank?
    log(:info, "Changing Service name:<#{service.name}> to <#{new_service_name}>")
  else
    new_service_name = "#{service.name}-#{Time.now.strftime('%Y%m%d-%H%M%S')}"
    log(:info, "Changing Service name:<#{service.name}> to <#{new_service_name}>")
  end
  service.name = new_service_name
  service.description = new_service_description
end

def get_intuit_template(intuit_template)
  template = $evm.instantiate("#{INTUIT_TEMPLATE_URI}/#{intuit_template}")
  error("Unable to locate template: [#{intuit_template}]") unless template
  log(:info, "Retrieved template configuration: [#{intuit_template}][#{template.attributes}]")
  template.attributes
end

# get template based on incoming dialog_option_[1-9]_[template|name]
def get_template(options_hash, tags_hash)
  intuit_template_tag = tags_hash[:intuit_template] || options_hash[:name]
  log(:info, "intuit_template: #{intuit_template_tag}")

  intuit_template = get_intuit_template(intuit_template_tag)
  image_name = intuit_template['image_name']
  customization_spec = intuit_template['customization_spec']

  templates = $evm.vmdb('miq_template').all.select do |t|
    $evm.log(:info, "Checking template #{t.name}")

    next unless t.name.match(/^#{Regexp.escape(image_name)}.*$/)
    next unless t.ext_management_system
    next unless t.ext_management_system.tagged_with?('intuit_hs', tags_hash[:intuit_hs])
    $evm.log(:info, "Passed check for intuit_hs #{tags_hash[:intuit_hs]}")
    if ["lvdca","qdca","qdcc"].include? tags_hash[:intuit_hs]
      $evm.log(:info, "Checking for intuit_sz #{tags_hash[:intuit_sz]}")
      next unless t.ext_management_system.tagged_with?('intuit_sz', tags_hash[:intuit_sz])
      $evm.log(:info, "Passed intuit_sz check")
    end
    next unless !t.archived && !t.orphaned && !t.retired

    valid_template = true
    if t.tags('prov_scope').first != 'all'
      log(:warn, "Template missing provisioning scope tag, skipping: [#{t.name}]")
      valid_template = false
    elsif t.tags('intuit_hpsa_agent_url').blank? || t.tags('intuit_hpsa_baseline').blank?
      log(:warn, "Template missing HPSA tags, skipping: [#{t.name}]")
      valid_template = false
    end
    valid_template
  end

  # if no templates were found
  error("Unable to locate specified template image: [#{image_name}][#{tags_hash[:intuit_hs]}]") if templates.blank?
  log(:info, "Number of templates found: #{templates.count}")

  # get the first template in the list
  template = templates.first
  options_hash[:template] = template.name
  options_hash[:template_ems_guid] = template.uid_ems
  options_hash[:template_image_name] = image_name
  options_hash[:customization_spec] = customization_spec
  log(:info, "Found template: #{template.name}")
  return template
end

def get_intuit_software_component(intuit_software_component_tag)
  intuit_software_component = $evm.instantiate("#{INTUIT_SOFTWARE_COMPONENT_URI}/#{intuit_software_component_tag}")
  error("Unable to locate software component: [#{intuit_software_component_tag}]") unless intuit_software_component
  log(:info, "Retrieved software component configuration: [#{intuit_software_component_tag}][#{intuit_software_component.attributes}]")
  intuit_software_component.attributes
end

def get_intuit_software_components(req_intuit_software_component_tags, opt_intuit_software_component_tags)
  log(:info, "Search for software components: [#{req_intuit_software_component_tags}][#{opt_intuit_software_component_tags}]")
  (req_intuit_software_component_tags + opt_intuit_software_component_tags).collect do |intuit_software_component_tag|
    get_intuit_software_component(intuit_software_component_tag)
  end
end

def get_intuit_server_type(intuit_server_type_tag)
  intuit_server_type = $evm.instantiate("#{INTUIT_SERVER_TYPE_URI}/#{intuit_server_type_tag}")
  error("Unable to locate server type: [#{intuit_server_type_tag}]") unless intuit_server_type
  log(:info, "Retrieved server type configuration: [#{intuit_server_type_tag}][#{intuit_server_type.attributes}]")
  intuit_server_type.attributes
end

def get_server_type(options_hash, tags_hash)
  intuit_template_tag = tags_hash[:intuit_template]
  intuit_template = get_intuit_template(intuit_template_tag)

  software_components = options_hash[:software_components]
  intuit_software_components = get_intuit_software_components(
      intuit_template['required_intuit_software_components'] || [],
      (software_components && software_components.split('|')) || [])
  intuit_server_type_tags = intuit_software_components.collect do |intuit_software_component|
    next if intuit_software_component['intuit_server_type'].blank?
    intuit_software_component['intuit_server_type']
  end.compact
  intuit_server_type_tags << 'os' if intuit_server_type_tags.blank?

  intuit_server_type_tag = intuit_server_type_tags.sort.join('_')

  intuit_server_type = get_intuit_server_type(intuit_server_type_tag)
  intuit_server_type['server_type']
end

# use this to name the vm or set it to changeme to run through the vm_name automate method
def get_vm_name(options_hash, tags_hash)
  if options_hash[:vm_name].nil?
    # payload
    vm_name = 'p'

    # security zone
    case tags_hash[:intuit_sz]
      when 'preprod'
        vm_name += 'prf'
      when 'dev'
        vm_name += 'dev'
      else
        vm_name += 'prd'
    end

    vm_name += options_hash[:project_acronym].strip().downcase()[0..2]
    #vm_name += get_server_type(options_hash, tags_hash)[0..1]
    log(:info, "Options Hash: #{options_hash.inspect}")
    vm_name += get_intuit_server_type(options_hash[:intuit_software_component])['server_type']

    case tags_hash[:intuit_hs]
      when "qdca"
        vm_name += '1'
      when "qdcb"
        vm_name += '2'
      when "qdcc"
        vm_name += '3'
      when "lvdca"
        vm_name += '4'
      when "qdcd"
        vm_name += '5'
      when "qdce"
        vm_name += '6'
      when "lvdcb"
        vm_name += '7'
      when 'qcyf01'
        vm_name += '8'
      when 'lasf01'
        vm_name += '9'
      when 'edca'
        vm_name += 'a'
      when 'sdca'
        vm_name += 'b'
    end
    #if tags_hash[:intuit_hs] =~ /lvdc/i
    #  vm_name += '4'
    #else
    #  vm_name += '6'
    #end

    vm_name += '$n{2}'

    options_hash[:vm_name] = vm_name
  end
end

# use this to determine the provision_type. Valid types are vmware, pxe
def get_provision_type(options_hash)
  options_hash[:provision_type] = 'vmware'
end

def count_allocated_cpu_in_host(host)
  #vms = host.vms.find_all
  vms = host.vms
  cpu_count = 0

  vms.each do |vm|
    cpu_count += vm.num_cpu.to_i
  end

  return cpu_count
end

# In the host, find all the VMs and sum up the allocated memory.
def sum_allocated_mem_in_host(host)
  #vms = host.vms.find_all
  vms = host.vms
  mem_sum = 0

  vms.each do |vm|
    mem_sum += vm.mem_cpu.to_i
  end

  return mem_sum
end

# get target cluster
def get_cluster(options_hash, tags_hash, template)
  security_zone = tags_hash[:intuit_sz]
  security_level = tags_hash[:intuit_security_level]
  fault_domain = tags_hash[:intuit_fd]

  requested_cpus = options_hash[:number_of_sockets]*options_hash[:cores_per_socket]
  requested_mem = options_hash[:vm_memory]

  ems = template.ext_management_system

  host_name = options_hash[:placement_host_name]

  log(:info, "placement_host_name: #{host_name}")



  clusters = ems.ems_clusters.select do |cluster|

    total_cpu_count = 0
    total_mem_count = 0
    requested_cpu_total = 1
    requested_mem_total = 1
    cluster_aggregate_mem = 1
    cluster_aggregate_cpu = 1

    #oversub_cpu_ratio = cluster.tagged_with?('intuit_cpu_oversub', (1..10).to_a) ? cluster.tags(category='intuit_cpu_oversub').first.to_i : 1
    #oversub_mem_ratio = cluster.tagged_with?('intuit_mem_oversub', (1..10).to_a) ? cluster.tags(category='intuit_mem_oversub').first.to_i : 1
    oversub_cpu_ratio = cluster.tags(category='intuit_cpu_oversub').first
    oversub_mem_ratio = cluster.tags(category='intuit_mem_oversub').first
    log(:info, "CPU before nil check: #{oversub_cpu_ratio}")
    log(:info, "MEM before nil check: #{oversub_mem_ratio}")
    oversub_cpu_ratio = 1 if oversub_cpu_ratio.nil?
    oversub_mem_ratio = 1 if oversub_mem_ratio.nil?

    if cluster.tagged_with?('intuit_ratio_scope', 'cluster')
      cluster.hosts.each {|host|
        total_cpu_count += count_allocated_cpu_in_host(host)
        total_mem_count += sum_allocated_mem_in_host(host)
      }

      cluster_aggregate_cpu = cluster.aggregate_logical_cpus.to_i
      cluster_aggregate_mem = cluster.aggregate_memory.to_i

      requested_cpu_total = total_cpu_count + requested_cpus
      requested_mem_total = total_mem_count + requested_mem

    end
    log(:info, "Cluster name: #{cluster.name}")
    log(:info, "CPU Ratio Limit: #{oversub_cpu_ratio}")
    log(:info, "MEM Ratio Limit: #{oversub_mem_ratio}")
    log(:info, "CPU Total After Request: #{requested_cpu_total}")
    log(:info, "MEM Total After Request: #{requested_mem_total}")
    log(:info, "CPU Ratio After Request: #{(requested_cpu_total.fdiv(cluster_aggregate_cpu))}")
    log(:info, "MEM Ratio After Request: #{(requested_mem_total.fdiv(cluster_aggregate_mem))}")
    log(:info, "Math check: #{((requested_cpu_total.fdiv(cluster_aggregate_cpu)) <= oversub_cpu_ratio.to_i)}")
    result = cluster.tagged_with?('prov_scope', 'all') &&
        cluster.tagged_with?('intuit_sz', security_zone) &&
        cluster.tagged_with?('intuit_security_level', security_level) &&
        cluster.tagged_with?('intuit_fd', fault_domain) &&
        ((requested_cpu_total.fdiv(cluster_aggregate_cpu)) <= oversub_cpu_ratio.to_i) &&
        ((requested_mem_total.fdiv(cluster_aggregate_mem)) <= oversub_mem_ratio.to_i)

    if result and not host_name.blank? and not cluster.hosts.detect { |h| h.name == host_name }
      log(:info, "Placement host #{host_name} not found, skipping cluster #{cluster.name}")
      result = false
    end

    result
  end.sort_by { |cluster| cluster.vms.count }
  log(:info, "Found #{clusters.count} Clusters")

  error("No target cluster available") if clusters.empty?

  cluster = clusters.first
  error("Unable to find suitable cluster") if cluster.nil?

  options_hash[:throttle_provision] = cluster.tags(category='intuit_throttle').first.to_s

  options_hash[:throttle_provision] = 'f' if cluster.tags(category='intuit_throttle').nil?

  log(:info, "Using Cluster: #{cluster.name}")
  cluster.id
end

# use this method to define extra provisioning options
def get_extra_options(options_hash, tags_hash, service)
  # Set the VM to autostart after provisioning
  options_hash[:vm_auto_start] = 'false'

  # Force BestFit to select the best host and storage
  options_hash[:placement_auto] = 'true'

  # Force thick provisioned disks
  options_hash[:disk_format] = 'thick'

  # VM ownership
  user = $evm.root['user']
  miq_group = user.miq_group
  options_hash[:owner_userid] = user.userid
  options_hash[:owner_group] = miq_group.description

  # BU tag

  ##Below is how we were getting BU tags, now we're getting them from the service_validate method

  #if tags_hash[:intuit_bu].nil?
  #I found this next code was what was causing the single letter value associated with bu
  #bu = miq_group.tags(:intuit_bu).first.first rescue nil
  #bu = miq_group.tags(:intuit_bu).first rescue nil
  #bu = $evm.root[:intuit_bu]
  #unless bu.blank?
  #  tags_hash[:intuit_bu] = bu
  #  log(:info, "Adding tag: intuit_bu/#{bu} from group: #{miq_group.description} to tags_hash")
  #end
  # end

  # service name
  options_hash[:service_name] = service.name if service

  # stuff the parent service id so that the VMs can be added to the service later
  unless $evm.root['service'].nil?
    # executed via service button
    service = $evm.root['service']
    options_hash[:service_id] = service.id
  else
    # executed via service catalog
    service = $evm.root['service_template_provision_task'].destination
    options_hash[:service_id] = service.id
  end
end

# use this to define sizing parameters
def get_sizing_parms(options_hash, tags_hash)
  unless tags_hash[:intuit_instance_size].nil?
    options = $evm.instantiate("/Intuit/Provisioning/Configuration/intuit_instance_size/#{tags_hash[:intuit_instance_size]}")
    error("Unable to locate instance size: [#{tags_hash[:intuit_instance_size]}]") if options.blank?
    log(:info, "Retrieved VM sizing parameters: [#{options.attributes}]")
    [:number_of_sockets, :cores_per_socket, :vm_memory].each { |x| options_hash[x] = options[x.to_s] unless options[x.to_s].blank? }
  end
end

# Populate arguments and submit provision request using incoming tags_hash and options_hash
def build_provision_request(tags_hash, options_hash)
  user       = $evm.root['user']
  user_id    = (user && user.userid) || 'admin'
  user_mail  = (!options_hash[:owner_email].blank? && options_hash[:owner_email]) ||
      (user && user.email) || 'cloud.admin@example.com'
  user_first = (user && user.first_name) || 'cloud'
  user_last  = (user && user.last_name) || 'user'

  request = {}
  request[:template_fields] = "name=#{options_hash[:template]}|ems_guid=#{options_hash[:template_ems_guid]}|request_type=template"

  valid_vm_args = [
      :vm_name, :number_of_sockets, :cores_per_socket, :vm_memory, :number_of_vms, :provision_type, :vm_auto_start,
      :vlan, :retirement, :retirement_warn, :vm_description, :number_of_sockets, :linked_clone,
      :vm_prefix, :network_adapters, :placement_auto, :ip_addr, :disk_format]
  vm_args = options_hash.select { |k, v| valid_vm_args.include? k }.to_a.collect { |v| v.join('=') }.join('|')
  valid_vm_args.each { |k| options_hash.delete(k) }
  request[:vm_fields] = vm_args
  request[:requester] = "user_name=#{user_id}|owner_first_name=#{user_first}|owner_last_name=#{user_last}|owner_email=#{user_mail}"
  request[:tags] = ''  # we cannot directly pass the tags as they might not exist in the local region
  request[:values] =
      (options_hash.collect { |k, v| "#{k.to_s}=#{v}" } + tags_hash.collect { |k, v| "tag_#{k.to_s}=#{v}" }).join('|')
  request[:ems_custom_attributes] = ''
  request[:miq_custom_attributes] = ''

  log(:info, "Building provision request with the following arguments: <#{request.inspect}>")
  return request
end

def parse_delimited_option(option)
  (option || '').split(/[,\n]/).reject { | item | item.empty? }.collect { |x| x.strip }
end

def get_vm_instance_options(options_hash)
  host_names   = parse_delimited_option(options_hash[:host_names])
  ip_addresses = parse_delimited_option(options_hash[:ip_addresses])
  target_hosts = parse_delimited_option(options_hash[:target_hosts])
  mgmt_ip_addresses = parse_delimited_option(options_hash[:mgmt_ip_addresses])
  mgmt_host_names = parse_delimited_option(options_hash[:mgmt_host_names])
  storage_ip_addresses = parse_delimited_option(options_hash[:storage_ip_addresses])
  storage_host_names = parse_delimited_option(options_hash[:storage_host_names])

  vm_instance_options = []
  (0...[host_names, ip_addresses, target_hosts, mgmt_ip_addresses, mgmt_host_names, storage_ip_addresses, storage_host_names].collect { |x| x.length }.max).each do |idx|
    vm_instance_option = {}
    vm_instance_option[:vm_name] = host_names[idx].downcase unless host_names[idx].blank?
    vm_instance_option[:ip_addr] = ip_addresses[idx] unless ip_addresses[idx].blank?
    vm_instance_option[:ip_address] = ip_addresses[idx] unless ip_addresses[idx].blank?
    vm_instance_option[:placement_host_name] = target_hosts[idx] unless target_hosts[idx].blank?
    vm_instance_option[:mgmt_ip_address] = mgmt_ip_addresses[idx] unless mgmt_ip_addresses[idx].blank?
    vm_instance_option[:mgmt_hostname] = mgmt_host_names[idx].downcase unless mgmt_host_names[idx].blank?
    vm_instance_option[:storage_ip_address] = storage_ip_addresses[idx] unless storage_ip_addresses[idx].blank?
    vm_instance_option[:storage_hostname] = storage_host_names[idx].downcase unless storage_host_names[idx].blank?
    vm_instance_options << vm_instance_option
  end
  vm_instance_options
end

def tag_service(options_hash, tags_hash, service)
  bu = tags_hash[:intuit_bu]
  service_hash = {}
  service_hash[:sn_application]     = options_hash[:application_name]
  service_hash[:sn_environment]     = options_hash[:environment]
  service_hash[:sn_execution_zone]  = options_hash[:execution_zone]
  service_hash[:sn_sox_compliant]   = options_hash[:sox_compliant]
  service_hash[:project_acronym]    = options_hash[:project_acronym]
  service_hash[:patch_cycle]        = tags_hash[:intuit_patch_cycle]
  service_hash[:patch_window]       = tags_hash[:intuit_patch_window]
  service_hash[:bu]                 = tags_hash[:intuit_bu]
  service_hash[:filter_bu]          = tags_hash[:intuit_filter_bu] = bu

  service_hash.each do |key, value|
    next if key == 'intuit_filter_bu' && value.blank? || value.nil?
    log(:info, "Tag info key: intuit_#{key} value: #{value}")
    if $evm.execute('tag_exists?', "intuit_#{key}", "#{value}")
      log(:info, "Tag exists")
      service.tag_assign("intuit_#{key}/#{value.downcase}")
    else
      description = value
      value = value.downcase.gsub(/\s/, '_').gsub(/\(/, '1234').gsub(/\)/, '4321').gsub(/\-/, '7890').gsub(/\//, '0987')
      $evm.execute('tag_create', "intuit_#{key}", :name => "#{value}", :description => "#{description}")
      log(:info, "Creating tag intuit_#{key}/#{value.downcase.chomp}")
      tag_response = service.tag_assign("intuit_#{key}/#{value.downcase.chomp}")
      $evm.log(:info, "Tag assign response: #{tag_response.inspect}")
    end
  end
end

require 'uri'

def escape_uri(value)
  URI.escape("#{value}", Regexp.new("[^#{URI::PATTERN::UNRESERVED}]"))
end

def get_intuit_dc()
  intuit_dc = $evm.instantiate(GET_CURRENT_INTUIT_DC_URI)['intuit_dc']
  error("Unable to determine current Intuit data center") if intuit_dc.blank?
  intuit_dc
end

def get_service_now_application_list(intuit_dc)
  result = $evm.instantiate("#{GET_SERVICE_NOW_APPLICATION_LIST_URI}?" \
    "intuit_dc=#{escape_uri(intuit_dc)}");
  result['application_list']
end

def get_service_now_environment_list(intuit_dc)
  result = $evm.instantiate("#{GET_SERVICE_NOW_ENVIRONMENT_LIST_URI}?" \
    "intuit_dc=#{escape_uri(intuit_dc)}");
  result['environment_list']
end

def get_service_now_execution_subzone_list(intuit_dc)
  result = $evm.instantiate("#{GET_SERVICE_NOW_EXECUTION_SUBZONE_LIST_URI}?" \
    "intuit_dc=#{escape_uri(intuit_dc)}");
  result['execution_subzone_list']
end

def clean_up_extra_dialog_options(extra_dialog_options,gsi_object)
  intuit_dc = get_intuit_dc()
  application_list = get_service_now_application_list(intuit_dc)
  execution_zone_list = get_service_now_execution_subzone_list(intuit_dc)
  environment_list = get_service_now_environment_list(intuit_dc)

  #find correct app name
  application_list.each do |app|
    gsi_object.tag_assign("phony_spo_tag/#{app['name']}")
    tag_converted_name = gsi_object.tags(:phony_spo_tag).first
    log(:info, "comparing #{tag_converted_name} and #{extra_dialog_options['dialog_option_0_application_name']}")
    if tag_converted_name.eql? extra_dialog_options['dialog_option_0_application_name']
      extra_dialog_options['dialog_option_0_application_name'] = app['name']
      break
    end
  end

  #find correct execution zone
  execution_zone_list.each do |execution_zone|
    gsi_object.tag_assign("phony_spo_tag/#{execution_zone['label']}")
    tag_converted_name = gsi_object.tags(:phony_spo_tag).first
    if tag_converted_name.eql?extra_dialog_options['dialog_option_0_execution_zone']
      extra_dialog_options['dialog_option_0_execution_zone'] = execution_zone['label']
      break
    end
  end

  #find correct environment
  environment_list.each do |environment|
    gsi_object.tag_assign("phony_spo_tag/#{environment['label']}")
    tag_converted_name = gsi_object.tags(:phony_spo_tag).first
    if tag_converted_name.eql?extra_dialog_options['dialog_option_0_environment']
      extra_dialog_options['dialog_option_0_application_environment'] = environment['name']
      break
    end
  end
  extra_dialog_options
end

def get_extra_gsi_dialog_options()
  gsi_object = $evm.vmdb('service').find_by_id($evm.root['dialog_project_template'])
  tags = gsi_object.tags
  extra_dialog_options = {}
  #convert tag to hash
  tags.each do |tag|
    tag_array = tag.split("/")
    extra_dialog_options['dialog_'+tag_array[0]] = tag_array[1]
  end
  extra_dialog_options = clean_up_extra_dialog_options(extra_dialog_options,gsi_object)
end


begin
  log(:info, 'CFME Automate Method Started')

  dump_root

  case $evm.root['vmdb_object_type']
    when 'service_template_provision_task'
      # executed via service catalog
      task = $evm.root['service_template_provision_task']

      log(:info, "Task Options: #{task.options}")
      bu = task.get_option(:intuit_bu)
      service = task.destination
      log(:info, "Detected Service:<#{service.name}> Id:<#{service.id}>")

      # Name Service
      dialog_service_name = $evm.root['dialog_service_name'] rescue nil
      dialog_service_description = $evm.root['dialog_service_description']
      name_service(service, dialog_service_name, dialog_service_description)
    when 'service'
      service = $evm.root['service']

      dialog_service_name = service.name
      dialog_service_description = service.description
    else
      raise "Invalid $evm.root['vmdb_object_type']:<#{$evm.root['vmdb_object_type']}>"
  end

  # build a hash of dialog_options
  dialog_options = Hash[$evm.root.attributes.sort.collect { |k, v| [k, v] if k.starts_with?('dialog_') }]
  log(:info, "dialog_options: #{dialog_options.inspect}")

  #Check if this was a provision with a project tempalte
  if !$evm.root['dialog_project_template'].blank?
    extra_dialog_options = get_extra_gsi_dialog_options()
    dialog_options.update Hash[extra_dialog_options.collect{|k, v| [k, v] if k.starts_with?('dialog_')}]
    log(:info, "dialog_options (after PT addition): #{dialog_options.inspect}")
  end

  tags_hash = get_tags_hash(dialog_options)
  options_hash = get_options_hash(dialog_options)

  builds = []
  options_hash.each do |build, options|
    next if build == 0
    if options[:number_of_vms].to_i > 0
      builds << build
    else
      log(:info, "Skipping build: #{build} due to options[:number_of_vms]=> #{options[:number_of_vms]}")
    end
  end

  base_options_hash = options_hash[0] || {}
  base_tags_hash = tags_hash[0] || []

  vm_provision_requests = []
  builds.each do |build|
    matching_options_hash = base_options_hash.merge(options_hash[build] || {})
    matching_tags_hash = base_tags_hash.merge(tags_hash[build] || {})

    # The software component field is a dynamic drop-down and for now is
    # hard-coded to the single option ["OS-only", "os_only"]. For some
    # reason, CF sets it to nil instead of "os_only", so we have to work
    # around that here.
    #  if matching_tags_hash[:intuit_software_component].nil?
    #    log(:warn, "No intuit_software_component value supplied, using default <os_only>")
    #    matching_tags_hash[:intuit_software_component] = "os_only"
    #  end

    if matching_options_hash[:intuit_software_component].nil?
      log(:warn, "No intuit_software_component value supplied, using default <os_only>")
      matching_options_hash[:intuit_software_component] = "os"
    end

    matching_tags_hash.each do |category, tag|
      process_tags(category, true, tag)
    end

    # parse the security_zone to get the tags for intuit_hs, intuit_sz, intuit_security_level, intuit_fd
    parse_security_zone(matching_options_hash, matching_tags_hash)

    # Get template
    template = get_template(matching_options_hash, matching_tags_hash)

    # Generate a VM name else set it to changeme
    get_vm_name(matching_options_hash, matching_tags_hash)

    # Set the provision type
    get_provision_type(matching_options_hash)

    # Set vCPU and vRAM based on T-shirt sizing parameters
    get_sizing_parms(matching_options_hash, matching_tags_hash)

    # build any overrides in this section
    get_extra_options(matching_options_hash, matching_tags_hash, service)

    matching_tags_hash[:intuit_bu] = bu

    tag_service(matching_options_hash, matching_tags_hash, service)

    #The :vlan value needs to be populated here, but it is not used to determine the VMs vlan id in the local region
    matching_options_hash[:vlan] = "global"

    # You may hard code any overrides and any additional parameters here
    log(:info, "Build:#{build} - matching_tags_hash #{matching_tags_hash}")
    log(:info, "Build:#{build} - matching_options_hash #{matching_options_hash}")

    vm_instance_options = get_vm_instance_options(matching_options_hash)
    (0...matching_options_hash[:number_of_vms].to_i).each do |vm_idx|
      log(:info, "Build:#{build}, VM:#{vm_idx + 1} - instance_options_hash #{vm_instance_options[vm_idx]}")
      instance_options_hash = matching_options_hash.merge(vm_instance_options[vm_idx] || {})

      # hard code number of VMs to 1 since we issue individual requests for VMs
      instance_options_hash[:number_of_vms] = 1
      # begin
      #   instance_options_hash[:placement_cluster_id] = \
      #     get_cluster(instance_options_hash, matching_tags_hash, template)
      # rescue Exception => err
      #   retry_method(err)
      # end
      # call build_provision_request using matching_tags_hash and instance_options_hash
      request_hash = build_provision_request(matching_tags_hash, instance_options_hash)
      vm_provision_requests << {
          :options => instance_options_hash,
          :tags    => matching_tags_hash,
          :request => request_hash
      }
    end
  end

  log(:info, "VM Provision Requests: [#{vm_provision_requests}]")
  if task.nil?
    $evm.root[:vm_provision_requests] = vm_provision_requests
  else
    task.set_option(:vm_provision_requests, vm_provision_requests)
  end
  log(:info, 'CFME Automate Method Ended')
  exit MIQ_OK

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
