###################################
#
# CFME Automate Method: wait_for_provision_complete
#
# Author: Jason Dillaman
#
# Notes: Retries until all associated VM provision requests have completed (or failed)
#
# Inputs: vm_provision_requests within task
#
# Mods:
#
###################################

# logging convenience method
def log(level, msg)
  @method = 'wait_for_provision_complete'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, msg)
  #$evm.root['ae_result'] = 'retry'
  #$evm.root['ae_retry_interval'] = '5.seconds'
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  log(:info, "Removing errored provision request")
  instance_hash = instance_get('/Customer/Intuit/Provisioning/ServiceMethods/WaitForProvisionComplete')
  log(:info, "Instance Orig: #{instance_hash['vm_provision_requests']}")
  instance_hash['vm_provision_requests'] = nil
  log(:info, "Instance New: #{instance_hash}")
  result = instance_update('/Customer/Intuit/Provisioning/ServiceMethods/WaitForProvisionComplete', instance_hash)
  log(:info, "Instance Result: #{result}")
  exit MIQ_OK
end

def dump_root()
  log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.each { |k| log(:info, "Root:<$evm.root> Attribute - #{k}")}
  log(:info, "Root:<$evm.root> End $evm.root.attributes")
  log(:info, "")
end

def instance_get(path)
  result = $evm.instance_get(path)
  # Returns Hash
  $evm.log('info',"Instance:<#{path}> properties:<#{result.inspect}>") if @debug
  return result
end

def instance_find(path)
  result =   $evm.instance_find(path)
  # Returns Hash
  #$evm.log('info',"Instance:<#{path}> properties:<#{result.inspect}>") if @debug
  return result
end

def instance_update(path, hash)
  result = $evm.instance_update(path, hash)
  if result
    $evm.log('info',"Instance: <#{path}> updated. Result:<#{result.inspect}>") if @debug
  else
    $evm.log('info',"Instance: <#{path}> not updated. Result:<#{result.inspect}>") if @debug
  end
  return result
end

def instance_exists(path)
  result = $evm.instance_exists?(path)
  if result
    $evm.log('info',"Instance:<#{path}> exists. Result:<#{result.inspect}>") if @debug
  else
    $evm.log('info',"Instance:<#{path}> does not exist. Result:<#{result.inspect}>") if @debug
  end
  return result
end

begin
  log(:info, 'CFME Automate Method Started')

  log(:info, "OBJECT TYPE: #{$evm.root['vmdb_object_type']}")

  service = nil
  request_ids = Array.new
  case $evm.root['vmdb_object_type']
    when 'service_template_provision_task'
      # executed via service catalog
      task = $evm.root['service_template_provision_task']
      service = task.destination
      vm_provision_requests = task.options[:vm_provision_requests]
      error("Missing VM provision request") unless vm_provision_requests

      # Clean up any retry status since it implies we timed-out retrying
      vm_provision_requests.each do |vm_provision_request|
        if vm_provision_request[:result] == 'retry'
          log(:warn, "Detected timed out VM provision request: [#{vm_provision_request}]")
          vm_provision_request[:result] = 'error'
          vm_provision_request[:reason] = 'Timed out initiating VM provision request'
        end
      end

      retry_requests = []
      status_messages = ""
      vm_provision_requests.each do |vm_provision_request|
        next unless vm_provision_request[:result] == 'ok'

        log(:info, "Processing VM provision request: [#{vm_provision_request}]")

        request_id = vm_provision_request[:request_id]
        log(:info, "Searching for VM provision request: [#{request_id}]")
        request = $evm.vmdb('miq_provision_request').find_by_id(request_id)
        if request.nil?
          log(:info, "Waiting for VM provision request VMDB entry")
          retry_requests << vm_provision_request
        else
          if request.miq_request_tasks.any? { |request_task| request_task.status == 'retry' }
            # Handle the case where one or more VMs are still being provisioned 
            # We don't want to bubble up an error just because a single VM failed to provision
            #log(:info, "request INSPECT: #{request.miq_request_tasks.inspect}")
            request.miq_request_tasks.each do |r_task|
              log(:info, "Request MESSAGE: #{r_task.message.inspect}")
              status_messages << ": #{r_task.message}"
            end
            log(:info, "Waiting for VM provision request to complete")
            retry_requests << vm_provision_request

          elsif request.status != 'Ok'
            # Handle the case where the provision request failed or was denied
            log(:warn, "VM provision request reported as failed: [#{request.message}] ")
            log(:info, "request INSPECT: #{request.miq_request.inspect}")
            vm_provision_request[:result] = 'error'
            vm_provision_request[:reason] = request.message
            request.miq_request.miq_provisions.each {|provisions|
              log(:info, "provision INSPECT: #{provisions.miq_provision_request.inspect}")
            }
            vm_provision_request[:best_fit_error] = request.miq_request.get_option(:best_fit_error) || {}

          elsif request.state != 'finished'
            # Handle a potential race at provisioning startup
            log(:info, "Waiting for VM provision request to complete")
            retry_requests << vm_provision_request
          end

          # Ensure that all child VMs are properly tagged to the service
          request.miq_request_tasks.each do |request_task|
            next if request_task.destination.nil? || request_task.destination.service
            log(:info, "Added VM to service: [#{request_task.destination.name}]")
            request_task.destination.add_to_service(service) if service
          end
        end
      end

      log(:info, "VM Provision Requests: [#{vm_provision_requests}]")
      task.set_option(:vm_provision_requests, vm_provision_requests)


      error_messages = ""
      error_requests = vm_provision_requests.select { |request| request[:result] != 'ok' }
      error_requests.each {|err_req|
        log(:info, "Error request: #{err_req[:reason]}")
        if err_req[:reason].include? "suitable host"
          err_req[:reason] = err_req[:best_fit_error]
        end
        error_messages << ": #{err_req[:reason]}"
      }

      error_msg = "#{error_requests.count} failed VM provision request#{error_requests.count > 1 ? 's' : ''}#{error_messages}" unless error_requests.blank?
      retry_msg = "#{retry_requests.count} pending VM provision request#{retry_requests.count > 1 ? 's' : ''}#{status_messages}" unless retry_requests.blank?
      reason_msg = [error_msg, retry_msg].compact.join(', ')
      if reason_msg.blank?
        # No errors and no pending requests - we are done.

        $evm.root['ae_result'] = 'ok'
        $evm.root['ae_reason'] = ''
      elsif retry_msg.blank?
        # No pending requests but at least one failed. If there's no successful
        # requests, delete the service.

        if service && error_requests.length == vm_provision_requests.length
          log(:info, "All requests failed, deleting service #{service}")
          service.remove_from_vmdb
        end

        error(error_msg)
      else
        log(:info, reason_msg)
        $evm.root['ae_result'] = 'retry'
        $evm.root['ae_reason'] = reason_msg
        #$evm.root['ae_retry_interval'] = '1.minute'
        #Modifying this to a shorter retry value to see how messages are returned from local region
        $evm.root['ae_retry_interval'] = '15.seconds'
      end
    when 'service'
      service = $evm.root['service']
      vm_provision_requests = $evm.root[:vm_provision_requests]
      if vm_provision_requests.blank?
        log(:info, "Add Vm to service retried")
        instance_hash = instance_get('/Customer/Intuit/Provisioning/ServiceMethods/WaitForProvisionComplete')
        request_id_string= instance_hash['vm_provision_requests']
        request_ids = request_id_string.split(',').reject{|x| x.empty?}
      else
        vm_provision_requests.each {|request|
          request_ids.push request[:request_id]
        }

        log(:info, "Adding #{request_ids} to service object")
        instance_hash = instance_get('/Customer/Intuit/Provisioning/ServiceMethods/WaitForProvisionComplete')
        log(:info, "Instance Orig: #{instance_hash['vm_provision_requests']}")
        instance_hash['vm_provision_requests'] = request_ids.join(",")
        log(:info, "Instance New: #{instance_hash}")
        result = instance_update('/Customer/Intuit/Provisioning/ServiceMethods/WaitForProvisionComplete', instance_hash)
        log(:info, "Instance Result: #{result}")
      end

      log(:info, "Found Provision Request: #{request_ids}")
      # log(:info, "Provision Request Array: #{request_array}")
      retry_requests = []
      status_messages = ""

      request_ids.each do |vm_provision_request|

        log(:info, "Processing VM provision request: [#{vm_provision_request}]")

        request_id = vm_provision_request


        log(:info, "Searching for VM provision request: [#{request_id}]")

        request = $evm.vmdb('miq_provision_request').find_by_id(request_id)
        log(:info, "Found Request #{request.inspect}")
        #next if request.status != 'Ok'
        if request.nil?
          log(:info, "Waiting for VM provision request VMDB entry")
          retry_requests.push vm_provision_request
        else
          log(:info, "Request Status: #{request.status}")
          #if request.miq_request_tasks.any? { |request_task| request_task.status == 'retry' }
          # Handle the case where one or more VMs are still being provisioned 
          # We don't want to bubble up an error just because a single VM failed to provision
          #  log(:info, "request INSPECT: #{request.miq_request_tasks.inspect}")
          #  request.miq_request_tasks.each do |task|
          #    log(:info, "Request MESSAGE: #{task.message.inspect}")
          #    status_messages << ": #{task.message}"
          #  end
          #  log(:info, "Waiting for VM provision request to complete")
          #  retry_requests << vm_provision_request
          #end
          # Ensure that all child VMs are properly tagged to the service
          if request.state == "finished"
            request.miq_request_tasks.each { |request_task|
              log(:info, "Added VM to service: [#{request_task.destination.name}]")
              request_task.destination.add_to_service(service) if service
            }
          else
            log(:info, "Waiting for provision to complete: #{request_id}")
            retry_requests.push request_id
          end
        end
      end

      log(:info, "Retry requests? #{retry_requests.inspect}")
      unless retry_requests.empty?

        $evm.root['ae_result'] = 'retry'
        $evm.root['ae_reason'] = "Waiting for request to complete"
        #$evm.root['ae_retry_interval'] = '1.minute'
        #Modifying this to a shorter retry value to see how messages are returned from local region
        $evm.root['ae_retry_interval'] = '15.seconds'
        log(:info, "Putting only retry requests into Insance")
        instance_hash = instance_get('/Customer/Intuit/Provisioning/ServiceMethods/WaitForProvisionComplete')
        log(:info, "Instance Orig: #{instance_hash['vm_provision_requests']}")
        instance_hash['vm_provision_requests'] = retry_requests.join(',')
        log(:info, "Instance New: #{instance_hash}")
        result = instance_update('/Customer/Intuit/Provisioning/ServiceMethods/WaitForProvisionComplete', instance_hash)
        log(:info, "Instance Result: #{result}")
      else
        instance_hash = instance_get('/Customer/Intuit/Provisioning/ServiceMethods/WaitForProvisionComplete')
        log(:info, "Instance Orig: #{instance_hash['vm_provision_requests']}")
        instance_hash['vm_provision_requests'] = nil
        log(:info, "Instance New: #{instance_hash}")
        result = instance_update('/Customer/Intuit/Provisioning/ServiceMethods/WaitForProvisionComplete', instance_hash)
        log(:info, "Instance Result: #{result}")
      end
    else
      raise "Invalid $evm.root['vmdb_object_type']:<#{$evm.root['vmdb_object_type']}>"
  end



  log(:info, 'CFME Automate Method Ended')
  exit MIQ_OK

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
