###################################
#
# CFME Automate Method: service_request_validation
#
# This method validates the service provision request using the values
# [max_vms, max_cpus, max_memory] from values in the model
#
###################################
begin
  # Method for logging
  def log(level, message, set_message=false)
    @method = 'service_request_validation'
    $evm.log(level, "#{@method} - #{message}")
    $evm.root['miq_request'].set_message(message) if $evm.root['miq_request'] && set_message
  end

  # dump_root
  def dump_root()
    log(:info, "Root: $evm.root Begin $evm.root.attributes")
    $evm.root.attributes.sort.each { |k, v| log(:info, "Root: $evm.root Attribute - #{k}: #{v}")}
    log(:info, "Root: $evm.root End $evm.root.attributes")
    log(:info, "")
  end

  def query_prov_options(parent_service_template, prov_option)
    options_array = []
    return options_array unless parent_service_template.respond_to?(:service_resources)
    parent_service_template.service_resources.each do |child_service_resource|

      # skip catalog item if generic for catalog bundles/items
      if parent_service_template.service_type == 'composite'
        next if child_service_resource.resource.prov_type == 'generic'
        log(:info, "Detected Service Catalog Bundle")
        child_service_resource.resource.service_resources.each do |grandchild_service_template_service_resource|
          log(:info, "Inspecting grandchild_service_template_service_resource.resource: #{grandchild_service_template_service_resource.resource.inspect}\n")
          log(:info, "Retrieving #{prov_option}=>#{grandchild_service_template_service_resource.resource.get_option(prov_option)}")
          options_array << grandchild_service_template_service_resource.resource.get_option(prov_option)
        end
      else
        next if parent_service_template.prov_type == 'generic'
        log(:info, "Detected Service Catalog Item")
        log(:info, "Inspecting child_service_resource.resource: #{child_service_resource.resource.inspect}\n")
        log(:info, "Retrieving #{prov_option}=>#{child_service_resource.resource.get_option(prov_option)}")
        options_array << child_service_resource.resource.get_option(prov_option)
      end

    end # parent_service_template_service_resources.each
    return options_array
  end

  # get_options_hash - Look for service dialog variables in the dialog options hash that start with "dialog_option_[0-9]"
  def get_options_hash(dialog_options)
    # Setup regular expression for service dialog tags
    options_regex = /^dialog_option_(\d*)_(\w*)/i
    options_hash = {}

    # Loop through all of the options and build an options_hash from them
    dialog_options.each do |k,v|
      option_key = k.downcase.to_sym
      if options_regex =~ k
        sequence_id = $2.to_i

        unless v.blank?
          log(:info, "Adding sequence_id: #{sequence_id} option_key: #{option_key.inspect} v: #{v.inspect} to options_hash")
          if options_hash.has_key?(sequence_id)
            options_hash[sequence_id][option_key] = v
          else
            options_hash[sequence_id] = { option_key => v }
          end
        end
      else
        # If options_regex does not match then stuff dialog options into options_hash[0]
        sequence_id = 0
        unless v.blank?
          log(:info, "Adding sequence_id: #{sequence_id} option_key: #{option_key.inspect} v: #{v.inspect} to options_hash")
          if options_hash.has_key?(sequence_id)
            options_hash[sequence_id][option_key] = v
          else
            options_hash[sequence_id] = { option_key => v }
          end
        end
      end # if options_regex =~ k
    end # dialog_options.each do
    log(:info, "Inspecting options_hash: #{options_hash.inspect}")
    return options_hash
  end

  # max_cpu_check
  def max_cpu_check(service_template, options_hash, reason_hash)
    # Add up all of the :cores_per_socket from the template(s) and the dialog options
    template_cpus_array = query_prov_options(service_template, :cores_per_socket)
    unless template_cpus_array.blank?
      template_cpus_totals = template_cpus_array.collect(&:to_i).inject(&:+)
      log(:info, "template_cpus_totals: #{template_cpus_totals.inspect}")
    end
    dialog_cpus_array = []
    options_hash.each do |sequence_id, options|
      dialog_cpus_array << options[:cores_per_socket] unless options[:cores_per_socket].blank?
    end
    unless dialog_cpus_array.blank?
      dialog_cpus_totals = dialog_cpus_array.collect(&:to_i).inject(&:+)
      log(:info, "dialog_cpu_totals: #{dialog_cpus_totals.inspect} ") unless dialog_cpus_totals.zero?
    end
    total_cpus_requested = template_cpus_totals.to_i + dialog_cpus_totals.to_i
    log(:info, "total_cpus_requested: #{total_cpus_requested.inspect} ")

    # Use value from model unless specified above
    model_max_cpus = nil || $evm.object['max_cpus'].to_i
    # Validate model_max_cpus if not nil or empty
    unless model_max_cpus.zero?
      log(:info,"Auto-Approval Threshold(Model): model_max_cpus=#{model_max_cpus} detected")
      if total_cpus_requested && (total_cpus_requested > model_max_cpus)
        log(:warn, "Auto-Approval Threshold: Amount of vCPUs requested: #{total_cpus_requested} exceeds: #{model_max_cpus}")
        reason_hash[:reason1] = "Requested CPUs #{total_cpus_requested} limit is #{model_max_cpus}"
      end
    end
  end

  # max_memory_check
  def max_memory_check(service_template, options_hash, reason_hash)
    # Add up all of the :vm_memory from the provisioning template(s) and the dialog options
    template_memory_array = query_prov_options(service_template, :vm_memory)
    unless template_memory_array.blank?
      log(:info, "template_memory_array: #{template_memory_array.inspect}")
      template_memory_totals = template_memory_array.collect(&:to_i).inject(&:+)
      log(:info, "template_memory_totals: #{template_memory_totals.inspect}")
    end
    dialog_memory_array = []
    options_hash.each do |sequence_id, options|
      dialog_memory_array << options[:vm_memory] unless options[:vm_memory].blank?
    end
    unless dialog_memory_array.blank?
      dialog_memory_totals = dialog_memory_array.collect(&:to_i).inject(&:+)
      log(:info, "dialog_memory_totals: #{dialog_memory_totals.inspect}") unless dialog_memory_totals.zero?
    end
    total_memory_requested = template_memory_totals.to_i + dialog_memory_totals.to_i
    log(:info, "total_memory_requested: #{total_memory_requested}")

    model_max_memory = nil || $evm.object['max_memory'].to_i
    # Validate model_max_memory if not 0
    unless model_max_memory.zero?
      log(:info,"Auto-Approval Threshold(Model): model_max_memory=#{model_max_memory} detected")
      if total_memory_requested && (total_memory_requested > model_max_memory)
        log(:warn, "Auto-Approval Threshold: Amount of vRAM requested: #{total_memory_requested} exceeds: #{model_max_memory}")
        reason_hash[:reason2] = "Requested Memory #{total_memory_requested}MB limit is #{model_max_memory}MB"
      end
    end
  end

  def group_check(miq_request, options_hash, reason_hash)
    
    $evm.log(:info, "User's role: #{miq_request.requester.miq_user_role_name}")
    unless miq_request.requester.miq_user_role_name == "EvmRole-super_administrator"
      reason_hash[:reason3] = "Not a member of the Administrator\s group, your request requires approval."
    end
  end

  def get_requested_resources(miq_request, tier)
    requested_cpu = 0
    requested_mem = 0
    #vms = Array.new
    #instance_sizes = Array.new
    requested_resources = {}

    vm_count = miq_request.options[:dialog]["dialog_option_#{tier}_number_of_vms"].to_i
    instance_size = miq_request.options[:dialog]["dialog_tag_#{tier}_intuit_instance_size"]

  #  vms.push miq_request.options[:dialog]["dialog_option_1_number_of_vms"].to_i
  #  vms.push (miq_request.options[:dialog]["dialog_option_2_number_of_vms"].to_i) if miq_request.options[:dialog]["dialog_option_2_number_of_vms"]
  #  vms.push (miq_request.options[:dialog]["dialog_option_3_number_of_vms"].to_i) if miq_request.options[:dialog]["dialog_option_3_number_of_vms"]

  #  instance_sizes.push miq_request.options[:dialog]["dialog_tag_1_intuit_instance_size"]
  #  instance_sizes.push (miq_request.options[:dialog]["dialog_tag_2_intuit_instance_size"]) if miq_request.options[:dialog]["dialog_tag_2_intuit_instance_size"]
  #  instance_sizes.push (miq_request.options[:dialog]["dialog_tag_3_intuit_instance_size"]) if miq_request.options[:dialog]["dialog_tag_3_intuit_instance_size"]

    log(:info, "VMS:#{vm_count} INSTANCE SIZES: #{instance_size}")

    #vms.each {|vm_count|
    #  instance_size = instance_sizes.shift
      vm_count.times do
        case instance_size
        when 'xsmall'
          requested_cpu += 1
          requested_mem += 1
        when 'small'
          requested_cpu += 1
          requested_mem += 2
        when 'small_highmem'
          requested_cpu += 1
          requested_mem += 4
        when 'medium'
          requested_cpu += 2
          requested_mem += 4
        when 'medium_highmem'
          requested_cpu += 2
          requested_mem += 8
        when 'large'
          requested_cpu += 4
          requested_mem += 8
        when 'large_highmem'
          requested_cpu += 4
          requested_mem += 16
        when 'xlarge'
          requested_cpu += 8
          requested_mem += 16
        when 'xlarge_highmem'
          requested_cpu += 8
          requested_mem += 32
        when 'xlarge_extrememem'
          requested_cpu += 8
          requested_mem += 64
        end
      end
    #}
    
    requested_resources[:cpu] = requested_cpu
    requested_resources[:mem] = requested_mem
    log(:info, "Requested Resources: #{requested_resources.inspect}")
    return requested_resources
  end

def assign_tag(group, tag_name, value)
  $evm.log(:info, "Assigning tag: #{tag_name} value: #{value}")
  if $evm.execute('tag_exists?', tag_name, value)
      $evm.log(:info, "Tag exists")
      group.tag_assign("#{tag_name}/#{value.downcase}")
   else
      $evm.execute('tag_create', tag_name, :name => "#{value}", :description => "#{value}")
      log(:info, "Creating tag: #{tag_name}/#{value.downcase.chomp}")
      tag_response = group.tag_assign("#{tag_name}/#{value.downcase.chomp}")
      $evm.log(:info, "Tag assign response: #{tag_response.inspect}")
   end
end


  log(:info, "CFME Automate Method Started")

  # dump all root attributes to the log
  dump_root

  # get the request object from root
  miq_request = $evm.root['miq_request']
  log(:info, "miq_request.id: #{miq_request.id} miq_request.options[:dialog]: #{miq_request.options[:dialog].inspect}")

  # Get dialog options from miq_request
  dialog_options = miq_request.options[:dialog]
  log(:info, "Inspecting Dialog Options: #{dialog_options.inspect}")
  options_hash = get_options_hash(dialog_options)

  # lookup the service_template object
  service_template = $evm.vmdb(miq_request.source_type, miq_request.source_id)
  log(:info, "service_template id: #{service_template.id} service_type: #{service_template.service_type} description: #{service_template.description}")

  reason_hash = {}


  log(:info, service_template.inspect)

  if ([999000000000002,999000000000003,999000000000010,999000000000010,999000000000011].include? service_template.id)
    intuit_bu = miq_request.requester.miq_group.tags(:intuit_bu).first

    instance_size = miq_request.options[:dialog]["dialog_tag_1_intuit_instance_size"]


    #intuit_hs = miq_request.options[:dialog]["dialog_option_1_hosting_segment"]
    #intuit_sz = miq_request.options[:dialog]["dialog_option_1_security_zone"].split("_")[1]
    
    hosting_segments = Array.new

    #hosting_segments.push(miq_request.options[:dialog]["dialog_option_1_hosting_segment"])
    #hosting_segments.push(miq_request.options[:dialog]["dialog_option_2_hosting_segment"]) if miq_request.options[:dialog]["dialog_option_2_hosting_segment"]
    #hosting_segments.push(miq_request.options[:dialog]["dialog_option_3_hosting_segment"]) if miq_request.options[:dialog]["dialog_option_3_hosting_segment"]

    hosting_segments.push(miq_request.options[:dialog]["dialog_option_1_hosting_segment"])
    hosting_segments.push(miq_request.options[:dialog]["dialog_option_2_hosting_segment"]) unless miq_request.options[:dialog]["dialog_option_2_hosting_segment"].blank?
    hosting_segments.push(miq_request.options[:dialog]["dialog_option_3_hosting_segment"]) unless miq_request.options[:dialog]["dialog_option_3_hosting_segment"].blank?

    hosting_segments.each_index {|tier_index|

      hosting_segment = hosting_segments[tier_index]

      mem_quota = miq_request.requester.miq_group.tags("intuit_mem_#{hosting_segment}_quota".to_sym).first.to_i
      cpu_quota = miq_request.requester.miq_group.tags("intuit_cpu_#{hosting_segment}_quota".to_sym).first.to_i

      log(:info, "MEM QUOTA: #{mem_quota}, CPU QUOTA #{cpu_quota}, BU #{intuit_bu}, HOSTING SEGMENT: #{hosting_segment}")

      requested_resources = get_requested_resources(miq_request, tier_index+1) 

      requested_cpu = requested_resources[:cpu]
      requested_mem = requested_resources[:mem]

      log(:info, "TOTAL MEM: #{requested_mem}")
      log(:info, "TOTAL CPU: #{requested_cpu}")

  #    log(:info, "TOTAL MEM: #{total_mem.fdiv(10000)}")
  #    log(:info, "TOTAL CPU: #{total_cpu.fdiv(10)}")

      new_cpu = cpu_quota - requested_cpu
      new_mem = mem_quota - requested_mem

      new_cpu = 0 if new_cpu < 0
       
      new_mem = 0 if new_mem < 0

      if mem_quota == 0
        reason_hash[:reason1] = "Memory Quota exhausted, request pending approval"
      end
      if cpu_quota == 0
        reason_hash[:reason2] = "CPU Quota exhausted, request pending approval"
      #elsif mem_quota < requested_mem
      #  (reason_hash[:reason1] ||= []) << "Request exceeds Memory quota limit. Quota: #{mem_quota} Allocation after request: #{new_mem} GBs."
        #miq_request.deny("admin", "MEM Quota Exceeded")
      #elsif cpu_quota < requested_cpu
      #  (reason_hash[:reason2] ||= []) << "Request exceeds CPU quota limit. Quota: #{cpu_quota} Allocation after request: #{new_cpu} CPUs."
        #miq_request.deny("admin", "CPU Quota Exceeded")
      end
      assign_tag(miq_request.requester.miq_group,"intuit_cpu_#{hosting_segment}_quota", "#{new_cpu}")
      assign_tag(miq_request.requester.miq_group,"intuit_mem_#{hosting_segment}_quota", "#{new_mem}")
    }
  end

  #group_check(miq_request, options_hash, reason_hash)

  # if approval required then send request into a pending state

  log(:info, "reason_hash: #{reason_hash.inspect}")
  unless reason_hash.blank?
    msg =  "Service Request was not auto-approved for the following reason(s): "
    msg += "(#{reason_hash[:reason1]}) " unless reason_hash[:reason1].blank?
    msg += "(#{reason_hash[:reason2]}) " unless reason_hash[:reason2].blank?
    msg += "(#{reason_hash[:reason3]}) " unless reason_hash[:reason3].blank?

    log(:info, "Message: #{msg}")

    miq_request.set_message(msg.chomp)
    #miq_request.set_message("Test")
    $evm.root['ae_result'] = 'error'
    $evm.object['reason'] = msg
  end

  miq_request.set_option(:intuit_bu,miq_request.requester.miq_group.tags(:intuit_bu).first)

  $evm.log(:info, "Added BU: #{miq_request.get_option(:intuit_bu)}")
  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

  # Ruby rescue
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  $evm.root['miq_request'].set_message("#{err}") if $evm.root['miq_request']
  exit MIQ_ABORT
end
