###################################
#
# CFME Automate Method: update_provision_status
#
# Notes: This method updates the provisioning status.
#
# Required inputs: status, status_state
# Mods: Wayne Chatham added rollback_ip to rollback_parameters
#       Maimuna Lubega added logic to push messages up to UI on retry
#		12/24/15 Wayne Chatham added Customer to path in create_rollback_request ae_namespace path
###################################
require 'socket'
require 'cgi'

# logging convenience method
def log(level, message)
  @method = 'update_provision_status'
  $evm.log(level, "#{@method} - #{message}")
end

def retry_method(msg, retry_time=1.minute)
  log(:warn, "Retrying in #{retry_time} seconds: [#{msg}]")
  $evm.root['ae_result'] = 'retry'
  $evm.root['ae_retry_interval'] = retry_time
  exit MIQ_OK
end

def lab_env?
  begin
    log(:info, "Is this a lab environment")
    hostname = Socket.gethostbyname(Socket.gethostname).first
    hostname.downcase!
    log(:info, "is this the king of the lab: #{hostname}")
    return hostname.include?("corplab")
  rescue SignalException, StandardError => e
    log(:info, "Unable to tell if this is the lab, defaulting to prod")
    return false
  end
end

def get_web_service_endpoint(intuit_dc)
  # The IP addresses should be stored separately in the automation datastore
  # under /Intuit/Integration/CloudForms/Configuration, instead of
  # hard-coding them here.

  evm_region = $evm.root['miq_server'].region_number

  case intuit_dc
    when "qdc"
      # cldfrgnui-qdc.corp.intuit.net
      return "10.158.32.52"
    when "lvdc"
      # cldfrgnui-lv.corp.intuit.net
      return "10.181.0.52"
    when "edc"
      return "10.1.66.103"
    when "sdc"
      return "10.1.98.103"
    when "ihplab"
      case evm_region
        when 509
          # cfltmrgnui01.corplab.intuit.net
          return "10.159.0.61"
        when 702
          # cfltmrgnui02.corplab.intuit.net
          return "10.159.0.63"
      end
  end

  log(:error, "No endpoint found for intuit_dc #{intuit_dc} and region #{evm_region}")
  return nil
end

def create_rollback_request(intuit_dc, prov, endpoint, rollback_parameters)
  parameters = [
      "intuit_dc=#{intuit_dc}",
      "dns_zone=#{prov.get_option(:dns_domain)}.",
      "target_name=#{prov.get_option(:vm_target_name)}"
  ]

  unless rollback_parameters['acquired_ip_addresses'].nil?
    v = rollback_parameters['acquired_ip_addresses'].join(',')
    parameters.push("acquired_ip_addresses=#{v}")
  end

  unless rollback_parameters['vm_name'].nil?
    v = rollback_parameters['vm_name']
    parameters.push("vm_name=#{v}")

    v = rollback_parameters['ems_id']
    parameters.push("ems_id=#{v}")
  end

  unless rollback_parameters['eqb_record_id'].nil?
    v = rollback_parameters['eqb_record_id']
    parameters.push("eqb_record_id=#{v}")
  end

  unless rollback_parameters['sn_vm_id'].nil?
    v = rollback_parameters['sn_vm_id']
    parameters.push("sn_vm_id=#{v}")
  end

  unless rollback_parameters['hpsa_vm_id'].nil?
    v = rollback_parameters['hpsa_vm_id']
    parameters.push("hpsa_vm_id=#{v}")
  end

  unless rollback_parameters['rollback_ip'].nil?
    v = rollback_parameters['rollback_ip']
    parameters.push("rollback_ip=#{v}")
  end

uri = "Customer/Intuit/Integration/CloudForms/Actions/CreateAutomationRequest?" + \
        URI.encode_www_form([['ip_address', endpoint],
                             ['intuit_dc', intuit_dc],
                             ['ae_namespace', 'Customer/Intuit/Provisioning/StateMachines'],
                             ['ae_class', 'VMProvisionRollback'],
                             ['ae_instance', 'Default'],
                             ['parameters', parameters.join('|')]])

  if not $evm.instantiate(uri)
    log(:error, "Failed to instantiate #{uri}")
  end
end

# Notify the requester about a failed provisioning attempt.
def failed_provisioning_email(prov, step, status)
  vm_name = prov.get_option(:vm_target_name)

  owner = $evm.root['user']
  to = nil
  to ||= prov.get_option(:owner_email) unless prov.nil?
  to ||= owner.email unless owner.nil?
  to ||= $evm.object['to_email_address']

  if to.nil?
    log(:warn, "Cannot determine recipient email for failed provisioning notification.")
    return
  end

  # Also send message to CF Engineering  (engineering_email_address)
  engineering_email_address = nil
  engineering_email_address ||= $evm.object['engineering_email_address']
  engineering_email_address ||= "TechCloudforms_Support@intuit.com"
  to += ",#{engineering_email_address}"

  from = nil
  if lab_env?
    from ||= $evm.object['from_email_address_lab']
    if from.nil?
      from = "cloudforms_lab@intuit.com"
    end
  else
    from ||= $evm.object['from_email_address']
    if from.nil?
      from = "cloudforms@intuit.com"
    end
  end
  subject = "Provisioning Failure for VM #{vm_name}"

  body = %{\
<p>Your request to provision the virtual machine</p>

<blockquote>#{vm_name}</blockquote>

<p>failed with the following error in step #{step}:</p>

<pre>#{status}</pre>

<p>More information may be available at <a href="https://cloudforms.intuit.net/miq_request/show_list">https://cloudforms.intuit.net/miq_request/show_list</a>.</p>

<p>Request: #{prov.miq_provision_request.id}<br>
Task: miq_provision_#{prov.id}</p>
}

  log(:info, "Sending email to <#{to}> from <#{from}> subject: <#{subject}>")
  $evm.execute(:send_email, to, from, subject, body)
end

begin
  log(:info, "CFME Automate Method Started")

  # Initialize variables
  prov = $evm.root['miq_provision']

  # Get variables from Server object
  server = $evm.root['miq_server']

  # Get State Machine
  state = $evm.current_object.class_name

  # Get current step
  step = $evm.current_object.current_field_name

  # Get status from input field status
  status = $evm.inputs['status']

  # Get status_state ['on_entry', 'on_exit', 'on_error'] from input field
  status_state = $evm.inputs['status_state']

  log(:info, "Server:<#{server.name}> Ae_Result:<#{$evm.root['ae_result']}> State:<#{state}> Step:<#{step}> Status_State:<#{status_state}> Status:<#{status}>")

  # Update Status for on_entry,on_exit
  if $evm.root['ae_result'] == 'ok'

    # Check to see if provisioning is complete
    if status == 'provision_complete'
      message = 'VM Provisioned Successfully'
      prov.finished(message)
    end
    prov.message = CGI::escape_html(status)
  end

  if $evm.root['ae_result'] == 'retry'
    ae_reason = $evm.root['ae_reason'] || "No reason"
    num_retry = $evm.root['ae_state_retries']
    count = "RETRY #{num_retry}: "
    log(:info, count + ae_reason )
    req_id = prov.miq_request_id
    req = $evm.vmdb('miq_request').find_by_id(req_id)
    req.set_message(count + ae_reason)
  end

  # Update Status for on_error and call methods to clean up resources that
  # have been allocated as part of this provisioning request.
  if $evm.root['ae_result'] == 'error'

    prov.message = CGI::escape_html(status)

    failed_provisioning_email(prov, step, status)

    log(:error, "Original error: #{status}")

    ae_result = $evm.root['ae_result']
    ae_reason = $evm.root['ae_reason']

    ems_id = prov.get_option(:src_ems_id)
    ems = $evm.vmdb('ext_management_system', ems_id)
    intuit_dc = ems.tags(:intuit_dc).first
    endpoint = get_web_service_endpoint(intuit_dc)
    rollback_parameters = prov.get_option(:rollback_parameters)

    create_rollback_request(intuit_dc, prov, endpoint, rollback_parameters)

    $evm.root['ae_result'] = ae_result
    $evm.root['ae_reason'] = ae_reason
  end

  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

    # Set Ruby rescue behavior
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_STOP
end
