###################################
#
# EVM Automate Method: PreProvision_Clone_to_VM
#
# Notes: This default method is used to apply PreProvision customizations during the cloning to a VM:
# 1. VLAN
# 2. VM Description/Annotations
# 3. Target VC Folder
# 4. Resource Pool
# 5. Tag Ineritance
#
###################################
begin
  # Method for logging
  def log(level, message)
    @method = 'PreProvision_Clone_to_VM'
    $evm.log(level, "#{@method} - #{message}")
  end

  # dump_root
  def dump_root()
    log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
    $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
    log(:info, "Root:<$evm.root> End $evm.root.attributes")
    log(:info, "")
  end

  log(:info, "CFME Automate Method Started")

  # dump all root attributes to the log
  dump_root

  # Get provisioning object
  prov = $evm.root['miq_provision']
  log(:info, "Provision:<#{prov.id}> Request:<#{prov.miq_provision_request.id}> Type:<#{prov.type}>")

  # Get template
  template = prov.vm_template
  #log(:info, "Inspecting Template Object: #{template.inspect}")

  # Get OS Type from the template platform
  product  = template.operating_system['product_name'] rescue ''
  log(:info, "Source Product: <#{product}>")

  ###################################
  # Set the customization spec here
  # If one selected in dialog it will be used,
  # else it will map the customization spec based on the
  # the entry below
  ###################################
  customization_spec = "my-custom-spec"

  # Skip automatic customization spec mapping if template is 'Other'
  unless product.include?("Other")
    if prov.get_option(:sysprep_custom_spec).nil?
      prov.set_customization_spec(customization_spec)
      log(:info, "Provisioning object updated - <:sysprep_custom_spec> = <#{customization_spec}>")
    end
  else
    log(:info, "Skipping automatic customization spec mapping")
  end


  ###################################
  # Was a VLAN selected in dialog?
  # If not you can set one here.
  ###################################
  default_vlan = "vlan1"
  default_dvs = "portgroup1"

  if prov.get_option(:vlan).nil?
    prov.set_vlan(default_vlan)
    #prov.set_dvs(default_dvs)
  end


  ###################################
  # Set the VM Description and VM Annotations  as follows:
  # The example would allow user input in provisioning dialog "vm_description"
  # to be added to the VM notes
  ###################################
  # Stamp VM with custom description
  unless prov.get_option(:vm_description).nil?
    vmdescription = prov.get_option(:vm_description)
    prov.set_option(:vm_description,vmdescription)
    log(:info, "Provisioning object <:vmdescription> updated with <#{vmdescription}>")
  end

  # Setup VM Annotations
  vm_notes =  "Owner: #{prov.get_option(:owner_first_name)} #{prov.get_option(:owner_last_name)}"
  vm_notes += "\nEmail: #{prov.get_option(:owner_email)}"
  vm_notes += "\nSource VM: #{prov.vm_template.name}"
  vm_notes += "\nCustom Description: #{vmdescription}" unless vmdescription.nil?
  prov.set_vm_notes(vm_notes)
  log(:info, "Provisioning object <:vm_notes> updated with <#{vm_notes}>")


  ###################################
  # Drop the VM in the targeted folder if no folder was chosen in the dialog
  # The VC folder must exist for the VM to be placed correctly else the
  # VM will placed along with the template
  # Folder starts at the Data Center level
  ###################################
  default_folder = 'DC1/Infrastructure/ManageIQ/SelfService'

  if prov.get_option(:placement_folder_name).nil?
    prov.get_folder_paths.each do |key, path|
      log(:info, "Dumping all folders:<#{key}> - <#{path}>")
    end
    prov.set_folder(default_folder)
    log(:info, "Provisioning object <:placement_folder_name> updated with <#{default_folder}>")
  else
    log(:info, "Placing VM in folder: <#{prov.get_option(:placement_folder_name)}>")
  end


  ############################################
  # Find and set the Resource Pool for a VM:
  ############################################
  # respool = prov.eligible_resource_pools.detect {|c| c.name.casecmp('MyResPool')==0}
  # prov.set_resource_pool(respool)


  ###################################
  #
  # Inherit parent VM's tags and apply
  # them to the cloned VM
  #
  ###################################

  # List of tag categories to carry over
  tag_categories_to_migrate = ['environment', 'department', 'location', 'function']

  # Assign variables
  prov_tags = prov.get_tags
  log(:info, "Provisioning Tags: <#{prov_tags.inspect}>")
  template_tags = template.tags
  log(:info, "Template Tags: <#{template_tags.inspect}>")

  # Loop through each source tag for matching categories
  template_tags.each do |cat_tagname|
    category, tag_value = cat_tagname.split('/')
    log(:info, "Processing Tag Category: <#{category}> Value: <#{tag_value}>")
    next unless tag_categories_to_migrate.include?(category)
    prov.add_tag(category, tag_value)
    log(:info, "Updating Provisioning Tags with Category: <#{category}> Value: <#{tag_value}>")
  end

  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

  # Set Ruby rescue behavior
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
