###################################
#
# CFME Automate Method: acquire_ip_addresses
#
# Author: Carsten Clasohm
#
# Notes: Acquire one or more IP addresses for a VM
#
# Inputs: miq_provision
#
# Mods: 
# - Wayne Chatham added rollback_ip to rollback_parameters
#   Will only cleanup data IP in rollback if result of AcquireIpAddress 
#   contains dns_record_added = true
###################################

require 'ipaddr'
require 'uri'

# logging convenience method
def log(level, msg)
  @method = 'acquire_ip_addresses'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, "#{msg}")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def retry_method(msg, retry_time=1.minute)
  log(:warn, "Retrying in #{retry_time} seconds: [#{msg}]")
  $evm.root['ae_result'] = 'retry'
  $evm.root['ae_retry_interval'] = retry_time
  exit MIQ_OK
end

def to_bool(value)
  return true if value == true || value =~ (/^(true|t|yes|y|1)$/i)
  return false if value == false || value.blank? || value =~ (/^(false|f|no|n|0|null)$/i)
  raise ArgumentError.new("invalid value for Boolean: \"#{value}\"")
end

def instantiate(uri)
  result = $evm.instantiate(uri)
  error("Failed to execute #{uri}") unless result

  case result['result']
    when 'error'
      error(result['reason'])
    when 'retry'
      retry_method(result['reason'])
  end

  result
end

begin
  prov = $evm.root['miq_provision']

  ems_id = prov.get_option(:src_ems_id)
  ems = $evm.vmdb('ext_management_system', ems_id)
  intuit_dc = ems.tags(:intuit_dc).first

  # Get the VM name and IP address that the user asked for. If the user did
  # not specify an IP address, we will ask Men & Mice to pick one.
  vm_name = prov.get_option(:vm_target_name)

  ip_address = prov.get_option(:ws_values)[:ip_address]

  unless ip_address.blank?
    log(:info, "Using pre-allocated data IP address: [#{ip_address}]")
  end

  create_storage_nic = to_bool(prov.get_option(:ws_values)[:create_storage_nic])

  tags = prov.get_tags
  hosting_segment = tags[:intuit_hs]
  security_zone = tags[:intuit_sz]
  security_level = tags[:intuit_security_level]
  fault_domain = tags[:intuit_fd]

  # Check if we are configuring a management interface for the VM. If so,
  # the positions of the data and storage NICs shift.

  create_mgmt_nic = false
  data_nic_idx = 0
  storage_nic_idx = 1

  # On Windows, the data NIC must be the last one, because that's the one
  # the DNS setting is applied to.
  custom_spec = prov.options[:sysprep_custom_spec]
  if create_storage_nic && custom_spec && custom_spec[1] == "cloudforms_windows"
    log(:info, "VMware customization for Windows detected, moving data NIC to last position")
    data_nic_idx = 1
    storage_nic_idx = 0
  end

  if hosting_segment == "qdca"
    create_mgmt_nic = true

    # If the VM has a management network interface, it's the first one. The
    # data and storage NICs shift position by one.
    data_nic_idx += 1
    storage_nic_idx += 1
  end

  # First get the data IP.
  rollback_ip = 'false'
  rollback_parameters = prov.get_option(:rollback_parameters)
  rollback_parameters['rollback_ip'] = rollback_ip
  prov.set_option(:rollback_parameters, rollback_parameters)
  if ['qcyf01','lasf01'].include? hosting_segment.downcase
    target_epg = prov.get_option(:ws_values)[:target_epg]

    ae_uri = "/Intuit/Integration/MenAndMice/Actions/AcquireIPAddressACI?" + \
             URI.encode_www_form([['intuit_dc', intuit_dc],
                                  ['ip_address', ip_address],
                                  ['hostname', vm_name],
                                  ['target_epg', target_epg]])
    result = instantiate(ae_uri)
  else
    network_db = "intuit_network_data/#{intuit_dc}_#{hosting_segment}_#{security_zone}_#{security_level}"

    ae_uri = "/Intuit/Integration/MenAndMice/Actions/AcquireIPAddress?" + \
           URI.encode_www_form([['intuit_dc', intuit_dc],
                                ['ip_address', ip_address],
                                ['hostname', vm_name],
                                ['network_db', network_db]])
    result = instantiate(ae_uri)
  end

  log(:info, "Result for DATA NETWORK: #{result}, IP ADDRESS: #{result['ip_address']}")

  acquired_hn = result['fqdn'].split('.')[0]
  acquired_ip = result['ip_address']

  #error("No free IP addresses in: #{intuit_dc} #{hosting_segment} #{security_zone} #{security_level}") if result['ip_address'].blank?
  if result['ip_address'].blank?
    log(:info, "Value of rollback_ip is #{rollback_ip}, hostname: #{acquired_hn}")
    error("No free IP addresses in: #{intuit_dc} #{hosting_segment} #{security_zone} #{security_level}")
  end
  vmware_network = result['vmware_network']
  vmware_network_is_dvs = result['vmware_network_is_dvs']
  dns_domain = result['dns_domain']
  fqdn = result['fqdn']

  dns1 = result['dns1']
  dns2 = result['dns2']
  dns_servers = dns2.blank? ? dns1 : "#{dns1}, #{dns2}"
  prov.set_option(:dns_servers, dns_servers)

  if result['dns_record_added']
    rollback_parameters['acquired_ip_addresses'] = [result['ip_address']]
    rollback_ip = 'true'
  else
    rollback_parameters['acquired_ip_addresses'] = []
    rollback_ip = 'false'
  end
  rollback_parameters['rollback_ip'] = rollback_ip
  prov.set_option(:rollback_parameters, rollback_parameters)
  log(:info, "Value of rollback_ip is #{rollback_ip} for ip address: #{acquired_ip}, hostname: #{acquired_hn}")
  # The vm_target_hostname attribute has to be set to the short hostname,
  # because the VMware customization automatically appends the domain name
  # to it.
  short_vm_name = vm_name.sub(%r{\..*$}, '')
  prov.set_option(:vm_target_hostname, short_vm_name)

  prov.set_option(:linux_host_name, fqdn)
  prov.set_option(:host_name, fqdn)
  prov.set_option(:hostname, fqdn)

  # This is used by other automation methods to convert the short hostname
  # to an FQDN.
  prov.set_option(:dns_domain, dns_domain)

  # On Linux systems, this will end up in /etc/resolv.conf.
  prov.set_option(:dns_suffixes, dns_domain)

  prov.set_option(:sysprep_domain_name, dns_domain)
  prov.set_option(:linux_domain_name, dns_domain)

  # We also need to make sure that the VM name includes the DNS domain.
  prov.set_option(:vm_target_name, "#{vm_name}.#{dns_domain}") unless vm_name.include? "."

  prov.set_network_adapter(data_nic_idx, \
                           {
                                           :is_dvs => vmware_network_is_dvs,
                                           :network => vmware_network,
                                           :devicetype => 'VirtualVmxnet3'
                                       })

  if vmware_network_is_dvs == 1
    prov.set_dvs(vmware_network)
  end
  # We only set the default gateway on the data NIC.
  prov.set_nic_settings(data_nic_idx, \
                        {
                                        :ip_addr => result['ip_address'],
                                        :subnet_mask => result['submask'],
                                        :gateway => result['gateway'],
                                        :addr_mode => ["static", "Static"]
                                    })

  if create_mgmt_nic
    mgmt_ip_address = prov.get_option(:ws_values)[:mgmt_ip_address]

    unless mgmt_ip_address.blank?
      log(:info, "Using pre-allocated management IP address: [#{mgmt_ip_address}]")
    end

    mgmt_hostname = prov.get_option(:ws_values)[:mgmt_hostname]
    if mgmt_hostname.blank?
      mgmt_hostname = "#{short_vm_name}m.#{dns_domain}"
    elsif mgmt_hostname == fqdn
      error("User-supplied management hostname #{mgmt_hostname} identical to data hostname #{fqdn}")
    end

    network_db = "intuit_network_management/#{intuit_dc}_#{hosting_segment}_#{security_zone}_#{security_level}"

    ae_uri = "/Intuit/Integration/MenAndMice/Actions/AcquireIPAddress?" + \
      URI.encode_www_form([['intuit_dc', intuit_dc],
                           ['ip_address', mgmt_ip_address],
                           ['hostname', mgmt_hostname],
                           ['network_db', network_db]])
    result = instantiate(ae_uri)

    error("failure in AcquireIPAddress") if result.nil?

    if result['dns_record_added']
      rollback_parameters['acquired_ip_addresses'].push(result['ip_address'])
      prov.set_option(:rollback_parameters, rollback_parameters)
    end

    log(:info, "Adding management NIC with network #{result['vmware_network']}")

    prov.set_network_adapter(0, \
                             {
                                  :is_dvs => result['vmware_network_is_dvs'],
                                  :network => result['vmware_network'],
                                  :devicetype => 'VirtualVmxnet3'
                              })

    prov.set_nic_settings(0, \
                          {
                               :ip_addr => result['ip_address'],
                               :subnet_mask => result['submask'],
                               :addr_mode=> ["static", "Static"]
                           })
    prov.set_option(:management_ip, result['ip_address'])
    prov.set_option(:management_netmask, result['submask'])
    prov.set_option(:management_gateway, result['gateway'])
    prov.set_option(:management_network, result['vmware_network'])
  end

  if create_storage_nic
    storage_ip_address = prov.get_option(:ws_values)[:storage_ip_address]

    unless storage_ip_address.blank?
      log(:info, "Using pre-allocated storage IP address: [#{storage_ip_address}]")
    end

    storage_hostname = prov.get_option(:ws_values)[:storage_hostname]
    if storage_hostname.blank?
      storage_hostname = "#{short_vm_name}s.#{dns_domain}"
    elsif storage_hostname == fqdn
      error("User-supplied storage hostname #{storage_hostname} identical to data hostname #{fqdn}")
    end

    if ['qcyf01','lasf01'].include? hosting_segment.downcase
      storage_epg = nil
      log(:info, "Security Level: #{security_level} SECURITY Zone: #{security_zone}")
      if ['publiccomp', 'sensitivecomp', 'nonpubliccomp'].include? security_level
        epg_tag = $evm.current[security_level]
        epg_sec_level = epg_tag.split(",").last
        epg_map = "cmp.#{epg_sec_level}"
        storage_epg = "uni/tn-storage/ap-array/epg-#{epg_map}.stor"
      elsif ['omn'].include? security_level
        epg_map = $evm.current[security_level]
        storage_epg = "uni/tn-storage/ap-array/epg-#{epg_map}.stor"
      else
        epg_sec_level = $evm.current[security_level]
        epg_sec_zone = $evm.current[security_zone]
        storage_epg = "uni/tn-storage/ap-array/epg-#{epg_sec_zone}.#{epg_sec_level}.stor"
      end

      log(:info, "Storage EPG: #{storage_epg}")

      ae_uri = "/Intuit/Integration/MenAndMice/Actions/AcquireIPAddressACI?" + \
               URI.encode_www_form([['intuit_dc', intuit_dc],
                                    ['ip_address', storage_ip_address],
                                    ['hostname', storage_hostname],
                                    ['target_epg', storage_epg]])
      result = instantiate(ae_uri)
    else
      network_db = "intuit_network_storage/#{intuit_dc}_#{hosting_segment}_#{security_zone}_#{security_level}"

      ae_uri = "/Intuit/Integration/MenAndMice/Actions/AcquireIPAddress?" + \
        URI.encode_www_form([['intuit_dc', intuit_dc],
                             ['ip_address', storage_ip_address],
                             ['hostname', storage_hostname],
                             ['network_db', network_db]])
      result = instantiate(ae_uri)
    end

    log(:info, "Result for STORAGE NETWORK: #{result}, IP ADDRESS: #{result['ip_address']}")

    error("failure in AcquireIPAddress") if result.nil?

    if result['dns_record_added']
      rollback_parameters['acquired_ip_addresses'].push(result['ip_address'])
      prov.set_option(:rollback_parameters, rollback_parameters)
    end

    log(:info, "Adding storage NIC with network #{result['vmware_network']}")

    prov.set_network_adapter(storage_nic_idx, \
                             {
                                                :is_dvs => result['vmware_network_is_dvs'],
                                                :network => result['vmware_network'],
                                                :devicetype => 'VirtualVmxnet3'
                                            })

    prov.set_nic_settings(storage_nic_idx, \
                          {
                                             :ip_addr => result['ip_address'],
                                             :subnet_mask => result['submask'],
                                             :addr_mode=> ["static", "Static"]
                                         })
  end

  # Log all of the provisioning options to the automation.log
  log(:info, "Provisioning option keys after acquire_ip_addresses:")
  prov.options.each { |k,v| log(:info, "Key:<#{k.inspect}> Value:<#{v.inspect}>") }

  $evm.root['ae_result'] = 'ok'
  $evm.root['ae_reason'] = ''

  log(:info, "Main: exiting main method")
  exit MIQ_OK
rescue SignalException => toerr
  error("Men and Mice timeout when updating DNS. [#{toerr}]")
rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
