###################################
#
# CFME Automate Method: cleanup_ip_addresses
#
# Author: Carsten Clasohm
#
# Notes:
#   Release one or more IP addresses that were acquired for a VM.
#   This is called if there is an error during the provisioning
#   process, to free up any addresses that acquire_ip_addresses
#   allocated for the VM.
#
# Inputs: miq_provision
#
# Mods:
# - Wayne Chatham Only release ip's if rollback_ip = true
###################################

# logging convenience method
def log(level, msg)
  @method = 'cleanup_ip_addresses'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, "#{msg}")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def retry_method(msg, retry_time=1.minute)
  log(:warn, "Retrying in #{retry_time} seconds: [#{msg}]")
  $evm.root['ae_result'] = 'retry'
  $evm.root['ae_retry_interval'] = retry_time
  exit MIQ_OK
end

def instantiate(uri)
  result = $evm.instantiate(uri)
  error("Failed to execute #{uri}") unless result

  case result['result']
    when 'error'
      error(result['reason'])
    when 'retry'
      retry_method(result['reason'])
  end

  result
end

begin
  log(:info, "Main: entering main method")

  task = $evm.root['automation_task']

  log(:info, "task.options[:attrs] = #{task.options[:attrs].inspect}")

  intuit_dc = task.options[:attrs][:intuit_dc]
  dns_zone = task.options[:attrs][:dns_zone]
  vm_name = task.options[:attrs][:target_name]
  rollback_ip = task.options[:attrs][:rollback_ip]
  log(:info, "Value for rollback_ip is #{rollback_ip} for VM Name #{vm_name}")

  unless task.options[:attrs][:acquired_ip_addresses].nil? || rollback_ip == 'false'
    acquired_ip_addresses = task.options[:attrs][:acquired_ip_addresses].split(',')

    #acquired_ip_addresses.each { |ip_address|
    ip_addresses = acquired_ip_addresses.join(",")
    log(:info, "Calling ReleaseIPAddress for IP #{ip_addresses} and VM Name #{vm_name} in #{dns_zone}")

    ae_uri = "/Intuit/Integration/MenAndMice/Actions/ReleaseIPAddress?" + \
               URI.encode_www_form([['intuit_dc', intuit_dc],
                                    ['ip_address', ip_addresses],
                                    ['dns_zone', dns_zone],
                                    ['vm_name', vm_name]])
    result = instantiate(ae_uri)

    log(:error, "failed to release IP address #{ip_address}") if result.nil?

    #}
  end

  $evm.root['ae_result'] = 'ok'
  $evm.root['ae_reason'] = ''

  log(:info, "Main: exiting main method")
  exit MIQ_OK

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
