###################################
#
# CFME Automate Method: cleanup_provider
#
# Author: Carsten Clasohm
#
# Notes: 
#   Delete a VM from its provider. This is called if there is an error
#   during the provisioning process, to automatically remove the
#   half-provisioned VM.
#
# Inputs: vm_name, ems_id
#
# Mods:
#
###################################

# logging convenience method
def log(level, msg)
  @method = 'cleanup_provider'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, "#{msg}")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def retry_method(msg, retry_time=1.minute)
  log(:warn, "Retrying in #{retry_time} seconds: [#{msg}]")
  $evm.root['ae_result'] = 'retry'
  $evm.root['ae_retry_interval'] = retry_time
  exit MIQ_OK
end

begin
  log(:info, "Main: entering main method")

  task = $evm.root['automation_task']
  task_attrs = task.options[:attrs]
  vm_name = task_attrs[:vm_name]

  unless vm_name.nil?
    ems_id = task.options[:attrs][:ems_id]
    ems = $evm.vmdb('ext_management_system', ems_id)

    vm = ems.vms.detect { |v| v.name == vm_name }

    if vm
      # Make sure that this VM was provisioned by us. This is a safeguard
      # for provisioning failures caused by naming collisions inside the
      # provider.
      if /\w*MIQ\sGUID/i =~ vm.v_annotation
        # We will need this later to delete the VM from the VMDB.
        task_attrs[:vm_id] = vm.id
        task.set_option(:attrs, task_attrs)

        power_state = vm.attributes['power_state']

        if power_state == "never"
          # If never then this VM is a template so exit the retirement state
          # machine.
          error("Invalid power state \"never\" for VM #{vm_name} on EMS #{ems.name}, is it a template?")
        elsif power_state != "off" && power_state != "suspended"
          vm.stop
          retry_method("Waiting for VM #{vm_name} on EMS #{ems.name} to power down.")
        end

        log(:info, "Deleting VM #{vm_name} from EMS #{ems.name}")
        vm.remove_from_disk
      else
        log(:warn, "VM #{vm_name} in EMS #{ems.name} missing MIQ GUID, not deleting during rollback")
      end
    else
      # Probably creation of the VM failed.
      log(:warn, "Cannot find VM #{vm_name} in EMS #{ems.name} during rollback")
    end
  end

  $evm.root['ae_result'] = 'ok'
  $evm.root['ae_reason'] = ''

  log(:info, "Main: exiting main method")
  exit MIQ_OK

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
