##################################
#
# CFME Automate Method: configure_vm
#
# Author: Wayne Chatham
#
# Notes: Configures VM settings. toolsUpgradePolicy set to manual.
#        toolsUpgradePolicy set to upgradeAtPowerCycle in upgrade_tools method.
# Inputs: miq_provision
#
# Mods:
#
###################################

require 'rbvmomi'

# logging convenience method
def log(level, msg)
  @method = 'configure_vm'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, "#{msg}")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg.to_s
  exit MIQ_OK
end

def get_vm(vim,types,searchfor,startobj=vim.rootFolder)
  # Allow types to be a string or an array.
  types = [types].flatten
  vms = vim.serviceContent.viewManager.CreateContainerView({
    :container => startobj,
    :type => types,
    :recursive => true
  }).view
  vm = vms.select { |v| v._ref.index(searchfor) == 0 }.first
  if vm
    msg = "Located vm: #{vm.config.name} uuid: #{vm.config.instanceUuid} moref: #{vm._ref}"
    log(:info,msg)
  else
    msg = "ERROR: Unable to locate vm with moref: #{searchfor}"
    error(msg)
  end
  return vm
end

def tools_spec_check(vm,prop,val)
  msg = "Looking up property: #{prop} on vm: #{vm.config.name}"
  log(:info,msg)
  result = ""
  toolsconf = vm.config.tools.props
  toolsconf.each do |item|
    checkprop = item[0].to_s
    checkval = item[1].to_s
    if checkprop == prop
      msg = "=> Property found: #{checkprop} with value: #{checkval}"
      log(:info,msg) 
      if checkval != val
        msg = "==> Requires configuration"
        log(:info,msg)
        result = true
      else
        msg = "==> Property is set correctly"
        log(:info,msg)
        result = false
      end
    end
  end
  return result
end

def configure_tools_settings(vm)
  msg = "Configuring VMware Tools settings"
  log(:info,msg)
  vmConfigSpec = VIM.VirtualMachineConfigSpec
  vmConfigSpec.tools = VIM.ToolsConfigInfo
  toolsChanges = []

  property='toolsUpgradePolicy'
  value='manual'
  result = tools_spec_check(vm,property,value)
  if result == true
    vmConfigSpec.tools.toolsUpgradePolicy = value
    toolsChanges << {property => value}
  end

  property='afterPowerOn'
  value='true'
  result = tools_spec_check(vm,property,value)
  if result == true
    vmConfigSpec.tools.afterPowerOn = value
    toolsChanges << {property => value}
  end

  property='afterResume'
  value='true'
  result = tools_spec_check(vm,property,value)
  if result == true
    vmConfigSpec.tools.afterResume = value
    toolsChanges << {property => value}
  end

  property='beforeGuestStandby'
  value='true'
  result = tools_spec_check(vm,property,value)
  if result == true
    vmConfigSpec.tools.beforeGuestStandby = value
    toolsChanges << {property => value}
  end

  property='beforeGuestShutdown'
  value='true'
  result = tools_spec_check(vm,property,value)
  if result == true
    vmConfigSpec.tools.beforeGuestShutdown = value
    toolsChanges << {property => value}
  end

  property='syncTimeWithHost'
  value='false'
  result = tools_spec_check(vm,property,value)
  if result == true
    vmConfigSpec.tools.syncTimeWithHost = value
    toolsChanges << {property => value}
  end

  #Reconfigure VM if changes required
  if toolsChanges.length != 0
    msg = "Reconfiguring VM"
    log(:info,msg)
    task = vm.ReconfigVM_Task(:spec => vmConfigSpec)
    msg = toolsChanges
    log(:info,msg)
    #Wait for ReconfigureVm_Task to complete
    loop do
      if task.info.state == "error"
        msg = "==> Error: Reconfigure VM task failed"
        err(msg)
        taskrun = false
      end
      if task.info.state != "success"
        taskrun = true
        msg ="=> Waiting for Reconfigure Task to complete: #{task.info.state}"
        log(:info,msg)
        sleep(5)
      else
        taskrun = false
        msg = "==> Reconfigure VM complete: #{task.info.state}"
        log(:info,msg)
      end
      break if (taskrun == false)
    end 
  end
end

def poweron_vm(vm)
	msg = "Powering on vm: #{vm.config.name}"
	log(:info,msg)
	vmstate = vm.runtime.powerState
	if vmstate != "poweredOn"
		vm.PowerOnVM_Task.wait_for_completion
	end
end

begin
  #Get VM info from CF
  prov = $evm.root['miq_provision']
  cfvm = prov.vm
  tags = prov.get_tags
  cfvmref = cfvm.ems_ref
  
  #Get VC info from CF VM
  vcserver = cfvm.ext_management_system.ipaddress
  vcuser = cfvm.ext_management_system.authentication_userid
  vcpwd = cfvm.ext_management_system.authentication_password
  
  #Connect to VC
  VIM = RbVmomi::VIM
  vim = VIM.connect host: vcserver, insecure: true, user: vcuser, password: vcpwd
  if vim
    msg = "Connected to vCenter: #{vcserver} as: #{vcuser}"
    log(:info,msg)
  else
    msg = "ERROR: Unable to connect to vCenter: #{vcserver} as: #{vcuser}"
    error(msg)
  end
  #Get VM
  vm = get_vm(vim,'VirtualMachine',cfvmref) 
    
  #Configure tools settings
  configure_tools_settings(vm)
  
  #PowerOn VM
  poweron_vm(vm)
    
  $evm.root['ae_result'] = 'ok'
  $evm.root['ae_reason'] = ''
  exit MIQ_OK
end
