###################################
#
# CFME Automate Method: provision_complete
#
# Notes: This method sends an e-mail when the following event is raised:
#
# Events: vm_provisioned
#
# Model Notes:
# 1. to_email_address - used to specify an email address in the case where the
#    vm's owner does not have an  email address. To specify more than one email
#    address separate email address with commas. (I.e. admin@example.com,user@example.com)
# 2. from_email_address - used to specify an email address in the event the
#    requester replies to the email
# 3. signature - used to stamp the email with a custom signature
#
###################################
require 'socket'

begin
  # logging convenience method
  def log(level, msg)
    @method = 'provision_complete'
    $evm.log(level, "#{@method}: #{msg}")
  end

  def error(msg)
    log(:error, "#{msg}")
    $evm.root['ae_result'] = 'error'
    $evm.root['ae_reason'] = msg.to_s
    exit MIQ_OK
  end

  def lab_env?
    begin
      log(:info, "Is this a lab environment")
      hostname = Socket.gethostbyname(Socket.gethostname).first
      hostname.downcase!
      log(:info, "is this the king of the lab: #{hostname}")
      return hostname.include?("corplab")
    rescue SignalException, StandardError => e
      log(:info, "Unable to tell if this is the lab, defaulting to prod")
      return false
    end
  end

  log(:info, "Main: entering main method")

  # Get vm from miq_provision object
  prov = $evm.root['miq_provision']
  vm = prov.vm
  raise "VM not found" if vm.nil?

  # Override the default appliance IP Address below
  appliance ||= $evm.root['miq_server'].ipaddress

  # Get VM Owner Name and Email
  owner = nil
  owner = $evm.root['user']
  log(:info, "VM Owner: #{owner.inspect}")

  to = nil
  to ||= prov.get_option(:owner_email) unless prov.nil?
  to ||= owner.email unless owner.nil?
  to ||= $evm.object['to_email_address']

  # Also send message to CF Engineering  (engineering_email_address)
  engineering_email_address = $evm.object['engineering_email_address']
  to += ",#{engineering_email_address}"

  # Also send message to CF Operations  (operations_email_address)
  operations_email_address = $evm.object['operations_email_address']
  to += ",#{operations_email_address}"

  if to.nil?
    log(:info, "Email not sent because no recipient specified.")
    exit MIQ_OK
  end

  # Assign original to_email_Address to orig_to for later use
  orig_to = to

  # Get from_email_address from model unless specified below
  from = nil
  if lab_env?
    from ||= $evm.object['from_email_address_lab']
  else
    from ||= $evm.object['from_email_address']
  end

  # Get signature from model unless specified below
  signature = nil
  signature ||= $evm.object['signature']

  # Get Escalation QB url
  eqb_record_url = nil
  eqb_record_url ||=vm.custom_get("eqb_record_url")

  # Set email Subject
  subject = "Your virtual machine request has Completed - VM: #{vm['name']}"


  # Set the opening body to Hello
  body = "Hello, "

  # VM Provisioned Email Body
  body += "<br><br>Your request to provision a virtual machine was approved and completed on #{Time.now.strftime('%A, %B %d, %Y at %I:%M%p')}. "
  body += "<br><br>Virtual machine #{vm['name']}<b> will be available in approximately 15 minutes</b>. "
  body += "<br><br>Plase note: The Opsware software policy remediation is still in progress.  Cloudforms will continue to monitor the policy remediation and will provide another update when the polciy remediation has finished."
  body += "<br><br>For Windows VM access is available via RDP and for Linux VM access is available via putty/ssh, etc. Or you can use the Console Access feature found in the detail view of your VM. "
  body += "<br><br>This VM will automatically be retired on #{vm['retires_on'].strftime('%A, %B %d, %Y')}, unless you request an extension. " if vm['retires_on'].respond_to?('strftime')
  body += " You will receive a warning #{vm['reserved'][:retirement][:warn]} days before #{vm['name']} set retirement date." if vm['reserved'] && vm['reserved'][:retirement] && vm['reserved'][:retirement][:warn]
  body += " As the designated owner you will receive expiration warnings at this email address: #{orig_to}"
  #this comes up empty
  parent_ip = $evm.object['parent_ip']
  body += "<br><br>If you are not already logged in, you can access and manage your virtual machine here <a href='https://cloudforms.intuit.net/vm_or_template/show/#{vm['id']}'>https://cloudforms.intuit.net/vm_or_template/show/#{vm['id']}'</a>"

  if !eqb_record_url.blank?
    body += "<br><br> A partial Escalation Quickbase record has been created - please complete the record at: #{eqb_record_url}"
  end
  body += "<br><br> If you have any issues with your new virtual machine please contact Support."
  body += "<br><br> Thank you,"
  body += "<br> #{signature}"

  # Send email
  log(:info, "Sending email to <#{to}> from <#{from}> subject: <#{subject}>")
  $evm.execute(:send_email, to, from, subject, body)

  log(:info, "Main: exiting main method")
  exit MIQ_OK

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
