###################################
#
# CFME Automate Method: set_unique_vm_name
#
# Author: Jason Dillaman
#
# Notes: Uses a Postgres advisory lock to ensure that VM names are uniquely assigned
#
# Inputs: miq_provision
#
###################################

require 'set'

LOCK_TIMEOUT_SECS = 600
NAME_SEQUENCE_REGEX = /\$n\{(\d+)\}/

def dump_root()
  $evm.log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Root:<$evm.root> End $evm.root.attributes")
  $evm.log(:info, "")
end

def error(msg)
  $evm.log(:error, msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def inject_with_advisory_lock(object)
  with_advisory_lock_code = <<EOF
  require 'zlib'
  require 'securerandom'

  def intuit_try_advisory_lock(hashcode)
    intuit_execute_successful?('pg_try_advisory_lock', hashcode)
  end

  def intuit_release_advisory_lock(hashcode)
    intuit_execute_successful?('pg_advisory_unlock', hashcode)
  end

  def intuit_execute_successful?(pg_function, hashcode)
    sql = "SELECT \#{pg_function}(\#{hashcode}, 0) AS \#{intuit_unique_column_name()}"
    result = connection.select_value(sql)
    (result == 't' || result == true)
  end

  def intuit_stable_hashcode(input)
    # Postgres requires a 31bit hashcode
    Zlib.crc32(input.to_s) & 0x7fffffff
  end

  def intuit_unique_column_name
    "t\#{SecureRandom.hex}"
  end
EOF
  object.object_send(:instance_eval, with_advisory_lock_code)
end

def with_advisory_lock(object, lock_name, timeout, &block)
  $evm.log(:info, "Attempting to acquire database advisory lock")
  hashcode = object.object_send(:intuit_stable_hashcode, lock_name)

  give_up_at = Time.now + timeout
  while Time.now < give_up_at do
    sleep(rand(0.1..15.0))
    $evm.log(:info, "An attempt to acquire database advisory lock for #{hashcode}")
    if object.object_send(:intuit_try_advisory_lock, hashcode)
      begin
        $evm.log(:info, "Acquired database advisory lock")
        yield
      ensure
        object.object_send(:intuit_release_advisory_lock, hashcode)
        $evm.log(:info, "Released database advisory lock")
      end
      return
    end
  end
  error("Failed to acquire database advisory lock")
end

def get_miq_provision()
  prov = $evm.root['miq_provision']
  error("Unable to retrieve provision task") unless prov
  prov
end

def get_requested_vm_name_components(prov)
  vm_name = prov.get_option(:vm_name).to_s.strip
  error("Requested VM name is blank") if vm_name.blank?

  vm_name_components = {
      :name => vm_name
  }

  if vm_name =~ NAME_SEQUENCE_REGEX
    vm_name_components.merge!({
                                  :prefix       => $`.to_s.downcase,
                                  :suffix       => $'.to_s.downcase,
                                  :index        => $&,
                                  :index_length => $1.to_i
                              })
  end
  $evm.log(:info, "Requested VM name: [#{vm_name_components}]")
  vm_name_components
end


def integer_to_name_index(index, index_length)
  return index.to_s.rjust(index_length, '0') if index < 100
  alpha_array = ("a".."z").to_a

  index -= 100
  result = ""
  until index.zero?
    index, r = index.divmod(26)
    result.prepend(alpha_array[r])
  end
  result.rjust(index_length, 'a')
end

def name_index_to_integer(name_index)
  return name_index.to_i if name_index.match(/\d+/)
  alpha_array = ("a".."z").to_a

  result = 0
  name_index = name_index.dup
  until (chr = name_index.slice!(0)).nil?
    result *= 26
    result += alpha_array.index(chr.downcase)
  end
  result + 100
end

def escape_postgres_string(value)
  value.gsub('_', '\_')
end

def get_vm_name_start_index(vm_name_prefix, index_length, vm_name_suffix)
  used_names = []
  start_idx = 0
  reg_val = Regexp.new("(#{Regexp.escape(vm_name_prefix)})([a-z0-9]{#{index_length}})(#{Regexp.escape(vm_name_suffix)})(?:\..+)?")
  $evm.vmdb('miq_provision').all.each do |p|
    used_names << {
        :name   => p.options[:vm_target_name],
        :active => p.miq_request.status == 'Error' || p.miq_request.state != 'finished'
    } if p.options[:vm_target_name].to_s.strip =~ reg_val
  end

  $evm.vmdb('vm_or_template').find(:all, :conditions => ['lower(name) ~ ?', "#{Regexp.escape(vm_name_prefix)}[a-z0-9]{#{index_length}}#{Regexp.escape(vm_name_suffix)}(\..+)?"]).each do |p|
    used_names << {
        :name   => p.name,
        :active => true
    } if p.name.to_s.strip =~ reg_val
  end

  used_name_indices = Set.new
  used_names.each do |used_name|
    next unless used_name[:name].match(reg_val)

    name_idx = name_index_to_integer($2.downcase)
    used_name_indices << name_idx if used_name[:active]
    start_idx = name_idx + 1 if name_idx >= start_idx
  end

  $evm.log(:info, "Determined VM name start index from provision requests: [#{start_idx}]")
  return start_idx, used_name_indices
end

def generate_unique_vm_name(prov, vm_name, vm_name_prefix, index_length, vm_name_suffix)
  start_index, used_name_indices = get_vm_name_start_index(vm_name_prefix, index_length, vm_name_suffix)

  name_index = integer_to_name_index(start_index, index_length)
  return "#{vm_name_prefix}#{name_index}#{vm_name_suffix}" if name_index.length == index_length

  # Need to search for a free spot in the used names list
  0.upto((10**2 + 26**2) - 1) do |x|
    next if used_name_indices.include?(x)
    return "#{vm_name_prefix}#{integer_to_name_index(x, index_length)}#{vm_name_suffix}"
  end
  error("Failed to generate unique VM name: [#{vm_name}]")
end

begin
  dump_root()

  exit MIQ_OK if $evm.root['miq_provision_request']

  prov = get_miq_provision()
  requested_vm_name_components = get_requested_vm_name_components(prov)

  vm_name = nil
  if requested_vm_name_components[:index].blank?
    vm_name = requested_vm_name_components[:name]
    $evm.log(:info, "Using provided VM name: [#{vm_name}]")
  else
    inject_with_advisory_lock(prov)
    with_advisory_lock(prov, requested_vm_name_components[:name], LOCK_TIMEOUT_SECS) do
      vm_name = generate_unique_vm_name(
          prov,
          requested_vm_name_components[:name],
          requested_vm_name_components[:prefix],
          requested_vm_name_components[:index_length],
          requested_vm_name_components[:suffix])

      $evm.log(:info, "Generated unique VM name: [#{vm_name}]")

      # Set the target name while holding the lock to prevent the race
      prov.set_option(:vm_target_name, vm_name)
    end
  end

  error("Failed to set unique VM name") if vm_name.blank?
  $evm.current['vmname'] = vm_name

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
