###################################
# **** USED IN INTUIT VM RETIRE PROCESS ****
###################################
#
# CFME Automate Method: VMRetirement_Emails
#
# Notes: This method sends out retirement emails when the following events are raised:
#
# Events: vm_retire_warn, vm_retired, vm_entered_retirement, vm_retire_initiated
#
# Model Notes:
# 1. to_email_address - used to specify an email address in the case where the
#    vm's owner does not have an  email address. To specify more than one email
#    address separate email address with commas. (I.e. admin@example.com,user@example.com)
# 2. from_email_address - used to specify an email address in the event the
#    requester replies to the email
# 3. signature - used to stamp the email with a custom signature
#
###################################
#
# 2015-11-11 - ew - adding vm_retire_initiated message
#
####################################
require 'socket'

begin
  # Method for logging
  def log(level, message)
    @method = 'VMRetirement_Emails'
    $evm.log(level, "#{@method} - #{message}")
  end

  # dump_root
  def dump_root()
    log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
    $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
    log(:info, "Root:<$evm.root> End $evm.root.attributes")
    log(:info, "")
  end

  # dump_vm
  def dump_vm( vm )
    log(:info, "VM:<#{vm.name}> Begin vm.attributes")
    vm.attributes.sort.each { |k, v| log(:info, "VM:<#{vm.name}> Attribute - #{k}: #{v}")}
    log(:info, "VM:<#{vm.name}> End vm.attributes")
    log(:info, "")
  end

  # dump_vm_tags
  def dump_vm_tags( vm )
    log(:info, "VM:<#{vm.name}> Begin vm.tags")
    vm.tags.sort.each { |k, v| log(:info, "VM:<#{vm.name}> Tag - #{k}: #{v}")}
    log(:info, "VM:<#{vm.name}> End vm.tags")
    log(:info, "")
  end

  log(:info, "CFME Automate Method Started")
  log(:info, "Intuit / Retirement / VMMethods / VMRetirement_Emails")

  def lab_env?
    begin
      log(:info, "Is this a lab environment")
      hostname = Socket.gethostbyname(Socket.gethostname).first
      hostname.downcase!
      log(:info, "is this the king of the lab: #{hostname}")
      return hostname.include?("corplab")
    rescue SignalException, StandardError => e
      log(:info, "Unable to tell if this is the lab, defaulting to prod")
      return false
    end
  end

  # dump all root attributes to the log
  dump_root

  # Look in the current object for a VM
  vm = $evm.object['vm']
  if vm.nil?
    vm_id = $evm.object['vm_id'].to_i
    vm = $evm.vmdb('vm', vm_id) unless vm_id == 0
  end

  # Look in the Root Object for a VM
  if vm.nil?
    vm = $evm.root['vm']
    if vm.nil?
      vm_id = $evm.root['vm_id'].to_i
      vm = $evm.vmdb('vm', vm_id) unless vm_id == 0
    end
  end

  #
  # Look in the Root Object for a Provision/Request
  #
  prov = $evm.root['miq_provision_request'] || $evm.root['miq_provision']
  if vm.nil?
    vm = prov.vm unless prov.nil?
  end

  #
  # No VM Found, exit
  #
  raise "VM not found" if vm.nil?

  # Dump VM attributes
  dump_vm( vm )
  dump_vm_tags( vm )

  # Get VM Name
  vm_name = vm.name
  log(:info, "vm_name: #{vm_name}")

  # Look at the Event Type in the Current Object or in the Root Object
  event_type = $evm.object['event'] || $evm.root['event_type']
  log(:info, "event_type: #{event_type}")

  # Override the default appliance below
  appliance = nil
  appliance ||= $evm.root['miq_server'].name
  log(:info, "appliance: #{appliance}")

  # Get VM Owner Name and Email
  evm_owner_id = vm.attributes['evm_owner_id']
  owner = nil
  owner = $evm.vmdb('user', evm_owner_id) unless evm_owner_id.nil?
  log(:info, "owner: #{owner}")

  # to_email_address from owner.email then from model if nil
  to = nil
  to ||= prov.get_option(:owner_email) unless prov.nil?
  to ||= owner.email unless owner.nil?
  to ||= $evm.object['to_email_address']

  # Also send message to CF Engineering  (engineering_email_address)
  engineering_email_address = nil
  engineering_email_address ||= $evm.object['engineering_email_address']
  engineering_email_address ||= "TechCloudforms_Support@intuit.com"
  to += ",#{engineering_email_address}"

  # Also send message to CF Operations  (operations_email_address)
  operations_email_address = $evm.object['operations_email_address']
  to += ",#{operations_email_address}"

  # Get from_email_address from model unless specified below
  from = nil
  if lab_env?
    from ||= $evm.object['from_email_address_lab']
  else
    from ||= $evm.object['from_email_address']
  end

  # Get signature from model unless specified below
  signature = nil
  signature ||= $evm.object['signature']

  ######################################
  #
  # VM Retirement Warning Email
  #
  ######################################
  if event_type == "vm_retire_warn"

    # email subject
    subject = "VM Retirement Warning for #{vm_name}"

    # Build email body
    body = "Hello, "
    body += "<br><br>Your virtual machine: [#{vm_name}] will be retired on [#{vm['retires_on']}]."
    body += "<br><br>If you need to use this virtual machine past this date please request an extension by contacting Support."
    body += "<br><br> Thank you,"
    body += "<br> #{signature}"
  end


  ######################################
  #
  # VM Retirement Exended Email
  #
  ######################################
  if event_type == "vm_retire_extend"

    # email subject
    subject = "VM Retirement Extended for #{vm_name}"

    # Build email body
    body = "Hello, "
    body += "<br><br>Your virtual machine: [#{vm_name}] will now be retired on [#{vm['retires_on']}]."
    body += "<br><br>If you need to use this virtual machine past this date please request an extension by contacting Support."
    body += "<br><br> Thank you,"
    body += "<br> #{signature}"
  end

  ######################################
  #
  # VM has entered Retirement Email
  #
  ######################################
  if event_type == "vm_entered_retirement"

    # email subject
    subject = "VM #{vm_name} has entered retirement"

    # Build email body
    body = "Hello, "
    body += "<br><br>Your virtual machine named [#{vm_name}] has been retired."
    body += "<br><br>You will have up to 3 days to un-retire this VM. Afterwhich time the VM will be deleted."
    body += "<br><br> Thank you,"
    body += "<br> #{signature}"
  end

  ######################################
  #
  #  VM Retirement Email
  #
  ######################################
  if event_type == "vm_retired"

    # email subject
    subject = "VM Retirement Alert for #{vm_name}"

    # Build email body
    body = "Hello, "
    body += "<br><br>Your virtual machine named [#{vm_name}] has been retired."
    body += "<br><br> Thank you,"
    body += "<br> #{signature}"
  end

  ######################################
  #
  #  VM Retire Initiated Email
  #
  ######################################
  if event_type == "vm_retire_initiated"

    # email subject
    subject = "VM Retirement Initiated for #{vm_name}"

    # Build email body
    body = "Hello, "
    body += "<br><br>Your virtual machine named [#{vm_name}] has entered the retirement process."
    body += "<br><br>Thank you,"
    body += "<br>#{signature}"
  end

  #
  # Send email
  #
  log(:info, "Sending email to <#{to}> from <#{from}> subject: <#{subject}>")
  $evm.execute('send_email', to, from, subject, body)


  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

    # Set Ruby rescue behavior
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_STOP
end
