###################################
#
# CFME Automate Method: get_input_validation_errors
# Author: Barry and Mai 2015-10-7
#
# Notes: Verifies input within a single tier on a dialog dorm. 
#        To add additional input, modify 3 locations:
#         1 - This method's $evm.inputs[] in the code 
#         2 - This method's inputs in Cloudforms UI
#         3 - This method's Instance value in Cloudforms UI
# 
#
# Inputs: 
#   $evm.inputs['number_of_vms'],
#   $evm.inputs['data_host_names'],
#   $evm.inputs['data_ip_addresses'],
#   $evm.inputs['mgmt_ip_addresses'],
#   $evm.inputs['mgmt_host_names'],
#   $evm.inputs['storage_ip_addresses'],
#   $evm.inputs['storage_host_names'],
#   $evm.inputs['hosting_segment'], 
#   $evm.inputs['intuit_tier_name'],   
#   $evm.inputs['intuit_template'], 
#   $evm.inputs['create_storage_nic'],
#   $evm.inputs['software_components'],
#   $evm.inputs['security_zone'],
#   $evm.inputs['target_epg'],
#   $evm.inputs['target_esx_hosts'],
#   $evm.inputs['owner_email']
#
# Mods:
#
###################################

require 'restclient'
require 'uri'
require 'json'
require 'resolv'

def log(level, msg)
  @method = 'get_input_validation_errors_tier1'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, "ERROR: #{msg}")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def retry_method(msg)
  log(:error, "RETRY: #{msg}")
  $evm.root['ae_result'] = 'retry'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def to_bool(value)
  return true if value == true || value =~ (/^(true|t|yes|y|1)$/i)
  return false if value == false || value.blank? || value =~ (/^(false|f|no|n|0|null)$/i)
  raise ArgumentError.new("invalid value for Boolean: \"#{value}\"")
end

def build_dialog_values(lf_array)
  # CFME does not properly handle sorted dynamic fields
  list_array = [['','']] + lf_array
  list_hash = {}
  list_array.each{ |k,v| list_hash.merge!({v=>k})}

  log(:info, "List array: #{list_array}")

  list_values = {
    'sort_by'       => :none,
    'data_type'     => :string,
    'required'      => false,
    'default_value' => list_array.first.last,
    'values'        => list_hash
  } 

  $evm.log('info', "Template drop-down: [#{list_values}]")
  list_values.each { |k,v| $evm.object[k] = v }
end

def valid_hostname?(name)
  hostname_regex = /^[a-zA-Z0-9]+([\-\.]{1}[a-zA-Z0-9]+)*(\.[a-z]{2,5})$/
  # We check for three dots as per Intuit's Standard for DNS zones
  return false if (name.count('.') != 3) || name.length > 63 || (hostname_regex =~ name).nil?
  return true 
end

def valid_ip_addr?(ip)
  return ip =~ Resolv::IPv4::Regex ? true : false  
end

def add_input_error(error_message)
  # Must insert error messages into dialog in array format with second value of "BREAK"
  @error_array.push [ "#{error_message}" , "BREAK" ]
end

def pluralize(address)
    return address + "s" if address == "hostname"
    return address + "es" if address == "IP address"
    raise ArgumentError.new("Invalid value for pluralize: \"#{value}\"")
end

def validate_data_and_log_errors(group, validation_type, data_string, all_host_names, all_ip_addrs, num_nodes, storage_nic, hosting_segment)
    
    unless data_string.blank?

      if group == "management" && hosting_segment != "qdca"
        add_input_error( @error_msgs[:wrong_management_region] % { group_type: validation_type})
        return
      end
      data_values = data_string.split(",").each do |value| 
        value.strip! 
        if validation_type == "hostname"
          valid_hostname?(value) == true ? all_host_names << value : add_input_error( @error_msgs[:invalid_hostname] % { group: group.capitalize, hostname: value })
        end
        if validation_type == "IP address"
          valid_ip_addr?(value) == true ? all_ip_addrs << value : add_input_error( @error_msgs[:invalid_ip] % { group: group.capitalize, ip: value }) 
        end
      end
      if group == "storage" && !data_values.blank? && !storage_nic
          add_input_error( @error_msgs[:no_storage_nic] % { group_type: pluralize(validation_type) })
      end          
      add_input_error( @error_msgs[:not_equal_to_nodes] % { group: group.capitalize , group_type: "#{ pluralize(validation_type)}" }) unless data_values.length == num_nodes
    
    end
end

begin

    log(:info, "Entering Get Input Validation Errors")
    log(:info, "Raw Inputs: #{$evm.inputs.inspect}")

    # Form Fields to be validated  
    input_fields = {
      num_nodes: $evm.inputs['number_of_vms'],
      data_hostnames:  $evm.inputs['data_host_names'],
      data_ip_addresses: $evm.inputs['data_ip_addresses'],
      management_ip_addresses: $evm.inputs['mgmt_ip_addresses'],
      management_hostnames: $evm.inputs['mgmt_host_names'],
      storage_ip_addresses: $evm.inputs['storage_ip_addresses'],
      storage_hostnames:  $evm.inputs['storage_host_names'],

      hosting_segment: $evm.inputs['hosting_segment'], 
      tier_name: $evm.inputs['intuit_tier_name'],
      software_component: $evm.inputs['intuit_software_component'],
      security_zone: $evm.inputs['security_zone'],   
      template: $evm.inputs['intuit_template'], 
      storage_nic: $evm.inputs['create_storage_nic'],
      target_epg: $evm.inputs['target_epg'],
      target_esx_hosts: $evm.inputs['target_esx_hosts'],
      owner_email: $evm.inputs['owner_email']
    }

    # Remove "_blank_" from all fields
    input_fields.each {|k, v| input_fields[k] = v.gsub "_blank_", ""  }

    # Correct type for certain fields
    input_fields[:num_nodes] = input_fields[:num_nodes].to_i
    input_fields[:storage_nic] = to_bool(input_fields[:storage_nic])

    log(:info, "Processed Inputs: #{input_fields.inspect}")
    
    # Dialog Error Messages

    @error_msgs = {
     invalid_ip: "Invalid %{group} IP Address : %{ip}",
     invalid_hostname: "Invalid %{group} Hostname : %{hostname}",
     no_storage_nic: "Must select 'Create Storage NIC' to specify Storage %{group_type}",
     wrong_management_region: "Must be in Region QDC-A to specify Management %{group_type}",
     not_equal_to_nodes: "Number of %{group} %{group_type} does not match number of nodes",
     invalid_email_format: "Email is formatted incorrectly",
     dup_hosts: "Duplicated Host Names: %{hostnames}",
     dup_ip_addrs: "Duplicated IP Addresses: %{ips}",
     blank_tier: "Tier Name field must contain a value",
     unapproved_qdca_template: "Invalid Template type for Hosting Segment QDCA",
     no_target_epg: "Must supply Target EPG for FAB Hosting Segments",
     invalid_software_component: "Software Component must not be blank or '-' ",
     no_security_zone: "Must supply a Security Zone"
    }

    # Error Reporting Array
    @error_array = Array.new

    # Contains all valid hostnames/IPs to check for duplicates
    all_host_names = Array.new
    all_ip_addrs = Array.new


    #Test: Email contains single \@ ; \.  ; non-whitespace chars ; letters-only top domain
    email_regex = /^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9.]+$/
    add_input_error( @error_msgs[:invalid_email_format] ) unless (email_regex=~ input_fields[:owner_email]) == 0

    #Tests: Valid Hostnames/IPs, Mgmt Hosts/IP are in QDCA, Storage NIC is checked for Storage Hosts/IPS, Number of nodes and Hosts/IPs are equivalent
    text_box_groups = ["data", "management", "storage"]

    text_box_groups.each do |group|
      validate_data_and_log_errors(group, "hostname", input_fields["#{group}_hostnames".to_sym], all_host_names, all_ip_addrs, input_fields[:num_nodes], input_fields[:storage_nic], input_fields[:hosting_segment])
      validate_data_and_log_errors(group, "IP address", input_fields["#{group}_ip_addresses".to_sym], all_host_names, all_ip_addrs, input_fields[:num_nodes],input_fields[:storage_nic], input_fields[:hosting_segment])
    end

    # Test: Are all host names and IPs unique on the form? 
    repeated_hostnames = all_host_names.select{ |hostname| all_host_names.count(hostname) > 1 }.uniq
    add_input_error( @error_msgs[:dup_hosts] % {hostnames: repeated_hostnames }) unless repeated_hostnames.blank?

    repeated_ip_addrs = all_ip_addrs.select{ |ip| all_ip_addrs.count(ip) > 1 }.uniq
    add_input_error( @error_msgs[:dup_ip_addrs] % {ips: repeated_ip_addrs }) unless repeated_ip_addrs.blank?

    #Test:  Tier name is not blank
    add_input_error( @error_msgs[:blank_tier] )  if input_fields[:tier_name].blank?

    #Test: Software Component is not blank or a ' - ' 
    component = input_fields[:software_component]
    add_input_error( @error_msgs[:invalid_software_component] ) if component.blank? || component == "-"

    #Test: Security Zone is not blank 
    add_input_error( @error_msgs[:no_security_zone] )  if input_fields[:security_zone].blank?

    #Test: Region QDC-A is currently limited to 4 templates
    qdca_allowed_oses = ["os_rhel_6u4" ,
                         "os_rhel_5u9", 
                         "os_win_2008_r2_sp1_ent_5_1_3", 
                         "os_win_2008_r2_sp1_standard_5_1_3"]

    if input_fields[:hosting_segment] == "qdca" && !qdca_allowed_oses.include?(input_fields[:template])
      add_input_error( @error_msgs[:unapproved_qdca_template] )
    end


    #Test: Target EPG supplied if QCY-Fab or LAS-Fab 
    target_epg_hosting_segments = [ "qcyf01", "lasf01"]
    if target_epg_hosting_segments.include?(input_fields[:hosting_segment]) && input_fields[:target_epg].blank?
      add_input_error( @error_msgs[:no_target_epg]) 
    end

    evm = $evm.vmdb('ext_management_system').all

    # No errors
    if @error_array.empty?
        @error_array.push ["No Errors Found", "BREAK"]
    end

    log(:info, "Final Array: #{@error_array}")
    build_dialog_values(@error_array)

rescue => err
     error("[#{err}]\n#{err.backtrace.join("\n")}")
end
