###################################
#
# CFME Automate Method: get_software_components
#
# Author: Jason Dillaman
#a
# Notes: Retrieve a list of all optional software components for the current template
#
# Inputs: intuit_template
#
###################################

PHAROS_SOFTWARE_COMPONENTS_CONFIG_URI = 'Intuit/Integration/PharosSoftwareComponents/Configuration'
INTUIT_SOFTWARE_COMPONENT_URI = "#{PHAROS_SOFTWARE_COMPONENTS_CONFIG_URI}/intuit_software_component"
INTUIT_TEMPLATE_URI = "#{PHAROS_SOFTWARE_COMPONENTS_CONFIG_URI}/intuit_template"

# logging convenience method
def log (level, msg)
  @method = 'get_software_components'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def get_intuit_software_components()
  software_components = {}
  $evm.instance_find("#{INTUIT_SOFTWARE_COMPONENT_URI}/*").collect do |intuit_software_component, _|
    software_component = $evm.instantiate("#{INTUIT_SOFTWARE_COMPONENT_URI}/#{intuit_software_component}")
    
    unless software_component
      log(:warn, "Failed to load software component: [#{intuit_software_component}]")
      next
    end
    
    log(:info, "Discovered Software Component: [#{intuit_software_component}][#{software_component.attributes}]")
    software_components[intuit_software_component] = software_component.attributes
  end
  return software_components
end

def get_template(intuit_template)
  template = $evm.instantiate("#{INTUIT_TEMPLATE_URI}/#{intuit_template}")
  error("Unable to locate template: [#{intuit_template}]") unless template
  log(:info, "Retrieved template configuration: [#{intuit_template}][#{template.attributes}]")
  template.attributes
end

def get_template_software_components(template)
  log(:info, "Template software components: [#{template['name']}][#{template['optional_intuit_software_components']}]")
  template['optional_intuit_software_components'] || []
end

def build_software_component_combinations(software_components, template_software_components)
  software_component_combos = []
  (1..template_software_components.length).each do |n|
    template_software_components.combination(n) do |combo|
      combo = combo.collect do |intuit_software_component|
        software_components[intuit_software_component].merge('intuit_software_component' => intuit_software_component)
      end.compact.sort_by do |software_component|
        software_component['name']
      end
      
      software_component_combos << {
        :id   => combo.collect { |x| x['intuit_software_component']}.join('|'),
        :name => combo.collect { |x| x['name']}.join(', ')
      }
    end
  end

  log(:info, "Software Component Combinations: [#{software_component_combos}]")
  software_component_combos
end

def build_dialog_values(software_components)
  # CFME does not properly handle sorted dynamic fields
  values = software_components.sort_by { |x| x[:name] }.to_a.collect { |x| [x[:name], x[:id]] }

  list_values = {
    'sort_by'       => :none,
    'data_type'     => :string,
    'required'      => values.length > 0,
    'values'        => [['OS-only', 'os_only']] + values
  }

  log('info', "Software Components drop-down: [#{list_values}]")
  list_values.each { |k,v| $evm.object[k] = v }
end

begin
  software_components = get_intuit_software_components()

  intuit_template = $evm.inputs['intuit_template']
  intuit_software_component_combos = []
  if intuit_template != '_blank_'
    template = get_template(intuit_template)
    template_software_components = get_template_software_components(template)
    intuit_software_component_combos = build_software_component_combinations(software_components, template_software_components)
  end

  build_dialog_values(intuit_software_component_combos)
  
rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
