##################################
#
# CFME Automate Method: get_templates
#
# Author: Jason Dillaman, Daniel Trieu
#
# Notes: Retrieve a list of all online templates
#
# Changes:
#   2015-12-03 Daniel Trieu updated the instance ref to Customer domain
#                           updated the values in list_values to comply with CF3.2
#
###################################

PHAROS_SOFTWARE_COMPONENTS_CONFIG_URI = '/Customer/Intuit/Integration/PharosSoftwareComponents/Configuration'
INTUIT_SOFTWARE_COMPONENT_URI = "#{PHAROS_SOFTWARE_COMPONENTS_CONFIG_URI}/intuit_software_component"
INTUIT_TEMPLATE_URI = "#{PHAROS_SOFTWARE_COMPONENTS_CONFIG_URI}/intuit_template"

def log(level, msg)
  @method = 'get_templates'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def dump_root()
  log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  log(:info, "Root:<$evm.root> End $evm.root.attributes")
  log(:info, "")
end

def valid_template?(template)
  template_name = template['name']
  log(:warn, "Template flagged as offline: [#{template_name}]") unless template['online']
  template['online'] &&
      valid_software_components?(template_name, template['required_software_components']) &&
      valid_software_components?(template_name, template['optional_software_components'])
end

def valid_software_component(software_component)
  software_component = $evm.instantiate("#{INTUIT_SOFTWARE_COMPONENT_URI}/#{software_component}")
  software_component
end

def valid_software_components?(template_name, software_components)
  software_components ||= []
  invalid_components = software_components.reject do |software_component|
    valid_software_component?(software_component)
  end
  log(:warn, "Detected invalid software components: [#{template_name}][#{invalid_components}]") unless invalid_components.blank?
  invalid_components.blank?
end

def valid_template_image?(template_image_names, image_name)
  valid = template_image_names.find { |template_name| template_name.match(/^#{Regexp.escape(image_name)}.*$/) }
  log(:info, "Detected invalid template image: [#{image_name}]") unless valid
  valid
end

def get_valid_template_image_names()
  template_image_names = []
  $evm.vmdb('miq_template').all.each do |template|
    template_image_names << template.name if !template.archived && !template.orphaned && !template.retired &&
        !template.tags('intuit_hpsa_agent_url').blank? &&
        !template.tags('intuit_hpsa_baseline').blank? &&
        template.tagged_with?('prov_scope','all')
  end
  log(:info, "Valid template images: [#{template_image_names}]")
  template_image_names
end

def get_templates(template_image_names)
  $evm.instance_find("#{INTUIT_TEMPLATE_URI}/*").collect do |intuit_template, _|
    template = $evm.instantiate("#{INTUIT_TEMPLATE_URI}/#{intuit_template}")
    log(:info, "Discovered Template: [#{template.attributes}]")

    next unless valid_template?(template)
    next unless valid_template_image?(template_image_names, template['image_name'])
    template['intuit_template'] = intuit_template
    template
  end.compact
end

def build_dialog_values(templates)
  # CFME does not properly handle sorted dynamic fields
  values = templates.sort_by { |x| x['name'] }.to_a.collect { |x| [x['intuit_template'], x['name']] }

  values_array = [['','']] + values
  values_hash = {}
  values_array.each { |k,v| values_hash.merge!({k=>v})}

  list_values = {
      'sort_by'       => :none,
      'data_type'     => :string,
      'required'      => true,
      'default_value' => '',
      'values'        => values_hash
  }

  log('info', "Template drop-down: [#{list_values}]")
  list_values.each { |k,v| $evm.object[k] = v }
end

begin
  dump_root()

  template_image_names = get_valid_template_image_names()
  templates = get_templates(template_image_names)
  build_dialog_values(templates)

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
