#
# EVM Automate Method: Auto_Tag
#
# Notes: This method is used to Auto Tag VMs based on ASCII file content
# extracted from VM Analysis
#
#
###################################
begin
  @method = 'Auto_Tag'
  $evm.log("info", "===== EVM Automate Method: <#{@method}> Started")

  # Turn of verbose logging
  @debug = true


  #
  # Method: Process Categories & Tags
  #
  def process_categories( vm, category, tag )

    # Convert to lower case and replace all non-word characters with underscores
    category_name = category.downcase.gsub(/\W/, '_')
    tag_name = tag.downcase.gsub(/\W/, '_')
    $evm.log("info", "Converted category name: <#{category_name}>")
    $evm.log("info", "Converted tag name: <#{tag_name}>")

    # if the category exists else create it
    if $evm.execute('category_exists?', category_name)
      $evm.log("info", "Category <#{category_name}> exists")
    else
      $evm.log("info", "Category <#{category_name}> doesn't exist, creating category")
      $evm.execute('category_create', :name => category_name, :single_value => true, :description => "#{category}")
    end

    # if the tag exists else create it
    if $evm.execute('tag_exists?', category_name, tag_name)
      $evm.log("info", "Tag <#{tag_name}> exists")
    else
      $evm.log("info", "Adding new tag <#{tag_name}> in Category <#{category_name}>")
      $evm.execute('tag_create', category_name, :name => tag_name, :description => "#{tag}")
    end

    # Tag VM with category/tag information
    if vm.tagged_with?(category_name,tag_name)
      $evm.log("info", "VM already tagged with <#{tag_name}> in Category <#{category_name}>")
    else
      $evm.log("info", "Tagging VM with new <#{tag_name}> tag in Category <#{category_name}>")
      vm.tag_assign("#{category_name}/#{tag_name}")
    end
  end


  #
  # Method: Update VC Custom Attributes
  #
  def process_customattributes( vm, category, tag )
    custom_value = vm.ems_custom_get(category)

    # Only apply EMS custom attributes if they are different or do not exist
    unless custom_value == tag
      $evm.log("info", "Updating EMS Custom Attribute <#{category}> with <#{tag}>")
      vm.ems_custom_set(category, tag)
    else
      $evm.log("info", "Skipping Update of EMS Custom Attribute <#{category}> with <#{tag}>")
    end
  end


  #
  # Find the VM
  #
  vm = $evm.root['vm']
  return if vm.nil?


  # Regular Expression to match the following construct:
  # 000A0500Business Unit|CORE
  # 1) ^\w{8} - match the 1st 8 word characters
  # 2) (\D.*) - match the next alpha character and up to the pipe
  # 3) (.*) - match everything after the pipe
  reg_ex = /^\w{8}(\D.*)\|(.*)/

  # Log the current custom attributes on the VM
  custom_keys = vm.ems_custom_keys
  custom_keys.each do |ck|
    ck_val = vm.ems_custom_get(ck)
    $evm.log("info", "Found EMS Custom Attribute: <#{ck}> with value: <#{ck_val}>") if @debug
  end

  # Loop and display file content only for targeted file
  vm_files = vm.files
  vm_files.each do |f|
    # if the file name "computer.mnv" collected in the analysis profile exists
    # note: you can also specify a absolute path as well i.e. "c:/progra~1/CA/computer.mnv"
    if f.name.downcase.include?("computer.mnv")
      contents = f.contents
      lines = contents.split("\n")

      $evm.log("info", "File Name: <#{f.name}>")
      $evm.log("info", "File Size: <#{f.size}>")
      $evm.log("info", "File lines: <#{lines.length}>")
      lines.each_with_index do |line, idx|
        # if the regular expression successfully matches else skip the line
        if reg_ex =~ line.chomp
          category = $1
          tag = $2
          $evm.log("info", "Processing Line#: <#{idx+1}> Line: <#{line}> Category: <#{category}> Tag: <#{tag}>")
          process_categories(vm, category, tag)
          process_customattributes(vm, category, tag)
        else
          $evm.log("info", "Skipping Line#: <#{idx+1}> Line: <#{line}>")
        end
      end
    end
  end


  #
  # Exit method
  #
  $evm.log("info", "===== EVM Automate Method: <#{@method}> Ended")
  exit MIQ_OK

  #
  # Set Ruby rescue behavior
  #
rescue => err
  $evm.log("error", "<#{@method}>: [#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
