##################################
#
# EVM Automate Method: cluster_cpu_utilization
#
# Notes:  This method will attempt to power on or off Grid Application VM based on an alert
#         It is called in three ways; high cpu utilization, low cpu utilization, and hourly for business hours check
#  The criteria is:
#                      Not business hours: (currently for testing between the hours of 5 and 17)
#                      Cluster is tagged with tag category: eligible tag:  true
#                      Vms within the cluster are tagged with category:  application:  tag: grid
#                      Check for power state
#  Code changed and consolidated based on updated logic flow:
#                      1. Is cluster eligible?
#                      2. Are any VMs tagged as grid?
#                      3. Is it a weekend?
#                       if so, is weekend processing allowed?
#                       if not, is it during business hours?
#                      4. how did we get here?  utilization values? - high, low , periodic
#                       periodic means that we are here as a result of a business hours hourly alert
#                       and that means that we are trying to shut down grid vms when they shouldnt be running.
###################################
begin
  @method = 'cluster_cpu_utilization'

  $evm.log("info", "GRIDPROCESSING ===== EVM Automate Method: <#{@method}> Started")
  # Turn on verbose logging
  debug = true
  def get_eligible_vms(ems_cluster, current_power_state)
    grid_vms = nil
    hosts = ems_cluster.hosts
    hosts.each  do  |h|
      vms = h.vms
      if current_power_state == "all"
        $evm.log("info","GRIDPROCESSING Searching for eligible grid VMs." )
        grid_vms = h.vms.find_all { |v| v.tagged_with?('application', 'grid') }
        $evm.log("info","GRIDPROCESSING Found <#{grid_vms.length}> eligible grid VMs.  Power state not checked. " )
      else
        $evm.log("info","GRIDPROCESSING Searching for eligible grid VMs that are currently powered #{current_power_state}" )
        grid_vms = h.vms.find_all { |v| v.tagged_with?('application', 'grid') &&  v.power_state == current_power_state}
        $evm.log("info","GRIDPROCESSING Found <#{grid_vms.length}> eligible grid VMs that are currently powered #{current_power_state}" )
      end
    end
    return grid_vms
  end
  def power_on_vms(grid_vms, number_of_vms)
    count = 0
    $evm.log("info","GRIDPROCESSING Powering on <#{number_of_vms}> VMs")
    grid_vms.each  do  |g|
      if count < number_of_vms
        $evm.log("info","GRIDPROCESSING Found eligible grid vm: <#{g.name}>")
        $evm.log("info","GRIDPROCESSING Powering on Grid Application workload vm: <#{g.name}> ")
        g.start
        $evm.log("info","GRIDPROCESSING Powered on Grid Application workload vm: <#{g.name}> ")
      end
      count += 1
    end
    return count
  end

  def power_off_vms(grid_vms, number_of_vms)
    count = 0
    return if grid_vms.length == 0
    $evm.log("info","GRIDPROCESSING Powering off <#{number_of_vms}> VMs")
    grid_vms.each  do  |g|
      if count < number_of_vms
        $evm.log("info","GRIDPROCESSING Found eligible grid vm: <#{g.name}>")
        $evm.log("info","GRIDPROCESSING Powering off Grid Application workload vm: <#{g.name}> ")
        g.stop
        $evm.log("info","GRIDPROCESSING Powered off Grid Application workload vm: <#{g.name}> ")
        count += 1
      end
    end
    return count
  end
  def get_grid_options
    params = {}
    $evm.log("info", "GRIDPROCESSING Listing Object Attributes:")
    obj = $evm.object
    obj.attributes.sort.each { |k, v| $evm.log("info", "GRIDPROCESSING \t#{k}: #{v}")}
    $evm.log("info", "GRIDPROCESSING ===========================================")

    params[:number_of_vms] = obj['number_of_vms'].to_i
    # get the number of vms and business hours from the instance
    if params[:number_of_vms].nil?
      $evm.log("info", "GRIDPROCESSING Number of vms not specified in instance - default is 1 VM")
      number_of_vms = 1
    else
      $evm.log("info", "GRIDPROCESSING Instance specified number of vms is : #{params[:number_of_vms]}")
    end
    params[:start_business_hour] = obj['start_business_hour'].to_i
    if params[:start_business_hour].nil?
      $evm.log("info", "GRIDPROCESSING Business hour start not specified in instance - defaulting to 9 am")
      start_business_hour = 9
    else
      $evm.log("info", "GRIDPROCESSING Business hour start specified is : #{params[:start_business_hour]}")
    end
    params[:end_business_hour] = obj['end_business_hour'].to_i
    if params[:end_business_hour].nil?
      $evm.log("info", "GRIDPROCESSING Business hour end not specified in instance - defaulting to 5 pm")
      end_business_hour = 17
    else
      $evm.log("info", "GRIDPROCESSING Business hour end specified is : #{params[:end_business_hour]}")
    end
    params[:weekend_processing] = obj['weekend_processing']
    if params[:weekend_processing].nil?
      $evm.log("info", "GRIDPROCESSING Weekend Processing not specified in instance - defaulting to weekend processing")
      weekend_processing = true
    else
      $evm.log("info", "GRIDPROCESSING Weekend Processing specified is : #{params[:weekend_processing]}")
    end
    # get the utilization type (high, low, periodic)
    params[:utilization_type] = obj['utilization_type'].downcase
    if params[:utilization_type].nil?
      $evm.log("info", "GRIDPROCESSING Utilization type not specified in instance")
    else
      $evm.log("info", "GRIDPROCESSING Utilization type specified is : #{params[:utilization_type]}")
    end
    return params
  end
  ### start here
  options = {}
  options = get_grid_options
  $evm.log("info","GRIDPROCESSING get_grid_options params: #{options.inspect}")
  # 1. Only check for eligible grid cluster on low utilization.  We still want to shut down grid VMs in any case.
  # need the cluster information to check for tag
  ems_cluster = $evm.root['ems_cluster']
  if options[:utilization_type] == 'low'
    unless ems_cluster.tagged_with?('eligible', 'true')
      $evm.log("info","GRIDPROCESSING Attempt to start a Grid Application workload on an ineligible Cluster: <#{ems_cluster.name}>")
      exit MIQ_OK
    end
  end
  # 2. Are any VMs tagged as grid?   Note - no check for current power state here
  grid_vms = get_eligible_vms(ems_cluster,"all")
  eligible_vm_count = grid_vms.length
  if eligible_vm_count == 0
    $evm.log("info","GRIDPROCESSING No Grid VMS found on Cluster: <#{ems_cluster.name}> .")
    exit MIQ_OK
  else
    $evm.log("info","GRIDPROCESSING Found : <#{eligible_vm_count}> VMs.")
  end
  t = Time.now
  $evm.log("info","GRIDPROCESSING The day of the week is : #{t.wday}")
  $evm.log("info","GRIDPROCESSING The hour of the day is : #{t.hour}")
  # 3. Is it the weekend, and if so, is weekend processing allowed?
  if t.wday == 0 || t.wday == 6
    if options[:weekend_processing] == false
      $evm.log("info", "GRIDPROCESSING It is the weekend. Not performing Grid Application processing during weekends.")
      $evm.log("info", "GRIDPROCESSING Find ALL Grid VMs that are powered on and power them off here.")
      grid_vms = get_eligible_vms(ems_cluster,'on')
      eligible_vm_count = grid_vms.length
      $evm.log("info", "GRIDPROCESSING Found <#{eligible_vm_count}> VMs to be powered off.")
      count = power_off_vms(grid_vms, eligible_vm_count)
      exit MIQ_OK
    else
      $evm.log("info", "GRIDPROCESSING It is the weekend, but weekend processing has been specified.")
    end
  else 
    # not the weekend, is it during business hours?
    if t.hour >= options[:start_business_hour] && t.hour <= options[:end_business_hour]-1
      $evm.log("info", "GRIDPROCESSING Current time falls within the business hours specified. Not Performing Grid Application processing during business hours.")
      $evm.log("info", "GRIDPROCESSING Find ALL Grid VMs that are powered on and power them off here.")
      grid_vms = get_eligible_vms(ems_cluster,'on')
      eligible_vm_count = grid_vms.length
      $evm.log("info", "GRIDPROCESSING Found <#{eligible_vm_count}> VMs to be powered off.")
      count = power_off_vms(grid_vms, eligible_vm_count)
      exit MIQ_OK
    end
  end

  # 4. How did we get called?  high util, low util, or periodic business hours/weekend check?
  case options[:utilization_type]
  when 'periodic'
    $evm.log("info", "GRIDPROCESSING Utilization type periodic specified specified - no more work to do: #{options[:utilization_type]}")
    exit MIQ_OK
  when 'high'
    $evm.log("info", "GRIDPROCESSING High Threshold processing  ")
    # Commented by bucklast 19/7/2011
    # $evm.log("info", "GRIDPROCESSING High Threshold processing - number of vms set to 1 ")
    # options[:number_of_vms] = 1
    # End comment block bucklast 19/7/2011
    grid_vms = get_eligible_vms(ems_cluster,'on')
    power_off_vms(grid_vms, options[:number_of_vms])   unless grid_vms.length == 0
  when 'low'
    $evm.log("info", "GRIDPROCESSING Low Threshold processing  ")
    grid_vms = get_eligible_vms(ems_cluster,'off')
    power_on_vms(grid_vms, options[:number_of_vms])    unless grid_vms.length == 0
  else
    $evm.log("info", "GRIDPROCESSING Cannot continue without Utilization type of high, low or periodic")
    exit MIQ_OK
  end

  #
  # Exit method
  #
  $evm.log("info", "GRIDPROCESSING ===== EVM Automate Method: <#{@method}> Ended")
  exit MIQ_OK
  #
  # Set Ruby rescue behavior
  #
rescue => err
  $evm.log("error", "<#{@method}>: [#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
