###################################
#
# EVM Automate Method: ServiceNow_CMDB_CI_Host
#
# Notes: This method calls Service-Now via SOAP/XML to insert/update
# the manageiq_template table in Service-Now
#
###################################
begin
  @method = 'ServiceNow_CMDB_CI_Host'
  @log_prefix = "[#{@method}]"
  $evm.log("info", "#{@log_prefix} - EVM Automate Method Started")

  # Turn of verbose logging
  @debug = true


  ###########################
  #
  # Method: callServiceNow
  #
  # inputs: username, password, wsdl_uri, soap_action, body_hash
  #
  ###########################
  def callServiceNow(username, password, wsdl_uri, soap_action, body_hash)
    # Require Savon Ruby Gem
    require "savon"
    require 'httpi'

    HTTPI.log_level = :info # changing the log level
    HTTPI.log       = false # diable HTTPI logging
    HTTPI.adapter   = :net_http # [:httpclient, :curb, :net_http]

    # Setup Savon Configuration
    Savon.configure do |config|
      config.log        = false      # disable Savon logging
      config.log_level  = :fatal      # changing the log level
    end

    # Set up SOAP Connection to WSDL
    client = Savon::Client.new do |wsdl,http,wsse|
      wsdl.document = wsdl_uri
      http.auth.ssl.verify_mode = :none
      http.auth.basic username, password
    end

    $evm.log("info","#{@log_prefix} - Namespace: #{client.wsdl.namespace.inspect}") if @debug
    $evm.log("info","#{@log_prefix} - Endpoint: #{client.wsdl.endpoint.inspect}") if @debug
    $evm.log("info","#{@log_prefix} - Actions: #{client.wsdl.soap_actions.inspect}") if @debug

    # Call Service-Now
    # Service-Now SOAP Actions: [:insert, :update, :delete_multiple, :get_keys, :delete_record, :get, :get_records]
    response = client.request soap_action do |soap|
      # Build the body of the XML elements
      soap.body = body_hash
    end

    # Convert xml response to a hash
    hash_response = response.to_hash["#{soap_action}_response".to_sym]
    $evm.log("info","#{@log_prefix} - Inspecting Service-Now #{soap_action} Response:<#{hash_response.inspect}>")
    return hash_response
  end


  vm = $evm.root['host']
  raise "#{@log_prefix} - Host object not found" if vm.nil?
  $evm.log("info","#{@log_prefix} - Detected Host:<#{host.name}>")

  # Get username from model else set below
  username = nil
  username ||= $evm.object['username']

  # Get password from model else set below
  password = nil
  password ||= $evm.object.decrypt('password')

  # Get wsdl_uri from model else set below
  wsdl_uri = nil
  wsdl_uri ||= $evm.object['wsdl_uri']

  get_records_hash = {'u_manageiq_template' => host.name}
  $evm.log("info","#{@log_prefix} - Searching Service-Now with parameters:<#{get_records_hash.inspect}>")
  record_hash = callServiceNow(username, password, wsdl_uri, :get_records, get_records_hash)

  # If a record is found then update the record
  unless record_hash.nil?

    # Strip out the service-now variables that we cannot modify except for :sys_id
    update_hash = record_hash[:get_records_result].dup.delete_if {|k,v| k.to_s[0,2] != 'u_' && k != :sys_id}

    # Convert symbols to strings as Service-Now does not work well with hash symbols
    update_hash = Hash[update_hash.map{ |k, v| [k.to_s, v] }]

    # Update guids for template and EMS
    update_hash['u_ems_guid'] = host.ext_management_system.guid
    update_hash['u_template_guid'] = host.guid
    $evm.log("info","#{@log_prefix} - Updating Service-Now cmdb_ci_esx_server record for ESX Host:<#{host.name}> with <#{update_hash.inspect}>")
    updateCI = callServiceNow(username, password, wsdl_uri, :update, update_hash)
  else
    # If a record is NOT found then insert a new record
    insert_hash = {}
    $evm.log("info","#{@log_prefix} - No Service-Now ESX Host found for Host:<#{host.name}>")

    # Build hash of fields to be inserted here
    insert_hash['discovery_source'] = "ManageIQ EVM"
    insert_hash['name'] = host.name
    insert_hash['url'] = "https://#{host.ipaddress}" unless host.ipaddress.nil?
    insert_hash['asset_tag'] = host.hardware.service_tag unless host.hardware.service_tag.nil?
    insert_hash['ip_address'] = host.ipaddress unless host.ipaddress
    insert_hash['manufacturer'] = host.hardware.manufacturer unless host.hardware.manufacturer.nil?
    insert_hash['model_number'] = host.hardware.model unless host.hardware.model.nil?
    insert_hash['os'] = host.vmm_product unless host.vmm_product.nil?
    insert_hash['os_service_pack'] = host.operating_system.service_pack unless host.operating_system.service_pack.nil?
    insert_hash['os_version'] = host.vmm_version unless host.vmm_version
    insert_hash['cpu_speed'] = host.hardware.cpu_speed unless host.hardware.cpu_speed.nil?
    insert_hash['ram'] = host.hardware.memory_cpu
    insert_hash['cpu_count'] = host.hardware.numvcpus
    insert_hash['cpu_type'] = host.hardware.cpu_type
    insert_hash['vendor'] = host.vmm_vendor

    #                  <category></category>
    #                  <comments></comments>
    #                  <company></company>
    #                  <correlation_id></correlation_id>
    #                  <cpu_core_count></cpu_core_count>
    #                  <></cpu_count>
    #                  <></cpu_manufacturer>
    #                  <dns_domain></dns_domain>
    #                  <dr_backup></dr_backup>
    #                  <fault_count></fault_count>
    #                  <firewall_status></firewall_status>
    #                  <first_discovered></first_discovered>
    #                  <floppy></floppy>
    #                  <form_factor></form_factor>
    #                  <gl_account></gl_account>
    #                  <hardware_status></hardware_status>
    #                  <hardware_substatus></hardware_substatus>
    #                  <host_domain></host_domain>
    #                  <host_name></host_name>
    #                  <install_date></install_date>
    #                  <install_status></install_status>
    #                  <invoice_number></invoice_number>
    #                  <justification></justification>
    #                  <last_discovered></last_discovered>
    #                  <lease_id></lease_id>
    #                  <location></location>
    # insert_hash['mac_address'] =
    #                  <managed_by></managed_by>
    #                  <operational_status></operational_status>
    #                  <order_date></order_date>
    #                  <os_address_width></os_address_width>
    #                  <os_domain></os_domain>
    #                  <owned_by></owned_by>

    #                  <po_number></po_number>
    #<purchase_date>
    #<repair_contract_id></repair_contract_id>
    #<returned_from_repair></returned_from_repair>
    #<sent_for_repair></sent_for_repair>
    #<serial_number>
    #<short_description></short_description>
    #<start_date></start_date>
    #<subcategory></subcategory>
    #<support_group></support_group>
    #<supported_by></supported_by>
    #<url></url>
    #warranty_expiration>

    $evm.log("info","#{@log_prefix} - Inserting Service-Now cmdb_ci_esx_server record for ESX Host:<#{host.name}> with <#{insert_hash.inspect}>")
    insertCI = callServiceNow(username, password, wsdl_uri, :insert, insert_hash)
  end

  #
  # Exit method
  #
  $evm.log("info", "#{@log_prefix} - EVM Automate Method Ended")
  exit MIQ_OK

  #
  # Set Ruby rescue behavior
  #
rescue => err
  $evm.log("error", "#{@log_prefix} - [#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
