###################################
#
# CFME Automate Method: POC_PreProvision
#
# Notes: This default method is used to apply PreProvision customizations for VMware, RHEV and Amazon provisioning
#
###################################
begin
  # Method for logging
  def log(level, message)
    @method = 'POC_PreProvision'
    $evm.log(level, "#{@method} - #{message}")
  end

  # dump_root
  def dump_root()
    log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
    $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
    log(:info, "Root:<$evm.root> End $evm.root.attributes")
    log(:info, "")
  end

  #################################
  #
  # Method: process_vmware
  # Notes: Process vmware specific provisioning options
  #
  #################################
  def process_vmware(prov)
    # Choose the sections to process
    set_vlan = false
    set_folder = true
    set_resource_pool = false
    set_notes = true

    # Get information from the template platform
    template = prov.vm_template
    product  = template.operating_system['product_name'].downcase
    bitness = template.operating_system['bitness']
    log(:info, "Template:<#{template.name}> Vendor:<#{template.vendor}> Product:<#{product}> Bitness:<#{bitness}>")

    if set_vlan
      ###################################
      # Was a VLAN selected in dialog?
      # If not you can set one here.
      ###################################
      default_vlan = "vlan1"
      default_dvs = "portgroup1"

      if prov.get_option(:vlan).nil?
        log(:info, "Provisioning object <:vlan> updated with <#{default_vlan}>")
        prov.set_vlan(default_vlan)
        #prov.set_dvs(default_dvs)
      end
    end

    if set_folder
      ###################################
      # Drop the VM in the targeted folder if no folder was chosen in the dialog
      # The vCenter folder must exist for the VM to be placed correctly else the
      # VM will placed along with the template
      # Folder starts at the Data Center level
      ###################################
      datacenter = template.v_owning_datacenter
      default_folder = "#{datacenter}/Discovered virtual machine"

      if prov.get_option(:placement_folder_name).nil?
        prov.get_folder_paths.each { |key, path| log(:info, "Eligible folders:<#{key.inspect}> - <#{path.inspect}>") }
        prov.set_folder(default_folder)
        log(:info, "Provisioning object <:placement_folder_name> updated with <#{default_folder}>")
      else
        log(:info, "Placing VM in folder: <#{prov.get_option(:placement_folder_name)}>")
      end
    end

    if set_resource_pool
      if prov.get_option(:placement_rp_name).nil?
        ############################################
        # Find and set the Resource Pool for a VM:
        ############################################
        default_resource_pool = 'MyResPool'
        respool = prov.eligible_resource_pools.detect {|c| c.name.casecmp(default_resource_pool)==0}
        log(:info, "Provisioning object <:placement_rp_name> updated with <#{respool.name}>")
        prov.set_resource_pool(respool)
      end
    end

    if set_notes
      ###################################
      # Set the VM Description and VM Annotations  as follows:
      # The example would allow user input in provisioning dialog "vm_description"
      # to be added to the VM notes
      ###################################
      # Stamp VM with custom description
      unless prov.get_option(:vm_description).nil?
        vmdescription = prov.get_option(:vm_description)
        prov.set_option(:vm_description,vmdescription)
        log(:info, "Provisioning object <:vmdescription> updated with <#{vmdescription}>")
      end

      # Setup VM Annotations
      vm_notes =  "Owner: #{prov.get_option(:owner_first_name)} #{prov.get_option(:owner_last_name)}"
      vm_notes += "\nEmail: #{prov.get_option(:owner_email)}"
      vm_notes += "\nSource Template: #{template.name}"
      vm_notes += "\nCustom Description: #{vmdescription}" unless vmdescription.nil?
      prov.set_vm_notes(vm_notes)
      log(:info, "Provisioning object <:vm_notes> updated with <#{vm_notes}>")
    end
  end # end process_vmware


  #################################
  #
  # Method: process_redhat
  # Notes: Process redhat specific provisioning options
  #
  #################################
  def process_redhat(prov)
    # Choose the sections to process
    set_vlan = true
    set_notes = true

    # Get information from the template platform
    template = prov.vm_template
    product  = template.operating_system['product_name'].downcase
    log(:info, "Template:<#{template.name}> Vendor:<#{template.vendor}> Product:<#{product}>")

    if set_vlan
      # Set default VLAN here if one was not chosen in the dialog?
      default_vlan = "rhevm"

      if prov.get_option(:vlan).nil?
        prov.set_vlan(default_vlan)
        log(:info, "Provisioning object <:vlan> updated with <#{default_vlan}>")
      end
    end

    if set_notes
      ###################################
      # Set the VM Description and VM Annotations  as follows:
      # The example would allow user input in provisioning dialog "vm_description"
      # to be added to the VM notes
      ###################################
      # Stamp VM with custom description
      unless prov.get_option(:vm_description).nil?
        vmdescription = prov.get_option(:vm_description)
        prov.set_option(:vm_description,vmdescription)
        log(:info, "Provisioning object <:vmdescription> updated with <#{vmdescription}>")
      end

      # Setup VM Annotations
      vm_notes =  "Owner: #{prov.get_option(:owner_first_name)} #{prov.get_option(:owner_last_name)}"
      vm_notes += "\nEmail: #{prov.get_option(:owner_email)}"
      vm_notes += "\nSource Template: #{template.name}"
      vm_notes += "\nCustom Description: #{vmdescription}" unless vmdescription.nil?
      prov.set_vm_notes(vm_notes)
      log(:info, "Provisioning object <:vm_notes> updated with <#{vm_notes}>")
    end
  end


  #################################
  #
  # Method: process_amazon
  # Notes: Process Amazon specific provisioning options
  #
  #################################
  def process_amazon(prov)

  end # end process_amazon


  #################################
  #
  # Method: process_openstack
  # Notes: Process Openstack specific provisioning options
  #
  #################################
  def process_openstack(prov)

  end # end process_openstack

  log(:info, "CFME Automate Method Started")
  
  # dump all root attributes to the log
  dump_root

  # Get provisioning object
  prov = $evm.root['miq_provision']
  log(:info, "Provision:<#{prov.id}> Request:<#{prov.miq_provision_request.id}> Type:<#{prov.type}>")

  # Build case statement to determine which type of processing is required
  case prov.type
  when 'MiqProvisionRedhatViaIso', 'MiqProvisionRedhatViaPxe', 'MiqProvisionRedhat';  process_redhat(prov)
  when 'MiqProvisionVmware', 'MiqProvisionVmwareViaPxe';                              process_vmware(prov)
  when 'MiqProvisionAmazon';                                                          process_amazon(prov)
  when 'MiqProvisionOpenstack';                                                       process_openstack(prov)
  else log(:info, "Provision Type:<#{prov.type}> does not match, skipping processing")
  end

  # Log all of the provisioning options to the automation.log
  prov.options.each { |k,v| log(:info, "Provisioning Option Key:<#{k.inspect}> Value:<#{v.inspect}>") }

  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

  # Set Ruby rescue behavior
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_STOP
end
