###################################
#
# CFME Automate Method: POC_VMProvisionRequest
#
# Author: Kevin Morey
#
# Notes: Simple method to kick off an provisioning request from a service dialog or an automation request via API call
#
# Inputs: dialog_option_[0-9]_template, dialog_option_[0-9]_sizing, dialog_tag_[0-9]_environment, etc...
#
###################################
begin
  def log(level, msg, update_message=false)
    @method = 'POC_VMProvisionRequest'
    $evm.log(level,"#{@method}: #{msg}")
    $evm.root['automation_task'].message = msg if $evm.root['automation_task'] && update_message
    $evm.root['service_template_provision_task'].message = msg if $evm.root['service_template_provision_task'] && update_message
  end

  # dump_root
  def dump_root()
    log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
    $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
    log(:info, "Root:<$evm.root> End $evm.root.attributes")
    log(:info, "")
  end

  # Look for service dialog variables in the dialog options hash that start with "dialog_tag_[0-9]",
  def get_tags_hash(dialog_options)
    # Setup regular expression for service dialog tags
    tags_regex = /^dialog_tag_(\d*)_(.*)/
    tags_hash = {}
    # Loop through all of the tags and build an options_hash from them
    dialog_options.each do |k, v|
      if tags_regex =~ k
        build = $1
        tag_category = $2.to_sym
        tag_value = v.downcase
        log(:info, "build:<#{build.inspect}> tag_category:<#{tag_category.inspect}> tag_value:<#{tag_value.inspect}>")
        unless tag_value.blank?
          log(:info, "Adding category:<#{tag_category.inspect}> tag:<#{tag_value.inspect}> to tags_hash")
          (tags_hash[build] ||={})[tag_category] = tag_value
        end
      end
    end
    log(:info, "Inspecting tags_hash:<#{tags_hash.inspect}>")
    return tags_hash
  end

  # Look for service dialog variables in the dialog options hash that start with "dialog_option_[0-9]",
  def get_options_hash(dialog_options)
    options_hash = {}
    # Loop through all of the options and build an options_hash from them
    dialog_options_regex = /^dialog_option_(\d*)_(.*)/
    dialog_options.each do |k, v|
      if dialog_options_regex =~ k
        build = $1
        option_key = $2.to_sym
        option_value = v
        unless option_value.blank?
          log(:info, "Adding option_key:<#{option_key.inspect}> option_value:<#{option_value.inspect}> to options_hash")
          (options_hash[build] ||={})[option_key] = option_value
        end
      end
    end
    log(:info, "Inspecting options_hash:<#{options_hash.inspect}>")
    return options_hash
  end

  # Loop through all tags from the dialog and create the categories and tags automatically
  def process_tags(category, single_value, tag)
    # Convert to lower case and replace all non-word characters with underscores
    category_name = category.to_s.downcase.gsub(/\W/,'_')
    tag_name = tag.downcase.gsub(/\W/,'_')
    log(:info, "Converted category name:<#{category_name}> Converted tag name: <#{tag_name}>")

    # if the category exists else create it
    unless $evm.execute('category_exists?', category_name)
      log(:info, "Category <#{category_name}> doesn't exist, creating category")
      $evm.execute('category_create', :name => category_name, :single_value => single_value, :description => "#{category}")
    end
    # if the tag exists else create it
    unless $evm.execute('tag_exists?', category_name, tag_name)
      log(:info, "Adding new tag <#{tag_name}> in Category <#{category_name}>")
      $evm.execute('tag_create', category_name, :name => tag_name, :description => "#{tag}")
    end
  end

  # get template guid based on incoming dialog_option_[1-9]_[template|name]
  def get_template_guid(options_hash)
    template_name = options_hash[:template] || options_hash[:name]
    log(:info, "template_name: <#{template_name}>")

    # Search for all templates that have a management system & are tagged with prov_scope/all & match the option template or name
    # Then load balance them across different vCenters
    templates = $evm.vmdb('miq_template').all.select do |template|
      template.ext_management_system && template.tagged_with?('prov_scope', 'all') && template.name == template_name
    end.sort { |template1, template2| template1.ext_management_system.vms.count <=> template2.ext_management_system.vms.count }

    # if no templates were found
    raise 'No templates found!' if templates.empty?

    # get the first template in the list
    template = templates.first

    # set the template and the guid
    options_hash[:template] = template.name
    options_hash[:guid] = template.guid
    return template
  end

  # use this to name the vm or set it to changeme to run through the vm_name automate method
  def get_vm_name(options_hash)
    if options_hash[:vm_name].nil?
      options_hash[:vm_name] = 'changeme'
    end
  end

  # use this to determine the provision_type. Valid types are vmware, pxe
  def get_provision_type(options_hash)
    options_hash[:provision_type] = 'vmware'
  end

  # use this section to set the network/vlan
  def get_network(options_hash)
    if options_hash[:vlan].nil?

    end
  end

  # use this method to define extra provisioning options
  def get_extra_options(options_hash, tags_hash)

    case tags_hash[:environment]
    when 'dev'
      #If tag environment = dev, retire vm in one week (in seconds) and warn 3 days prior
      options_hash[:retirement] = 604800
      options_hash[:retirement_warn] = 259200
    when 'test'
      # if tag environment = test, retire vm in two days (in seconds) with no warning
      options_hash[:retirement] = 172800
    else
      # Default retirement of 4 weeks and warn 1 week if no environment tag is chosen
      options_hash['retirement'] = 2419200
      options_hash['retirement_warn'] = 604800
    end
    # Set the VM to autostart after provisioning
    options_hash[:vm_auto_start] = 'true'

    # Force BestFit to select the best host and storage
    options_hash[:placement_auto] = 'true'
  end

  # use this to define sizing parameters
  def get_sizing_parms(options_hash)
    unless options_hash[:sizing].nil?
      case options_hash[:sizing]
      when 'small'
        options_hash[:cores_per_socket] = '1'
        options_hash[:vm_memory] = '2048'
      when 'medium'
        options_hash[:cores_per_socket] = '2'
        options_hash[:vm_memory] = '2048'
      when 'large'
        options_hash[:cores_per_socket] = '4'
        options_hash[:vm_memory] = '4096'
      when 'xlarge'
        options_hash[:cores_per_socket] = '8'
        options_hash[:vm_memory] = '8192'
      else
        # Default to xtra small
        options_hash[:cores_per_socket] = '1'
        options_hash[:vm_memory] = '1024'
      end
    end
  end

  # remove_service
  def remove_service(stp_task)
    # Get destination service object
    service = stp_task.destination
    unless service.nil?
      log(:info, "Removing Service:<#{service.name}> Id:<#{service.id}> from CFME database")
      service.remove_from_vmdb
    end
  end


  # Populate arguments and submit provision request using incoming tags_hash and options_hash
  def build_provision_request(tags_hash, options_hash)
    user = $evm.root['user']
    if user.nil?
      user_id = 'admin'
      user_mail = $evm.object['to_email_address'] || 'cloud.admin@exmaple.com'
      user_first = 'cloud'
      user_last = 'admin'
    else
      user_id = user.userid
      if user.email.nil?
        user_mail = $evm.object['to_email_address'] || 'cloud.admin@exmaple.com'
      else
        user_mail = user.email
      end
      user_first = user.first_name || 'cloud'
      user_last = user.last_name || 'user'
    end

    # arg1 = version
    args = ['1.1']

    # arg2 = templateFields
    args << "name=#{options_hash[:template]}|request_type=template|guid=#{options_hash[:guid]}"

    # arg3 = vmFields
    valid_vm_args = [:vm_name, :cores_per_socket, :vm_memory, :number_of_vms, :provision_type, :vm_auto_start, :vlan, :retirement, :retirement_warn]
    valid_vm_args += [:vm_description, :number_of_sockets, :linked_clone, :vm_prefix, :network_adapters, :placement_auto]
    vm_args = options_hash.select { |k, v| valid_vm_args.include? k }.to_a.collect { |v| v.join('=') }.join('|')
    #Remove any hash keys there were used in the vm_args
    valid_vm_args.each { |k| options_hash.delete(k) }
    args << vm_args

    # arg4 = requester
    args << "user_name=#{user_id}|owner_first_name=#{user_first}|owner_last_name=#{user_last}|owner_email=#{user_mail}"

    # arg5 = tags
    args << tags_hash.collect { |k, v| "#{k.to_s}=#{v}" }.join('|')

    # arg6 = WS Values | put all dialog_option variables in ws_values hash for later use in the state machine
    args << options_hash.collect { |k, v| "#{k.to_s}=#{v}" }.join('|')

    # arg7 = emsCustomAttributes
    args << nil

    # arg8 = miqCustomAttributes
    args << nil

    log(:info, "Building provision request with the following arguments: <#{args.inspect}>")
    #raise 'BAIL'
    request_id = $evm.execute('create_provision_request', *args)
    log(:info, "Inspecting request_id: <#{request_id.inspect}>")
    return request_id
  end


  # Beginning
  log(:info, 'CFME Automate Method Started', true)

  # dump all root attributes to the log
  dump_root

  # build a hash of dialog_options
  dialog_options = Hash[$evm.root.attributes.sort.collect { |k, v| [k, v] if k.starts_with?('dialog_') }]
  log(:info, "dialog_options: #{dialog_options.inspect}")

  tags_hash = get_tags_hash(dialog_options)
  log(:info, "Inspecting tags_hash: #{tags_hash.inspect}")
  options_hash = get_options_hash(dialog_options)
  log(:info, "Inspecting tags_hash: #{options_hash.inspect}")

  builds = {}
  options_hash.each do |build, options|
    sequence = options["dialog_option_#{build}_build_sequence"]
    (builds[sequence] ||= []) << build
  end

  # looping through all build sequences
  build_sequences = builds.keys.sort
  build_sequences.each do |sequence|
    matching_builds = builds[sequence]
    # concurrently build all builds in 'matching_builds'
    log(:info, "Inspecting: matching_builds = <#{matching_builds.inspect}>")

    matching_builds.each do |build|
      unless options_hash[0].nil?
        matching_options_hash = options_hash[0].merge(options_hash[build] || {})
      else
        matching_options_hash = options_hash[build] || {}
      end

      unless tags_hash[0].nil?
        matching_tags_hash = tags_hash[0].merge(tags_hash[build] || {})
      else
        matching_tags_hash = tags_hash[build] || {}
      end
      matching_tags_hash.each do |category, tag|
        process_tags( category, true, tag )
      end

      # Get template, guid
      template = get_template_guid(matching_options_hash)

      # Generate a VM name else set it to changeme
      get_vm_name(matching_options_hash)

      # Set the provision type
      get_provision_type(matching_options_hash)

      # Get network based on management system name
      get_network(matching_options_hash) unless matching_options_hash[:vlan].nil?

      # Set vCPU and vRAM based on option_sizing parameters
      get_sizing_parms(matching_options_hash) unless matching_options_hash[:sizing].nil?

      # hard code/override any options in this method
      get_extra_options(matching_options_hash, matching_tags_hash)

      # You may hard code any overrides and any additional parameters here
      log(:info, "Build:#{build} - matching_tags_hash #{matching_tags_hash}")
      log(:info, "Build:#{build} - matching_options_hash #{matching_options_hash}")

      # call build_provision_request using matching_tags_hash and matching_options_hash
      request_id = build_provision_request(matching_tags_hash, matching_options_hash)
      log(:info, "Build:#{build} - Provision request #{request_id} for #{matching_options_hash[:vm_name]} successfully submitted", true)
    end # matching_builds.each do
  end

  # Exit method
  log(:info, 'CFME Automate Method Ended', true)
  $evm.root['automation_task'].finished("#{@method} completed successfully") unless $evm.root['automation_task'].nil?
  remove_service($evm.root['service_template_provision_task']) unless $evm.root['service_template_provision_task'].nil?
  exit MIQ_OK

  # Set Ruby rescue behavior
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  $evm.root['automation_task'].finished("#{err}") unless $evm.root['automation_task'].nil?
  $evm.root['service_template_provision_task'].finished("#{err}") unless $evm.root['service_template_provision_task'].nil?
  remove_service('service_template_provision_task') unless $evm.root['service_template_provision_task'].nil?
  exit MIQ_ABORT
end
