###################################
#
# EVM Automate Method: RunProgramInGuest
#
# Notes: This method will run a program on a guest VM via VIX SDK/API.
# 
# Requirements: 
#   a) The latest VIX SDK can be downloaded from: http://www.vmware.com/support/developer/vix-api
# and must be installed on the EVM Appliance for this script to function
#   b) Guest VM must have VMware Tools running
#
# Model Inputs: vmrun_path,linux_guest_username,linux_guest_password,linux_guest_program,
#  windows_guest_username,windows_guest_password,windows_guest_program
#
###################################
begin

  def log (level, msg)
    @method = 'RunProgramInGuest'
    $evm.log(level, "<#{@method}>: #{msg}")
  end

  @method = 'RunProgramInGuest'
  log(:info, "EVM Automate Method Started")

  # Turn of debugging
  @debug = true

  ####################################
  #
  # Method: runProgramInGuest
  # Notes: call vmrun with parameters
  # Returns: true|false
  #
  ####################################
  def runProgramInGuest(vm, vmrun_path, ems, guest_username, guest_password, guest_program, guest_options)
    cmd  = "#{vmrun_path}"
    cmd += " -h https://#{ems.ipaddress}:443/sdk"
    cmd += " -u \"#{ems.authentication_userid}\""
    cmd += " -p \"#{ems.authentication_password}\""
    cmd += " -gu \"#{guest_username}\""
    cmd += " -gp \"#{guest_password}\""
    cmd += " runProgramInGuest"
    cmd += " \"[#{vm.storage_name}]#{vm.location}\""
    cmd += " \"#{guest_program}\""
    cmd += " #{guest_options}" unless guest_options.blank?


    log(:info, "Calling external cmd:<#{cmd}>") if @debug
    results = system(cmd)
    #$evm.log("info","VMrun command <runProgramInGuest> completed with results:<#{results}>")
    return results
  end

  ####################################
  #
  # Method: fileExistsInGuest
  # Notes: call vmrun with parameters
  # Returns: true|false
  #
  ####################################
  def fileExistsInGuest(vm, vmrun_path, ems, guest_username, guest_password, guest_program)

    cmd  = "#{vmrun_path}"
    cmd += " -h https://#{ems.ipaddress}:443/sdk"
    cmd += " -u \"#{ems.authentication_userid}\""
    cmd += " -p \"#{ems.authentication_password}\""
    cmd += " -gu \"#{guest_username}\""
    cmd += " -gp \"#{guest_password}\""
    cmd += " fileExistsInGuest"
    cmd += " \"[#{vm.storage_name}]#{vm.location}\""
    cmd += " \"#{guest_program}\""

    log(:info, "Calling external cmd:<#{cmd}>") if @debug
    results = system(cmd)
    #$evm.log("info","VMrun command <fileExistsInGuest> completed with results:<#{results}>")
    return results
  end


  ###################################
  #
  # Method: boolean
  # Returns: true/false
  ###################################
  def boolean(string)
    return true if string == true || string =~ (/(true|t|yes|y|1)$/i)
    return false if string == false || string.nil? || string =~ (/(false|f|no|n|0)$/i)

    # Return false if string does not match any of the above
    log(:info, " - Invalid boolean string:<#{string}> detected. Returning false") if @debug
    return false
  end

  vm = nil
  case $evm.root['vmdb_object_type']
    when 'vm'
      vm = $evm.root['vm']
      log(:info, "Got vm object from $evm.root['vm']")
    when 'miq_provision'
      vm = $evm.root['miq_provision'].vm
      log(:info, "Got vm object from $evm.root['miq_provision']")
  end
  raise "#{@method} - VM object not found" if vm.nil?
  log(:info, " - VM Found:<#{vm.name}>")

  # Since this is provisioning we need to put in retry logic to wait until IP Addresses are populated.
  unless vm.ipaddresses.empty?
    non_zeroconf = false
    vm.ipaddresses.each do |ipaddr|
      non_zeroconf = true unless ipaddr.match(/^(169.254|0)/)
      log(:info, "VM:<#{vm.name}> IP Address found #{ipaddr} (#{non_zeroconf})")
    end
    if non_zeroconf
      log(:info, "VM:<#{vm.name}> IP addresses:<#{vm.ipaddresses.inspect}> present.")
      $evm.root['ae_result'] = 'ok'
    else
      log(:warn, "VM:<#{vm.name}> IP addresses:<#{vm.ipaddresses.inspect}> not present.")
      $evm.root['ae_result'] = 'retry'
      $evm.root['ae_retry_interval'] = '1.minute'
      exit MIQ_OK
    end
  else
    log(:warn, "VM:<#{vm.name}> IP addresses:<#{vm.ipaddresses.inspect}> not present.")
    $evm.root['ae_result'] = 'retry'
    $evm.root['ae_retry_interval'] = '1.minute'
    exit MIQ_OK
  end

  # Sleep for VM

  # Get OS Type from the VM platform
  product  = vm.operating_system['product_name'].downcase
  raise "#{@method} - VM OS product cannot be found" if vm.nil?
  log(:info, " - VM Source Product:<#{product}>") if @debug

  ems = vm.ext_management_system
  raise "EMS object not found" if ems.nil?
  log(:info, " - VM Source EMS:<#{ems.name}>") if @debug

  # Set linux_enabled below else get value from model
  linux_enabled = nil
  linux_enabled ||= $evm.object['linux_enabled']

  # Set window_enabled below else get value from model
  windows_enabled = nil
  windows_enabled ||= $evm.object['windows_enabled']


  # Set vmrun_path below or get from model
  #vmrun_path = '/usr/bin/vmrun'
  vmrun_path = nil
  vmrun_path ||= $evm.object['vmrun_path']
  raise "#{@method} - File '#{vmrun_path}' does not exist" unless File.exist?(vmrun_path)


  # Set guest_user and guest_password and guest_program
  if product.include?("linux") && boolean(linux_enabled)
    guest_username = $evm.object['linux_guest_username']
    guest_password = $evm.object.decrypt('linux_guest_password')
    guest_program  = $evm.object['linux_guest_program']
    guest_options = $evm.object['linux_options']
    log(:info, "VIX Processing on Linux detected")
  elsif product.include?("windows") && boolean(windows_enabled)
    guest_username = $evm.object['windows_guest_username']
    guest_password = $evm.object.decrypt('windows_guest_password')
    guest_program  = $evm.object['windows_guest_program']
    guest_options = $evm.object['windows_options']
    log(:info, "VIX Processing on Windows detected")
  else
    log(:info, "Skipping method")
    exit MIQ_OK
  end
  
  raise "#{@method} - Invalid Guest OS Username:<#{guest_username}>" if guest_username.nil?
  raise "#{@method} - Invalid Guest OS Password:<#{guest_password}>" if guest_password.nil?
  raise "#{@method} - Invalid Guest OS Program:<#{guest_program}>" if guest_program.nil?

  fileExists = fileExistsInGuest(vm, vmrun_path, ems, guest_username, guest_password, guest_program)

  if product.include?("windows")
    unless fileExists
      log(:info, "fileExists failed (#{fileExists}) using #{guest_username}/#{guest_password}, trying 'Administrator' as an alternative")
      guest_username = "Administrator"
      fileExists = fileExistsInGuest(vm, vmrun_path, ems, guest_username, guest_password, guest_program)
    end

    unless fileExists
      log(:info, "fileExists failed (#{fileExists}) using #{guest_username}/#{guest_password}, trying '.\\Administrator' as an alternative")
      guest_username = ".\\Administrator"
      fileExists = fileExistsInGuest(vm, vmrun_path, ems, guest_username, guest_password, guest_program)
    end

    unless fileExists
      guest_password = "Intuit01!"
      log(:info, "fileExists failed using #{guest_username}/#{guest_password}, trying alternate password")
      fileExists = fileExistsInGuest(vm, vmrun_path, ems, guest_username, guest_password, guest_program)
    end
  end
  # Check to see if guest_program exists
  if fileExists
    log(:info, "VMrun command <fileExistsInGuest> completed with results:<#{fileExists}>") if @debug
    # Call runProgramInGuest method
    results = runProgramInGuest(vm, vmrun_path, ems, guest_username, guest_password, guest_program, guest_options)
    if results
      log(:info, "VMrun command <runProgramInGuest> completed with results:<#{results}>") if @debug
    else
      log(:info, "VMrun command <runProgramInGuest> failed with results:<#{results}>") if @debug
    end
  else
    log(:info, "VMrun command <fileExistsInGuest> failed with results:<#{fileExists}>") if @debug
  end

  #
  # Exit method
  #
  log(:info, "EVM Automate Method Ended")
  exit MIQ_OK

  #
  # Set Ruby rescue behavior
  #
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
