###################################
#
# EVM Automate Method: RunProgramInGuestViaButton
#
# Notes: This method will run a program on a guest VM via VIX SDK/API.
# 
# Requirements: 
#   a) The latest VIX SDK can be downloaded from: http://www.vmware.com/support/developer/vix-api
# and must be installed on the EVM Appliance for this script to function
#   b) Guest VM must have VMware Tools running
#   c) Guest VM must already have the script being run
#
# Model Inputs: vm, vmrun_path,linux_guest_username,linux_guest_password,linux_guest_program,
#  windows_guest_username,windows_guest_password,windows_guest_program
#
###################################
# Method for logging
def log(level, message)
  @method = 'RunProgramInGuestViaButton'
  $evm.log(level, "#{@method} - #{message}")
end

begin
  log(:info, "EVM Automate Method Started")


  def copyFileFromHostToGuest(vm, vmrun_path, ems, guest_username, guest_password, srcfile, destfile)
    cmd = "#{vmrun_path}"
    cmd += " -h https://#{ems.ipaddress}:443/sdk"
    cmd += " -u \"#{ems.authentication_userid}\""
    cmd += " -p \"#{ems.authentication_password}\""
    cmd += " -gu \"#{guest_username}\""
    cmd += " -gp \"#{guest_password}\""
    cmd += " copyFileFromHostToGuest"
    cmd += " \"[#{vm.storage_name}]#{vm.location}\""
    cmd += " #{srcfile} #{destfile}"
    log(:info, "Calling external cmd:<#{cmd}>")
    results = system(cmd)
    return results
  end

  ####################################
  #
  # Method: runProgramInGuest
  # Notes: call vmrun with parameters
  # Returns: true|false
  #
  ####################################
  def runProgramInGuest(vm, vmrun_path, ems, guest_username, guest_password, guest_program)
    cmd  = "#{vmrun_path}"
    cmd += " -h https://#{ems.ipaddress}:443/sdk"
    cmd += " -u \"#{ems.authentication_userid}\""
    cmd += " -p \"#{ems.authentication_password}\""
    cmd += " -gu \"#{guest_username}\""
    cmd += " -gp \"#{guest_password}\""
    cmd += " runProgramInGuest"
    cmd += " \"[#{vm.storage_name}]#{vm.location}\""
    cmd += " \"#{guest_program}\""
    #cmd += " <Arguments go here>"

    log(:info,"Calling external cmd:<#{cmd}>")
    results = system(cmd)
    #log(:info,"VMrun command <runProgramInGuest> completed with results:<#{results}>")
    return results
  end

  ####################################
  #
  # Method: fileExistsInGuest
  # Notes: call vmrun with parameters
  # Returns: true|false
  #
  ####################################
  def fileExistsInGuest(vm, vmrun_path, ems, guest_username, guest_password, guest_program)
    cmd  = "#{vmrun_path}"
    cmd += " -h https://#{ems.ipaddress}:443/sdk"
    cmd += " -u \"#{ems.authentication_userid}\""
    cmd += " -p \"#{ems.authentication_password}\""
    cmd += " -gu \"#{guest_username}\""
    cmd += " -gp \"#{guest_password}\""
    cmd += " fileExistsInGuest"
    cmd += " \"[#{vm.storage_name}]#{vm.location}\""
    cmd += " \"#{guest_program}\""

    log(:info,"Calling external cmd:<#{cmd}>")
    results = system(cmd)
    #log(:info,"VMrun command <fileExistsInGuest> completed with results:<#{results}>")
    return results
  end


  ###################################
  #
  # Method: boolean
  # Returns: true/false
  ###################################
  def boolean(string)
    return true if string == true || string =~ (/(true|t|yes|y|1)$/i)
    return false if string == false || string.nil? || string =~ (/(false|f|no|n|0)$/i)

    # Return false if string does not match any of the above
    log(:info,"Invalid boolean string:<#{string}> detected. Returning false")
    return false
  end

  # Get Provisioning object
  vm = $evm.root['vm']
  raise "VM object not found" if vm.nil?
  log(:info,"VM Found:<#{vm.name}>")

  # Get OS Type from the VM platform
  product  = vm.operating_system['product_name'].downcase
  raise "VM OS product cannot be found" if vm.nil?
  log(:info,"VM Source Product:<#{product}>")

  ems = vm.ext_management_system
  raise "EMS object not found" if ems.nil?
  log(:info,"VM Source EMS:<#{ems.name}>")

  # Set linux_enabled below else get value from model
  linux_enabled = nil
  linux_enabled ||= $evm.object['linux_enabled']

  # Set window_enabled below else get value from model
  windows_enabled = nil
  windows_enabled ||= $evm.object['windows_enabled']


  # Set vmrun_path below or get from model
  #vmrun_path = '/usr/bin/vmrun'
  vmrun_path = nil
  vmrun_path ||= $evm.object['vmrun_path']
  raise "File '#{vmrun_path}' does not exist" unless File.exist?(vmrun_path)


  # Set guest_user and guest_password and guest_program
  if product.include?("linux") && boolean(linux_enabled)
    guest_username = $evm.object['linux_guest_username']
    guest_password = $evm.object.decrypt('linux_guest_password')
    guest_program = $evm.object['linux_guest_program']
    log(:info,"VIX Processing on Linux detected")
  elsif product.include?("windows") && boolean(windows_enabled)
    guest_username = $evm.object['windows_guest_username']
    guest_password = $evm.object.decrypt('windows_guest_password')
    guest_program = $evm.object['windows_guest_program']
    log(:info,"VIX Processing on Windows detected")
  else
    log(:info,"Skipping method")
    exit MIQ_OK
  end
  
  guest_program  = $evm.root['guest_program'] if guest_program.nil?


  raise "Invalid Guest OS Username:<#{guest_username}>" if guest_username.nil?
  raise "Invalid Guest OS Password:<#{guest_password}>" if guest_password.nil?
  raise "Invalid Guest OS Program:<#{guest_program}>" if guest_program.nil?

  fileExists = fileExistsInGuest(vm, vmrun_path, ems, guest_username, guest_password, guest_program)
  # Check to see if guest_program exists
  if fileExists
    log(:info,"VMrun command <fileExistsInGuest> completed with results:<#{fileExists}>")
    # Call runProgramInGuest method
    results = runProgramInGuest(vm, vmrun_path, ems, guest_username, guest_password, guest_program)
    if results
      log(:info,"VMrun command <runProgramInGuest> completed with results:<#{results}>")
      vm.custom_set("#{guest_program}", 'Success')
    else
      log(:info,"VMrun command <runProgramInGuest> failed with results:<#{results}>")
      vm.custom_set("#{guest_program}", 'Failed')      
    end
  else
    log(:info,"VMrun command <fileExistsInGuest> failed with results:<#{fileExists}>")
  end


  #
  # Exit method
  #
  log(:info, "EVM Automate Method Ended")
  exit MIQ_OK

  #
  # Set Ruby rescue behavior
  #
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
